;
;     Account Object
;                                        (C)2001 Omoikane Inc.

(define (configure-acc)
  
  ; vars
  (define context 'admin)
  (define reserve '())     ; list of (list <acc> <pass>)
  (define policy 'ws)      ; ws $B$^$?$O(B sv

  ; libs
  (define (user? a)
    (= 0 (sh "grep '^~a:' /etc/passwd >/dev/null" a)))
  
  ; methods
  (define (init c)
    (set! context c))
  
  (define (abort)
    (set! reserve '()))
  
  (define (passwd a p)
    (let ((tmp (unix-tempname)))
      (let ((r (cond
                 ((not (user? a))
                   (_ "No such user"))
                 ((not (= 0 (sh "touch ~a" tmp)))
                   (_ "Cannot exec touch command"))
                 ((not (= 0 (sh "chmod og-rw ~a" tmp)))
                   (_ "Cannot exec chown "))
                 ((not (and (fprintf tmp
                              (format #f "[QANDA]\n.*new UNIX password:.*\n~a\n\n" p))
                            (= 0 (system-at "/dev/null"
                                   (format #f "wrap -f ~a -- passwd ~a 2>&1 >/dev/null " tmp a)))))
                   (_ "passwd failed"))
                 (else
                   #t))))
        (sh "rm -f ~a" tmp)
        r)))
  
  (define (adduser a p name g)
    (let ((tmp (unix-tempname)))
      (let ((r (cond
                 ((user? a)
                   (_ "User already exists"))
                 ((not (= 0 (sh "touch ~a" tmp)))
                   (_ "Cannot exec touch command"))
                 ((not (= 0 (sh "chmod og-rw ~a" tmp)))
                   (_ "Cannot exec chown "))
                 ((not (and (fprintf tmp
                              (format #f
                                (string-append
                                  "[QANDA]\n.*new UNIX password:.*\n~a\n\n"
                                  "[QANDA]\n.*Full Name.*\n~a\n\n"
                                  "[QANDA]\n.*Room Number.*\n\n\n"
                                  "[QANDA]\n.*Phone.*\n\n\n"
                                  "[QANDA]\n.*Other .*\n\n\n"
                                  "[QANDA]\ninformation correct.*\ny\n\n")
                                p name))
                            (= 0 (system-at "/dev/null"
                                   (format #f "wrap -f ~a -- adduser --no-create-home ~a 2>&1 >/dev/null " tmp a)))))
                   (_ "adduser command failed"))
                 ((not (andmap (lambda (x)
                                 (= 0 (sh "addgroup ~a ~a" a x)))
                               g))
                   (_ "addgroup command failed"))
                 (else
                   #t))))
        (sh "rm -f ~a" tmp)
        r)))
  
  (define (deluser a)
    (= 0 (sh "userdel ~a" a)))
  
  (define (future a p name)
    (cond
      ((user? a)
        (_ "User already exists"))
      (else
        (let ((r (assoc a reserve)))
          (cond
            ((eq? #f r)
              (set! reserve (cons (list a p name) reserve))
              #t)
            (else
              (_ "User already planed to be created")))))))
  
  (define (force)
    (let ((r (map (lambda (x) (adduser (car x) (cadr x) (caddr x)))
                  (reverse reserve))))
      (set! reserve '())
      r))
  
  (define (get-user-list)
    (define (get i)
      (cond
        ((< 5000 i) '())
        (else
          (let ((p (unix-getpwuid i)))
            (cond
              ((eq? #f p) (get (1+ i)))
              (else (cons p (get (1+ i)))))))))
    (get 1000))
  
  (define (suggest)
    (let ((sv? (equal? policy 'sv)))
      (list
        (list "audio" (_ "audio device access") #t)
        (list "cdrom" (_ "CD-ROM access") (if sv? #f #t))
        (list "floppy" (_ "floppy disk access") (if sv? #f #t))
        (list "operator" (_ "system shutdown") (if sv? #f #t))
        (list "staff" (_ "/usr/local write access") (if sv? #f #t))
        (list "video" (_ "frame buffer access") (if sv? #f #t)))))
  
  (define (set-policy p)
    (case p
      ((sv ws)
        (set! policy p)
        #t)
      (else
        #f)))
  
  (make-object
    'abort           abort
    'adduser         adduser
    'deluser         deluser
    ;'future          future
    ;'force           force
    'get-user-list   get-user-list
    'init            init
    'passwd          passwd
    'shadow-on       (lambda () (= 0 (sh "shadowconfig on")))
    'set-policy      set-policy
    'suggest         suggest
    ))
    