
(define (configure-boot obj-part obj-mod)
  ; var
  (define context 'admin)
  (define entry '())
  (define root "/")
  (define opt "")
  (define timeout 13)
  (define logid 0)
  
  ; libs
  (define (done? f)
    (let ((r (getfile f)))
      (cond
        ((eq? #f r)
          #f)
        (else
          (sh "rm ~a" f)
          (cond
            ((not (null? (string-match r "Done")))
              #t)
            (else
              (fprintf (format #f "/tmp/grubfail-~a" logid) r)
              (set! logid (1+ logid))
              (cond
                ((not (null? (string-match r "Error 17:")))
                  1)
                (else
                  0))))))))
  
  (define (check-entry e)
    (cond
      ((or (not (list? e))
           (null? e))
        #f)
      (else
        (let ((l (length e)))
          (case (car e)
            ((L)
              (inst-log "debug : ~a ~a ~a ~a\n" 
                   (< 7 l)
                   (number? (list-ref e 3))
                   (number? (list-ref e 6))
                   (null? (string-match (list-ref e 7) "\t")))
              
              (and (< 7 l)
                   (number? (list-ref e 3))
                   (number? (list-ref e 6))
                   (null? (string-match (list-ref e 7) "\t"))))
            ((B)
              (= 3 l))
            ((P)
              (and (= 4 l)
                   (number? (list-ref e 3))))
            ((F)
              (and (= 5 l)
                   (number? (list-ref e 3))))
            (else
              #f))))))
  
  (define (grub-setup r t)
    (let* ((tmp (unix-tempname))
           (fin (lambda (m)
                  (sh "mv ~a /tmp/grub-setup.log" tmp)
                  m)))
      (cond
        ((not (= 0 (sh "echo -e 'root (~a)\nsetup (~a)'| /sbin/grub --device-map=~a/boot/grub/device-map >~a 2>&1"
                       r t root tmp)))
          (fin (_ "Failed to run grub command.")))
        ((= 1 (sh "grep 'succeeded' ~a >/dev/null" tmp))
          (fin (_ "Could not install grub (script failed). Check /tmp/grub-setup.log. \nThis might happen when you use USB FD. If so, you might need to modify /boot/grub/device-map. ")))
        ((not (= 0 (sh "grep 'succeeded' ~a >/dev/null" tmp)))
          (fin (_ "Could not confirm grub's result. (no grep?)")))
        (else
          (sh "rm ~a" tmp)
          #t))))
  
  ; $B%+!<%M%k%$%a!<%8$,B8:_$9$k%(%s%H%jCf:G=i$N%P!<%8%g%s$rC5$7$^$9(B
  (define (search-default-version)
    (let* ((r (obj-part 'path-mnt "/"))
           (suf (case (list-ref r 6)
                  ((linux-xfs) "mini-xfs")
                  (else "mini-woxfs"))))
      (define (search l)
        (cond
          ((null? l)
            (cons "" '()))
          (else
            (let* ((x (car l))
                   (v (string-gsub (string-gsub (list-ref x 4) ".*/vmlinuz-" "")
                        "-[^-]\\+$" "")))
              (cond
                ((unix-access? (format #f "~a/boot/vmlinuz-~a-~a" root v suf)
                               '(read))
                  (cons v x))
                (else
                  (search (cdr l))))))))
      (search (filter (lambda (x)
                        (and (not (null? r))
                             (eq? 'L (car x))
                             (equal? (car r) (list-ref x 5))
                             (equal? (list-ref r 5) (list-ref x 6))))
                      entry))))

  (define (root-name did n)
    (let ((old (obj-part 'oldname did n)))
      (cond
        ((and (< 12 (string-length old))
              (equal? "/dev/ataraid" (substring old 0 12)))
          (obj-part 'hexname did n))
        (else
          old))))
    
  
  ; method
  (define (add-entry e)
    (inst-log "adding : ~a\n" e)
    (cond
      ((check-entry e)
        (set! entry (cons e entry))
        (inst-log "added : ~a\n" entry)
        #t)
      (else
        (inst-log "failed to add : ~a\n" entry)
        #f)))
  
  (define (add-default v)
    (let* ((t (format #f "~a (~a)"
               (let ((r (parse-awk #\tab #t "/var/lib/ogl/etc/title")))
                 (cond
                   ((or (equal? #f r) (null? r) (null? (car r)))
                     "SomeOGL")
                   (else
                     (caar r))))
               v))
           (boot? (obj-part 'have-/boot?))
           (p (obj-part 'path-mnt (if boot? "/boot" "/")))
           (root (obj-part 'path-mnt "/")))
      (cond
        ((or (null? p)
             (null? root))
          #f)
        (else
          (add-entry
            (list 'L t (car p) (list-ref p 5)
                  (format #f "~a/vmlinuz-~a" (if boot? "" "/boot") v)
                  (car root) (list-ref root 5)
                  opt
                  (if (obj-mod 'need-bl-driver?)
                    (format #f "~a/initrd-~a" (if boot? "" "/boot") v)
                    "")
                  ""))))))
  
  (define (auto-mbr)
    (let ((dl (obj-part 'list-did)))
      (cond
        ((or (not (= 1 (length dl)))
             (not (obj-part 'whole-part? (car dl))))
          #f)
        ((not (null? (filter (lambda (x)
                               (case (list-ref x 6)
                                 ((linux-ext2 linux-ext3 linux-xfs linux-reiser linux-jfs swap)
                                   #f)
                                 (else
                                   #t)))
                             (obj-part 'list-mnt))))
          #f)
        (else
          (inst-log "auto-mbr installing : ~a\n" (car dl))
          (install (car dl))))))
  
  (define (auto-pbr)
    (let ((r (obj-part 'boot-mnt)))
      (cond
        ((or (null? r)
             (< 4 (cadr r)))
          #f)
        (else
          (inst-log "auto-pbr installing : ~a\n" r)
          (apply install r)))))
  
  (define (install . l)  ; <did> [n]
    (cond
      ((or (null? l) (< 2 (length l)))
        #f)
      (else
        (let* ((part? (= 2 (length l)))
               (n (if part? (1- (cadr l)) 0))
               (r (obj-part 'path-mnt (if (obj-part 'have-/boot?) "/boot" "/")))
               ;(b (obj-part 'mnt-biosnum "/boot"))
               (d (obj-part 'get-biosnum (car l)))
               (s (if part? (format #f "hd~a,~a" d n)
                            (format #f "hd~a" d)))
               (did (car l))
               (backup (format #f "~a/var/lib/ogl/etc/boot-backup-~a~a" root did
                         (if part? (format #f "-~a" n) ""))))
          (inst-log "backup : ~a\n" backup)
          (cond
            ((< d 0)
              (inst-log "Could not get bios number.\n")
              #f)
            ((null? r)
              (inst-log "Could not get root partition.\n")
              #f)
            (else
              (cond
                ((equal? 'error (unix-filetype backup))
                  (if (not (= 0 (sh "dd if=~a of=~a count=~a"
                                    (obj-part 'devfsname did)
                                    backup
                                    (if part? 1 63))))
                    (inst-log "grub install : failed to backup."))))
              (let* ((tmp (case context ((inst2) "/tmp/grub.log") (else (unix-tempname))))
                     (devm (case context
                             ((inst2)
                               (let ((f "/tmp/dmap"))
                                 (obj-part 'make-devicemap f)
                                 f))
                             (else
                               "/boot/grub/device-map")))
                     (cmd (format #f "echo -e 'root (hd~a,~a)\nsetup (~a)'| /sbin/grub --device-map=~a | tee ~a"
                                      (obj-part 'get-biosnum (car r)) (1- (list-ref r 5))
                                      s
                                      devm
                                      tmp))
                     ; $B$"$H$G(B grub-setup $B$KCV$-49$($k(B
                     (x (case context
                          ((inst2)
                            (obj-part 'unmount)
                            (let ((r (system-at "logger" cmd)))
                              (obj-part 'domount)
                              r))
                          (else
                            (system-at "logger" cmd)))))
                (inst-log "install cmd  : ~a\n" cmd)
                (inst-log "install exit : ~a\n" x)
                (cond
                  ((not (= 0 x))
                    #f)
                  (else
                    (let ((grub-result (done? tmp)))
                      (cond
                        ((eq? #t grub-result)
                          (cond
                            ((= 1 (length l))
                              (let ((dev (obj-part 'devfsname did)))
                                (sh "mkdir ~a/var/lib/ogl/md5sum" root)
                                (sh "dd if=~a count=1 bs=446 | md5sum >~a/var/lib/ogl/md5sum/mbr-first-~a"
                                    dev root did)
                                (sh "dd if=~a count=62 skip=1 | md5sum >~a/var/lib/ogl/md5sum/mbr-rest-~a"
                                    dev root did)
                                (sh "dd if=~a count=63 | md5sum >~a/var/lib/ogl/md5sum/mbr-all-~a"
                                    dev root did))))
                          #t)
                        (else
                          (inst-log "grub install : done test failed.\n")
                          grub-result))))))))))))
  
  ; $B$J$<$+(B system-at $B$N(B logger $B$,8z$+$J$$!#(Bsh ... | logger $B$b8z$+$J$$!#(B
  
  (define (remove-default v)
    (let* ((p (format #f "~a/vmlinuz-~a"
                      (if (obj-part 'have-/boot?) "" "/boot")
                      v))
           (f (lambda (x) 
                (cond
                  ((or (< (length x) 6)
                       (not (eq? 'L (car x))))
                    #f)
                  (else (equal? p (list-ref x 4))))))
           (d (filter f entry)))
      (set! entry (filter (lambda (x) (not (f x)))
                          entry))
      (length d)))
  
  (define (load-default)
    (let ((r (parse-awk #\tab #t "/var/lib/ogl/etc/boot-default")))
      (cond
        ((eq? #f r)
          #f)
        ((not (and (= 2 (length r))
                   (= 1 (length (car r)))
                   (= 1 (length (cadr r)))))
          #f)
        ((eq? #f (string->number (car (cadr r))))
          (inst-log "boot load : line2 of boot-default is not number.\n")
          #f)
        (else
          (set! opt (caar r))
          (set! timeout (string->number (car (cadr r))))
          #t))))
  
  (define (load-entry)
    (let ((r (parse-awk #\tab #t "/var/lib/ogl/etc/boot-entry")))
      (cond
        ((eq? #f r)
          #f)
        (else
          (let ((s (map (lambda (x)
                          (if (null? x)
                            #f
                            (let ((a (string->symbol (car x)))
                                  (l (length x)))
                              (case a
                                ((L)
                                  (if (< l 8)
                                    #f
                                    (let ((n0 (string->number (list-ref x 3)))
                                          (n1 (string->number (list-ref x 6))))
                                      (cond
                                        ((or (not (number? n0))
                                             (not (number? n1)))
                                          #f)
                                        (else
                                          (list a (cadr x) (caddr x) n0 (list-ref x 4)
                                                (list-ref x 5) n1 (list-ref x 7)
                                                (if (< l 9) "" (list-ref x 8))
                                                (if (< l 10) "" (list-ref x 9))))))))
                                ((B)
                                  (if (not (= 3 l))
                                    #f
                                    (list a (cadr x) (caddr x))))
                                ((P)
                                  (if (not (= 4 l))
                                    #f
                                    (let ((n (string->number (list-ref x 3))))
                                      (cond
                                        ((not (number? n))
                                          #f)
                                        (else
                                          (list a (cadr x) (caddr x) n))))))
                                ((F)
                                  (if (not (= 5 l))
                                    #f
                                    (let ((n (string->number (list-ref x 3))))
                                      (cond
                                        ((not (number? n))
                                          #f)
                                        (else
                                          (list a (cadr x) (caddr x) n (list-ref x 4)))))))
                                (else
                                  #f)))))
                        r)))
            (inst-log "load-entry s : ~a\n" s)
            (cond
              ((andmap check-entry s)
                (set! entry s)
                #t)
              (else
                #f)))))))
  
  (define (load-it)
    (if (not (load-default))
      (inst-log "Failed to load boot-default."))
    (if (not (load-entry))
      (inst-log "Failed to load boot-entry."))
    #t)
  
  (define (write-var)
    (fprintf "/var/lib/ogl/etc/boot-default"
      (format #f "~a\n~a\n" opt timeout))
    (fprintf "/var/lib/ogl/etc/boot-entry"
      (apply string-append
        (map (lambda (x)
               (cond
                 ((null? x) "")
                 (else
                   (case (car x)
                     ((L)
                       (cond
                         ((not (= (length x) 10))
                           (inst-log "illegal entry : ~a\n" x)
                           "")
                         (else
                           (format #f "L\t~a\t~a\t~a\t~a\t~a\t~a\t~a\t~a\t~a\n"
                                   (cadr x) (caddr x) (list-ref x 3)
                                   (list-ref x 4) (list-ref x 5) (list-ref x 6)
                                   (list-ref x 7) (list-ref x 8) (list-ref x 9)))))
                     ((B)
                       (cond
                         ((not (= 3 (length x))) "")
                         (else
                           (format #f "B\t~a\t~a\n" (cadr x) (caddr x)))))
                     ((P)
                       (cond
                         ((not (= 4 (length x))) "")
                         (else
                           (format #f "P\t~a\t~a\t~a\n" (cadr x) (caddr x) (list-ref x 3)))))
                     ((F)
                       (cond
                         ((not (= 5 (length x))) "")
                           (format #f "B\t~a\t~a\t~a\t~a\n" ar x)
                             (cadr x) (caddr x) (list-ref x 3) (list-ref x 4)))
                     (else
                       "")))))
           entry))))
  
  (define (make-menu.lst f)
    (fprintf f
      (apply string-append
        (cons
          (format #f "timeout ~a\ndefault 0\nfallback 0\n\n" timeout)
          (map (lambda (x)
                 (cond
                   ((null? x) "")
                   (else
                     (case (car x)
                       ((L)
                         (cond
                           ((not (= (length x) 10))
                             (inst-log "illegal entry : ~a\n" x)
                             "")
                           (else
                             (format #f "title ~a\nroot(hd~a,~a)\nkernel ~a root=~a ~a~a~a\n\n"
                                     (cadr x)
                                     (obj-part 'get-biosnum (caddr x)) (1- (list-ref x 3))
                                     (list-ref x 4)
                                     (root-name (list-ref x 5) (list-ref x 6))
                                     (list-ref x 7)
                                     (if (equal? "" (list-ref x 8)) ""
                                       (format #f "\ninitrd ~a\n" (list-ref x 8)))
                                     (list-ref x 9)))))
                       ((B)
                         (cond
                           ((not (= 3 (length x))) "")
                           (else
                             (format #f "title ~a\nrootnoverify (hd~a)\nchainloader +1\n\n"
                                     (cadr x) (obj-part 'get-biosnum (caddr x))))))
                       ((P)
                         (cond
                           ((not (= 4 (length x))) "")
                           (else
                             (format #f "title ~a\nrootnoverify (hd~a,~a)\nchainloader +1\n\n"
                                     (cadr x)
                                     (obj-part 'get-biosnum (caddr x))
                                     (list-ref x 3)))))
                       ((F)
                         (cond
                           ((not (= 5 (length x))) "")
                             (format #f "# not implemented yet.\n")))
                       (else
                         "")))))
               entry))))
    #t)
  
  (define (commit)
    (write-var)
    (cond
      ((equal? context 'inst1)
        (inst-log "illegal context\n")
        #f)
      (else
        #t)))
  
  (define (make-fd format?)
    (let* ((fd (obj-part 'get-fdd))
           (rt (obj-part 'path-mnt "/"))
           (tmp (unix-tempname))
           (tmp2 (unix-tempname))
           (umount (lambda () (= 0 (sh "umount ~a" tmp))))
           (suf (case (list-ref rt 6)
                  ((linux-xfs) "mini-xfs")
                  (else "mini-woxfs")))
           (r1 (search-default-version))
           (v (car r1))
           (x (cdr r1))
           (initrd? (obj-mod 'need-bl-driver?)))
      (cond
        ((equal? "" fd)
          (_ "Could not find FD drive."))
        ((equal? "" v)
          (_ "Could not find kernel image which can be used for boot fd."))
        ((and format?
              (not (= 0 (sh "superformat ~a" fd))))
          (_ "Failed to do low-level format."))
        ((not (= 0 (sh "mkdosfs -I ~a" fd)))
          (_ "Failed to make DOS file system."))
        ((not (and (= 0 (sh "mkdir ~a" tmp))
                   (= 0 (sh "mount -t vfat ~a ~a" fd tmp))))
          (_ "Failed to mount floppy"))
        ((not (and (= 0 (sh "mkdir -p ~a/boot/grub" tmp))
                   (= 0 (sh "cp ~a/usr/lib/grub/i386-pc/stage1 ~a/boot/grub/" root tmp))
                   (= 0 (sh "cp ~a/usr/lib/grub/i386-pc/stage2 ~a/boot/grub/" root tmp))
                   (= 0 (sh "cp ~a/boot/vmlinuz-~a-~a ~a/vmlinuz" root v suf tmp))))
          (umount)
          (_ "Failed to copy required files."))
        ((and initrd?
              (not (= 0 (sh "cp ~a/boot/initrd-~a ~a/initrd" root v tmp))))
          (umount)
          (_ "Failed to copy required files (initrd). Disk full?"))
        ((not (fprintf (format #f "~a/boot/grub/menu.lst" tmp)
                (string-append "timeout 30\n"
                               "default 0\n"
                               "fallback 1\n\n\n"
                               (format #f "title ~a\n" (cadr x)) 
                               "root (fd0)\n"
                               (format #f "kernel /vmlinuz root=~a ~a\n"
                                 (root-name (list-ref x 5) (list-ref x 6))
                                 ;(obj-part 'oldname (list-ref x 5) (list-ref x 6))
                                 (list-ref x 7))
                               (if initrd? "initrd /initrd" "")
                               "\n")))
          (umount)
          (_ "Failed to write menu.def"))
        ((not (umount))
          (_ "Could not unmount."))
        (else
          (let ((r (grub-setup "fd0" "fd0")))
            (cond
              ((string? r)
                r)
              (else
                (eq? #t (done? tmp2)))))))))
  
  (define (make-initrd v)
    (cond
      ((not (obj-mod 'need-bl-driver?))
        #t)
      ((equal? "" v)
        (inst-log "could not find entry.\n")
        #f)
      (else
        (= 0 (sh (case context
                   ((inst2) "chroot /target mkinitrd /boot/initrd-~a ~a")
                   (else "mkinitrd force /boot/initrd-~a ~a"))
                 v v)))))
  
  (define (make-default-initrd)
    (let* ((r (search-default-version))
           (v (car r)))
      (make-initrd v)))
  
  (define (save)
   'sucess)
  
  (define (init c)
    (set! context c)
    (case context
      ((inst1 inst2 recovery)
        (set! root "/target")))
    #t)
  
  (define (set-option x)
    (set! opt x)
    #t)
  
  (define (add-option x)
    (set! opt 
      (if (equal? "" opt) x (format #f "~a ~a" opt x)))
    #t)
  
  (define (set-timeout x)
    (set! timeout x)
    #t)
  
  (define (replace n x)
    (define (rep n x l)
      (cond
        ((null? l) l)
        ((= 0 n) (cons x (cdr l)))
        (else
          (cons (car l) (rep (1- n) x (cdr l))))))
    (set! entry (rep n x entry))
    #t)
  
  (define (set-default n)
    (let ((l (length entry)))
      (cond
        ((or (< n 0) (< l n))
          #f)
        (else
          (set! entry
                (append (list (list-ref entry n))
                        (list-sub entry 0 n)
                        (if (= l n)
                          '()
                          (list-sub entry (1+ n) l))))
          #t))))
  
  (define (remove-entry n)
    (let ((l (length entry)))
      (cond
        ((or (< n 0) (< l n))
          #f)
        (else
          (set! entry
                (append (list-sub entry 0 n)
                        (if (= l n)
                          '()
                          (list-sub entry (1+ n) l))))
          #t))))
  
  (define (list-recoverable)
    (map (lambda (x) (string-gsub x ".*boot-backup-" ""))
      (filter (lambda (x)
                (not (null? (string-match x "boot-backup-"))))
              (unix-read-directory2 "/var/lib/ogl/etc"))))
  
  (define (recover-check did)
    (define (check cmd f)
      (let ((s1 (md5sum (format #f "|~a" cmd)))
            (s2 (getfile f)))
        (inst-log "recover-check s1:~a s2:~a\n" s1 s2)
        (cond
          ((or (not (string? s1)) (eq? #f s2))
            #f)
          ((< (string-length s2) 32)
            #f)
          (else
            (equal? s1 (substring s2 0 32))))))
    
    (let* ((dev (obj-part 'devfsname did))
           (c0 (check (format #f "dd if=~a count=1 bs=446" dev)
                      (format #f "/var/lib/ogl/md5sum/mbr-first-~a" did)))
           (c1 (check (format #f "dd if=~a count=62 skip=1" dev)
                      (format #f "/var/lib/ogl/md5sum/mbr-rest-~a" did)))
           (c2 (check (format #f "dd if=~a count=63" dev)
                      (format #f "/var/lib/ogl/md5sum/mbr-all-~a" did))))
      (inst-log "recover-check : ~a ~a ~a \n" c0 c1 c2)
      (cond
        (c2
          'ok)
        ((and c0 c1)
          'partition)
        (else
          'modified))))
  
  (define (recover did all?)
    (let* ((dev (obj-part 'devfsname did))
           (backup (format #f "/var/lib/ogl/etc/boot-backup-~a" did)))
      (cond
        ((not (unix-access? backup '(read)))
          #f)
        (all?
          (and (= 0 (sh "dd if=~a of=~a count=63" backup dev))
               (= 0 (sh "rm ~a" backup))))
        (else
          (let ((tmp (unix-tempname)))
            (and (= 0 (sh "dd if=~a of=~a count=1 bs=446" backup tmp))
                 (= 0 (sh "dd if=~a bs=1 skip=446 count=66 >>~a" dev tmp))
                 (= 0 (sh "dd if=~a skip=1 count=62 >>~a" backup tmp))
                 (= 0 (sh "dd if=~a of=~a count=63" tmp dev))
                 (= 0 (sh "rm ~a ~a" tmp backup))))))))
  
  (make-object
    'add-default         add-default
    'add-entry           add-entry
    'add-option          add-option
    'auto-mbr            auto-mbr
    'auto-pbr            auto-pbr
    'commit              commit
    'conf                (lambda () (list (list "/boot/grub/menu.lst" make-menu.lst #t)))
    'entry               (lambda () entry)
    'get-option          (lambda () opt)
    'get-timeout         (lambda () timeout)
    'init                init
    'install             install
    'load                load-it
    'make-fd             make-fd
    'make-default-initrd make-default-initrd
    'make-initrd         make-initrd
    'mid-conf            (lambda () (list "/var/lib/ogl/etc/boot-default" "/var/lib/ogl/etc/boot-entry"))
    'replace             replace
    'save                save
    'set-option          set-option
    'set-timeout         set-timeout
    'remove-default      remove-default
    'remove-entry        remove-entry
    'recover             recover
    'recover-check       recover-check
    'list-recoverable    list-recoverable
    'set-default         set-default
    ))
