;
;    Crontab
;                               (C) 2002 Omoikane Inc.
;

(define (conf-cron obj-sv)
  
  ; vars
  
  (define tbl '())
  
  
  ; libs
  
  (define (wton w)
    (case w
      ((sun) 0)
      ((mon) 1)
      ((tue) 2)
      ((wed) 3)
      ((thu) 4)
      ((fri) 5)
      ((sat) 6)
      (else -1)))
  
  (define (ntos n)
    (if (< n 0) "*" (format #f "~a" n)))
  
  
  ; methods
  
  (define (make-crontab f)
    (fprintf f
      (apply string-append
        (append
          (list
            "# /etc/crontab: system-wide crontab\n"
            "# this file was generated by ogl-admin\n"
            "# see man 5 crontab for more info\n"
            "\n"
            "SHELL=/bin/sh\n"
            "PATH=/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin\n"
            "\n"
            "# m h dom mon dow user  command\n"
            "25 6    * * *   root    test -e /usr/sbin/anacron || run-parts --report /etc/cron.daily\n"
            "47 6    * * 7   root    test -e /usr/sbin/anacron || run-parts --report /etc/cron.weekly\n"
            "52 6    1 * *   root    test -e /usr/sbin/anacron || run-parts --report /etc/cron.monthly\n"
            "#\n\n")
          (map (lambda (x)
                 (format #f "# for ~a\n~a ~a  ~a ~a ~a  ~a    ~a\n"
                   (car x)
                   (ntos (cadr x))
                   (ntos (caddr x))
                   (ntos (list-ref x 3))
                   (ntos (list-ref x 4))
                   (ntos (wton (list-ref x 5)))
                   (list-ref x 6) (list-ref x 7)))
               tbl)
          (list "\n")))))
  
  (define (load-it)
    (cond
      ((not (unix-access? "/var/lib/ogl/etc/cron" '(read)))
        #t)
      (else
        (let ((r (ttable-read "/var/lib/ogl/etc/cron" '((sym int int int int sym str str)))))
          (cond
            ((not (list? r))
              #f)
            (else
              (set! tbl r)
              #t))))))
  
  (define (commit)
    (ttable-write "/var/lib/ogl/etc/cron" tbl))
  
  (define (restart-if-run)
    (if (obj-sv 'start? "cron")
      (obj-sv 'restart "cron")
      #t))
  
  (define (update k n0 n1 n2 n3 w u c)
    (cond
      ((not (and (andmap symbol? (list k w))
                 (andmap integer? (list n0 n1 n2 n3))
                 (andmap string? (list u c))))
        #f)
      (else
        (set! tbl
          (append
            (filter (lambda (x) (not (equal? k (car x))))
                    tbl)
            (list (list k n0 n1 n2 n3 w u c))))
        #t)))
  
  (define (remove k)
    (set! tbl
      (filter (lambda (x) (not (equal? k (car x))))
        tbl))
    #t)
  
  (define (get k)
    (let ((r (assoc k tbl)))
      (cond
        ((eq? #f r)
          '())
        (else
          (cdr r)))))
  
  (make-object
    'load             load-it
    'conf             (lambda () (list (list "/etc/crontab" make-crontab #t)))
    'commit           commit
    'mid-conf         (lambda () (list "/var/lib/ogl/etc/crontab"))
    'restart-if-run   restart-if-run
    'update           update
    'get              get
    'remove           remove
    ))


