;;;
;;; set error-handler and suspend-handler
;;;
;;; error-handler is called when interpreter hits error.
;;; suspend-handler is called when ^Z is pressed.
;;;
(let* ()
  (define rep-level)
  (define rep-frames)
  (define (rep-loop env)
    (let loop ()
      (display "scheme")
      (if (> rep-level 0)
          (display rep-level))
      (let ((exp (cond 
                   (debug-with-readline? (read/rl "> "))
                   (else (display "> ") (read)))))
        (if (not (eof-object? exp))
            (begin
              (write (eval exp env))
              (newline)
              (loop))))))

  (define (error-print error-msg)
    (format #t "~s: " (car error-msg))
    (apply format #t (cdr error-msg))
    (newline))

  (define (push-frame! frame)
    (set! rep-frames (cons frame rep-frames))
    (set! rep-level (1+ rep-level)))
  (define (pop-frame!)
    (set! rep-frames (cdr rep-frames))
    (set! rep-level (1- rep-level)))

  (define (rep-error-handler . error-msg)
    (error-print error-msg)
    (let loop ((intr-level (enable-interrupts)))
      (if (> intr-level 0)
          (loop (enable-interrupts))))
    (let loop ()
      (if (call-with-current-continuation
           (lambda (control-point)
             (push-frame! control-point)
             (rep-loop (the-environment))
             #f))
          (begin
            (pop-frame!)
            (loop))))
    (newline)
    (pop-frame!)
    (let ((next-frame (car rep-frames)))
      (next-frame #t)))

  (define top-level-environment (the-environment))

  (define (top-level)
    (let ((eh error-handler))
      (set! error-handler rep-error-handler)
      (let loop ()
        (if (call-with-current-continuation
             (lambda (control-point)
               (set! rep-frames (list control-point))
               (set! top-level-control-point control-point)
               (set! rep-level 0)
               (rep-loop top-level-environment)
               #f))
            (loop)))
      (set! error-handler eh)))

  (set! error-handler
        (lambda error-msg
          (snewt-finish)
          (display (_ "Unrecoverble error occurred.\nYou can evaluate Scheme expression for debug.\n"))
          (error-print error-msg)
          (top-level)
          (exit 1)))
  (set! snewt-suspend-handler
        (lambda ()
          (display (_ "Installer is suspended.\nYou can evaluate Scheme expression for debug.\nIf you want to return installer, type ^D.\n"))
	  (top-level))))

;;; for debug
(define frame-list)
(define (debug-init) (set! frame-list (backtrace-list)))
(define (frame n) (list-ref frame-list n))
(define debug-with-readline? #f)
