;
;     Directory Service Object
;
;                                  (C)2002 Omoikane Inc.
;


(define (conf-dirsv obj-sv)
  
  ; var
  (define ds '())
  (define postconf '())
  
  ; lib
  (define (get-it k)
    (let ((r (assoc k ds)))
      (cond
        ((eq? #f r) r)
        (else
          (cdr r)))))
  
  (define (update-it k . l)
    (set! ds
      (append (map (lambda (x) (filter (not (eq? k (car x)))))
                   ds)
              (list (cons k l))))
    #t)
  
  (define (make-yp.conf f)
    (fprintf f
      (let ((r (assoc 'nis ds)))
        (cond
          ((eq? #f r) "")
          ((equal? (cadr r) "")
            (format #f "domain ~a broadcast\n" (caddr r)))
          (else
            (format #f "domain ~a\nypserver ~a\n" (caddr r) (cadr r)))))))
  
  (define (make-ldap.conf f)
    (fprintf f
      (let ((r (assoc 'ldap ds)))
        (cond
          ((eq? #f r) "")
          (else
            (format #f "HOST ~a\nBASE ~a\nSASL_SECPROPS none\n"
              (if (< 0 (caddr r))
                (format #f "~a:~a" (cadr r) (caddr r))
                (cadr r))
              (list-ref r 3)))))))
  
  (define (make-defaultdomain f)
    (fprintf f
      (let ((r (assoc 'nis ds)))
        (cond
          ((eq? #f r) "")
          (else (format #f "~a\n" (caddr r)))))))
  
  (define (make-nss f)
    (fprintf f
      (let ((add (apply string-append
                   (map (lambda (x)
                          (case (car x)
                            ((nis) " nis")
                            ((ldap) " ldap")
                            (else "")))
                        ds))))
        (string-append
          "# This is configuration of GNU Name Service Switch functionality,\n"
          "# generated by ogl-tools.\n"
          "# If you have the `glibc-doc' and `info' packages installed, try:\n"
          "# `info libc \"Name Service Switch\"' for information about this file.\n\n"
          "passwd:      compat" add "\n"
          "group:       compat" add "\n"
          "shadow:      compat" add "\n\n"
          "hosts:       files dns\n"
          "networks:    files\n\n"
          "protocols:   db files\n"
          "services:    db files\n"
          "ethers:      db files\n"
          "rpc:         db files\n\n"
          "netgroup:    nis\n"))))
  
  (define (make-pam-passwd f)
    (fprintf f
      (string-append
        "#\n"
        "# The PAM configuration file for the Shadow `passwd' service\n"
        "#\n"
        "\n"
        "# The standard Unix authentication modules, used with NIS (man nsswitch) as\n"
        "# well as normal /etc/passwd and /etc/shadow entries. For the login service,\n"
        "# this is only used when the password expires and must be changed, so make\n"
        "# sure this one and the one in /etc/pam.d/login are the same. The \"nullok\"\n"
        "# option allows users to change an empty password, else empty passwords are\n"
        "# treated as locked accounts.\n"
        "#\n"
        "# (Add `md5' after the module name to enable MD5 passwords the same way that\n"
        "# `MD5_CRYPT_ENAB' would do under login.defs).\n"
        "#\n"
        "# The \"obscure\" option replaces the old `OBSCURE_CHECKS_ENAB' option in\n"
        "# login.defs. Also the \"min\" and \"max\" options enforce the length of the\n"
        "# new password.\n"
        "\n"
        (cond
          ((eq? (assoc 'ldap ds) #f) "")
          (else
            "password   sufficient pam_ldap.so\n"))
        "password   required   pam_unix.so md5 nullok obscure min=4 max=8\n"
        "\n"
        "# Alternate strength checking for password. Note that this\n"
        "# requires the libpam-cracklib package to be installed.\n"
        "# You will need to comment out the password line above and\n"
        "# uncomment the next two in order to use this.\n"
        "# (Replaces the `OBSCURE_CHECKS_ENAB', `CRACKLIB_DICTPATH')\n"
        "#\n"
        "# password required       pam_cracklib.so retry=3 minlen=6 difok=3\n"
        "# password required       pam_unix.so use_authtok nullok md5\n")))
  
  ; method
  
  (define (stop-server)
    (and (obj-sv 'remove "nscd")
         (obj-sv 'remove "nis")))
  
  (define (set-postconf)
    (let* ((r0 (assoc 'nis ds))
           (r1 (assoc 'ldap ds)))
      (and
        (if (eq? #f r0)
           #t
           (= 0 (sh "nisdomainname ~a" (caddr r0))))
        (obj-sv 'remove "nscd")
        (obj-sv 'remove "nis")
        (cond
          ((and (eq? #f r0) (eq? #f r1))       ; local only
            #t)
          ((eq? #f r0)                         ; LDAP
            (set! postconf
              (list (list (_ "LDAP client setting")
                          (_ "Starting LDAP client ...")
                          (lambda () (obj-sv 'start "nscd")))))
            #t)
          (else                                ; NIS
            (set! postconf
              (list (list (_ "NIS client setting")
                          (_ "Starting NIS client ...")
                          (lambda ()
                            (and (obj-sv 'start "nis")
                                 (obj-sv 'start "nscd"))))))
            #t)))))
  
  (define (commit)
    (ttable-write "/var/lib/ogl/etc/dirsv" ds))
  
  (define (load-it)
    (cond
      ((not (unix-access? "/var/lib/ogl/etc/dirsv" '(read)))
        #t)
      (else
        (let ((r (ttable-read "/var/lib/ogl/etc/dirsv"
                   '(("nis" str str) ("ldap" str int str)))))
          (cond
            ((eq? r #f) #f)
            (else
              (set! ds
                (map (lambda (x) (cons (string->symbol (car x)) (cdr x)))
                     r))
              #t))))))
  
  (define (reset)
    (set! ds '())
    #t)
  
  (define (update-ldap h p b)
    (cond
      ((or (equal? "" h) (equal? "" b))
        #f)
      (else
        (update-it 'ldap h p b))))
  
  (define (update-nis s d)
    (cond
      ((equal? "" d)
        #f)
      (else
        (update-it 'nis s d))))
  
  (make-object
    'commit          commit
    'conf            (lambda () (list (list "/etc/yp.conf" make-yp.conf #t)
                                      (list "/etc/ldap/ldap.conf" make-ldap.conf #t)
                                      (list "/etc/defaultdomain" make-defaultdomain #t)
                                      (list "/etc/nsswitch.conf" make-nss #t)
                                      (list "/etc/pam.d/passwd" make-pam-passwd #f)))
    'get-ldap        (lambda () (get-it 'ldap))
    'get-nis         (lambda () (get-it 'nis))
    'info            (lambda () ds)
    'load            load-it
    'mid-conf        (lambda () (list "/var/lib/ogl/etc/dirsv"))
    'postconf        (lambda () postconf)
    'set-postconf    set-postconf
    'reset           reset
    'stop-server     stop-server
    'update-ldap     update-ldap
    'update-nis      update-nis
    ))
