#!/bin/sh
string=?;exec /usr/bin/elk -c -l $0 -p /usr/lib/ogl/scm:/usr/lib/elk/scm:/usr/lib/elk/obj:`pwd` $* 2>/dev/null

(load "gtk.scm")
(load "gtk-obj.scm")
(load "gtk-middle.scm")
(load "util.scm")
(load "obj.scm")
(load "net.scm")
(load "service.scm")
(load "prod.scm")

(define debug? #f)

(set! inst-log-context 'user)

(set! sh-log "/dev/null")

(define rc (format #f "~a/.glloginrc" (unix-getenv "HOME")))

(define (fail-reason)
  (let* ((obj-net (configure-network))
         (obj-sv (configure-service))
         (obj-pd (conf-prod obj-net obj-sv)))
    (obj-pd 'load)
    (let ((pri (obj-pd 'get-primary)))
      (cond
        ((equal? "" pri)
          (sgtk-message-error
            (_ "This host doesn't seem to be configured enough. Configure ORCA with ogl-admin first.")))
        ((or (not (unix-connect? pri 8000))
             (not (unix-connect? pri 9000)))
          (sgtk-message-error
            (_ "Could not access to the ORCA server. Make sure that ORCA server(s) are running with ogl-admin' ORCA configuration.")))
        (else
          (sgtk-message-error
            (_ "Could not run ORCA client (glclient).\nCheck if ORCA configuration/installation is completed. (or ask ARMA Net for support.)")))))))

(define (wait-exit tmp t tout)
  (cond
    ((< tout t)
      #f)
    (else
      (unix-sleep 1)
      (let ((r0 (system-at "/dev/null" (format #f "grep '^GLEXITWITH' ~a" tmp)))
            (r1 (system-at "/dev/null" (format #f "grep '^Connected' ~a" tmp))))
        (if debug?
          (format #t "checking ~a : ~a ~a ... \n" tmp r0 r1))
        (if (or (= 0 r0) (= 0 r1))
          #t
          (wait-exit tmp (1+ t) tout))))))
;(if (= 0 (sh "grep '^GLEXITWITH' ~a >&/dev/null" tmp))


(define (orca-test m id pass setlogin?)
  (let ((new (sgtk-window-new-toplevel-named (_ "ORCA login")))
        (v (sgtk-vbox-new #f 10))
        (v1 (sgtk-vbox-new #f 10))
        (l (sgtk-label-normalized-new
             (string-append m
               (_ "Enter ORCA operator ID and password.\nOr select 'cancel' not to start ORCA client."))))
        (eb (sgtk-entrybox #f
              (list (list (_ "User ID") id #t)
                    (list (_ "Password") pass #f))))
        (cl (sgtk-checkbox
              (list (list (_ "Save this ID/password") #t)
                    (list (_ "Show this menu at the login") setlogin?))))
        (bb (sgtk-ok/cancel #t)))
    (sgtk-container-set-border-width v1 10)
    (sgtk-container-add-many v1 l (eb 'widget) (cl 'widget))
    (sgtk-container-add-many v v1 (bb 'widget))
    (sgtk-container-add-many new v)
    (sgtk-widget-show-all new)
    (sgtk-main)
    (cl 'freeze)
    (case (bb 'info)
      ((back)
        (sgtk-widget-destroy new)
        #f)
      (else
        (let* ((r (eb 'info))
               (c (car (cl 'info)))
               (tmp (if debug? "/tmp/glclient.log" (unix-tempname)))
               (rm (lambda () (if debug? #t (= 0 (sh "rm ~a" tmp))))))
          (sgtk-widget-destroy new)
          (sgtk-update)
          (sh "(cd $HOME >/dev/null ; LANG=~a glclient -user ~a -pass ~a -style /home/orca/orcarc panda:orca00 ; echo 'GLEXITWITH' $?) | tee ~a &"
              (let ((g (unix-getenv "LANG")))
                (if (equal? "ja" g) "ja_JP.eucJP" g))
              (car r) (cadr r) tmp)
          (let ((r1 (wait-exit tmp 0 20)))
            (if debug? (sh "cp ~a /tmp/gllogin.log" tmp))
            (cond
              ((not r1)
                (sgtk-message-error
                  (_ "Starting the login program (gllogin) timed out.")))
              ((= 0 (sh "grep 'can not.*authentication' ~a >/dev/null" tmp))
                (rm)
                (orca-test (_ "Authentication Error.\n\n") id "" setlogin?))
              ((not (= 0 (system-at "/dev/null" (format #f "grep '^Connected' ~a" tmp))))
                (rm)
                (fail-reason)
                #t)
              (else
                (rm)
                (and (fprintf rc
                       (string-append
                         (if (and c (not (equal? "" (car r))) (not (equal? "" (cadr r))))
                           (format #f "~a\t~a\n" (car r) (cadr r))
                           "")
                         (format #f "login\t~a\n" (if (cadr (cl 'info)) "on" "off"))))
                     (sh "chmod og-rw ~a" rc))
                #t))))))))

(sgtk-start)
(textdomain "ogl-admin" "/usr/share/locale")
(let* ((r (ttable-read rc '((str str))))
       (a (command-line-args))
       (login? (ormap (lambda (x) (equal? "start" x)) a)))
  (if (ormap (lambda (x) (equal? "debug" x)) a)
    (set! debug? #t))
  (cond
    ((or (eq? #f r) (null? r))
      (orca-test "" "" "" #t))
    (else
      (let* ((noset-login? (ormap (lambda (x) (and (equal? (car x) "login") (equal? (cadr x) "off")))
                                  r)))
        (cond
          ((and login? noset-login?)
            (exit 0))
          ((= 2 (length r))
            (orca-test "" (caar r) (cadr (car r)) (not noset-login?)))
          (else
            (orca-test "" "" "" (not noset-login?))))
        (sh "touch ~a ; chmod og-rw ~a" rc rc)))))

 