

(define (gtk-ui-flushlog w)
  (define (rep n)
    (let ((l (unix-read-directory2 "/tmp/sup")))
      (cond
        ((< 6 n)
          #f)
        ((<= (length l) 2)
          #t)
        (else
          (unix-sleep 2)
          (rep (1+ n))))))
  (let ((l (unix-read-directory2 "/tmp/sup")))
    (let ((r (if (< 2 (length l))
               (sgtk-show-while w 
                 (_ "Flushing log ...")
                 (lambda () (rep 0)))
               #t)))
      (sh "rm -rf /tmp/sup")
      r)))

(define (gtk-ui-xtest m c)
  ; m : if not #f, message on continue button.
  ; c : closure to continue
  
  (define (text n)
    (format #f (_ "Automatically return to the console within ~s sec") n))
  
  (define sec 15)
  
  (unix-setenv "DISPLAY" ":0.0")
  (unix-sleep 2)
  (cond
    ((not (sgtk-start))
      (inst-log "Could not initialize Gtk.\n")
      #f)
    (else
      (let ((w (sgtk-window-new-toplevel-named (_ "X test")))
            (l (sgtk-label-new (text sec)))
            (v (sgtk-vbox-new #f 10))
            (hr (sgtk-hseparator-new))
            (hbox (sgtk-hbox-new #f 10))
            (b0 (if m (sgtk-button-new-with-label m) m))
            (b1 (sgtk-button-new-with-label (_ "Return to the console")))
            (push? 0))
            
        (define (roop/10 n)
          (cond
            ((< n 0) #f)
            ((not (= push? 0)) #t)
            (else
              (unix-usleep 100000)
              (sgtk-update)
              (roop/10 (1- n)))))
        
        (define (roop n)
          (cond
            ((< n 0) #t)
            (else
              (let ((r (roop/10 10)))
                (cond
                  (r r)
                  (else
                    (sgtk-label-set-text l (text n))
                    (sgtk-update)
                    (roop (1- n))))))))
        (sgtk-signal-connect b1 "clicked" (lambda (w) (set! push? 2)))
        (cond
          (m
            (sgtk-signal-connect b0 "clicked" (lambda (w) (set! push? 1)))
            (sgtk-container-add-many hbox b0 b1))
          (else
            (sgtk-container-add-many hbox b1)))
        (sgtk-container-add-many v l hr hbox)
        (sgtk-container-add w v)
        (sgtk-window-set-position w 1)
        (sgtk-widget-show-all w)
        (roop sec)
        (sgtk-widget-destroy w)
        (if (eq? push? 1)
          (c))
        #t))))

(define (gtk-ui-timezone w obj-time)
  (let* ((tz (append (obj-time 'get-root) (obj-time 'get-table)))
         (v  (sgtk-vbox-new #f 10))
         (ll (sgtk-label-new (_ "Select appropriate time zone for your location.")))
         (tr (make-simple-tree #t #t
               (map (lambda (x)
                      (if (list? x)
                        (map (lambda (y) (list (sgtk-label-new y) #f))
                             x)
                        (list (sgtk-label-new x) #f)))
                    tz)))
         (a  (sgtk-alignment-new 0.8 0.8 0.8 0.8))
         (bb (sgtk-ok/cancel #t)))
    (define (tree-get r)
      (cond
        ((= 1 (length r))
          (list-ref tz (car r)))
        ((= 2 (length r))
          (list-ref (list-ref tz (car r))
                    (cadr r)))
        (else
          "")))
    (sgtk-container-add a (tr 'widget))
    (sgtk-container-add-many v ll a)
    (sgtk-box-pack-start v (bb 'widget) #f #f 0)
    (sgtk-paned-pack2 w v #f #f)
    (sgtk-widget-show-all w)
    (sgtk-main)
    (sgtk-container-remove w v)
    (cond
      ((null? (tr 'info))
        #f)
      (else
        (obj-time 'set-tz (tree-get (car (tr 'info))))))))

(define (gtk-ui-hwclock w obj-time)
  (let ((r (sgtk-choice w 
             (string-append
               (_ "You can select two different types of clock setting.\n\n")
               (_ "The machine has its own hardware clock which can be set with BIOS. ")
               (_ "This is called 'hardware clock'. ")
               (_ "On the other hand, OS has its own expression of time, according to the location. ")
               (_ "This is called 'local time'.\n\n")
               (_ "You can set local time to hardware clock or GMT. ")
               (_ "If you have several Unix like OS installed, it might be better to set it to GMT.")
               (_ "Otherwise you would better to set it to hardware clock.\n\n")
               (_ "Would the local time be set to hardware clock?"))
             "Button_No" "Button_Yes")))
    (obj-time 'set-hwclock (= r 1))
    #t))
; ʤ󤫰㤦

(define (gtk-ui-kern w obj-med obj-boot)
  (inst-log "=== gtk-ui-kern\n")
  (let* ((tmp (unix-tempname)))
    (obj-med 'cp (format #f "ogl/inst/kernel/desc~a" (if (lang-ja?) "-ja" "")) tmp)
      (let ((tbl (ttable-read tmp '((str str str str)))))
        ;(format #t "~s\n" tbl)
        (cond
          ((or (not (list? tbl)) (null? tbl))
            (sgtk-message-error
              (_ "Could not read kernel description table. Aborting, sorry.")))
          (else
            (let* ((inst-ver (get-proc-version)))
              (define (default n l)
                (cond
                  ((null? l) 0)
                  ((equal? (caar l) inst-ver) n)
                  (else (default (1+ n) (cdr l)))))
              (let* ((d (default 0 tbl))
                     (v0 (sgtk-vbox-new #f 10))
                     (v1 (sgtk-vbox-new #t 10))
                     (h (sgtk-hbox-new #t 10))
                     (a (sgtk-alignment-new 0.8 0.8 0.8 0.8))
                     (a1 (sgtk-alignment-new 0.0 0.0 1.0 1.0))
                     (dummyl (map (lambda (x) " ") tbl))
                     (vr (apply make-vradio (cons #f (cons d dummyl))))
                     (cl (sgtk-checkbox
                           (map (lambda (x) 
                                  (list (sprintf "%-20s      "
                                          (string-cut (caddr x) 20))
                                        (equal? (car x) inst-ver)))
                                 tbl)))
                     (ll (sgtk-label-normalized-new
                           (string-append
                             (_ "You can install several kernels here. Please check the kernel which you want to install.")
                             (_ "\nA checker in the first left column means default kernel. Those on second column mean to be installed."))))
                     (bb (sgtk-bbox-ok)))
              (sgtk-box-pack-start v1 ll #t #t 0)
              (sgtk-container-add a1 (vr 'widget))
              (sgtk-box-pack-start h a1 #f #f 0)
              (sgtk-box-pack-start h (cl 'widget) #f #f 0)
              ; ˾ɤ֤Τ񤷤Τ check box ¦ΥåäƤޤ
              (sgtk-container-add v1 h)
              (sgtk-box-pack-start v1 a #t #t 0)
              (sgtk-container-add v0 v1)
              (sgtk-box-pack-start v0 (bb 'widget) #f #f 0)
              (sgtk-paned-pack2 w v0 #t #f)
              (sgtk-widget-show-all w)
              (sgtk-main)
              (cl 'freeze)
              (sgtk-container-remove w v0)
              (cond
                ((not (= 0 (bb 'info)))
                  (sgtk-message-error (_ "This process cannot be skipped."))
                  (gtk-ui-kern w obj-med obj-boot))
                ((null? (map (lambda (x) x) (cl 'info)))
                  (sgtk-message-error (_ "You have to install at least one kernel."))
                  (gtk-ui-kern w obj-med obj-boot))
                (else
                  (let ((v (sgtk-vbox-new #f 10))
                        (l (sgtk-label-new (_ "Installing kernel ...")))
                        (p (sgtk-progress-bar-new)))
                    (sgtk-container-add v l)
                    (sgtk-box-pack-start v p #f #f 100)
                    (sgtk-paned-pack2 w v #t #f)
                    (sgtk-widget-show-all v)
                    (sgtk-update)
                    (let* ((r (progress-start
                                (format #f "(PROGRESS_FD=9 /scm/install-kernel.scm ~a 9>&1 2>/tmp/klog 1>/tmp/klog ; echo $? >/tmp/exit-kinst)| tee /tmp/instk.log"
                                  (apply string-append
                                    (map (lambda (x y)
                                           (if y (format #f " ~a" (car x)) ""))
                                         tbl
                                         (smap (lambda (n x)
                                                 (or x (= n (vr 'info))))
                                               (cl 'info)))))
                                (lambda (f s)
                                  (sgtk-progress-bar-update p f)
                                  (sgtk-update)))))
                      (sgtk-widget-destroy v)
                      (let ((r' (parse-awk #\space #t "/tmp/exit-kinst")))
                        (cond
                          ((not (equal? '(("0")) r'))
                            (inst-log "kernel installation exited with ~a.\n" r')
                            (sgtk-message-fatalerror
                              (format #f (_ "Kernel installation failed (~a).\n") r')))
                          (else
                            (map (lambda (x y)
                                   (if y (obj-boot 'add-default (car x))))
                                 tbl
                                 (smap (lambda (n x)
                                         (and x (not (= n (vr 'info)))))
                                       (cl 'info)))
                            (obj-boot 'add-default (car (list-ref tbl (vr 'info))))
                            (obj-boot 'set-timeout 3)
                            #t))))))))))))))

(define (gtk-ui-install-br w obj-boot obj-part first?)
  (define title (_ "Boot Record Installation"))
  (define (check-win)
    (let ((l (filter (lambda (x)
                       (case (car x)
                         ((dev empty) #f)
                         (else (id-win? (list-ref x 8)))))
                     (obj-part 'list-all-part #f))))
      (define (add t p)
        (obj-boot 'add-entry (list 'P t (cadr p) (1- (list-ref p 3)))))
      (inst-log "adding boot loader for : ~a\n" l)
      (if (= 1 (length l))
        (add "Windows" (car l))
        (smap (lambda (n x)
                (add (format #f "Windows(~a)" n) x))
              l))))
  
  (inst-log "=== gtk-ui-install-br\n")
  (let ((l (obj-part 'hw-disc))
        (p (obj-part 'boot-mnt)))
    (cond
      ((or (null? l) (null? p))
        (sgtk-message-error
          (_ "Disc drive or boot partition couldn't be found.")))
      (else
        (let* ((ext? (< 0 (caddr p)))
               (desc0 (obj-part 'did-desc (car p) (if ext? (caddr p) (cadr p))))
               (boot-part? (list-ref p 3))
               (desc-p (cons
                         (format #f (_ "top region of ~a") desc0)
                         (not ext?)))
               (desc-m (map (lambda (x)
                              (cons
                                (format #f (_ "top region of ~a") (obj-part 'hw-desc x))
                                (and (not (equal? 'ataraid (list-ref x 3)))
                                     (equal? (car x) (car p)))))
                            l))
               (cl (sgtk-checkbox
                     (map (lambda (x) (list (car x) (cdr x)))
                          (if boot-part? (cons desc-p desc-m) desc-m))))
               (bb (sgtk-ok/cancel #t))
               (v0 (sgtk-vbox-new #f 10))
               (v1 (sgtk-vbox-new #t 10))
               (a (sgtk-alignment-new 0.8 0.8 0.8 0.8))
               (ll (sgtk-label-normalized-new
                     (string-append
                       (_ "Select regions where boot loader will be installed.\n")
                       (if (ormap (lambda (x) (equal? 'ataraid (list-ref x 3))) l)
                         (_ "If you want to use ATA RAID device as root partition, it is recommended not to install the boot loader and instead to use a boot FD (or other media). ")
                         "")
                       (if boot-part?
                         ""
                         (_ "(XFS partitions are not listed because boot loader cannot be installed. )"))
                       (if (and boot-part? ext?)
                         (format #f (_ "'~a' is an extended partition where boot partions resides.\n") desc0)
                         "")))))
          (sgtk-container-add v0 v1)
          (sgtk-container-add a (cl 'widget))
          (sgtk-box-pack-start v1 ll #t #t 0)
          (sgtk-box-pack-start v1 a #f #f 0)
          (sgtk-box-pack-start v0 (bb 'widget) #f #f 0)
          (sgtk-paned-pack2 w v0 #t #f)
          (sgtk-widget-show-all w)
          (sgtk-main)
          (cl 'freeze)
          (sgtk-container-remove w v0)
          (cond
            ((not (equal? (bb 'info) 'forward))
              (inst-log "canceld\n")
              #f)
            ((not (ormap (lambda (x) x) (cl 'info)))
              (cond
                ((not first?)
                  #f)
                ((= 1 (sgtk-choice w
                        (string-append
                          (_ "Without boot loader, the system won't be able to boot. ")
                          (_ "In this case, you have to use boot fd or other (already installed some) boot loader. ")
                          (_ "\n\nDo you proceed without boot loader installation?"))
                        "Button_No" "Button_Yes"))
                  #f)
                (else
                  (gtk-ui-install-br w obj-boot obj-part first?))))
            (else
              (let ((r1 (if (and boot-part? (car (cl 'info)))
                          (obj-boot 'install (car p) (if ext? (caddr p) (cadr p)))
                          #t))
                    (r2 (map (lambda (x y)
                               (if y
                                 (obj-boot 'install (car x))
                                 #t))
                             l (if boot-part? (cdr (cl 'info)) (cl 'info)))))
                (cond
                  ((andmap (lambda (x) (eq? #t x))
                           (cons r1 r2))
                    (if first? (check-win))
                    #t)
                  ((= 1 (sgtk-choice w
                          (apply string-append
                            (append
                              (list
                                (_ "Failed to install grub into regions below. ")
                                (if (and (not first?)
                                      (andmap (lambda (x) (equal? 1 x))
                                        (cons r1 r2)))
                                  (_ "This may be that grub tried to use running journaling file systems and cannot read it. You don't need to care match about this but should try later (or after next boot). ")
                                  "")
                                "\n\n"
                                (if (eq? #t r1) "" (format #f "    ~a\n" (car desc-p))))
                              (map (lambda (x y) (if (eq? #t y) "" (format #f "    ~a\n" (car x))))
                                   desc-m r2)
                              (list
                                (_ "\nDo you retry boot loader install?"))))
                        "Button_No" "Button_Yes"))
                    (gtk-ui-install-br w obj-boot obj-part first?))
                  (else
                    #f))))))))))

(define (gtk-ui-base w obj-med)
  (let ((l (sgtk-label-normalized-new (_ "Installing base files. This step takes a few minutes.\nPlease wait a moment ..."))))
    (sgtk-paned-pack2 w l #t #f)
    (sgtk-widget-show-all l)
    (sgtk-update)
    (let ((r (obj-med 'pipe "ogl/inst/base.tgz" "gunzip | (cd /target ; tar x)")))
      (if (eq? r #f)
        (sgnome-dialog-run
          (sgnome-message-box-new (_ "Failed to get base file.")
            "error" "Button_Ok")))
      (unix-sync)
      (sgtk-widget-destroy l)
      r)))

(define (gtk-ui-part w obj-part default-fs mount?)
  ; default-fs : default type
  ; w : parent widget
  
  (define o obj-part)
  
  (define (tree-get n m)
    (list-ref (obj-part 'list-part
                (car (list-ref (obj-part 'hw-disc) n)))
              m))
  
  (define (start)
    (let ((v0 (sgtk-vbox-new #f 10))
          (v1 (sgtk-vbox-new #t 10))
          (a (sgtk-alignment-new 0.8 0.8 0.8 0.8))
          (vr (make-vradio #f 1 (_ "Automatic") (_ "Edit with Menu")))
          (l (sgtk-label-normalized-new
               (_ "Select method for partitioning. \nWhen you select 'automatic', you will get a confirmation before the actual writing.\n")))
          (hr0 (sgtk-hseparator-new))
          (gb (sgtk-ok/cancel #t)))
      (sgtk-container-add v0 v1)
      (sgtk-container-add a (vr 'widget))
      (sgtk-box-pack-start v1 l #t #t 0)
      (sgtk-box-pack-start v1 a #f #f 0)
      (sgtk-box-pack-start v0 (gb 'widget) #f #f 0)
      (sgtk-paned-pack2 w v0 #t #f)
      (sgtk-widget-show-all w)
      (sgtk-main)
      (sgtk-container-remove w v0)
      (cond
        ((eq? 'back (gb 'info))
          #f)
        (else
          (case (vr 'info)
            ((0)
              (cond
                ((not (automatic))
                  (inst-log "automatic failed.\n")
                  (obj-part 'abort)
                  (start))
                ((not (and (swap-check) (fs-capacity-check)))
                  (start))
                (else
                  (let ((r (do-format #f #f)))
                    (inst-log "do-format result : ~a\n" r)
                    (cond
                      ((eq? r #f)
                        (start))
                      (else
                        #t))))))
            ((1)
              (cond
                ((not (mfdisk))
                  (inst-log "mfdisk canceled.\n")
                  (obj-part 'abort)
                  (start))
                ((not (and (swap-check) (fs-capacity-check)))
                  (start))
                (else
                  (let ((r (do-format #t #f)))
                    (inst-log "do-format result : ~a\n" r)
                    (cond
                      ((eq? r #f)
                        (start))
                      (else
                        #t))))))
            (else
              #f))))))
  
  (define (root-check)
    (cond
      ((null? (filter (lambda (x) (equal? "/" (list-ref x 4)))
                      (o 'list-mnt)))
        (sgnome-dialog-run
          (sgnome-message-box-new (_ "No root partition set.\n")
            "error" "Button_Ok"))
        #f)
      (else
        #t)))

  (define (partition-tree stop?)
    (let ((t (map (lambda (x)
                    (let ((d (car x)))
                      (cons
                        (list (sgtk-label-new (obj-part 'did-desc d)) #f)
                        (map (lambda (y)
                                (list (sgtk-label-new (obj-part 'hw-mid-print #f y)) y))
                             (obj-part 'list-part d)))))
                  (obj-part 'hw-disc))))
      (make-simple-tree stop? #f t)))
  
  (define (automatic)
    (let* ((v (sgtk-vbox-new #f 10))
           (b (sgtk-ok/cancel #t))
           (tr (partition-tree #f))
           (l (sgtk-label-normalized-new 
               (string-append
                 (_ "Disk(s) is/are partitioned as shown below.\n")
                 (_ "Mark regions which you plan to leave to the installer.\n\n")
                 (_ "To leave a whole disk, mark every region on the disk. (A mark on the disk will be ignored)\n")
                 (if (null? (obj-part 'hw-ataraid)) ""
                   (_ "IDE RAID is shown as single drive and logical drive. Be carefull not to specify both at a time.\n"))))))
        (sgtk-container-add-many v l (tr 'widget))
        (sgtk-paned-pack2 w v #t #f)
        (sgtk-box-pack-start v (b 'widget) #f #f 0)
        (sgtk-widget-show-all w)
        (tr 'expand)
        (sgtk-main)
        (sgtk-container-remove w v)
        (cond
          ((not (eq? 'forward (b 'info)))
            (inst-log "canceled\n")
            #f)
          (else
            (let ((ap (conf-autopart obj-part)))
              (cond
                ((not (ap 'init-tree (tr 'info)))
                  (sgtk-message-error (_ "Failed to do automatic partition(0).\n"))
                  (obj-part 'abort)
                  #f)
                ((eq? #f (ap 'analyze))
                  (sgtk-message-error (_ "Failed to do automatic partitioning(1).\n"))
                  (obj-part 'abort)
                  #f)
                (else
                  (let* ((v (sgtk-vbox-new #f 10))
                         (b (sgtk-ok/cancel #t))
                         (tr (partition-tree #f))
                         (l (sgtk-label-normalized-new 
                              (string-append
                                (_ "Disk(s) will be partitioned as shown below.\n")
                                (_ "If you are satisfied with the partition, press 'ok'.\n")))))
                      (sgtk-container-add-many v l)
                      (sgtk-container-add-many v (tr 'widget))
                      (sgtk-paned-pack2 w v #t #f)
                      (sgtk-box-pack-start v (b 'widget) #f #f 0)
                      (sgtk-widget-show-all w)
                      (tr 'expand)
                      (sgtk-main)
                      (sgtk-container-remove w v)
                      (cond
                        ((not (equal? 'forward (b 'info)))
                          (obj-part 'abort)
                          #f)
                        ((not (sgtk-message-while w "" (_ "Writing on to the disk(s) ...")
                                (lambda () (obj-part 'commit-part))))
                          (fprintf "/tmp/install-finished" "")
                          (sgtk-message-info
                            (string-append
                               (_ "Kernel could not read newly created partition. \n\n")
                               (_ "If you use IDE RAID device (or other similar one), this isn't error. ")
                               "\n"
                               (_ "Reboot system once, choose 'Edit with Menu'and don't modify partition next time. ")
                               (_ "\n\nPressing Ok will reboot system. ")))
                          (exit 0))
                        (else
                          #t))))))))))
  
  (define (mfdisk)
    (let ((v (sgtk-vbox-new #f 10))
          (tr (partition-tree #t))
          (a (sgtk-alignment-new 0 0.8 1 0.8))
          (l (sgtk-label-normalized-new 
              (string-append
                (_ "Disk(s) is/are partitioned as shown below.\n\n")
                (_ "Click partition which you want to modify."))))
          (b (sgtk-ok/cancel #t)))
      (sgtk-container-add a (tr 'widget))
      (sgtk-box-pack-start v l #f #f 0)
      (sgtk-container-add-many v a)
      (sgtk-box-pack-start v (b 'widget) #f #f 0)
      (sgtk-paned-pack2 w v #t #f)
      (sgtk-widget-show-all w)
      (tr 'expand)
      (sgtk-main)
      ;(format #t "mfdisk button ~a\n" (b 'info))
      (case (b 'info)
        ((none)
          (inst-log "tree got :~a\n" (tr 'info))
          (let* ((x (car (tr 'info)))
                 (p (tree-get (car x) (cadr x))))
            (inst-log "p got :~a\n" p)
            (case (car p)
              ((empty)
                (create-partition p))
              (else
                (modify-partition p))))
          (inst-log "pop up fin\n")
          (sgtk-container-remove w v)
          (mfdisk))
        ((back)
          (sgtk-container-remove w v)
          #f)
        (else
          (cond
            ((not (root-check))
              (sgtk-container-remove w v)
              (mfdisk))
            (else
              (let ((r (equal? 'forward (b 'info))))
                (if r
                   (sgtk-message-while w "" (_ "Writing on to the disk(s) ...")
                     (lambda () (obj-part 'commit-part))))
                (sgtk-container-remove w v)
                r)))))))
  
  (define (get-fs-sym s)
    (case (if (string? s) (string->symbol s) s)
      ((XFS) 'linux-xfs)
      ((Ext2) 'linux-ext2)
      ((Ext3) 'linux-ext3)
      ((ReiserFS) 'linux-reiser)
      ((JFS) 'linux-jfs)
      ((Swap) 'swap)
      ((Extended) 'ext)
      (else
        (inst-log "unknown fs type string : ~a\n" s)
        'linux-ext2)))
  
  (define (modify-partition p)
    (let* ((new (sgtk-window-new-toplevel-named (_ "Modify Partition")))
           (v (sgtk-vbox-new #f 10))
           (l (sgtk-label-new (_ "Enter new parameter for this partition, or push delete.")))
           (hb1 (sgtk-hbox-new #f 10))
           (lt1 (sgtk-label-new (_ "File System Type")))
           (om (make-option-menu #f
                 (case (car p) ((swap) 0)
                               ((linux-ext2) 2)
                               ((linux-ext3) 3)
                               ((linux-reiser) 4)
                               ((linux-jfs) 5)
                               (else 1))
                 '("Swap" "XFS" "Ext2" "Ext3" "ReiserFS" "JFS")))
           (prev-mnt (obj-part 'search-mnt p))
           (eb (sgtk-entrybox #f
                 (if mount?
                   (list (list (_ "Mount Point") prev-mnt #t))
                   '())))
           (bb (sgtk-bbox #t
                 (sgtk-button-new-with-label (_ "Modify"))
                 (sgtk-button-new-with-label (_ "Delete"))
                 (sgnome-stock-button "Button_Cancel"))))
      (sgtk-container-add-many hb1 lt1 (om 'widget))
      (sgtk-container-add-many v l hb1 (eb 'widget) (bb 'widget))
      (sgtk-container-add new v)
      (sgtk-widget-show-all new)
      (sgtk-grab-add new)
      (sgtk-widget-set-sensitive w #f)
      (sgtk-main)
      (eb 'freeze)
      (sgtk-widget-destroy new)
      (sgtk-widget-set-sensitive w #t)
      (let ((mnt (if mount? (car (eb 'info)) prev-mnt))
            (t (get-fs-sym (om 'info))))
        (case (bb 'info)
          ((2) #t)
          ((1)
            (and (if (equal? "" mnt) #t (obj-part 'del-mnt mnt))
                 (obj-part 'delete-part (cadr p) (list-ref p 3))))
          (else
            (cond
              ((and (equal? 'swap t)
                    (and (not (equal? mnt ""))
                         (not (equal? mnt "swap"))))
                (sgtk-message-error (_ "Swap and mount point cannot be specified at the same time.")))
              ;((or (equal? "" mnt)
              ;     (not mount?))
              ;  (sgtk-message-error (_ "Specify mount point.")))
              ((equal? 'ext t)
                (if (not (equal? mnt ""))
                  (sgtk-message-error
                    (_ "Mount point for extended partition was ignored.\n"))))
              ((not (obj-part 'modify-part (cadr p) (list-ref p 3) t))
                (sgtk-message-error (_ "Failed to change partition ID.")))
              ((equal? "" mnt)
                (obj-part 'del-mnt prev-mnt))
              (else
                (let ((r (o 'add-mnt (cons t (cdr p)) mnt)))
                  (cond
                    ((not (eq? #t r))
                      (sgtk-message-error
                        (format #f (_ "Could not change mount point.\nReason:~a") r)))))))
           #t)))))
  
  (define (create-partition p)
    (define (ask s e ext?)
      (let* ((did (cadr p))
             (u (* (obj-part 'unit-size did) (obj-part 'block-size did)))
             (mn (i/ (* u (1+ (- e s))) 1024))
             (m (format #f "~a" mn)))
        (let ((new (sgtk-window-new-toplevel-named (_ "Create Partition")))
              (v (sgtk-vbox-new #f 10))
              (l (sgtk-label-new (_ "Enter parameters for this partition.")))
              (eb (sgtk-entrybox #f
                    (append
                      (list (list (_ "Size (in MB)") (format #f "~a" (caddr p)) #t))
                      (if mount? (list (list (_ "Mount Point") "/" #t)) '()))))
              (hb2 (sgtk-hbox-new #f 10))
              (lt (sgtk-label-new (_ "File System Type")))
              (om (make-option-menu #f 1 (if ext? '("Swap" "XFS" "Ext2" "Ext3" "ReiserFS" "JFS" "Extended")
                                                  '("Swap" "XFS" "Ext2" "Ext3" "ReiserFS" "JFS"))))
              (bb (sgtk-ok/cancel #t)))
          (sgtk-container-add-many hb2 lt (om 'widget))
          (sgtk-container-add-many v l (eb 'widget) hb2 (bb 'widget))
          (sgtk-container-add new v)
          (sgtk-widget-show-all new)
          (sgtk-widget-set-sensitive w #f)
          (sgtk-grab-add new)
          (sgtk-widget-set-sensitive w #t)
          (sgtk-main)
          (eb 'freeze)
          (sgtk-widget-destroy new)
          (cond
            ((not (equal? 'forward (bb 'info)))
              #f)
            (else
              (let ((n (string->number (car (eb 'info))))
                    (t (get-fs-sym (om 'info))))
                (cond
                  ((eq? n #f)
                    (sgtk-message-error (_ "Number character is only allowed for size.")))
                  ((not (and (< 0 n)
                             (<= n mn)))
                    (sgtk-message-error
                       (string-append
                         (_ "Entered value exceeds the valid range for partition size.\n")
                         (sprintf (_ "Enter value between %d and %d.\n") 0 mn))))
                  (else
                    (inst-log "p:~a\n" p)
                    (let ((i (if (equal? 'e-empty (list-ref p 3)) 0 -1))
                          (e (if (= n (caddr p))
                                 (list-ref p 5)
                                 (max s (+ s (1- (i/ (* n 1024 1024) u))))))
                          (mnt (if mount? (cadr (eb 'info)) "")))
                      (cond
                        ((if (equal? 'swap t)
                             (and (not (equal? mnt ""))
                                  (not (equal? mnt "swap")))
                             (equal? mnt "swap"))
                          (sgtk-message-error (_ "Swap and mount point cannot be specified at the same time.")))
                        ((not (obj-part 'create-part did t i s e))
                          (sgtk-message-error (_ "Failed to create partition.")))
                        ((or (not mount?)
                             (equal? "" mnt))
                          #f)
                        ((equal? 'ext t)
                          (if (not (equal? mnt ""))
                            (sgtk-message-error
                              (_ "Mount point for extended partition was ignored.\n"))))
                        (else
                         (let ((r (obj-part 'add-mnt (obj-part 'get-part did s e) mnt)))
                            (cond
                              ((not (eq? #t r))
                                (sgtk-message-error
                                  (format #f (_ "Could not set mount point.\nReason:~a") r)))))))
                      #t
                      )))))))))
    
    (cond
       ((eq? 'e-empty (list-ref p 3))
         (ask (list-ref p 4) (list-ref p 5) #f))
       (else
         (let* ((did (cadr p))
                (pr (obj-part 'rest-primary did))
                (ep? (obj-part 'have-extended did)))
           (cond
             ((null? pr)
               (sgtk-message-error
                 (string-append
                   (_ "You are only able to have 4 primary partitions at most.\n")
                   (_ "To create more primary/extended partition, first delete some other partition."))))
             (else
               (ask (list-ref p 4) (list-ref p 5) (not ep?))))))))
  
  (define (part-to-dummy-mnt l)
    (map (lambda (x)
           (list (cadr x) #f (list-ref x 5) (list-ref x 6) ""
                 (list-ref x 3) (car x)))
         l))
  
  ;
  ; ̾ΰեޥåȴؿdo- ȤĤġǽ confirm 롣
  ;
  (define (do-format q? new?)
    (early-swapoff-check)
    (let ((new (if new? '()
                        (o 'list-always-mnt #f))))
      (inst-log "do-format new:~s\n" new)
      
      (define (confirm)
        (let ((v (sgtk-vbox-new #f 10))
              (b (sgtk-ok/cancel #t))
              (cl (sgtk-checkbox (map (lambda (x)
                                        (list (obj-part 'did-desc (car x) (list-ref x 5)) #t))
                                      new)))
              (l (sgtk-label-normalized-new 
                   (string-append
                     (_ "Checked partitions are going to be formatted.\n")
                     (_ "If you have a partition not to be formatted, uncheck it.\n\n")))))
          (sgtk-container-add-many v l (cl 'widget))
          (sgtk-paned-pack2 w v #t #f)
          (sgtk-box-pack-start v (b 'widget) #f #f 0)
          (sgtk-widget-show-all w)
          (sgtk-main)
          (cl 'freeze)
          (inst-log "confirm button : ~a\n" (b 'info))
          (cond
            ((equal? (b 'info) 'back)
              (sgtk-container-remove w v)
              #f)
            (else
              (let ((r (filter (lambda (x) (not (eq? #f x)))
                         (map (lambda (z y) (if z y #f))
                              (cl 'info) new))))
                (sgtk-container-remove w v)
                r)))))
      
      (define (run pl)
        (let ((v (sgtk-vbox-new #f 10))
              (pg (map (lambda (x) (sgtk-progress-bar-new))
                       pl))
              (ls (map (lambda (x) (sgtk-label-new (obj-part 'devfsname (car x) (list-ref x 5))))
                       pl))
              (l (sgtk-label-new (_ "Formatting ..."))))
          (sgtk-paned-pack2 w v #t #f)
          (sgtk-container-add v l)
          (map (lambda (x y)
                 (let ((h (sgtk-hbox-new #f 10)))
                   (sgtk-container-add-many h x y)
                   (sgtk-box-pack-start v h #f #f 0)
                  ; (sgtk-container-add v h)
                   ))
               ls pg)
          (sgtk-widget-show-all w)
          (sgtk-update)
          (map (lambda (x y)
                 (let ((d (obj-part 'devfsname (car y) (list-ref y 5))))
                   (sh "dd if=/dev/zero of=~a bs=1024 count=65" d)
                   (progress-start
                     (let ((log (if (equal? "logger" sh-log) "/dev/null" sh-log)))
                       (format #f "PROGRESS_FD=9 ~a ~a 9>&1 2>~a 1>~a | tee /tmp/mkfslog "
                               (o 'format-command (list-ref y 6)) d log log))
                     (lambda (f s)
                       ;(format #t "got ~a\n" f)
                       (sgtk-progress-bar-update x f)
                       (sgtk-update)))
                   (if (equal? 'swap (list-ref y 6))
                     (sh "swapon ~a" d))))
               pg pl)
          (sgtk-container-remove w v)
          #t))
      
      (let ((l (if q? (confirm) new)))
        (cond
          ((eq? #f l)
            (inst-log "do-format returns ~a\n" l)
            #f)
          ((null? l)
            (inst-log "do-format returned null\n")
            #f)
          (else
            (run l))))))
  
  (define (sanity-check)
    (let ((l (filter (lambda (x) (not (obj-part 'sane-table? (car x))))
                     (obj-part 'hw-disc))))
      (cond
        ((null? l)
          #t)
        (else
          (let ((r (sgtk-choice w
                     (apply string-append
                       (append
                         (list (_ "Disk(s) below seems to have a broken partition table.\n\n"))
                         (map (lambda (x)
                                (format #f "  - ~a" (obj-part 'did-desc (car x))))
                              l)
                         (list
                           (_ "\n\nDo you initialize a partition table of this device(s)? ")
                           (_ "If not, you need to initialize it manually (ie using dd or fdisk) before installation")))))))
            (cond
              ((not (= 1 r))
                #t)
              ((not (andmap
                      (lambda (x)
                        (= 0 (sh "dd if=/dev/zero of=~a count=1" (obj-part 'devfsname (car x)))))
                      l))
                (sgtk-message-error (_ "Could not initialize partition. (no dd command?)")))
              (else
                (obj-part 'abort)
                #t)))))))
  
  (define (swap-check)
    (inst-log "Checking swap ...\n")
    (let ((mem (unix-filesize "/proc/kcore"))
          (sw (apply +
                (map (lambda (x)
                       (case (list-ref x 6)
                         ((swap)
                           (* (obj-part 'block-size (car x))
                              (obj-part 'unit-size (car x))
                              (1+ (- (list-ref x 3) (list-ref x 2)))))
                         (else
                           0)))
                     (obj-part 'list-mnt))))
          (retry (lambda () (begin (obj-part 'abort) #f))))
      (inst-log "mem :~a, sw:~a\n" mem sw)
      (cond
        ((< (* 255 1024 1024) (+ mem sw))
          #t)
        ((equal? 'orca (product-type))
          (sgtk-message-error
            (string-append
              (_ "In order to run ORCA server, the machine should virtually have at least 256MB memory. ")
              (_ "Set partition again and increase swap size. ")))
          (retry))
        ((= 0 (sgtk-choice-new (_ "Partition Setting")
                (string-append
                  (_ "In order to run most application, it is recommended that the machine virtually has at least 256MB memory. ")
                  (_ "Do you set partition again?"))
                (_ "Yes") (_ "No")))
          (obj-part 'abort)
          (retry))
        (else
          #t))))
  
  (define (fs-capacity-check)
    (inst-log "checking capacity ...\n")
    (let* ((sz (apply +
                 (map (lambda (x)
                        (case (list-ref x 6)
                          ((swap)
                            0)
                          (else
                            (* (obj-part 'block-size (car x))
                               (obj-part 'unit-size (car x))
                               (1+ (- (list-ref x 3) (list-ref x 2)))))))
                      (obj-part 'list-mnt))))
           (szg (/ sz (* 1024 1024 1024))))
      (inst-log "sz : ~a, sgz : ~a\n" sz szg)
      (if (equal? 'orca (product-type))
        (cond
          ((< 10.0 szg)
            #t)
          ((= 0 (sgtk-choice-new (_ "Warning")
                  (string-append
                    (if (< szg 2.0)
                      (_ "Too small free disk space. ")
                      (_ "Small free disk space. "))
                    "\n\n"
                    (if (< szg 3.0)
                      (_ "In order to install ORCA, there should be at least 3GB of free space. ")
                      "")
                    (_ "If you plan to use the machine as simple ORCA client, this may not be a problem. (if so, at least 2GB of free space is required) ")
                    (_ "If you plan to run ORCA server, free space should be have more than 20GB. ")
                    (_ "\n\nDo you retry the partition setting?"))
                  (_ "Yes") (_ "No")))
            (obj-part 'abort)
            #f)
          (else
            #t))
        (cond
          ((< 3.0 szg)
            #t)
          ((= 0 (sgtk-choice-new (_ "Warning")
                  (string-append
                    (cond
                      ((< szg 1.4)
                        (_ "Free disk space is so small that an average installation cannot be accomplished. "))
                      ((< szg 2.0)
                        (_ "Free disk space is too small. "))
                      (else
                        (_ "Free disk space is small. ")))
                    (_ "\n\nIn order to run most application, it is recommended you have at least 3GB of free space. ")
                    (_ "\n\nDo you retry the partition setting?"))
                  (_ "Yes") (_ "No")))
            (obj-part 'abort)
            #f)
          (else
            #t)))))
  
;  (define (start-start)
;    (and (start)
;      (cond
;        ((not (and (swap-check) (fs-capacity-check)))
;          (start-start))
;        (else
;          #t))))
  
  (cond
    ((and (not mount?)
       (case (o 'get-context)
         ((inst3 admin) #t)
         (else #f)))
      (cond
        ((mfdisk)
          (sgtk-message-info
            (string-append
              (_ "You need to manually format the newly created partitions. ")
              (_ "When you modify the already mounted drive, you also need to reboot before formatting."))))
        (else
          #t)))
    (else
      (cond
        ((not (sanity-check))
          #f)
        ((not (start))
          #f)
        (mount?
          (let ((c (o 'check-mnt)))
            (cond
              ((eq? #f (o 'domount))
                (sgnome-dialog-run
                  (sgnome-message-box-new (_ "Fatail error. Cannot mount formated partition.")
                    "error" "Button_Ok"))
                #f)
              ((eq? c #t)
                #t)
              (else
                (sgnome-dialog-run
                  (sgnome-message-box-new
                    (format #f (_ "Fatal Error. Wrong parititons : ~a") c)
                    "error" "Button_Ok"))
                #f))))
        (else
          #t)))))

(define (gtk-ui-bootfd w inst? obj-boot obj-part)
  (inst-log "==== gtk-ui-bootfd \n")
  (let* ((f (obj-part 'get-fdd)))
    (cond
      ((equal? f "")
        (sgtk-message-error (_ "Could not find floppy device.")))
      (else
        (let* ((old? (or (equal? f "/dev/fd0") (equal? f "/dev/floppy/0")))
               (v  (sgtk-vbox-new #f 10))
               (ll (sgtk-label-new 
                     (if inst?
                       (_ "Remove installer FD from drive and insert new floppy disk.\n")
                       (_ "Insert new floppy disk.\n"))))
               (cl (sgtk-checkbox (list (list (_ "format before making disk") #t))))
               (bb (sgtk-ok/cancel #t)))
          (sgtk-container-add v ll)
          (if old? (sgtk-container-add v (cl 'widget)))
          (sgtk-box-pack-start v (bb 'widget) #f #f 0)
          (sgtk-paned-pack2 w v #f #f)
          (sgtk-widget-show-all w)
          (sgtk-main)
          (cl 'freeze)
          (cond
            ((not (equal? 'forward (bb 'info)))
              (sgtk-container-remove w v)
              #f)
            (else
              (let ((f? (car (cl 'info)))
                    (ll (sgtk-label-new (_ "Creating boot floppy disk ..."))))
                (sgtk-container-remove w v)
                (sgtk-paned-pack2 w ll #t #f)
                (sgtk-widget-show-all w)
                (sgtk-update)
                (let ((r (obj-boot 'make-fd (and old? f?))))
                  (sgtk-widget-destroy ll)
                  (cond
                    ((string? r)
                      (sgtk-message-error
                        (format #f (_ "Could not create boot floppy disk.\nReason:~a\n\n") r))
                      (gtk-ui-bootfd w inst? obj-boot obj-part))
                    (else
                      #t)))))))))))

(define (gtk-ui-bios-order w obj-part)

  (define (if->str x)
    (sprintf "   %s   " (obj-part 'hw-ifname (car x) (cadr x))))

  (define (make-up n l)
    (cond
      ((or (= n 0) (null? l)) l)
      ((not (= n 1))
        (cons (car l) (make-up (1- n) (cdr l))))
      ((= 1 (length l))
        l)
      (else
        (cons (cadr l)
          (cons (car l) (cddr l))))))
  
  (define (rep g)
    (let* ((v  (sgtk-vbox-new #f 10))
           (ll (sgtk-label-new (_ "Selected item will moves upward.\n")))
           (lb (apply sgtk-labelbox
                 (cons #t (smap (lambda (n x) (format #f "~a.  ~a  \n"
                                                         (1+ n) (if->str x)))
                                g))))
           (bb (sgtk-ok/cancel #t)))
      (sgtk-container-add-many v ll)
      (sgtk-box-pack-start v (lb 'widget) #f #f 0)
      (sgtk-box-pack-start v (bb 'widget) #f #f 0)
      (sgtk-paned-pack2 w v #f #f)
      (sgtk-widget-show-all w)
      (sgtk-main)
      (sgtk-container-remove w v)
      (case (bb 'info)
        ((forward)
          g)
        ((back)
          #f)
        (else
          (let ((r (make-up (lb 'info) g)))
            (if (list? r) (rep r) #f))))))
  
  (let ((g (obj-part 'hw-get-biosorder)))
    (cond
      ((< (length g) 2)
        #t)
      (else
        (let ((r (sgtk-choice w
                   (apply string-append
                     (append
                       (list
                         (_ "Several type of disc interfaces are found. ")
                         (_ "Either to boot or to install boot-loader correctly, you need to specify drive order in BIOS.")
                         (_ "Order of line below should be exactly as same as that of BIOS. ")
                         (_ "Do you set up BIOS drive order?\n\n"))
                       (map (lambda (x) (format #f "  ~a  \n" (if->str x)))
                            g)))
                   "Button_No" "Button_Yes")))
          (cond
            ((not (= r 1))
              #t)
            (else
              (let ((r (rep g)))
                (cond
                  ((list? r)
                    (obj-part 'hw-set-biosorder r))
                  (else
                    #f))))))))))

(define (gtk-ui-power-management w obj-mod obj-boot)
  (let* ((acpi-valid? (not (null? (string-match (obj-boot 'get-option) "acpi=on"))))
         (apm-valid? (not (null? (filter (lambda (x) (equal? 'apm (car x)))
                          (obj-mod 'table)))))
         (opt (string-gsub (string-gsub (obj-boot 'get-option) "acpi=[^ ]\\+" "")
                           "  " " "))
         (v0 (sgtk-vbox-new #f 10))
         (v1 (sgtk-vbox-new #t 10))
         (a (sgtk-alignment-new 0.8 0.8 0.8 0.8))
         (lb (sgtk-labelbox #t
               (_ "Use APM") (_ "Use ACPI") (_ "No Power Management")))
         (m (sgtk-label-normalized-new 
              (string-append
                (_ "Specify the method for power management.\n")
                (_ "Linux can do two ways of power management : APM and ACPI\n\n")
                (_ "APM is classical and safe. ")
                (_ "Although Linux ACPI is still under development and doesn't work enough, ")
                (_ "some recent notebook PCs lack APM and might require ACPI in order to work PCI device properly.")
                (_ "\n\nCurrent status : ")
                (cond
                  (acpi-valid? (_ "ACPI valid "))
                  (apm-valid? (_ "APM valid "))
                  (else (_ "No Power management")))
                "\n")))
         (hr0 (sgtk-hseparator-new))
         (bb (sgtk-ok/cancel #t)))
    (sgtk-container-add v0 v1)
    (sgtk-container-add a (lb 'widget))
    (sgtk-box-pack-start v1 m #t #t 0)
    (sgtk-box-pack-start v1 a #f #f 0)
    (sgtk-box-pack-start v0 (bb 'widget) #f #f 0)
    (sgtk-paned-pack2 w v0 #t #f)
    (sgtk-widget-show-all w)
    (sgtk-main)
    (sgtk-container-remove w v0)
    (case (bb 'info)
      ((forward)
        #t)
      ((back)
        #f)
      (else
        (case (lb 'info)
          ((0)
            (if (not apm-valid?)
              (obj-mod 'setmod "apm" "" 'inst/set))
            (if acpi-valid?
              (obj-boot 'set-option opt))
            #t)
          ((1)
            (if apm-valid?
              (obj-mod 'rmmod "apm"))
            (if (not acpi-valid?)
              (obj-boot 'set-option (format #f "~a acpi=on" opt)))
            #t)
          ((2)
            (if apm-valid?
              (obj-mod 'rmmod "apm"))
            (if acpi-valid?
              (obj-boot 'set-option opt))
            #t)
          (else
            #f))))))

