
(define (gtk-ui-atok-key w obj-pd obj-sv)
  (inst-log "gtk-ui-atok-key =====")
  (define title (_ "ATOK X Keybinding"))
  (let ((dir "/usr/lib/im/locale/ja/atokserver"))
    (cond
      ((not (unix-access? dir '(read)))
        (inst-log "gtk-ui-atok-key : could not access to the atox server directory. skipping.")
        #f)
      (else
        (let* ((v0 (sgtk-vbox-new #f 10))
               (v1 (sgtk-vbox-new #f 10))
               (a (sgtk-alignment-new 0.8 0.8 0.8 0.8))
               (l (sgtk-label-normalized-new
                   (string-append
                     (_ "Select a keybinding to invoke the ATOK X. ")
                     "\n\n"
                     (_ "ATOK X has two way to invoke : 'Ctrl + Space' or 'Shift + space' ")
                     (_ "If you would like to use an editor from emacs family, it is recommended to choose 'Shift + Space'. ")
                     (_ "Other than these bindings, 'Zenkaku' key can be also used. "))))
               (vr (make-vradio #f 0 (_ "Shift + Space") (_ "Ctrl + Space")))
               (hr (sgtk-hseparator-new))
               (bb (sgtk-boxed-ok)))
          (sgtk-label-set-justify l (sgtk-val "GTK_JUSTIFY_LEFT"))
          (sgtk-paned-pack2 w v0 #t #f)
          (sgtk-container-add v0 v1)
          (sgtk-container-add a (vr 'widget))
          (sgtk-box-pack-start v1 l #t #t 0)
          (sgtk-box-pack-start v1 a #f #f 0)
          (sgtk-box-pack-start v0 bb #f #f 0)
          (sgtk-widget-show-all v0)
          (sgtk-main)
          (sgtk-container-remove w v0)
          (case (vr 'info)
            ((1) #t)
            (else
              (sh "touch ~a/shift_space" dir)
              (if (obj-sv 'start? "atokx")
                (obj-sv 'restart "atokx"))
              #t)))))))

(define (gtk-ui-additional w inst? obj-add)
  (define title (_ "Additional package installation"))
  
  (define (license n)
    (= 0 (sh "~agless ~a"
           (if (lang-ja?)
             "LANG=ja_JP.eucJP "
             "")
           (list-ref (obj-add 'license) n))))
  
  (define (ask n)
    (let ((new (sgtk-window-new-toplevel-named (_ "Change Password/Deletion")))
          (v (sgtk-vbox-new #f 10))
          (l (sgtk-label-normalized-new (list-ref (obj-add 'asking) n)))
          (bb (sgtk-bbox #t
                (sgnome-stock-button "Button_Ok")
                (sgnome-stock-button "Button_Cancel")
                (sgtk-button-new-with-label (_ "View Once")))))
      (sgtk-container-add-many v l)
      (sgtk-box-pack-start v (bb 'widget) #f #f 0)
      (sgtk-container-add new v)
      (sgtk-widget-show-all new)
      (sgtk-widget-set-usize new 480 120)
      (sgtk-main)
      (sgtk-widget-destroy new)
      (case (bb 'info)
        ((0)
          (install n))
        ((2)
          (sgtk-update)
          (license n)
          (ask n))
        (else
          #f))))
  
  (define (install n)
    (sgtk-show-while-new title
      (list-ref (obj-add 'waiting) n)
      (lambda ()
        (= 0 (sh (list-ref (obj-add 'installer) n))))))
  
  (define (show-all)
    (let* ((v (sgtk-vbox-new #f 10))
           (ll (sgtk-label-normalized-new
                 (apply string-append
                   (cons
                     (_ "Status of additional packages has been changed as below:\n\n")
                     (map (lambda (x)
                            (sprintf "  %-12s  \n        --- %s\n" (car x) (cdr x)))
                          (obj-add 'status-all))))))
           (bb (sgtk-bbox-ok)))
      (sgtk-box-pack-start v ll #t #f 10)
      (sgtk-box-pack-start v (bb 'widget) #f #f 0)
      (sgtk-container-add w v)
      (sgtk-widget-show-all w)
      (sgtk-main)
      (sgtk-container-remove w v)
      #t))
  
  (define (start)
    (let* ((v (sgtk-vbox-new #f 10))
           (h (sgtk-hbox-new #f 10))
           (l (sgtk-label-normalized-new
                (_ "Select additional software which you would like to install.")))
           (n -1)
           (ll (let ((v (sgtk-vbox-new #f 10)))
                 (smap (lambda (m x y)
                         (let* ((h (sgtk-hbox-new #f 10))
                                (l0 (sgtk-label-new x))
                                (l1 (cond
                                      ((equal? y (_ "installed"))
                                        (sgtk-label-new y))
                                      (else
                                        (let ((b (sgtk-button-new-with-label y)))
                                          (sgtk-signal-connect b "clicked"
                                            (lambda (w)
                                              (set! n m)
                                              (sgtk-main-quit)))
                                          b)))))
                           (sgtk-container-add-many h l0 l1)
                           (sgtk-box-pack-start v h #f #f 0)))
                        (obj-add 'desc)
                        (obj-add 'status))
                 v))
           (st (obj-add 'status))
           (lb (apply sgtk-labelbox (cons #t st)))
           (bb (sgtk-ok/cancel #t)))
      (sgtk-container-add-many v l ll)
      (sgtk-box-pack-start v (bb 'widget) #f #f 0)
      (sgtk-container-add w v)
      (sgtk-widget-show-all w)
      (sgtk-main)
      (sgtk-container-remove w v)
      (case (bb 'info)
        ((back)
          #f)
        ((forward)
          (show-all))
        (else
          (cond
            ((< n 0)
              #f)
            ((equal? (list-ref st n) (_ "installed"))
              (start))
            (else
              (license n)
              (ask n)
              (start)))))))
  
  (cond
    ((not (obj-add 'init (if inst? 'inst3 'admin) #t))
      (sgtk-message-info
        (if inst?
          (string-append
            (_ "Currently this host doesn't seems to be ready for addtional package installation. ")
            (_ "Mostly this is because of lacking of network connectivity. ")
            (_ "You can install additional packages later with ogl-admin. "))
          (_ "Could not read addtional package table nor get package selection. Skipping."))))
    (else
      (let ((l (obj-add 'desc)))
        (cond
          ((null? l)
            (if (not inst?)
              (sgtk-message-info
                (_ "No additional package was found which is to be installed.")))
            #t)
          (else
            (start)))))))

(define (gtk-ui-default-media w obj-med obj-na)
  (cond
    ((not (= 1 (sgtk-choice w 
                 (string-append
                   (_ "Do you want to set the default package source to the ARMA Net web?\n")
                   (_ "After setting it, upgrading will be expected very smooth with the ARMA Net web. Otherwise default package source will remain CD-ROM."))
                 "Button_No" "Button_Yes")))
      #t)
    (else
      (obj-med 'set 'http "www.omoikane.co.jp" "arma_2.1_updates/deb"
                (format #f "D~a" (obj-na 'get-id)) (obj-na 'get-hashed) "" -1)
      (sgtk-message-info
        (_ "The default package source has been changed to the ARMA Net.\nIf you are using HTTP proxy, run the administration tool (ogl-admin) after installation is completed."))
      #t)))

(define (gtk-ui-orca w obj-pd patch obj-prof obj-cr)
  (cond
    ((not (equal? 'orca (obj-pd 'product-type)))
      (sgtk-message-error
        (_ "This product does not seem to be ORCA edition. Aborting.")))
    (else
      (let ((t (obj-pd 'get-orcatype)))
        (case t
          ((unset)
            (gtk-ui-orca-type w obj-pd patch obj-prof obj-cr))
          (else
            (cond
              ((< (obj-pd 'get-initstep) 100)
                (case t
                  ((primary)
                    (gtk-ui-orca-type-primary w obj-pd patch obj-cr))
                  ((secondary)
                    (gtk-ui-orca-type-secondary w obj-pd patch))
                  ((single)
                    (gtk-ui-orca-type-single w obj-pd patch))
                  (else
                    (gtk-ui-orca-type-client w obj-pd))))
              (else
                (gtk-ui-orca-conf w obj-pd patch obj-cr)))))))))

(define (gtk-ui-orca-type w obj-pd patch obj-prof obj-cr)
  (cond
    ((not (equal? 'unset (obj-pd 'get-orcatype)))
      (sgtk-message-error
        (_ "This host is already configured."))
      #f)
    (else
      (let* ((l (list (_ "Primary Server of Dual server") (_ "Secondary Server of Dual server")
                      (_ "Single Server") (_ "Client")))
             (ll (sgtk-label-normalized-new
                   (string-append
                     (_ "Select ORCA host type configuration.\n")
                     (_ "Be careful that once you select type of host, it cannot be changed later. "))))
             (v (sgtk-vbox-new #f 10))
             (a (sgtk-alignment-new 0.8 0.8 0.8 0.8))
             (lb (apply sgtk-labelbox (cons #t l)))
             (bb (sgtk-ok/cancel #t)))
        (sgtk-container-add a (lb 'widget))
        (sgtk-container-add v ll)
        (sgtk-box-pack-start v a #f #f 0)
        (sgtk-box-pack-start v (bb 'widget) #f #f 0)
        (sgtk-paned-pack2 w v #t #f)
        (sgtk-widget-show-all w)
        (sgtk-main)
        (sgtk-container-remove w v)
        (case (bb 'info)
          ((back)
            #f)
          ((forward)
            #t)
          (else
            (let ((r1 (sgtk-choice w
                        (format #f (_ "This host will be set as ORCA host type\n\n  : ~a\n\nAre ok with this setting?")
                          (list-ref l (lb 'info)))
                        "Button_No" "Button_Yes")))
              (case r1
                ((0)
                  (gtk-ui-orca-type w obj-pd patch obj-prof obj-cr))
                (else
                  (case (lb 'info)
                    ((0 1 2)
                      (obj-prof 'update-line "psql default host" "PGHOST=localhost\nexport PGHOST")
                      (obj-prof 'update-line "psql default port" "PGPORT=5432\nexport PGPORT")))
                  (case (lb 'info)
                    ((0)
                      (gtk-ui-orca-type-primary w obj-pd patch obj-cr))
                    ((1)
                      (gtk-ui-orca-type-secondary w obj-pd patch))
                    ((2)
                      (gtk-ui-orca-type-single w obj-pd patch))
                    ((3)
                      (gtk-ui-orca-type-client w obj-pd patch))
                    (else
                      #t)))))))))))

(define (gtk-ui-orca-primary-mtacheck title obj-pd)
  (cond
    ((obj-pd 'have-mail?)
      #t)
    ((not (= 0 (sgtk-choice-new title
                 (string-append
                   (_ "Mail server seems not to be running. ")
                   "\n\n"
                   (_ "With the dual ORCA configuration, unix user 'orca' can accept the alert mail from log checker, which sees the DB replicator log and will send alert if it find the error. ")
                   (_ "This function may help the administrator to find a early corruption of the dual system. ")
                   (_ "In order to get this alert, you need to start mail server. ")
                   "\n\n"
                   (_ "Do you start the mail server?"))
                 (_ "Yes") (_ "No"))))
      #f)
    ((not (sgtk-show-while-new title
            (_ "Starting the mail server ...")
            (lambda () (obj-pd 'start-mail))))
      (sgtk-message-error
        (_ "Failed to start the mail server. Please investigate the system later.")))
    (else
      #t)))

(define (gtk-ui-orca-type-primary w obj-pd patch obj-cr)
  (define title (_ "Primary Server Setting"))
  (define (set-host?)
    (cond
      ((< (obj-pd 'get-initstep) 20)
        #t)
      (else
        (= 0 (sgtk-choice-new title
               (_ "Do you reconfigure secondary server address setting?")
               (_ "Yes") (_ "No"))))))
  (obj-pd 'set-orcatype 'primary)
  (obj-pd 'stop-server)
  (cond
    ((and (set-host?)
          (not (gtk-ui-orca-primary-host w #t obj-pd patch)))
      #f)
    (else
      (let* ((n (obj-pd 'get-initstep))
             (gl (obj-pd 'get-gl)))
        (cond
          ((not (eq? #t (obj-pd 'secondary-test (cadr (obj-pd 'get-gl)))))
            (sgtk-message-info
              (string-append
                (_ "The ORCA primary server has been partially configured. ")
                "\n\n"
                (_ "To complete the configuration, please configure and start the secondary server first. ")
                (_ "Then run the ogl-admin on the primary server(this host) and select ORCA configuration again. ")))
            #f)
          ((not (and (obj-pd 'wait-server)
                     (if (< n 60) (gtk-ui-orca-init w obj-pd) #t)
                     (if (< n 80) (gtk-ui-orca-master w obj-pd) #t)))
            #f)
          (else
            (gtk-ui-orca-schedule w obj-pd obj-cr)
            (gtk-ui-orca-primary-mtacheck title obj-pd)
            (obj-pd 'set-initstep 100)
            (cond
              ((= 1 (sgtk-choice-new title
                      (_ "ORCA primary server was configured. Do you start ORCA server now?")
                      (_ "Yes") (_ "No")))
                (obj-pd 'run-server))
              (else
                #t))))))))

(define (gtk-ui-orca-type-secondary w obj-pd patch)
  (obj-pd 'set-orcatype 'secondary)
  (obj-pd 'update-secondary "127.0.0.1" 8000)
  (patch 'commit)
  (obj-pd 'commit)
  (obj-pd 'set-initstep 20)
  (gtk-ui-orca-secondary-host w #t obj-pd patch))

(define (gtk-ui-orca-type-single w obj-pd patch)
  (obj-pd 'set-orcatype 'single)
  (obj-pd 'update-primary "127.0.0.1" 8000)
  (patch 'commit)
  (obj-pd 'commit)
  (obj-pd 'set-initstep 20)
  (let ((n (obj-pd 'get-initstep)))
    (cond
      ((not (and (obj-pd 'wait-server)
                 (if (< n 60) (gtk-ui-orca-init w obj-pd) #t)
                 (if (< n 80) (gtk-ui-orca-master w obj-pd) #t)))
        #f)
      (else
        (obj-pd 'set-initstep 100)
        (cond
          ((= 0 (sgtk-choice-new (_ "Single Server Setting")
                  (_ "ORCA was configured. Do you start ORCA server now?")
                  (_ "Yes") (_ "No")))
            (obj-pd 'run-server))
          (else
            #t))))))

(define (gtk-ui-orca-type-client w obj-pd patch)
  (and (obj-pd 'set-orcatype 'client)
       (gtk-ui-orca-host w obj-pd patch)))

(define (gtk-ui-orca-check-wait title m obj-pd)
  (cond
    ((equal? 'wait (obj-pd 'stat-server))
      #t)
    ((not (= 0 (sgtk-choice-new title
                 (string-append
                   m
                   "\n"
                   (_ "Do you want to change the server status now?"))
                 (_ "Yes") (_ "No"))))
      #f)
    (else
      (let ((r (sgtk-show-while-new title
                 (_ "Changing server status to 'wait' ...")
                 (lambda () (obj-pd 'wait-server)))))
        (cond
          ((null? r)
            #t)
          (else
            (sgtk-message-error
              (format #f (_ "Could not change service(s) status below: \n~a")
                (list-foldstring " " r)))
            #f))))))

(define (gtk-ui-orca-hospid w obj-pd)
  (define title (_ "Hostpital ID Setting"))
  (cond
    ((not (gtk-ui-orca-check-wait title
            (_ "To change hostpital ID, the ORCA server should be in the wait status. ")
            obj-pd))
      #f)
    ((not (gtk-ui-secondary-check obj-pd))
      #f)
    (else
      (let ((new (sgtk-window-new-toplevel-named title))
            (v (sgtk-vbox-new #f 10))
            (l (sgtk-label-new (_ "Enter Hostpital ID.\n")))
            (eb (sgtk-entrybox #f
                  (list (list (_ "Hostpital ID :") (obj-pd 'get-medid) #t))))
            (bb (sgtk-ok/cancel #t)))
        (sgtk-container-add-many v l (eb 'widget))
        (sgtk-box-pack-start v (bb 'widget) #f #f 0)
        (sgtk-container-add new v)
        (sgtk-widget-show-all new)
        (sgtk-main)
        (eb 'freeze)
        (sgtk-widget-destroy new)
        (case (bb 'info)
          ((back)
            (sgtk-message-info
              (_ "Hospital ID was not modified."))
            #f)
          ((forward)
            (let ((r1 (sgtk-show-while-new title
                        (_ "Changing ID ...\nThis step takes a few minites. Please wait a moment.")
                        (lambda () (obj-pd 'update-medid (car (eb 'info)))))))
              (cond
                (r1
                  (sgtk-message-info
                    (format #f (_ "Hospital ID was successfully modified."))))
                (else
                  (sgtk-message-error
                    (_ "Could not change hospital ID, sorry. Please execute /home/orca/orca-hospid-set.sh script manually."))))))
          (else
            #f))))))

(define (gtk-ui-orca-conf w obj-pd patch obj-cr)
  (let* ((gl (obj-pd 'get-gl))
         (t (obj-pd 'get-orcatype))
         (v0 (sgtk-vbox-new #f 10))
         (v1 (sgtk-vbox-new #f 10))
         (ll (sgtk-label-normalized-new (_ "Select ORCA configuration below.")))
         (s0 (format #f (_ "Server Status / current ~a") (obj-pd 'desc-server)))
         (s1 (_ "Host configuration"))
         (s2 (_ "Master Password setting"))
         (s3 (_ "Operator Account setting"))
         (s4 (_ "Hospital ID setting"))
         (s5 (_ "Secondary server data recovery"))
         (s6 (_ "Data Refresh Schedule"))
         (s7 (_ "Primary server data recovery"))
         (a (sgtk-alignment-new 0.8 0.8 0.8 0.8))
         (lb (apply sgtk-labelbox
               (cons #t (append (case t
                                  ((primary)   (list s0 s1 s2 s3 s4 s5 s6 s7))
                                  ((secondary) (list s0 s1 s2 s3))
                                  ((single)    (list s0 s2 s3 s4))
                                  (else        (list s1)))))))
         (bb (sgtk-bbox-ok)))
    (sgtk-container-add v0 v1)
    (sgtk-container-add a (lb 'widget))
    (sgtk-box-pack-start v1 ll #t #t 0)
    (sgtk-box-pack-start v1 a #f #f 0)
    (sgtk-box-pack-start v0 (bb 'widget) #f #f 0)
    (sgtk-paned-pack2 w v0 #t #f)
    (sgtk-widget-show-all w)
    (sgtk-main)
    (sgtk-container-remove w v0)
    (case (bb 'info)
      ((0) #t)
      (else
        (case (lb 'info)
          ((0)
            (if (equal? t 'client)
              (gtk-ui-orca-host w obj-pd patch)
              (gtk-ui-orca-server w obj-pd patch)))
          ((1)
            (if (equal? t 'single)
              (gtk-ui-orca-master w obj-pd)
              (gtk-ui-orca-host w obj-pd patch)))
          ((2)
            (if (equal? t 'single)
              (gtk-ui-orca-acc w obj-pd)
              (gtk-ui-orca-master w obj-pd)))
          ((3)
            (if (equal? t 'single)
              (gtk-ui-orca-hospid w obj-pd)
              (gtk-ui-orca-acc w obj-pd)))
          ((4)
            (gtk-ui-orca-hospid w obj-pd))
          ((5)
            (gtk-ui-orca-recover-secondary w obj-pd))
          ((6)
            (gtk-ui-orca-schedule w obj-pd obj-cr))
          ((7)
            (gtk-ui-orca-recover-primary w obj-pd)))
        (gtk-ui-orca-conf w obj-pd patch obj-cr)))))


(define (gtk-ui-orca-schedule w obj-pd obj-cr)
  (let* ((s (obj-cr 'get 'orca))
         (v (sgtk-vbox-new #f 10))
         (h (sgtk-hbox-new #f 10))
         (v' (sgtk-vbox-new #f 10))
         (h' (sgtk-vbox-new #f 10))
         (a (sgtk-alignment-new 0.8 0.8 0.8 0.8))
         (ll (sgtk-label-normalized-new
               (string-append
                 (if (null? s)
                   (string-append
                     (_ "You are strongly recommended to set automatic data refresh for secondary server. ")
                     (_ "With automatic data refresh, data synchronization will be more steady data than simple DB duplication. ")
                     "\n\n"
                     (_ "To set the data refresh schedule, select the schedule day and time, and click 'Ok'.")
                     (_ "If do not set, select 'cancel'. "))
                   (string-append
                     (_ "Data refresh is scheduled as below:\n\n")
                     (_ "To change the data refresh schedule, select the schedule day and time, and click 'Ok'.")
                     (_ "If do not change, select 'cancel'. "))))))
         (om (make-option-nmenu #f
               (if (null? s)
                 0
                 (let ((wk (list-ref s 4)))
                   (case wk
                     ((none) 1)
                     (else (+ 2 (weekton wk))))))
               (list (_ "No Schedule") (_ "Every day") (_ "Every Sunday") (_ "Every Monday") (_ "Every Tuesday")
                     (_ "Every Wednesday") (_ "Every Thursday") (_ "Every Friday") (_ "Every Saturday"))))
         (l0  (sgtk-label-new "            "))
         (lh  (sgtk-label-new " : "))
         (l1  (sgtk-label-new "    "))
         (aj1 (sgtk-adjustment-new (if (null? s) 0.0 (* 1.0 (cadr s))) 0.0 23.0 1.0  4.0 0.0))
         (sp1 (sgtk-spin-button-new aj1 0.5 0))
         (aj2 (sgtk-adjustment-new (if (null? s) 0.0 (* 1.0 (car s)))  0.0 59.0 1.0 10.0 0.0))
         (sp2 (sgtk-spin-button-new aj2 0.5 0))
         (bb (sgtk-ok/cancel #t)))
    (sgtk-box-pack-start v' (om 'widget) #t #f 40)
    ;(sgtk-box-pack-start h' v' #t #f 40)
    (map (lambda (x) (sgtk-box-pack-start h x #f #f 0))
         (list l0 v' l1 sp1 lh sp2))
    (sgtk-container-add a h)
    (sgtk-container-add-many v ll a)
    ;(sgtk-box-pack-start v v' #t #t 0)
    (sgtk-box-pack-start v (bb 'widget) #f #f 0)
    (sgtk-paned-pack2 w v #t #f)
    (sgtk-widget-show-all w)
    (sgtk-main)
    (let ((n1 (sgtk-spin-button-get-value-as-int sp1))
          (n2 (sgtk-spin-button-get-value-as-int sp2)))
      (sgtk-container-remove w v)
      (case (bb 'info)
        ((back) #f)
        (else
          (case (om 'info)
            ((0)
              (obj-cr 'remove 'orca)
              #t)
            (else
              (let ((w (if (= (om 'info) 1) 'none
                         (list-ref (list 'sun 'mon 'tue 'wed 'thu 'fri 'sat) (- (om 'info) 2)))))
                 (cond
                   ((obj-cr 'update 'orca n2 n1 -1 -1 w "root" (obj-pd 'auto-refresh-cmd))
                     (sgtk-message-info
                       (_ "Automatic data refresh was successfully configured.")))
                   (else
                     (sgtk-message-error "Failed to set.")))))))))))

(define (gtk-ui-orca-recover-secondary w obj-pd)
  (let ((hs (obj-pd 'get-secondary)))
    (cond
      ((equal? "" hs)
        (sgtk-message-info (_ "Secondary server is not set yet."))
        #f)
      ((not (unix-connect? hs 5432))
        (sgtk-message-error
          (format #f (_ "Could not connect to the secondary server (~a). In order to start recovery, login the secondary server and change the status to 'wait' with ogl-admin. ")
                  hs))
        #f)
      ((not (= 1 (sgtk-choice w
                   (string-append
                     (_ "With this recovery process, data in the secondary server will be copied from the primary server. After the recovery, data in the primary and the secondary will be identical. \n\n")
                     (_ "In order to recover the secondary server, you have to change the primary server status to 'semi wait'. ")
                     (_ "This change will interrupt ORCA service during the recovery. ")
                     (_ "The recovery process will take very long time, accodring to the amount of data. ")
                     (_ "Are you Ok to start the recovery? "))
                   "Button_No" "Button_Yes")))
        #f)
      (else
        (let ((r1 (sgtk-show-while w
                    (_ "Changing server status ... ")
                    (lambda () (obj-pd 'semiwait-server)))))
          (let ((r2 (sgtk-show-while w
                      (_ "Recovering data ... ")
                      (lambda () (obj-pd 'recover-secondary)))))
            (cond
              ((not (= 0 r2))
                (sgtk-message-error
                  (format #f (_ "Recovery failed (~a).") r2))
                #f)
              ((= 1 (sgtk-choice w
                      (string-append
                        (_ "Data was successfully recovered. ")
                        (_ "Do you restart ORCA services? "))
                      "Button_No" "Button_Yes"))
                (sgtk-show-while w
                  (_ "Changing server status to 'start' ... ")
                  (lambda () (obj-pd 'run-server)))
                #t)
              (else
                (sgtk-show-while w
                  (_ "Changing server status to 'wait' ... ")
                  (lambda () (obj-pd 'wait-server)))
                #t))))))))

(define (gtk-ui-orca-secondary-recover-check obj-pd)
  (cond
    ((not (obj-pd 'secondary-busy?))
      #t)
    (else
      (= 0 (sgtk-choice-new (_ "Warning")
              (string-append
                (_ "It is possible that the (automatic) data recovery may be proceeding. If so, a system change on the secondary server may cause the serious inconsystemcy with the dual DB. ")
                "\n\n"
                (_ "Do you want to continue?"))
              (_ "Yes") (_ "No"))))))

(define (gtk-ui-orca-recover-primary w obj-pd)
  (define title (_ "Primary server recovery"))
  (let ((hs (obj-pd 'get-secondary)))
    (cond
      ((equal? "" hs)
        (sgtk-message-info (_ "Secondary server is not set yet."))
        #f)
      ((unix-connect? hs 8000)
        (sgtk-message-error
          (string-append
            (_ "Secondary server seems to be running. ('work' state). ")
            (_ "In order to start recovery, login the secondary server and change the status to 'wait' with ogl-admin. ") hs)))
      ((not (unix-connect? hs 5432))
        (sgtk-message-error
          (string-append
            (format #f (_ "Could not connect to the secondary server (~a). ") hs)
            (_ "In order to start recovery, login the secondary server and change the status to 'wait' with ogl-admin. "))))
      ((not
        (and 
          (= 1 (sgtk-choice w
                 (string-append
                   (_ "With this recovery process, data in the primary server will be copied from the secondary server. After the recovery, data in the primary and the secondary will be identical. \n\n")
                   (_ "This menu should be only used when the primary server (this host) is just after a (physical) crush. ")
                   (_ "Othersize data in the primary will be damaged seriously. ")
                   "\n\n"
                   (_ "In order to recover the primary server, you have to change the primary server status to 'semi wait'. ")
                   (_ "The recovery process will take very long time, accodring to the amount of data. ")
                   (_ "Are you Ok to start the recovery? "))
                 "Button_No" "Button_Yes"))
          (= 1 (sgtk-choice w
                 (string-append
                   (_ "Confirmation. \n\n")
                   (_ "I ask you again because this is a very critical process. ")
                   "\n\n\n"
                   (_ "   DATA WILL BE COPIED FROM THE SECONDARY SERVER TO THE PRIMARY SERVER (THIS HOST)")
                   "\n\n"
                   (_ "Are you really Ok to start the recovery? "))
                 "Button_No" "Button_Yes"))))
        #f)
      (else
        (let ((r1 (sgtk-show-while w
                    (_ "Changing server status ... ")
                    (lambda () (obj-pd 'semiwait-server)))))
          (let ((r2 (sgtk-show-while w
                      (_ "Recovering data ... ")
                      (lambda () (obj-pd 'recover-primary)))))
            (cond
              ((not (= 0 r2))
                (sgtk-message-error
                  (format #f (_ "Recovery failed (~a).") r2))
                #f)
              ((= 1 (sgtk-choice w
                      (string-append
                        (_ "Data was successfully recovered. ")
                        (_ "Do you restart ORCA services? "))
                      "Button_No" "Button_Yes"))
                (sgtk-show-while w
                  (_ "Changing server status to 'start' ... ")
                  (lambda () (obj-pd 'run-server)))
                #t)
              (else
                (sgtk-show-while w
                  (_ "Changing server status to 'wait' ... ")
                  (lambda () (obj-pd 'wait-server)))
                #t))))))))

(define (gtk-ui-orca-initcheck obj-pd)
  (cond
    ((< 40 (obj-pd 'get-initstep))
      #t)
    (else
      (sgtk-message-error
        (_ "ORCA server is not initialized yet. Please change server status 'Waited' or 'Started' first.")))))

(define (gtk-ui-orca-local-dbcheck title obj-pd)
  (cond
    ((= 0 (sh "echo \"select usr from orca_acc\" | su orca -c 'psql -h localhost orca' 2>&1"))
      #t)
    ((not (= 0 (sgtk-choice-new title
                 (string-append
                   (_ "Could not access to the local database. ")
                   (_ "In order to modify account information, the server should be in 'wait' or 'start' status. ")
                   "\n\n"
                   (_ "Do you change the server status to 'wait' now? Otherwise setting will be aborted."))
                 (_ "Yes") (_ "No"))))
      #f)
    (else
      (let ((r (sgtk-show-while-new (_ "Changing Server")
                  (_ "Changing service status to 'wait' ...")
                  (lambda () (obj-pd 'wait-server)))))
        (cond
          ((null? r)
            (sgtk-message-info
              (_ "Service(s) status are successfully changed.")))
          (else
            (sgtk-message-error
              (string-append
                (format #f (_ "Could not change service(s) status below: \n~a")
                  (list-foldstring " " r))
                "\n"
                (_ "Please check the system first.")))))))))

(define (gtk-ui-orca-master w obj-pd)
  (define title (_ "Master Password Setting"))
  (cond
    ((not (gtk-ui-orca-initcheck obj-pd))
      #f)
    ((not (gtk-ui-orca-secondary-check w obj-pd))
      #f)
    ((not (gtk-ui-orca-local-dbcheck title obj-pd))
      #f)
    (else
      (let* ((new (sgtk-window-new-toplevel-named title))
             (v (sgtk-vbox-new #f 10))
             (l (sgtk-label-new (_ "Enter new master user password twice.\n")))
             (eb (sgtk-entrybox #f
                   (list (list (_ "Password") "" #f)
                         (list (_ "Password(confirm)") "" #f))))
             (bb (sgtk-ok/cancel #t))
             (obj-orca (conf-orca)))
        (sgtk-container-add-many v l (eb 'widget))
        (sgtk-box-pack-start v (bb 'widget) #f #f 0)
        (sgtk-container-add new v)
        (sgtk-widget-show-all new)
        (sgtk-main)
        (eb 'freeze)
        (sgtk-widget-destroy new)
        (cond 
          ((not (equal? 'forward (bb 'info)))
            (sgtk-message-info (_ "Master password has not been modified."))
            #f)
          (else
            (let* ((r (eb 'sinfo))
                   (p1 (car r))
                   (p2 (cadr r)))
              (cond
                ((not (equal? p1 p2))
                  (sgtk-message-error (_ "Passwords don't match..\n"))
                  (gtk-ui-orca-master w obj-pd))
                (else
                  (let ((r1 (obj-orca 'passwd "ormaster" p1)))
                    (cond
                      ((string? r1)
                        (sgtk-message-error r1))
                      (else
                        (sgtk-message-info (_ "Master password has been successfully modified."))))))))))))))

(define (gtk-ui-secondary-check obj-pd)
  (cond
    ((or (not (equal? 'primary (obj-pd 'get-orcatype)))
         (unix-connect? (obj-pd 'get-secondary) 5432))
      #t)
    ((equal? "" (obj-pd 'get-secondary))
      (gtk-message-error
        (_ "Secondary server is not specified yet.")))
    (else
      (let ((new (sgtk-window-new-toplevel-named (_ "Warning")))
            (v (sgtk-vbox-new #f 10))
            (h (sgtk-hbox-new #f 15))
            (l (sgtk-label-normalized-new
                 (string-append
                   (_ "Secondary server does not seem to be in service.\n\n")
                   (_ "With dual host configuration, primary server should not start unless secondary server has trouble. ")
                   (_ "Otherwise primary and secondary DB will be in inconsistent status.")
                   (_ "\n\nDo you proceed anyway?"))))
            (bb (sgtk-bbox #t
                  (sgnome-stock-button "Button_Ok")
                  (sgnome-stock-button "Button_Cancel"))))
       (sgtk-container-set-border-width h 20)
       (sgtk-container-add h l)
       (sgtk-container-add-many v h (bb 'widget))
       (sgtk-container-add new v)
       (sgtk-widget-show-all new)
       (sgtk-grab-add new)
       ;(sgtk-widget-set-sensitive w #f)
       (sgtk-main)
       (sgtk-widget-destroy new)
       ;(sgtk-widget-set-sensitive w #t)
       (= (bb 'info) 0)))))

(define (gtk-ui-orca-init w obj-pd)
  (define obj-orca (conf-orca))
  (cond
    ((or (case (obj-pd 'get-orcatype)
            ((primary single) #f)
            (else #t))
         (< 60 (obj-pd 'get-initstep)))
      #t)
    ((and (< (obj-pd 'get-initstep) 40)
          (not (sgtk-show-while w
                 (_ "Initializing ORCA server.\nThis step takes a few minutes. Please wait a moment.")
                 (lambda ()
                   (and (obj-pd 'wait-server)
                        (obj-pd 'init-orca)
                        (obj-orca 'make-master)
                        (obj-pd 'commit))))))
      (sgtk-message-error
        (_ "Failed to initialize ORCA. Make sure that ORCA installation is complete. (or consult ARMA Net support.)")))
    (else
      (obj-pd 'set-initstep 40)
      (let rep ((r (or (< 60 (obj-pd 'get-initstep))
                       (gtk-ui-orca-hospid w obj-pd))))
        (cond
          (r
            (obj-pd 'set-initstep 60)
            (obj-pd 'commit)
            r)
          ((not (= 1 (sgtk-choice w
                       (string-append
                         (_ "Initialization could not be finished until hospital ID is set. ")
                         (_ "ORCA cannot work without setting hospital ID. \n\n")
                         (_ "Do you set hospital ID again?"))
                       "Button_No" "Button_Yes")))
            #f)
          (else
            (rep (gtk-ui-orca-hospid w obj-pd))))))))

(define (gtk-ui-orca-server w obj-pd patch)
  (let* ((t (obj-pd 'get-orcatype))
         (new (sgtk-window-new-toplevel-named (_ "Server Status")))
         (v (sgtk-vbox-new #f 10))
         (l (sgtk-label-normalized-new
              (string-append
                 (format #f (_ "Current server status is : ~a\n\n") (obj-pd 'desc-server))
                 (_ "   'Start' : start all server\n")
                 (_ "   'Stop'  : stop all server\n")
                 (_ "   'Wait'  : start DB server/stop application server\n"))))
         (bb (sgtk-bbox #t
               (sgtk-button-new-with-label (_ "Start"))
               (sgtk-button-new-with-label (_ "Stop"))
               (sgtk-button-new-with-label (_ "Wait"))
               (sgnome-stock-button "Button_Ok"))))
    (sgtk-container-add-many v l (bb 'widget))
    (sgtk-container-add new v)
    (sgtk-widget-show-all new)
    (sgtk-grab-add new)
    (sgtk-widget-set-sensitive w #f)
    (sgtk-main)
    (sgtk-widget-destroy new)
    (sgtk-widget-set-sensitive w #t)
    (case (bb 'info)
      ((0)
        (cond
          ((not (gtk-ui-secondary-check obj-pd))
            #f)
          ((and (equal? 'secondary t)
                (unix-connect? (obj-pd 'get-primary) 5432))
            (sgtk-message-error
              (string-append
                (_ "Primary server seem to be in service.\n\n")
                (_ "With dual host configuration, secondary ORCA server cannot start when primary server works.")))
            #f)
          ((and (equal? 'secondary t)
                (not (= 1 (sgtk-choice w
                            (string-append
                              (_ "Warninig!\n\n")
                              (_ "This host is secondary server and ORCA server should not be run unless the primary server is down.\n\n")
                              (_ "Though primary server does not seem to be in service, you are warned because it is important.\n\n")
                              (_ "Do you proceed?"))
                            "Button_No" "Button_Yes"))))
            #f)
          (else
            (gtk-ui-orca-init w obj-pd)
            (let ((r1 (sgtk-show-while-new (_ "Staring Server") 
                        (_ "Staring server ...")
                        (lambda () (obj-pd 'run-server)))))
              (cond
                ((null? r1)
                  (sgtk-message-info (_ "Service(s) are successfully started.")))
                (else
                  (sgtk-message-info
                    (format #f (_ "Service(s) ~a could not be started.")
                      (list-foldstring " " r1)))
                  #t))))))
      ((1)
        (let ((r1 (sgtk-show-while-new (_ "Stoppping Server") 
                    (_ "Stopping server ...")
                    (lambda () (obj-pd 'stop-server)))))
          (cond
            ((null? r1)
              (sgtk-message-info (_ "Service(s) are successfully stopped.")))
            (else
              (sgtk-message-error
                (format #f (_ "Could not stop service(s) below: ~a")
                  (list-foldstring " " r1)))))))
      ((2)
        (let ((r1 (sgtk-show-while-new (_ "Changing Server") 
                    (_ "Changing server status to 'wait' ...")
                    (lambda () (obj-pd 'wait-server)))))
          (gtk-ui-orca-init w obj-pd)
          (cond
            ((null? r1)
              (sgtk-message-info
                (_ "Service(s) status are successfully changed.")))
            (else
              (sgtk-message-error
                (format #f (_ "Could not change service(s) status below: \n~a")
                  (list-foldstring " " r1)))))))
      (else
        #t))))


(define (gtk-ui-orca-primary-secondary-test obj-pd h)
  (define title (_ "Secondary Access Test"))
  (let ((r (sgtk-show-while-new title
             (_ "Accessing to the secondary server ...")
             (lambda () (obj-pd 'secondary-test h)))))
    (cond
      ((eq? #t r)
        #t)
      (else
        (sgtk-message-error
          (format #f (_ "Could not access to the secondary server. \nReason : ~a") r))))))

(define (gtk-ui-orca-primary-host w first? obj-pd patch)
  (define title (_ "Secondary Server Address"))
  (define (not-changed)
    (sgtk-message-info
      (_ "The secondary server address has not been changed. "))
    #t)
  (let* ((n (obj-pd 'get-initstep))
         (gl (obj-pd 'get-gl))
         (st (obj-pd 'stat-server))
         (t (obj-pd 'get-orcatype))
         (new (sgtk-window-new-toplevel-named (_ "Server Status")))
         (v (sgtk-vbox-new #f 10))
         (l (sgtk-label-new
              (_ "Enter ORCA secondary server host address (hostname or IP).\n")))
         (prim (obj-pd 'get-primary))
         (sec (obj-pd 'get-secondary))
         (eb (sgtk-entrybox #f
               (list (list (_ "Secondary Server : ") sec #t))))
         (bb (sgtk-ok/cancel #t)))
    (sgtk-container-add-many v l (eb 'widget) (bb 'widget))
    (sgtk-container-add new v)
    (sgtk-widget-show-all new)
    (sgtk-grab-add new)
    (sgtk-widget-set-sensitive w #f)
    (sgtk-main)
    (eb 'freeze)
    (sgtk-widget-destroy new)
    (sgtk-widget-set-sensitive w #t)
    (case (bb 'info)
      ((back) #f)
      (else
        (let* ((h0 (car (eb 'info))))
          (cond
            ((not (gtk-ui-orca-resolv-check obj-pd h0))
              (gtk-ui-orca-secondary-host w first? obj-pd patch))
            (else
              (let* ((n (obj-pd 'get-initstep))
                     (con? (gtk-ui-orca-primary-secondary-test obj-pd h0))
                     (c0 (if con?
                           #t
                           (= 0 (sgtk-choice-new (_ "Warning")
                                  (format #f (_ "Could not get response from server ~a. Do you proceed anyway?") h0)
                                  (_ "Yes") (_ "No"))))))
                (cond
                  ((not c0)
                    (gtk-ui-orca-primary-host w first? obj-pd patch))
                  (else
                    (obj-pd 'set-initstep 20)
                    (obj-pd 'update-primary "127.0.0.1" 8000)
                    (obj-pd 'update-secondary h0 8000)
                    (patch 'commit)
                    (obj-pd 'commit)
                    (cond
                      ((equal? h0 (cadr gl))
                        (not-changed))
                      ((and (not first?)
                         (not (= 0 (sgtk-choice-new title
                                     (case st
                                       ((work wait semiwait)
                                         (string-append
                                           (_ "In order to reflect the system change, ORCA server has to be needed to restart. ")
                                           (_ "Do you restart the ORCA server now?")))
                                       (else
                                         (_ "Do you start the ORCA server now?")))
                                     (_ "Yes") (_ "No")))))
                        #t)
                      ((not (and (obj-pd 'stop-server)
                              (case st
                                ((wait) (obj-pd 'wait-server))
                                ((semiwait) (obj-pd 'semiwait-server))
                                (else (obj-pd 'run-server)))))
                        (sgtk-message-error
                          (_ "Failed to restart servers. Please check the system first.")))
                      (else
                        (sgtk-message-info
                          (_ "ORCA primary server has been successfully configured."))))))))))))))

(define (gtk-ui-orca-secondary-host w first? obj-pd patch)
  (define title (_ "Primary Server Address"))
  (let* ((n (obj-pd 'get-initstep))
         (gl (obj-pd 'get-gl))
         (st (obj-pd 'stat-server))
         (t (obj-pd 'get-orcatype))
         (new (sgtk-window-new-toplevel-named (_ "Server Status")))
         (v (sgtk-vbox-new #f 10))
         (l (sgtk-label-new
              (_ "Enter ORCA primary server address (host or IP) .\n")))
         (prim (obj-pd 'get-primary))
         (sec (obj-pd 'get-secondary))
         (eb (sgtk-entrybox #f
               (list (list (_ "Primary Server : ") prim #t))))
         (bb (sgtk-ok/cancel #t)))
    (inst-log "orca-host a : ~s, ~s\n" eb bb)
    (sgtk-container-add-many v l (eb 'widget) (bb 'widget))
    (inst-log "b\n")
    (sgtk-container-add new v)
    (sgtk-widget-show-all new)
    (sgtk-grab-add new)
    (sgtk-widget-set-sensitive w #f)
    (sgtk-main)
    (eb 'freeze)
    (sgtk-widget-destroy new)
    (sgtk-widget-set-sensitive w #t)
    (case (bb 'info)
      ((back) #f)
      (else
        (let* ((h0 (car (eb 'info))))
          (cond
            ((not (gtk-ui-orca-resolv-check obj-pd h0))
              (gtk-ui-orca-secondary-host w first? obj-pd patch))
            (else
              (let* ((n (obj-pd 'get-initstep)))
                (sgtk-message-info
                  (if (not (obj-pd 'connect-backend? h0))
                    (string-append
                      (_ "Could not connect to the primary server. In most case this is not an error, because secondary server is configured first. ")
                      "\n\n"
                      (_ "Please do not forget to configure the primary server after finishing ogl-admin. "))
                    (string-append
                      (_ "Primary server seems to be already working. ")
                      (_ "If you configure the secondary server (this host) for the first time, please do not forget to run 'Secondary server data recovery' menu of ogl-admin on the primary server. "))))
                (obj-pd 'set-initstep 20)
                (obj-pd 'update-secondary "127.0.0.1" 8000)
                (obj-pd 'update-primary h0 8000)
                (patch 'commit)
                (obj-pd 'commit)
                (obj-pd 'set-initstep 100)
                (cond
                  ((equal? h0 (cadr gl))
                    (not-changed))
                  ((and (not first?)
                     (not (= 0 (sgtk-choice-new title
                                 (case st
                                   ((work wait semiwait)
                                     (string-append
                                       (_ "In order to reflect the system change, ORCA server has to be needed to restart. ")
                                       (_ "Do you restart the ORCA server now?")))
                                   (else
                                     (_ "Do you start the ORCA server now?")))
                                 (_ "Yes") (_ "No")))))
                    #t)
                  ((not (sgtk-show-while-new title
                          (_ "(Re)starting server status to 'wait' ...")
                          (lambda () (and (obj-pd 'stop-server)
                                          (obj-pd 'wait-server)))))
                    (sgtk-message-error
                      (_ "Failed to restart servers. Please check the system first.")))
                  (else
                    (sgtk-message-info
                      (_ "ORCA secondary server has been successfully configured."))))))))))))

(define (gtk-ui-orca-resolv-check obj-pd h)
  (cond
    ((obj-pd 'ip-or-resolv? h)
      #t)
    (else
      (sgtk-message-info
        (format #f (_ "Address '~a' is neither IP address nor resolvable name. ") h))
      #f)))

(define (gtk-ui-orca-host w obj-pd patch)
  (let* ((t (obj-pd 'get-orcatype))
         (new (sgtk-window-new-toplevel-named (_ "Server Status")))
         (v (sgtk-vbox-new #f 10))
         (l (sgtk-label-new (_ "Enter ORCA Server host or IP.\n")))
         (prim (obj-pd 'get-primary))
         (sec (obj-pd 'get-secondary))
         (eb (sgtk-entrybox #f
               (list (list (_ "(Primary) Server : ") prim #t)
                     (list (_ "Secondary Server (if exists) :") sec #t))))
         (bb (sgtk-ok/cancel #t)))
    (inst-log "orca-host a : ~s, ~s\n" eb bb)
    (sgtk-container-add-many v l (eb 'widget) (bb 'widget))
    (inst-log "b\n")
    (sgtk-container-add new v)
    (sgtk-widget-show-all new)
    (sgtk-grab-add new)
    (sgtk-widget-set-sensitive w #f)
    (sgtk-main)
    (eb 'freeze)
    (sgtk-widget-destroy new)
    (sgtk-widget-set-sensitive w #t)
    (case (bb 'info)
      ((back) #f)
      (else
        (let* ((h0 (car (eb 'sinfo)))
               (h1 (cadr (eb 'info)))
               (single? (equal? "" h1)))
          (cond
            ((not (andmap (lambda (x) (gtk-ui-orca-resolv-check obj-pd x))
                          (if single? (list h0) r)))
              (gtk-ui-orca-host w obj-pd patch))
            (else
              (let* ((n (obj-pd 'get-initstep))
                     (con? (obj-pd 'connect? h0))
                     (c0 (if con?
                           #t
                           (= 0 (sgtk-choice-new (_ "Warning")
                                  (format #f (_ "Could not get response from server ~a. Do you proceed anyway?") h0)
                                  (_ "Yes") (_ "No")))))
                     (c1 (if single?
                           #t
                           (= 0 (sgtk-choice-new (_ "Warning")
                                  (format #f (_ "Could not get response from server ~a. Do you proceed anyway?") h1)
                                  (_ "Yes") (_ "No"))))))
                (inst-log "x\n")
                (cond
                  ((not (and c0 c1))
                    #f)
                  (else
                    (obj-pd 'set-initstep 20)
                    (obj-pd 'update-primary h0 8000)
                    (if (not single?)
                      (obj-pd 'update-secondary h1 8000))
                    (gtk-ui-orca-init w obj-pd)
                    (obj-pd 'set-initstep 100)
                    (sgtk-message-info
                      (_ "ORCA client has been successfully configured."))))))))))))

(define (gtk-ui-orca-secondary-check w obj-pd)
  (cond
    ((not (equal? 'secondary (obj-pd 'get-orcatype)))
      #t)
    (else
      (= 1 (sgtk-choice-new (_ "Warning")
             (string-append
               (_ "Warning!\n\nThis host is set as secondary ORCA server. ")
               (_ "If primary server is working, don't edit user information on the secondary. Instead do it on the primary. \n\n")
               (_ "Otherwise setting user on the secondary may cause system inconsistency. \n\n")
               (_ "Do you proceed anyway?"))
             (_ "No") (_ "Yes"))))))

(define (gtk-ui-orca-acc w obj-pd)
  (define title (_ "ORCA User Configuration"))
  (define obj-orca (conf-orca))
  
  (define (adduser u)
    (let ((new (sgtk-window-new-toplevel-named (_ "Add User")))
          (v (sgtk-vbox-new #f 10))
          (l (sgtk-label-new (format #f (_ "Enter ORCA user ID and password (twice).\n"))))
          (eb (sgtk-entrybox #f
                (list (list (_ "User ID") "" #t)
                      (list (_ "Password") "" #f)
                      (list (_ "Password(confirm)") "" #f))))
          (bb (sgtk-ok/cancel #t)))
      (sgtk-container-add-many v l (eb 'widget))
      (sgtk-box-pack-start v (bb 'widget) #f #f 0)
      (sgtk-container-add new v)
      (sgtk-widget-show-all new)
      (sgtk-main)
      (eb 'freeze)
      (sgtk-widget-destroy new)
      (cond
        ((not (equal? 'forward (bb 'info)))
          (sgtk-message-info (_ "Account not generated.\n")))
        (else
          (let* ((r (eb 'sinfo))
                 (p1 (cadr r))
                 (p2 (caddr r)))
            (sgtk-widget-destroy new)
            (cond
              ((set-include? (car r) u)
                (sgtk-message-error (format #f (_ "User ~a already exists.") (car r)))
                (adduser u))
              ((not (equal? p1 p2))
                (sgtk-message-error (_ "Passwords don't match..\n"))
                (adduser u))
              (else
                (let ((r1 (obj-orca 'adduser (car r) p1)))
                  (cond
                    ((string? r1)
                      (sgtk-message-error r1))
                    (else
                      (sgtk-message-info
                        (string-append
                          (_ "Account successfully generated.\n\n")
                          (_ "Notice that this account is not valid yet. In order to validate this operator account, you have to configure further information of this account via ORCA menu. ")
                          (_ "Login ORCA first with 'ormaster' account and configure it.")))))))))))))
  
  (define (deluser n)
    (cond
      ((not (= 0 (sgtk-choice-new (_ "User deletion")
                   (format #f (_ "Are you ok to remove user ~a ?") n)
                   (_ "Yes") (_ "No"))))
        #f)
      (else
        (let ((r1 (obj-orca 'deluser n)))
          (cond
            ((string? r1)
              (sgtk-message-error r1))
            (else
              (sgtk-message-info
                (string-append
                  (format #f (_ "User ~a successfully removed.\n\n") n)
                  (_ "Notice that this account is not fully remove yet. In order toremove this account completely, you have to configure further information of this account via ORCA menu. ")
                  (_ "Login ORCA first with 'ormaster' account and configure it.")))))))))
  
  (define (edituser u)
    (let ((new (sgtk-window-new-toplevel-named (_ "Change Password/Deletion")))
          (v (sgtk-vbox-new #f 10))
          (l (sgtk-label-new (format #f (_ "Enter password for user ~a (twice), or choose 'delete'.\n") u)))
          (eb (sgtk-entrybox #f
                (list (list (_ "Password") "" #f)
                      (list (_ "Password(confirm)") "" #f))))
          (bb (sgtk-bbox #t
                (sgtk-button-new-with-label (_ "Delete"))
                (sgnome-stock-button "Button_Cancel")
                (sgnome-stock-button "Button_Ok"))))
      (sgtk-container-add-many v l (eb 'widget))
      (sgtk-box-pack-start v (bb 'widget) #f #f 0)
      (sgtk-container-add new v)
      (sgtk-widget-show-all new)
      (sgtk-main)
      (eb 'freeze)
      (sgtk-widget-destroy new)
      (case (bb 'info)
        ((0)
          (deluser u))
        ((1)
          (sgtk-message-info (_ "Account was not modified.\n"))
          #f)
        ((2)
          (let* ((r (eb 'sinfo))
                 (p1 (car r))
                 (p2 (cadr r)))
            (cond
              ((not (equal? p1 p2))
                (sgtk-message-error (_ "Passwords don't match..\n"))
                (edituser u))
              (else
                (let ((r1 (obj-orca 'passwd u p1)))
                  (cond
                    ((string? r1)
                      (sgtk-message-error r1))
                    (else
                      (sgtk-message-info (_ "Password successfully modified.")))))))))
        (else
          #t))))
  
  (cond
    ((not (gtk-ui-orca-initcheck obj-pd))
      #f)
    ((not (gtk-ui-orca-secondary-check w obj-pd))
      #f)
    ((not (gtk-ui-orca-local-dbcheck title obj-pd))
      #f)
    (else
      (let ((u (obj-orca 'user-list)))
        (cond
          ((not (list? u))
            (sgtk-message-error
              (string-append
                (_ "Could not access to the local database. ")
                (_ "Please check the system first."))))
          (else
            (let* ((v0 (sgtk-vbox-new #f 10))
                   (v1 (sgtk-vbox-new #t 10))
                   (a (sgtk-alignment-new 0.8 0.8 0.8 0.8))
                   (lb (apply sgtk-labelbox (cons #t u)))
                   (ll (sgtk-label-normalized-new
                         (string-append
                           (_ "With this menu, you can edit operator account used for ORCA.\n\n")
                           (_ "To change password or delete account, select existing operator account (ID). ")
                           (_ "To create new operator account, select 'new account' button. "))))
                   (hr0 (sgtk-hseparator-new))
                   (bb (sgtk-bbox #t
                         (sgtk-button-new-with-label (_ "New Account"))
                         (sgnome-stock-button "Button_Ok"))))
             (sgtk-container-add v0 v1)
             (sgtk-container-add a (lb 'widget))
             (sgtk-box-pack-start v1 ll #t #t 0)
             (sgtk-box-pack-start v1 a #f #f 0)
             (sgtk-box-pack-start v0 (bb 'widget) #f #f 0)
             (sgtk-paned-pack2 w v0 #t #f)
             (sgtk-widget-show-all w)
             (sgtk-main)
             (sgtk-container-remove w v0)
             (case (bb 'info)
               ((1)
                 #t)
               (else
                 (sgtk-widget-set-sensitive w #f)
                 (case (bb 'info)
                   ((0)
                     (adduser u))
                   (else
                     (edituser (list-ref u (lb 'info)))))
                 (sgtk-widget-set-sensitive w #t)
                 (gtk-ui-orca-acc w obj-pd))))))))))

(define (gtk-ui-ide w obj-ide obj-part)
  
  (define (gtk-measure d)
    (let ((new (sgtk-window-new-toplevel-named (_ "Measuring Transfer")))
          (v (sgtk-vbox-new #f 10))
          (l (sgtk-label-new (_ "measuring speed ...")))
          (p (sgtk-progress-bar-new)))
      (sgtk-container-add-many v l p)
      (sgtk-container-add new v)
      (sgtk-widget-show-all new)
      (sgtk-update)
      (let* ((y (obj-ide 'test-tool d (lambda (f s)
                                       (if (not (equal? "" s))
                                         (sgtk-label-set-text l s))
                                         (sgtk-progress-bar-update p f)
                                         (sgtk-update))))
             (r (progress-start (car y) (cadr y))))
        (sgtk-widget-destroy new)
        (if (not (= 0 r))
          (sgtk-message-error
            (_ "Could not execute measurement. Maybe command not found or drive not ready."))))))
  
  (define (get-default n l x)
    (cond
      ((null? l) 0)
      ((equal? (caar l) x) n)
      (else (get-default (1+ n) (cdr l) x))))
  
  (let* ((v (sgtk-vbox-new #f 10))
         (h (sgtk-hbox-new #f 10))
         (l (obj-ide 'table))
         (lb (sgtk-label-normalized-new
               (string-append
                 (_ "Select xfer mode.\n\n")
                 (_ "Warning!\n\nBe carefull to choose the mode. Selecting inappropriate value will cause the system freeze very easily.\n\n")
                 (_ "If you are not sure of the spec of IDE device, ATA33 or PIO may be safe.\n\n")
                 (_ "If the system freezes or responds much slower after the menu, reboot(reset) the computer and select safer IDE mode.")
                 )))
         (ll (let ((v (sgtk-vbox-new #f 10)))
                   (map (lambda (x)
                          (sgtk-box-pack-start v
                          (sgtk-label-new
                            (let ((s
                            (format #f "~a ~a"
                              (string-cut (obj-part 'did-desc (car x)) 12)
                              (cadr x))))
                              (inst-log "s:~a\n" s)
                              (cadr x)))
                          ; ɤ٥뤬ޤФʤȤΤǤҤȤޤ model ̾ˡ
                          #t #f 10))
                        l)
               v))
         (vo (sgtk-vbox-new #f 10))
         (ol (map (lambda (x)
                    (let* ((d (car x))
                           (m (obj-ide 'mode-list x))
                           (o (sgtk-option
                                (get-default 0 m (caddr x))
                                (map cadr m) (map car m))))
                      (sgtk-box-pack-start vo (o 'widget) #t #f 10)
                      o))
                  l))
         (vs (sgtk-vbox-new #f 10))
         (sl (map (lambda (x)
                    (let ((a (sgtk-label-new
                               (let ((n (list-ref x 4)))
                                 (if (< n 0) ""
                                   (format #f "@ ~a MB/s" n))))))
                      (sgtk-box-pack-start vs a #t #f 10)
                      a))
                  l))
         (bl-info -1)
         (bl (let ((v (sgtk-vbox-new #f 10)))
               (smap (lambda (n x)
                       (let ((b (sgtk-button-new-with-label (_ "Measure"))))
                         (sgtk-signal-connect b "clicked"
                           (lambda (w)
                             (set! bl-info n)
                             (sgtk-main-quit)))
                         (sgtk-box-pack-start v b #t #f 10)))
                     l)
               v))
         (bb (sgtk-ok/cancel #t)))
    (sgtk-box-pack-start h ll #t #f 10)
    (sgtk-box-pack-start h vo #t #t 0)
    (sgtk-box-pack-start h bl #t #f 10)
    (sgtk-box-pack-start h vs #t #f 10)
    (sgtk-box-pack-start v lb #t #f 10)
    (sgtk-container-add-many v h)
    (sgtk-box-pack-start v (bb 'widget) #f #f 0)
    (sgtk-container-add w v)
    (sgtk-widget-show-all w)
    (sgtk-main)
    (sgtk-container-remove w v)
    (case (bb 'info)
      ((back)
        (obj-ide 'abort))
      ((forward)
        (let ((r (smap (lambda (n x)
                         (let ((v ((list-ref ol n) 'info)))
                           (cond
                             ((not (obj-ide 'try-mode (car x) v))
                               #f)
                             (else
                               (obj-ide 'set-mode (car x) v)))))
                       l)))
          (cond
            ((not (ormap (lambda (r) (eq? r #f)) r))
              #t)
            ((= 0 (sgtk-choice-new (_ "Error")
                    (string-append
                      (_ "Some IDE device or IDE interface does not accept the mode. ")
                      "\n"
                      (_ "Do you want to configure IDE mode again? "))
                    (_ "Yes") (_ "No")))
              (gtk-ui-ide w obj-ide obj-part))
            (else
              #f))))
      (else
        (cond
          ((equal? bl-info -1)
            #f)
          (else
            (let* ((d (car (list-ref l bl-info)))
                   (o (list-ref ol bl-info))
                   (m (o 'info)))
              (cond
                ((not (obj-ide 'try-mode d m))
                  (sgtk-message-error
                    (_ "IDE device or IDE interface does not accept the mode.\n\nMode was not changed.\n")))
                (else
                  (obj-ide 'set-mode d m)
                  (gtk-measure d))))
            (gtk-ui-ide w obj-ide obj-part)))))))


(define (gtk-module-sound w obj-mod)
  (define (ask m)
    (let ((new (sgtk-window-new-toplevel-named (_ "Modify Driver")))
          (v (sgtk-vbox-new #f 10))
          (ll (sgtk-label-normalized-new
                (format #f (_ "How do you modify setting for ~a") m)))
          (bb (sgtk-bbox #t
                (sgtk-button-new-with-label (_ "Set First"))
                (sgtk-button-new-with-label (_ "Delete"))
                (sgtk-button-new-with-label (_ "Cancel")))))
      (sgtk-container-add-many v ll (bb 'widget))
      (sgtk-container-add new v)
      (sgtk-widget-show-all new)
      (sgtk-main)
      (sgtk-grab-remove new)
      (sgtk-widget-destroy new)
      (case (bb 'info)
        ((0)
          (obj-mod 'set-first m))
        ((1)
          (obj-mod 'rmmod m))
        (else
          #f))))
  (obj-mod 'setmod-init)
  (let ((l (filter (lambda (x)
                     (case (cadr x)
                       ((un/not inst/not)
                         #f)
                       (else
                         (obj-mod 'sound-target? (car x)))))
                   (obj-mod 'table))))
    (define (insmod)
      (map (lambda (x) (obj-mod 'rmmod (car x)))
           l)
      (obj-mod 'rmmod 'snd-mixer-oss)
      (obj-mod 'rmmod 'snd-pcm-oss)
      (map (lambda (x) (obj-mod 'insmod (car x) ""))
           l)
      #t)
    (inst-log "module-sound : ~a\n" l)
    (cond
      ((< (length l) 2)
        (insmod))
      (else
        (let ((v0 (sgtk-vbox-new #f 10))
              (v1 (sgtk-vbox-new #t 10))
              (a (sgtk-alignment-new 0.8 0.8 0.8 0.8))
              (lb (apply sgtk-labelbox
                    (cons #t (map (lambda (x) (obj-mod 'desc (car x)))
                                          l))))
              (ll (sgtk-label-normalized-new
                    (string-append
                      (_ "System has several sound device. Drivers are going to be loaded according to the order below. \n\n")
                      (_ "Note that a few driver can be only used as primary driver.\n\n")
                      (_ "Proceed with this driver order?"))))
              (hr0 (sgtk-hseparator-new))
              (bb (sgtk-ok/cancel #t)))
          (sgtk-container-add v0 v1)
          (sgtk-container-add a (lb 'widget))
          (sgtk-box-pack-start v1 ll #t #t 0)
          (sgtk-box-pack-start v1 a #f #f 0)
          (sgtk-box-pack-start v0 (bb 'widget) #f #f 0)
          (sgtk-paned-pack2 w v0 #t #f)
          (sgtk-widget-show-all w)
          (sgtk-main)
          (sgtk-widget-set-sensitive v0 #f)
          (sgtk-container-remove w v0)
          (case (bb 'info)
            ((none)
              (ask (car (list-ref l (lb 'info))))
              (gtk-module-sound w obj-mod))
            ((forward)
              (insmod))
            ((back)
              #f)
            (else
              #t)))))))


(define (gtk-ui-volume w obj-vol)
  (let ((l (obj-vol 'list)))
    (case (length l)
      ((0)
        (sgtk-message-info
          (string-append
            (_ "No sound device (mixer device) was found. ")
            (_ "If the system has ISA sound devices, please install appropriate driver manually with 'Driver Setting' of ogl-admin. ")
            (_ "(In that case, you may also need to configure with isapntools. )"))))
      (else
        (let* ((v (sgtk-vbox-new #f 10))
               (h (sgtk-hbox-new #f 10))
               (lb (sgtk-label-new (_ "Set volume :")))
               (ll (let ((v (sgtk-vbox-new #f 10)))
                     (apply sgtk-container-add-many
                       (cons v 
                         (smap (lambda (n x)
                                 (sgtk-label-new (sprintf (_ "%6s sound device") (ordinal-string #t n))))
                               l)))
                     v))
               (hl (let ((v (sgtk-vbox-new #f 10)))
                     (apply sgtk-container-add-many
                       (cons v
                         (smap (lambda (n x)
                                 ((sgtk-hscalebox
                                    (sprintf "" (ordinal-string #t n))
                                    (lambda (v) (obj-vol 'set n v))
                                    (obj-vol 'get n) 0 100 5 0 0)
                                  'widget))
                               l)))
                     v))
               (bl (let ((v (sgtk-vbox-new #f 10)))
                     (smap (lambda (n x)
                             (let ((b (sgtk-button-new-with-label (_ "Ring"))))
                               (sgtk-signal-connect b "clicked"
                                 (lambda (w)
                                   (let ((t (car x)))
                                     (obj-vol 'apply t)
                                     (obj-vol 'test t))))
                               (sgtk-box-pack-start v b #t #f 10)))
                            l)
                     v))
               (bb (sgtk-ok/cancel #t)))
          (sgtk-box-pack-start h ll #t #f 10)
          (sgtk-box-pack-start h hl #t #t 0)
          (sgtk-box-pack-start h bl #t #f 10)
          (sgtk-box-pack-start v lb #t #f 10)
          (sgtk-container-add-many v h)
          (sgtk-box-pack-start v (bb 'widget) #f #f 0)
          (sgtk-container-add w v)
          (sgtk-widget-show-all w)
          (sgtk-main)
          (sgtk-container-remove w v)
          (equal? 'forward (bb 'info)))))))


(define (gtk-ui-package w)
  (let ((v (sgtk-vbox-new #f 10))
        (l (sgtk-label-new (_ "Installing Packages ...")))
        (p (sgtk-progress-bar-new)))
    ;(sgtk-container-add-many v l p)
    (sgtk-container-add v l)
    (sgtk-box-pack-start v p #f #f 100)
    (sgtk-paned-pack2 w v #t #f)
    (sgtk-widget-show-all v)
    (sgtk-update)
    (let ((r (progress-start
               "/usr/sbin/ogl/pkginst"
               (lambda (f s)
                 (if (not (equal? "" s))
                   (sgtk-label-set-text l s))
                 (sgtk-progress-bar-update p f)
                 (sgtk-update)))))
      (sgtk-widget-destroy v)
      (cond
        ((and (= 0 r) (not (unix-access? "/tmp/inst/pkgfail" '(read))))
          #t)
        ((= 1 (sgtk-choice w
                (string-append
                  (_ "Package installation failed. \n")
                  (let ((r (ttable-read "/tmp/inst/pkgfail" '((str int)))))
                    (cond
                      ((and (list? r) (not (null? r)))
                        (let ((p (caar r))
                              (e (cadr (car r))))
                          (format #f (_ "package ~a ~a\n\nDo you want to skip this package?") p
                            (if (= 220 e)
                              (_ "timed out. \n")
                              (format #f (_ "exited with ~a .\n") e)))))
                      (else
                        (_ "Will you skip a failed package and continue installation?")))))
                "Button_No" "Button_Yes"))
          (gtk-ui-package w))
        (else
          #f)))))

(define (gtk-check-package obj-med f)
  (let ((r (parse-awk #\space #t f)))
    (cond
      ((or (eq? #f r)
           (ormap null? r))
        #f)
      (else
        (let ((l (obj-med 'verify-inst (map car r))))
          (cond
            ((null? l)
              #t)
            ((< 20 (length l))
              (sgtk-message-error
                (_ "A lot of packages are not installed successfully.\n")))
            (else
              (sgtk-message-error
                (apply string-append
                  (cons
                    (_ "Following Package(s) are not installed:\n\n")
                    (map (lambda (x) (format #f "~a " x))
                         l)))))))))))

(define (gtk-ui-root-password w obj-acc)
  (let ((v (sgtk-vbox-new #f 10))
        (l (sgtk-label-new (_ "Enter Password for root account twice.\n")))
        (eb (sgtk-entrybox #f
              (list (list (_ "Password") "" #f)
                    (list (_ "Password(confirm)") "" #f))))
        (bb (sgtk-ok/cancel #t)))
    (sgtk-container-add-many v l (eb 'widget))
    (sgtk-box-pack-start v (bb 'widget) #f #f 0)
    (sgtk-container-add w v)
    (sgtk-widget-show-all w)
    (sgtk-main)
    (cond
      ((not (equal? 'forward (bb 'info)))
        (sgtk-container-remove w v)
        (sgnome-dialog-run
          (sgnome-message-box-new (_ "Root password was not changed.\n")
            "info" "Button_Ok"))
        #f)
      (else
        (let* ((r (eb 'sinfo))
               (p1 (car r))
               (p2 (cadr r)))
          (sgtk-container-remove w v)
          (cond
            ((not (equal? p1 p2))
              (sgnome-dialog-run
                (sgnome-message-box-new (_ "Passwords don't match..\n")
                  "info" "Button_Ok"))
              (gtk-ui-root-password w obj-acc))
            (else
              (let ((r (obj-acc 'passwd "root" p1)))
                (cond
                  ((string? r)
                    (sgnome-dialog-run
                      (sgnome-message-box-new 
                        (format #f (_ "Password was not changed.\nReason : ~a\n") r)
                        "error" "Button_Ok"))
                    #f)
                  (else
                    #t))))))))))

(define (gtk-ui-adduser w obj-acc)
  (let ((v (sgtk-vbox-new #f 10))
        (ll (sgtk-label-new (_ "Enter User name and password twice.\n")))
        (eb (sgtk-entrybox #f
              (list (list (_ "Login name") "" #t)
                    (list (_ "Full name") "" #t)
                    (list (_ "Password") "" #f)
                    (list (_ "Password(confirm)") "" #f))))
        (bb (sgtk-ok/cancel #t)))
    (sgtk-container-add-many v ll (eb 'widget))
    (sgtk-box-pack-start v (bb 'widget) #f #f 0)
    (sgtk-container-add w v)
    (sgtk-widget-show-all w)
    (sgtk-main)
    (cond
      ((not (equal? 'forward (bb 'info)))
        (sgtk-container-remove w v)
        (sgtk-message-error (_ "Account not generated.\n")))
      (else
        (let* ((r (eb 'sinfo))
               (p1 (car r))
               (p2 (cadr r))
               (p3 (caddr r))
               (p4 (list-ref r 3)))
          (sgtk-container-remove w v)
          (cond
            ((not (equal? p3 p4))
              (sgtk-message-error (_ "Passwords don't match..\n"))
              (gtk-ui-adduser w obj-acc))
            ((ormap (lambda (x) (equal? "" x)) r)
              (sgtk-message-error (_ "Account not generated.\n")))
            (else
              (let ((v (sgtk-vbox-new #f 10))
                    (b (sgtk-ok/cancel #t))
                    (cl (sgtk-checkbox
                          (map (lambda (x)
                                 (list (sprintf "%-8s : %s" (car x) (cadr x))
                                       (caddr x)))
                               (obj-acc 'suggest))))
                    (l (sgtk-label-normalized-new 
                         (_ "Check groups to which new user will be added. Groups represent access ability to device or command execution"))))
                (sgtk-container-add-many v l (cl 'widget))
                (sgtk-paned-pack2 w v #t #f)
                (sgtk-box-pack-start v (b 'widget) #f #f 0)
                (sgtk-widget-show-all w)
                (sgtk-main)
                (cl 'freeze)
                (inst-log "confirm button : ~a\n" (b 'info))
                (cond
                  ((equal? (b 'info) 'back)
                    (sgtk-container-remove w v)
                    #f)
                  (else
                    (let ((r2 (obj-acc 'adduser (car r) (caddr r) (cadr r)
                                (filter (lambda (x) (not (eq? #f x)))
                                  (map (lambda (x y) (if y (car x) y))
                                    (obj-acc 'suggest)
                                    (cl 'info))))))
                      (sgtk-container-remove w v)
                      (cond
                        ((string? r2)
                          (sgtk-message-error
                            (format #f (_ "Could not create user.\nReason : ~a\n") r2)))
                        (else
                          (let ((r3 (sgtk-choice w
                                      (_ "Account successfully generated.\nDo you create more user account?")
                                      "Button_No" "Button_Yes")))
                            (if (= 0 r3)
                              #t
                              (gtk-ui-adduser w obj-acc))))))))))))))))

(define (gtk-ui-ldap w obj-ds)
  (let* ((r0 (obj-ds 'get-ldap))
         (h (if (eq? r0 #f) "" (car r0)))
         (p (if (or (eq? r0 #f) (< (cadr r0) 0))
                ""
                (format #f "~a" (cadr r0))))
         (b (if (eq? r0 #f) "" (caddr r0)))
         (v (sgtk-vbox-new #f 10))
         (ll (sgtk-label-new (_ "Enter LDAP server hostname and base for authentication.\n")))
         (cb (sgtk-checkbox (list (list (_ "disable LDAP") #f))))
         (eb (sgtk-entrybox #f
               (list (list (_ "hostname ") h #t)
                     (list (_ "port ") p #t)
                     (list (_ "base ") b #t))))
         (bb (sgtk-ok/cancel #t)))
    (sgtk-container-add-many v ll (cb 'widget) (eb 'widget))
    (sgtk-box-pack-start v (bb 'widget) #f #f 0)
    (sgtk-container-add w v)
    (sgtk-widget-show-all w)
    (sgtk-main)
    (cb 'freeze)
    (cond
      ((not (equal? 'forward (bb 'info)))
        (sgtk-container-remove w v)
        #f)
      (else
        (let* ((c (cb 'info))
               (r (eb 'info))
               (p' (if (equal? "" (cadr r))
                       -1
                       (string->number (cadr r)))))
          (sgtk-container-remove w v)
          (cond
            ((car c)
              (if (and (obj-ds 'reset)
                       (obj-ds 'stop-server))
                (sgtk-message-info (_ "Disabled LDAP.\nAfter this, system authentication relies only on local user account.\n"))
                (sgtk-message-error (_ "Disabling LDAP failed anyway."))))
            ((eq? #f p')
              (sgtk-message-error (_ "Digits is allow for port number.\n"))
              (gtk-ui-ldap w obj-ds))
            ((not (and (obj-ds 'reset)
                       (obj-ds 'update-ldap (car r) p' (caddr r))
                       (obj-ds 'set-postconf)))
              (sgtk-message-error (_ "Failed to set ldap configuration\n")))
            (else
              #t)))))))

(define (gtk-ui-nis w obj-ds)
  (let* ((r0 (obj-ds 'get-nis))
         (h (if (eq? r0 #f) "" (cadr r0)))
         (b (if (eq? r0 #f) "" (car r0)))
         (v (sgtk-vbox-new #f 10))
         (ll (sgtk-label-normalized-new
               (string-append
                 (_ "Enter NIS domain and server.\n")
                 (_ "If using broadcast, leave 'server' entry empty."))))
         (cb (sgtk-checkbox (list (list (_ "disable NIS") #f))))
         (eb (sgtk-entrybox #f
               (list (list (_ "NIS domain ") h #t)
                     (list (_ "Server ") b #t))))
         (bb (sgtk-ok/cancel #t)))
    (sgtk-container-add-many v ll (cb 'widget) (eb 'widget))
    (sgtk-container-add-many v ll (eb 'widget))
    (sgtk-box-pack-start v (bb 'widget) #f #f 0)
    (sgtk-container-add w v)
    (sgtk-widget-show-all w)
    (sgtk-main)
    (cb 'freeze)
    (cond
      ((not (equal? 'forward (bb 'info)))
        (sgtk-container-remove w v)
        #f)
      (else
        (let* ((c (cb 'info))
               (r1 (eb 'info)))
          (sgtk-container-remove w v)
          (cond
            ((car c)
              (if (and (obj-ds 'reset)
                       (obj-ds 'stop-server))
                (sgtk-message-info (_ "Disabled NIS.\nAfter this, system authentication relies only on local user account.\n"))
                (sgtk-message-error (_ "Disabling NIS failed anyway."))))
            ((and (equal? "" (car r1))
                  (equal? "" (cadr r1)))
              (sgtk-message-info (_ "NIS client was not modified.\n"))
              #f)
            ((equal? "" (car r1))
              (sgtk-message-info (_ "It is needed to specify NIS domain."))
              (gtk-ui-nis w obj-ds))
            ((not (and (obj-ds 'reset)
                       (obj-ds 'update-nis (cadr r1) (car r1))
                       (obj-ds 'set-postconf)))
              (sgtk-message-error (_ "Failed to set NIS configuration.")))
            (else
              #t)))))))

(define (gtk-ui-user-account w obj-ds obj-acc)
  (let ((v0 (sgtk-vbox-new #f 10))
        (v1 (sgtk-vbox-new #t 10))
        (a (sgtk-alignment-new 0.8 0.8 0.8 0.8))
        (lb (sgtk-labelbox #t
              (_ "Local Account Creation") (_ "LDAP client setting") (_ "NIS client setting")))
        (m (sgtk-label-normalized-new (_ "Select menu below\n")))
        (hr0 (sgtk-hseparator-new))
        (bb (sgtk-boxed-ok)))
    (sgtk-container-add v0 v1)
    (sgtk-container-add a (lb 'widget))
    (sgtk-box-pack-start v1 m #t #t 0)
    (sgtk-box-pack-start v1 a #f #f 0)
    (sgtk-box-pack-start v0 bb #f #f 0)
    (sgtk-paned-pack2 w v0 #t #f)
    (sgtk-widget-show-all w)
    (sgtk-main)
    (sgtk-container-remove w v0)
    (case (lb 'info)
      ((0)
        (gtk-ui-adduser w obj-acc)
        (gtk-ui-user-account w obj-ds obj-acc))
      ((1)
        (gtk-ui-ldap w obj-ds)
        (gtk-ui-user-account w obj-ds obj-acc))
      ((2)
        (gtk-ui-nis w obj-ds)
        (gtk-ui-user-account w obj-ds obj-acc))
      (else
        #t))))

(define (gtk-ui-check-media obj-med)
  (cond
    ((and (eq? #t (obj-med 'open))
          (eq? #t (obj-med 'verify)))
      (obj-med 'close)
      #t)
    (else
      (obj-med 'close)
      (sgtk-message-error
        (_ "Could not open media.\nCheck your install media again.\n")))))

(define (gtk-ui-selection w f)
  (define (ask)
    (let ((r0 (parse-awk #\tab #t "/var/lib/ogl/etc/product")))
      (cond
        ((not (and (not (eq? #f r0))
                   (= 1 (length r0))
                   (= 1 (length (car r0)))
                   (< 2 (string-length (caar r0)))))
          '())
        (else
          (let* ((p (string->symbol (substring (caar r0) 0 2)))
                 (v0 (sgtk-vbox-new #f 10))
                 (v1 (sgtk-vbox-new #f 10))
                 (a (sgtk-alignment-new 0.8 0.8 0.8 0.8))
                 (l (sgtk-label-new (_ "Select a course of installation.")))
                 (vr (apply make-vradio
                       (cons #f
                         (case p       ;  obj-pd ϤƤäۤ褤
                           ((SV)
                             (list 2 (_ "Minimum (core system only)") (_ "Standard Desktop") (_ "Server") (_ "Server + Desktop")))
                           ((OS OD SS SD)
                             (list 1 (_ "Minimum (core system only)") (_ "ORCA + Desktop") (_ "ORCA + Server") (_ "ORCA + Desktop + Server")))
                           (else 
                             (list 1 (_ "Minimum (core system only)") (_ "Standard Desktop")))))))
                 (hr (sgtk-hseparator-new))
                 ;(bb (sgtk-hbutton-box-new))
                 ;(b-ok (sgnome-stock-button "Button_Ok"))
                 (bb (sgtk-boxed-ok))
                 (orca? (case p ((OS OD SS SD) #t)
                                (else #f))))
               
            ;(sgtk-container-set-border-width bb 5)
            ;(sgtk-button-box-set-layout bb (sgtk-val "GTK_BUTTONBOX_END"))
            ;(sgtk-button-box-set-spacing bb 20)
            ;(sgtk-button-box-set-child-size bb 85 20)
            ;(sgtk-signal-connect b-ok "clicked" (lambda (w) (sgtk-main-quit)))
            ;(sgtk-container-add bb b-ok)
            
            (sgtk-label-set-justify l (sgtk-val "GTK_JUSTIFY_LEFT"))
            (sgtk-paned-pack2 w v0 #t #f)
            (sgtk-container-add v0 v1)
            (sgtk-container-add a (vr 'widget))
            (sgtk-box-pack-start v1 l #t #t 0)
            (sgtk-box-pack-start v1 a #f #f 0)
            (sgtk-box-pack-start v0 bb #f #f 0)
            ;(sgtk-box-pack-start v0 (bb 'widget) #f #f 0)
            
            (sgtk-widget-show-all v0)
            (sgtk-main)
            (sgtk-container-remove w v0)
            (case (vr 'info)
              ((0)
                (let ((s (sgtk-choice w 
                            (string-append
                              (_ "Warining!\n\n")
                              (_ "System will be configured very minimum. ")
                              (_ "After installation, you might have some difficulty with this system.\n\n")
                              (_ "Are you ok with minimum system?\n"))
                           "Button_No" "Button_Yes")))
                  (cond
                    ((= s 1)
                      '(#f #f #f #f #f))
                    (else
                      (ask)))))
              (else
                (let ((r3 (sgtk-choice-str w
                            (_ "Select a MTA (mail tranport agent).")
                            "qmail" "postfix")))
                  (let ((postfix? (= r3 1)))
                    (case (vr 'info)
                      ((1)  (list #t #t #f orca? postfix?))
                      ((2)  (list #t #f #t orca? postfix?))
                      (else (list #t #t #t orca? postfix?))))))))))))
  
  (let ((r (ask)))
    (cond
      ((null? r) #f)
      (else
        (let ((x (apply f r)))
          (cond
            ((not (eq? #t x))
              (sgtk-message-error
                (format #f
                  (string-append
                    (_ "Fatal Error!\n\n")
                    (_ "Could not create package selection.\nReason : ~a\n"))
                  x))
              #f)
            (else
              #t)))))))

(define (gtk-ui-service w obj-sv)
  (let* ((v (sgtk-vbox-new #f 10))
         (v1 (sgtk-vbox-new #f 10))
         (sc (sgtk-scrolled-window-new-nn))
         (a0 (sgtk-alignment-new 0.8 0.8 0.8 0.8))
         (a (sgtk-alignment-new 0.8 0.8 0.8 0.8))
         (cl (map (lambda (x)
                    (let ((c (sgtk-check-button-new-with-label
                               (sprintf "%-17s : %s" (car x) (list-ref x 3)))))
                      (sgtk-toggle-button-set-active c (list-ref x 4))
                      c))
                  (obj-sv 'list)))
         (l (sgtk-label-normalized-new 
               (string-append
                 (_ "With this menu, you can set a status of each service.")
                 (_ "Checked services are going to run. ")
                 (_ "Unchecked services will be stopped.\n"))))
         (bb (sgtk-ok/cancel #t))
         ;(t (sgtk-stdlayout w l v (bb 'widget)))
         )
    (sgtk-scrolled-window-set-policy sc
      (sgtk-val "GTK_POLICY_AUTOMATIC") (sgtk-val "GTK_POLICY_AUTOMATIC"))
    (sgtk-container-add a0 l)
    (sgtk-box-pack-start v a0 #f #f 0)
    (apply sgtk-container-add-many (cons v1 cl))
    (sgtk-scrolled-window-add-with-viewport sc v1)
    (sgtk-container-add a sc)
    ;(sgtk-box-pack-start a sc #f #f 0)
    (sgtk-box-pack-start v a #t #t 0)
    (sgtk-box-pack-start v (bb 'widget) #f #f 0)
    (sgtk-paned-pack2 w v #t #f)
    (sgtk-widget-show-all w)
    (sgtk-main)
    (cond
      ((not (eq? 'forward (bb 'info)))
        (sgtk-container-remove w v)
        ;(sgtk-container-remove w t)
        #f)
      (else
        (map (lambda (c x)
               (if c (obj-sv 'start x)
                     (obj-sv 'remove x)))
             (map sgtk-toggle-button-get-active cl)
             (map car (obj-sv 'list)))
        (sgtk-container-remove w v)
        ;(sgtk-container-remove w t)
        #t))))


(define (gtk-ui-recover-mbr w obj-boot obj-part)
  (define (proceed d all?)
    (let ((r (sgtk-choice w
               (string-append
                 (_ "This is last chance to quit. Boot record for device below will be rewritten with old boot record.\n\n    ")
                 (obj-part 'did-desc d)
                 (_ "\n\nIf this device is BIOS boot device, erasing grub might make this system unbootable.\n\n")
                 (_ "Do you proceed?"))
               "Button_No" "Button_Yes")))
      (case r
        ((0)
          #f)
        ((obj-boot 'recover d all?)
          (sgtk-message-info (_ "Previous boot record was recovered.\n")))
        (else
          (sgtk-message-error (_ "Failed to recover boot record.\n"))))))
  
  (let ((l (obj-boot 'list-recoverable)))
    (cond
      ((null? l)
        (sgtk-message-info (_ "No boot record is recoverable.\n"))
        #f)
      (else
        (let ((v0 (sgtk-vbox-new #f 10))
              (v1 (sgtk-vbox-new #t 10))
              (a (sgtk-alignment-new 0.8 0.8 0.8 0.8))
              (lb (apply sgtk-labelbox (cons #t
                                         (map (lambda (x)
                                                (format #f (_ "top region of ~a") (obj-part 'did-desc x)))
                                              l))))
              (ll (sgtk-label-normalized-new
                    (_ "\nSelect a device to recover previous boot record.\n")))
              (hr0 (sgtk-hseparator-new))
              (bb (sgtk-cancelbox)))
          (sgtk-container-add v0 v1)
          (sgtk-container-add a (lb 'widget))
          (sgtk-box-pack-start v1 ll #t #t 0)
          (sgtk-box-pack-start v1 a #f #f 0)
          (sgtk-box-pack-start v0 (bb 'widget) #f #f 0)
          (sgtk-paned-pack2 w v0 #t #f)
          (sgtk-widget-show-all w)
          (sgtk-main)
          (sgtk-container-remove w v0)
          (cond
            ((= 0 (bb 'info))
              (inst-log "canceld\n")
              #f)
            (else
              (let ((d (list-ref l (lb 'info))))
                (case (obj-boot 'recover-check d)
                  ((error)
                    (sgtk-message-error (_ "Could not recover.\n"))
                    #f)
                  ((modified)
                    (let ((r1 (sgtk-choice w
                                (string-append
                                  (_ "Boot record was altered since grub had been installed. System might be destroyed completely by forcing recover process.\n\n")
                                  (_ "Do you proceed anyway?"))
                                "Button_No" "Button_Yes")))
                      (case r1
                        ((1) (proceed d #f))
                        (else #f))))
                  ((partition)
                    (let ((r1 (sgtk-choice w
                                (string-append
                                  (_ "Partition table was altered. Previous boot-loader might not be able to boot correctly. \n\n")
                                  (_ "Do you proceed anyway?"))
                                "Button_No" "Button_Yes")))
                      (case r1
                        ((1) (proceed d #f))
                        (else #f))))
                  (else
                    (proceed d #t)))))))))))


