;
;     IDE configurator
;
;                             (C)2002 Omoikane Inc.
;

(define (conf-ide obj-part)
  
  ; var
  (define table '()) ; <did> <name:str> <set mode:int> <init mode:int> <speed:int>
  
  ; lib
  (define (cmd d m)
    (format #f "/sbin/hdparm ~a ~a"
      (if (< 63 m)
        (format #f "-c 1 -d 1 -m 16 -X ~a" m)
        "-d 0 -p 0")
      (obj-part 'devfsname d)))
  
  (define (update x)
    (let ((r (assoc (car x) table)))
      (cond
        ((eq? r #f)
          #f)
        (else
          (set! table
            (map (lambda (y)
                   (cond
                     ((equal? (car x) (car y))
                       (list (car y) (cadr y) (cadr x) (list-ref y 3) (list-ref y 4)))
                     (else
                       y)))
                 table))
          #t))))
  
  (define (update-result d n)
    (let ((r (assoc d table)))
      (cond
        ((eq? r #f)
          #f)
        (else
          (set! table
            (map (lambda (y)
                   (cond
                     ((equal? d (car y))
                       (list d (cadr y) (caddr y) (list-ref y 3) n))
                     (else
                       y)))
                 table))
          #t))))
  
  (define (mode n)
    (case n
      ((8)  "PIO")
      ((66) "ATA33 (UDMA2)")
      ((68) "ATA66 (UDMA4)")
      ((69) "ATA100 (UDMA5)")
      (else
       (cond
         ((< 63 n)
           (format #f "UDMA~a" (- n 64)))
         (else
           (_ "unknown"))))))
  
  ; method
  (define (apply-it d)
    (let ((r (assoc d table)))
      (cond
        ((eq? r #f) #f)
        (else
          (let* ((s (cmd d (caddr r)))
                 (e (sh s)))
            (if (not (= 0 e))
              (inst-log "idecond : command failed ~s\n" s))
            (= 0 e))))))
  
  (define (desc x)
    (cond
      ((not (= 5 (length x) ))
        "unknown")
      (else
        (let ((d (car x))
              (n0 (caddr x))
              (n1 (list-ref x 3))
              (v (list-ref x 4)))
          (sprintf "%26s %12s %14s%s"
            (string-cut (obj-part 'did-desc d) 26)
            (string-cut (cadr x) 12)
            (cond
              ((= n0 n1)
                (mode (caddr x)))
              (else
                (mode n0)))
                ;(format #f "~a (set ~a)" (mode n1) (mode n0))
            "")))))
            
            ;  (format #f (_ "@~aM/s") v)
  
  (define (init c)
    (set! table
      (filter (lambda (x) (not (eq? x #f)))
        (map (lambda (x)
               (cond
                 ((not (equal? 'ide (list-ref x 3)))
                   #f)
                 (else
                   (let ((m (hdd-idemode (obj-part 'devfsname (car x)))))
                     (list (car x) (cadr x) m m -1)))))
             (obj-part 'hw-info))))
    #t)
  
  (define (commit)
    (sh "update-rc.d ide defaults")
    (fprintf "/var/lib/ogl/etc/ideconf"
      (apply string-append
        (map (lambda (x) (format #f "~a\t~a\n" (car x) (caddr x)))
             table))))
  
  (define (load-it)
    (let ((r (parse-awk #\tab #t "/var/lib/ogl/etc/ideconf")))
      (cond
        ((eq? #f r)
          #t)
        (else
          (andmap (lambda (x)
                    (cond
                      ((not (= (length x) 2))
                        #f)
                      (else
                        (let ((n (string->number (cadr x))))
                          (cond
                            ((eq? #f n)
                              #f)
                            (else
                              (update (list (car x) n))))))))
                  r)))))
  
  (define (make-etc f)
    (let ((c (map (lambda (x) (format #f "        ~a\n" (cmd (car x) (caddr x))))
                  table)))
      (and (fprintf f
             (apply string-append
               (append
                  (list "#!/bin/sh\n"
                        "# this script was generated by ogl-tools\n\n"
                        "case \"$1\" in\n"
                        "    start)\n"
                        "        echo \"Setting IDE devices ...\"\n")
                  c
                  (list "        ;;\n"
                        "    stop)\n"
                        "        ;;\n"
                        "    restart)\n"
                        "        echo \"Reloading IDE setting...\"\n")
                  c
                  (list "        ;;\n"
                        "    *)\n"
                        "        echo \"Usage : /etc/init.d/ide {start|stop|reload}\"\n"
                        "        ;;\n"
                        "esac\nexit 0\n"))))
           (= 0 (sh "chmod 755 ~a" f)))))
  
  (define (abort)
    (andmap (lambda (x)
              (let ((d (car x))
                    (n0 (caddr x))
                    (n1 (list-ref x 3)))
                (cond
                  ((= n0 n1) #t)
                  (else
                    (update (list d n1))
                    (apply-it d)))))
            table))
  
  (define (test-tool d u)
    (define i 0)
    (define sum 0)
    (let ((r (assoc d table)))
      (cond
        ((eq? #f r)
          '())
        (else
          (let ((n 15))
          (list
            (format #f "xferrate -n ~a -l 200 -t 400 -p ~a -d 10" n (obj-part 'devfsname d))
              (lambda (x t)
                (inst-log "got ~a\n" t)
                (let* ((mk.1 (lambda (k) (/ (real->int (* k 10)) 10)))
                       (s (string-match t "[0-9.]\\+"))
                       (t' (if (null? s) #f (string->number (car s)))))
                  (cond
                    ((eq? #f t')
                      (u x (_ "    failed to measure")))
                    (else
                      (set! sum (+ sum t'))
                      (set! i (1+ i))
                      (if (= i n)
                        (update-result d (mk.1 (/ sum n))))
                      (u x (format #f (_ "    transfering @ ~a Mbytes/sec ")
                                      (mk.1 t')))))))))))))
  
  (define (mode-list d)
    (list
      (list 8  "PIO")
      (list 66 "ATA33 (UDMA2)")
      (list 68 "ATA66 (UDMA4)")
      (list 69 "ATA100 (UDMA5)")
      (list 70 "ATA133 (UDMA6)")))
  
  (define (set-mode d n)
    (set! table
      (map (lambda (x)
             (if (equal? (car x) d)
               (list d (cadr x) n (list-ref x 3) (list-ref x 4))
               x))
           table))
    #t)
  
  (define (try-mode d n)
    (inst-log "ideconf : trying ~s ~s\n" d n)
    (let ((r (assoc d table)))
      (cond
        ((eq? #f r)
          (inst-log "ideconf : incorrect device ~s in ~s\n" d table)
          #f)
        ((not (= 0 (sh (cmd d n))))
          (inst-log "ideconf : trying failed ~s:~s\n" d n)
          #f)
        (else
          (let ((m (hdd-idemode (obj-part 'devfsname d))))
            (cond
              ((= m -1)
                (inst-log "ideconf : could not get mode. maybe CD-ROM. ignoring.\n")
                #t)
              ((not (= n m))
                (inst-log "ideconf : ide mode not changed : ~s\n" d)
                (sh (cmd d (caddr r)))
                #f)
              (else
                #t)))))))
  
  (define (get-speed d)
    (let ((r (assoc d table)))
      (cond
        ((eq? r #f)
          -1)
        (else
          (list-ref r 4)))))
  
  (make-object
    'abort     abort
    'apply     apply-it
    'init      init
    'commit    commit
    'conf      (lambda () (list (list "/etc/init.d/ide" make-etc #f)))
    'desc      desc
    'get-speed get-speed
    'load      load-it
    'mid-conf  (lambda () (list "/var/lib/ogl/etc/ideconf"))
    'mode-list mode-list
    'n->mode   mode
;    'md5sum    md5sum
    'set-mode  set-mode
    'try-mode  try-mode
    'test-tool test-tool
    'table     (lambda () table)
    ))
