;
;         First Stage Installer
;
;                                              (C)2001,2002 Omoikane Inc.
;
(load "debug.scm")
(load "obj.scm")
(load "keyboard.scm")
(load "mod.scm")
(load "net.scm")
(load "newt1.scm")
(load "util.scm")
(load "media.scm")
(load "snewt.scm")
(load "part.scm")
(load "fdisk.scm")
(load "na.scm")
(load "profile.scm")


(define (map-2ndfs r) ; r : 2nd root
  (collect)
  (let rep ((d ""))
    (let ((l (unix-read-directory (string-append r "/" d))))
      (inst-log "mapping ~a\n" d)
      (map (lambda (x)
             (let* ((from (string-append r "/" d "/" x))
                    (to (string-append "/" d "/" x))
                    (ln (lambda () (sh "ln -sf ~a ~a" from to)))
                    (t (unix-filetype to)))
               (inst-log "checking ~a\n" (unix-filetype to))
               (cond
                 ((equal? 'dir t)
                   (rep to))
                 ((equal? 'error t)
                   (ln))
                 (else
                   #t))))
           (filter (lambda (y)
                     (and (not (string=? ".." y))
                          (not (string=? "." y))))
                   l)))))

(define (media-check)
  (sh "modprobe sr_mod")
  (sh "modprobe sg_mod")
  (repeat
    (lambda ()
      (newt-ui-media-default #t obj-med obj-part obj-na)
      (obj-med 'open)
      (cond
        ((obj-med 'verify)
          #t)
        (else
          (snewt-message (_ "Media Selection")
            (_ "Could not open media.\n"))
          (obj-med 'close)
          #f)))))

(define (check-to-change-media)
  (cond
    ((or (arma-check-title)
         (not (= 2 (snewt-choice (_ "Loading Next Stage")
                     (_ "If you have ARMA CD-ROM, you can install ARMA with this updated installer.\n")
                     (_ "Skip") (_ "Install ARMA")))))
      (if (not (and (obj-med 'open)
                    (obj-med 'verify)))
        (media-check)))
    (else
      ;(obj-med 'eject)
      (cond
        ((not (= 1 (snewt-choice (_ "Loading Next Stage")
                     (_ "Place ARMA Installer and press Ok.")
                     (_ "Ok") (_ "Cancel"))))
          (check-to-change-media))
        ((not (and (obj-med 'open)
                   (obj-med 'verify)))
          (snewt-message  (_ "Loading Next Stage")
            (_ "Could not open media.\n"))
          (check-to-change-media))
        (else
          (snewt-show-while (_ "Loading Next Stage")
              (_ "Re-loading next stage of installer. \nPlease wait a moment....") 54 3
              (lambda ()
                (sh "mount /dev/ram3 /1.6 -t minix")
                ; OGL Free CD $B$+$i%V!<%H$7$F$$$k>l9g$O(B loopback $BIU$-(B mount $B%3%^%s%I$r;H$&!#(B
                (cond
                  ((equal? 'cdrom (car (obj-med 'info)))
                    (obj-med 'link "ogl/inst/boot.img" "/tmp/boot.img")
                    (sh "mount /tmp/boot.img /mnt -t vfat -o loop")
                    (sh "gunzip -c /mnt/root.bin >/dev/ram2")
                    (sh "umount /mnt")
                    (sh "mount /dev/ram2 /mnt -t minix")
                    (sh "cp /mnt/var/lib/ogl/etc/* /var/lib/ogl/etc")
                    (sh "umount /mnt")))
                (obj-med 'pipe "ogl/inst/second.img" "cat >/dev/ram2")))
          #t)))))

(define (cdboot-mount-rep n)
  (cond
    ((< 200 n)
      (snewt-fatal-error "Could not mount ram image!\n"))
    (else
      (sh "cp /mod.img /dev/ram4")
      (cond
        ((= 0 (sh "mount /dev/ram4 /mod-cram/mod -t cramfs"))
          (inst-log "mod.img mounted at ~a time.\n" n)
          (sh "rm /mod.img")
          #t)
        (else
          (cdboot-mount-rep (1+ n)))))))

(define obj-kbd (configure-keyboard))
(define obj-mod (configure-module))
(define obj-net (configure-network))
(define obj-part (configure-part "/mnt"))
(define obj-prof (conf-prof))
(define obj-med (conf-media obj-net obj-part obj-prof))
(define obj-na (conf-na))

(set! sh-log "logger")

(make-initdev)
(system-at "/dev/tty2" "mkdir /var/log")
(let ((lang (unix-getenv "LANG")))
  (system-at "/dev/tty2" "syslogd")
  (if (string? lang)
    (unix-setenv "LANG" lang)))

(sh "/dev/tty2" "tail -f /var/log/messages >/dev/tty2 &")
(sh "ln -sf tty2 /dev/console")
; set console log level to <4>
(sh "echo '4 4 1 7' >/proc/sys/kernel/printk")
(sh "klogd")
(textdomain "inst1" "/usr/share/locale")

(obj-net 'init 'inst1)
(obj-med 'init 'inst1)
(obj-part 'init 'inst1)

(let ((lang (unix-getenv "LANG")))
  (fprintf "/var/lib/ogl/etc/lang"
    (cond
      ((eq? #f lang) "C")
      ((and (< 2 (string-length lang))
            (equal? "ja" (substring lang 0 2)))
        "ja_JP.eucJP")
      (else
        lang))))

(cond
  ((unix-access? "/cdboot" '(read))
    (sh "insmod /tmp/cramfs.o")
    (sh "mkdir /mod-cram")
    (sh "mkdir /mod-cram/mod")
    (cdboot-mount-rep 0)
    (map-2ndfs "/mod-cram")
    (sh "rm /mod.img")))

(if (not (eq? #f (unix-getenv "NOUSB")))
  (display
    (string-append
      (_ "loading USB keyboard modules.\n")
      (_ "If system stops here, then your hardware should have some trouble with Linux USB keyboard driver.\n")
      (_ "If it is the case, try 'nousb' or 'nousbeng' option at boot prompt.\n"))))

(if (lang-ja?)
  (obj-kbd 'set-type "jp106")
  (obj-kbd 'set-type "us"))
(obj-kbd 'init 'inst1)

(cond
  ((unix-access? "/proc/cons" '(write))
    (sh "echo 2 >/proc/cons")
    (unix-sleep 1)
    (sh "echo 0 >/proc/cons")))

(snewt-start)
(cond
  ((not (arma-check-title))
    (newt-stage-title (_ "Welcome"))
    (snewt-message (_ "Welcome")
      (string-append
        (_ "Welcome to OGL Free!\n")
        (_ "If you have trouble with installing and with utilizing OGL, you can utilize support from Omoikane by buying support point.\n")))))

(newt-ui-kbd #t obj-kbd #f)

(sh "depmod -a")
(if (not (= 0 (sh "modprobe cramfs")))
  (snewt-fatal-error "Could not install cramfs driver!\n"))

(obj-mod 'init 'inst1)
(obj-mod 'extra-probe)

(newt-module-select obj-mod)
(newt-ui-auto-probe obj-part obj-mod)
(newt-ui-more-driver obj-part obj-mod)

(obj-net 'init 'inst1)
(repeat (lambda () (newt-ui-net #t obj-net)))
(obj-mod 'setmod 'ipv6 "" 'un/set)

(obj-part 'init 'inst1)
(let loop ((dummy #t))
  (media-check)
  ;(snewt-message (_ "debug") "try it")
  
  (snewt-show-while (_ "Loading Next Stage")
    (_ "Loading next stage of installer. \nPlease wait a moment....") 54 3
    (lambda ()
      (obj-med 'pipe "ogl/inst/stage1_5.gz" "gunzip -c - >/dev/ram3")
      ;(unix-sleep 10000)
      (obj-med 'pipe "ogl/inst/second.img" "cat - >/dev/ram2")
      (obj-med 'cp "ogl/inst/second.img" "/dev/ram2")
      (obj-med 'cp "ogl/inst/second.img" "/dev/ram2")
      (obj-med 'cp "ogl/inst/second.img" "/dev/ram2")
      (obj-med 'cp "ogl/inst/second.img" "/dev/ram2")
      (sh "md5sum /dev/ram2 >/dev/null")
      (sh "cat /dev/ram2 >/dev/null")
      (sh "mount /dev/ram2 /2nd -t cramfs 1>/dev/null 2>&1")
      (obj-med 'cp "ogl/inst/second.img" "/dev/ram2")
      (sh "umount /2nd")
      (obj-med 'cp "ogl/inst/second.img" "/dev/ram2")
      ; cram $B$O$J$<$+%a%b%j$K$&$^$/H?1G$5$l$J$$$H$-$,$"$k!#$R$H$^$:7+$jJV$7%3%T!<(B
      ; $B$9$k$3$H$G2sHr$G$-$F$$$kLOMM!#860x$OITL@$@$,!"%$%s%9%H!<%i$G$O%a%b%j$r@)8B(B
      ; $B$7$?$[$&$,$h$$$+$b$7$l$J$$!#(B
      (obj-med 'pipe
               (format #f "ogl/inst/2nds~a.tgz" (caddr (get-proc-version-list)))
               "gunzip -c - | (cd / ; tar -x)")
      (unix-sync)))
  (let ((r (obj-med 'close)))
    (cond
      ((not (eq? #t r))
        (newt-fatal-error
          (format #f (_ "Could not close opened media. Reason:\n~a\n") r)))
      ((= 0 (sh "mount /dev/ram2 /2nd -t cramfs 1>/dev/null 2>&1"))
        (check-to-change-media))
      (else
        (snewt-message (_ "Error")
          (_ "Could not access to the second stage."))
        (loop #f)))))

(map-2ndfs "/2nd")
(sh "ln -sf /1.5/*.scm /scm")
;(sh "cp /1.5/video* /2nd/usr/lib/ogl/lib")
(fprintf "/bin/X" "#!/bin/sh\nexec XFree86 $*\n")
(sh "chmod +x /bin/X")
(sh "depmod -a -r")

;(collect)

(map (lambda (x)
       (if (not (= 0 (sh "modprobe ~a" x)))
         (inst-log "Warinig : could not load fs module ~a\n" x)))
  '(xfs ext2 reiserfs ext3 jfs vfat))
(sh "rm /mod/fs/*")

(obj-med 'commit)
(obj-med 'close)
(obj-net 'commit)
(obj-mod 'commit)
(obj-kbd 'commit)
(obj-part 'commit)
(obj-prof 'commit)
(sh "mkdir /etc/network")
(snewt-finish)
(exit 0)
