
(define (configure-keyboard)
  (define typelist0 '(("JP-106/109" . "jp106") ("US" . "us")))
  (define typelist1 '(("JP-106/109" . "usb-jp106") ("US" . "usb-us")))
  (define typelist '())
  (define type "jp106")
  (define caps "caps")
  (define context 'admin)
  (define root "/")
  (define usb? #f)
  
  (define (type-to-sym t)
    (cond
      ((or (equal? t "jp106") (equal? t "usb-jp106"))
        "jp106")
      (else
        "us")))
  
  (define (init c)
    (set! context c)
    (set! usb?
      (cond
        ((eq? #f (unix-getenv "NOUSB"))
          (case context
            ((inst1 recovery)
              (sh "depmod -a ; modprobe keybdev ; modprobe hid")
              (sh "mount none /proc/bus/usb -t usbdevfs")))
          (let ((u (parse-awk #\tab #t "/proc/bus/usb/devices")))
            (cond
              ((eq? #f u) #f)
              ((null? (filter (lambda (x)
                                (and (list? x)
                                     (not (null? x))
                                     (string? (car x))
                                     (= 65 (string-length (car x)))
                                     (string=? "Prot=01 Driver=hid" (substring (car x) 47 65))))
                              u))
                #f)
              (else
                #t))))
        (else
          #f)))
    (if usb?
      (cond
        ((lang-ja?)
          (set! type "usb-jp106")
          (set! typelist typelist1))
        (else
          (set! type "usb-us")
          (set! typelist (reverse typelist1))))
      (cond
        ((lang-ja?)
          (set! type "jp106")
          (set! typelist typelist0))
        (else
          (set! type "us")
          (set! typelist (reverse typelist0)))))
    (case c
      ((inst2 recovery)
        (set! root "/target"))))
  
  (define (set-type t)
    (cond
      ((set-include? t (map cdr typelist))
        (set! type t)
        #t)
      (else
        #f)))
  
  (define (apply-setting)
    (if (and (equal? type "usb-jp106")
             (unix-access? "/proc/keybdev" '(write)))
        (= 0 (sh "cat /usr/lib/ogl/lib/usb-jp106-map >/proc/keybdev")))
    (let ((path (case context
                  ((inst1 inst2 recovery) "cat /lib/keymaps/~a.kmap")
                  (else "zcat /usr/share/keymaps/i386/qwerty/~a.kmap.gz")))
          (tmp (unix-tempname
                 (case context
                   ((inst1 inst2 recovery) "/lib/keymaps")
                   (else "/tmp")))))
      (and
        (= 0 (sh "~a > ~a" (format #f path (type-to-sym type)) tmp))
        (case caps
          ((ex ctrl)
            (= 0 (sh "echo 'keycode 58 = Control' >>~a" tmp)))
          (else
            #t))
        (case caps
          ((ex)
            (= 0 (sh "echo 'keycode 29 = Caps_Lock' >>~a" tmp)))
          (else
            #t))
        (= 0 (sh "loadkeys ~a" tmp))
        (begin (sh "rm ~a" tmp) #t))))
  
  (define (commit)
    (and 
      (fprintf "/var/lib/ogl/etc/kbd"
        (format #f "~a\t~a\n" type caps))
      (cond
        ((eq? 'inst1 context)
          #t)
        ((not (= 0 (sh "~a /usr/bin/dumpkeys | ~a /bin/gzip -c - >~a/etc/console/boottime.kmap.gz"
                       (if (eq? 'inst2 context) "chroot /target" "")
                       (if (eq? 'inst2 context) "chroot /target" "")
                       root)))
          "dumpkey failed")
        (else
          (cond
            ((or (equal? type "usb-jp106")
                 (equal? type "usb-us"))
              (fprintf (format #f "~a/etc/rc.boot/usbkbd" root)
                (string-append
                  "#!/bin/sh\nmodprobe input\nmodprobe hid\nmodprobe keybdev\n"
                  "modprobe usb-uhci >&/dev/null\nmodprobe usb-ohci>&/dev/null\n"
                  ; $B%$%s%9%H!<%k;~$K$O$I$A$i$r;H$C$F$$$k$+H=JL$G$-$J$$$N$GN>J}=q$$$F$*$/!#(B
                  (if (equal? type "usb-jp106")
                    (string-append
                      "if [ -f /proc/keybdev ] && [ -f /usr/lib/ogl/lib/usb-jp106-map ]; then\n"
                      "  cat /usr/lib/ogl/lib/usb-jp106-map >/proc/keybdev\n"
                      "fi\n")
                    "")))
              (sh "chmod +x ~a/etc/rc.boot/usbkbd" root)
              (if (eq? 'inst2 context)
                (sh "cp /usr/lib/ogl/lib/usb-jp106-map /target/usr/lib/ogl/lib/usb-jp106-map"))))
          '#t))))
  
  (define (load-it)
    (let ((r (parse-awk #\tab #t "/var/lib/ogl/etc/kbd")))
      (cond
        ((eq? #f r)
          (_ "No configuration file.\n"))
        (else
          (set! type (caar r))
          (if (< 1 (length (car r)))
            (set! caps (string->symbol (cadr (car r)))))
          #t))))
  
  (define (nousb)
    (set! usb? #f)
    (= 0 (sh "rmmod keybdev hid input")))
  
  (define (set-caps x)
    (case x
      ((caps ex ctrl)
        (set! caps x)
        #t)
      (else
        #f)))
  
  (define (desc)
    (string-append
      (cond
        ((equal? type "jp106")     "JP-106/109")
        ((equal? type "usb-jp106") "JP-106/109(USB)")
        ((equal? type "us")        "US")
        ((equal? type "usb-us")    "US(USB)")
        (else (_ "unknown")))
      " "
      (case caps
        ((ex)   (_ "(exchanged caps/control)"))
        ((ctrl) (_ "(mapped caps to control)"))
        (else (_ "(normal)")))))
  
  (make-object
    'apply        apply-setting
    'commit       commit
    ;'conf         (lambda () (list (list "/etc/rc.boot/kbd" make-rc #f)))
    'desc         desc
    'get-context  (lambda () context)
    'init  init
    'set-caps     set-caps
    'set-type     set-type
    'get-type     (lambda () (list type caps))
    'list         (lambda () typelist)
    'load         load-it
    'mid-conf     (lambda () (list "/var/lib/ogl/etc/kbd"))
    'usb?         (lambda () usb?)
    'nousb        nousb
    ))
