;;
;;    Media Object
;;
;;                          (C)2001,2002 Omoikane Inc.
;;

(define (conf-media obj-net obj-part obj-prof)
  ;;
  ;;  var
  ;;
  (define context 'admin)
  (define media '())
  (define media-misc '())
  (define mnt "/mnt/deb")
  (define open? #f)
  (define debconf "/var/cache/debconf/config.dat")
  (define mirror '())
  
  ;;  utils
  (define (med-type x) (car x))
  (define (med-did x) (cadr x))
  (define (med-host x) (cadr x))
  (define (med-path x) (caddr x))
  (define (med-fstype x) (list-ref x 3))
  (define (med-part x) (list-ref x 4))
  
  (define (opened)
    (set! open? #t)
    #t)
  
  (define (nfs-usable?)
    (let ((r (parse-awk #\space #t "/proc/filesystems")))
      (cond
        ((eq? #f r)
          #f)
        ((null? (filter (lambda (x) (equal? x "nfs"))
                        (apply append r)))
          #f)
        (else
          #t))))
  
  (define (make-line l bin? src?)
    (case (car l)
      ((man)
        (format #f "~a\n" (cadr l)))
      (else
        (let ((x (case (car l)
                   ((ogl)
                     (cond
                       ((null? (cdr l))
                         "")
                       (else
                         (let ((m (car media)))
                           (format #f "~a~a~a ./"
                             (case m
                               ((http) "http://")
                               (else "file:/"))
                             (case m
                               (http (if (equal? "" (list-ref media 3))
                                         (cadr media)
                                         (format #f "~a:~a@~a"
                                           (list-ref media 3) (list-ref media 4) (cadr media))))
                               (else "mnt/deb"))
                             (case m
                               ((cdrom nfs) "/ogl/")
                               (else
                                 (format #f "/~a/ogl" (caddr media)))))))))
                   ((debian)
                     (format #f "~a ~a main contrib non-free"
                       (cadr l) (caddr l)))
                   ((nu)
                     (format #f "~a ~a/non-US main contrib non-free"
                       (cadr l) (caddr l)))
                   ((sec)
                     (format #f " ~a ~a/updates main contrib non-free"
                       (cadr l) (caddr l)))
                   (else
                     "#illegal line : ~a" l))))
         (string-append
           (if bin? (format #f "deb ~a\n" x) "")
           (if src? (format #f "deb-src ~a\n" x) ""))))))
  
  (define (parse-mirror)
    (define (fold k x l)
      (cond
        ((null? l)
          (if (equal? k "next")
            '()
            (list (cons k (reverse x)))))
        (else
          (let ((h (car l)))
            (cond
              ((or (null? h) (equal? h '("")))
                (cons (cons k (reverse x))
                      (fold "next" '() (cdr l))))
              ((equal? "Site:" (car h))
                (fold (cadr h) '() (cdr l)))
              (else
                (fold k (cons h x) (cdr l))))))))
    
    (let ((r (parse-awk #\space #t "/usr/share/base-config/Mirrors.masterlist")))
      (cond
        ((eq? #f r)
          #f)
        (else
          (set! mirror (fold "next" '() r))))))
  
  ;;
  ;;  method
  ;;
  (define (init x)
    (set! context x)
    (case x
      ((inst2 recovery) "/target"))
    (case x
      ((inst3 admin)
        (parse-mirror)))
    #t)
  
  (define (open-it l)
    (define (mnt-check)
      (case context
        ((inst1 inst2 recovery)
          (set! mnt (unix-tempname))
          (= 0 (sh "mkdir ~a" mnt)))
        (else
          #t)))
    (cond
      ((null? l)
        (_ "Media is not configured yet."))
      (open?
        (_ "Media already open."))
      (else
        (case (med-type l)
          ((block cdrom)
            (mnt-check)
            (if (= 0 (sh "mount -r -t ~a ~a ~a"
                        (fs->mountopt (med-fstype l))
                        (if (equal? 'cdrom (med-type l)) (obj-part 'devfsname (med-did l))
                                                         (obj-part 'devfsname (med-did l) (med-part l)))
                        mnt))
                (opened)
                (_ "Failed to mount block device")))
          ((nfs)
            (mnt-check)
            (case context
              ((inst1 inst2 recovery)
                (sh "modprobe nfs")
                (cond
                  ((not (= 7 (length l)))
                    (inst-log "wrong format of media list : ~s\n" l)
                    #f)
                  ((= 0 (sh "mount -r -t nfs -o nolock ~a:~a ~a"
                                 (obj-net 'resolv (med-host l)) (med-path l) mnt))
                    (opened))
                  ((nfs-usable?)
                    (_ "Failed to mount nfs"))
                  (else
                    (_ "NFS module not loaded"))))
              (else
                (if (= 0 (sh "mount -r -t nfs -o nolock ~a:~a ~a"
                                 (obj-net 'resolv (med-host l)) (med-path l) mnt))
                    (opened)
                    (_ "Failed to mount media")))))
          ((http)
            (opened))
          (else
            (_ "Method Not Implemented"))))))
  
  (define (close-it l)
    (cond
      ((null? l)
        (_ "Media is not configured yet."))
      ((not open?)
        (_ "Media is not opened."))
      (else
        (case (car l)
          ((nfs smb cdrom block)
            (cond
              ((not (= 0 (sh "umount ~a" mnt)))
                (_ "Failed to unmount media"))
              (else
                (case context 
                  ((inst1 inst2 recovery)
                    (sh "rmdir ~a" mnt)))
                (set! open? #f)
                #t)))
          ((http)
            (set! open? #f)
            #t)
          (else
            (_ "Method Not Implemented"))))))
  
  (define (media-arg-check l)
    (case (car l)
      ((cdrom)
        (and (= 4 (length l))
             (string? (cadr l)) (string? (caddr l)) (symbol? (list-ref l 3))))
      ((block)
        (and (= 5 (length l))
             (string? (cadr l)) (string? (caddr l)) (symbol? (list-ref l 3))
             (integer? (list-ref l 4))))
      (else
        (and (= 7 (length l))
             (string? (cadr l)) (string? (caddr l)) (string? (list-ref l 3))
             (string? (list-ref l 4)) (string? (list-ref l 5)) (integer? (list-ref l 6))))))

  (define (set . l)
    (inst-log "media set, l:~s, open?:~s\n" l open?)
    (cond
      ((null? l)
        (set! media l)
        #t)
      ((not (media-arg-check l))
        (inst-log "illegal line : ~s\n" l)
        #f)
      (else
        (let ((x (open-it l)))
          (if (not (eq? #t x))
            x
            (let ((y (close-it l)))
              (cond
                ((not (eq? #t y))
                  y)
                (else
                  (set! media l)
                  (case (car media)
                    ((http)
                      (cond
                        ((equal? "" (list-ref media 5))
                          (obj-prof 'remove-line "http proxy"))
                        (else
                          (obj-prof 'update-line "http proxy"
                            (format #f "http_proxy=\"http://~a:~a/\"\nexport http_proxy"
                              (list-ref media 5) (list-ref media 6)))))))
                  #t))))))))
  
  (define (reset)
    (set! media '()))
  
  (define (pipe f c)
    (define (last l)
      (cond ((not (list? l)) nil)
            ((null? l) nil)
            ((null? (cdr l)) (car l))
            (else (last (cdr l)))))
    (if (null? media)
      #f
      (let ((l (case (car media)
                 ((nfs smb cdrom)
                   (format #f "cat ~a/~a" mnt f))
                 ((block)
                   (format #f "cat ~a/~a/~a" mnt (med-path media) f))
                 (http
                   ; $B$"$H$G(B
                   "")
                 (else
                   ""))))
        (= 0 (sh "~a | ~a" l c)))))
  
  (define (ls p)
    (let ((rr (lambda (x)
                (let ((r (unix-read-directory2 x)))
                  (cond
                    ((< (length r) 2)
                      #f)
                    (else
                      (cddr r)))))))
      (case (car media)
        ((nfs smb cdrom)
          (rr (format #f "~a/~a" mnt p)))
        ((block)
          (rr (format #f "~a/~a/~a" mnt (med-path media) p)))
        (else
          #t))))
  
  (define (cp src dest)
    (pipe src (format #f "cat >~a" dest)))
  
  (define (make-etc f)
    (fprintf f
      (apply string-append
        (cons
          (make-line (cons 'ogl media) #t #t)
          (map (lambda (x) (make-line x #t #t))
               media-misc)))))
  
  (define (load-it)
    (cond
      ((not (unix-access? "/var/lib/ogl/etc/media" '(read)))
        #t)
      (else
        (let ((r (ttable-read "/var/lib/ogl/etc/media"
                   '(("ogl" "cdrom" str str sym)
                     ("ogl" "block" str str sym int)
                     ("ogl"    str  str str str str str int)
                     ("deb" str str)
                     ("nu" str str)
                     ("sec" str str)
                     ("man" str)))))
          (cond
            ((eq? #f r)
              #f)
            (else
              (let ((lo (filter (lambda (x) (equal? "ogl" (car x)))
                                 r))
                    (lr (filter (lambda (x) (not (equal? "ogl" (car x))))
                                r)))
                (cond
                  ((not (and (or (null? lo) (= 1 (length lo)))
                             (andmap (lambda (x)
                                       (if (= 2 (length x))
                                          (ormap (lambda (y) (equal? y (cadr x)))
                                            '("unstable" "testing" "stable"))
                                          #t))
                                     lr)))
                    #f)
                  (else
                    (if (not (null? lo))
                      (let ((x (car lo)))
                        (set! media (cons (string->symbol (cadr x)) (cddr x)))))
                    (set! media-misc
                      (map (lambda (x) (cons (string->symbol (car x)) (cdr x)))
                           lr))
                    #t)))))))))
  
  (define (commit)
    (ttable-write "/var/lib/ogl/etc/media"
      (if (null? media)
         media-misc
         (cons (cons 'ogl media) media-misc))))
  
  (define (verify)
    (case context
      ((inst1 inst2 recovery)
        (let ((t (unix-tempname)))
          (cp "ogl/inst/checker" t)
          (let ((r (< 0 (unix-filesize t))))
            (sh "rm ~a" t)
            r)))
      (else
        (eq? #t (verify-deb (list (cons 'ogl media)))))))
  
  (define (verify-deb l)
    (case context
      ((inst1 inst2 recovery)
        "Context does not match. ")
      (else
        (let* ((tmp (unix-tempname))
               (a (apply string-append
                    (map (lambda (x) (make-line x #t #f))
                         l))))
          (inst-log "applying apt-line : ~s\n" a)
          (cond
            ((not (fprintf tmp a))
              "temporary file error.")
            (else
              (let ((n (sh "wrap -t 15 -- apt-get -o APT::Get::List-Cleanup=false -o Dir::Etc::sourcelist=~a update"
                           tmp)))
                (inst-log "media : apt-get returns ~a\n" n)
                (sh "rm ~a" tmp)
                (case n
                  ((0) #t)
                  ((220) (_ "apt-get has been timed out. "))
                  (else (format #f (_ "apt-get exited with code ~a . ") n))))))))))
  ; block media $B$O%^%&%s%H$7$J$$$H%@%a(B
  
  (define (eject)
    (cond
      ((and (< 2 (length media))
            (equal? 'cdrom (car media)))
        (let ((d (obj-part 'devfsname (cadr media))))
          (or (eject-cdrom d)
              (eject-cdrom d))))
      (else
        #f)))
  
  (define (link x y)
    (cond
      ((not (and (list? media)
                 (< 1 (length media))))
        #f)
      (else
        (case (car media)
          ((cdrom)
            (= 0 (sh "ln -sf ~a/~a ~a" mnt x y)))
          (else
            #f)))))
  
  (define (verify-inst l)
    (let* ((cmd "|gawk '/^Package:/{p=$2}/^Status:/{if(p!=\"\" && $2==\"install\" && $3==\"ok\" && $4==\"installed\") print p}/^$/{p=\"\"}' </var/lib/dpkg/status")
           (r (parse-awk #\space #t cmd)))
      (cond
        ((not (list? r))
          #f)
        ((ormap null? r)
          #f)
        (else
          (let ((r' (map car r)))
            (list-set- equal? l r'))))))
  
  (define (debconf-off)
    (let ((r (parse-awk #\space #t
               (string-append
                 "|gawk '/^Name:/{p=$2}/^Value:/{if(p==\"debconf/frontend\") print $2}' <"
                 debconf))))
      (cond
        ((not (and (list? r) (= 1 (length r)) (= 1 (length (car r)))))
          (inst-log "failed to get debconf's old value.\n")
          #f)
        ((not (fprintf "/var/lib/ogl/etc/debconf" (caar r)))
          (inst-log "failed to save debconf's old value.\n")
          #f)
        (else
          (let* ((tmp (unix-tempname))
                 (r1 (sh "gawk '/^Name:/{p=$2}/^Value:/{if(p==\"debconf/frontend\") print \"Value: Noninteractive\" ; else print $0} !/^Value:/{print $0}' <~a >~a"
                         debconf tmp)))
            (cond
              ((not (= 0 r1))
                (inst-log "failed to converting debconf config.\n")
                #f)
              ((not (and (= 0 (sh "cp -f ~a ~a-old" debconf debconf))
                         (= 0 (sh "mv -f ~a ~a" tmp debconf))
                         (= 0 (sh "chmod 600 ~a" debconf))))
                (inst-log "failed to copy debconf config.\n")
                #f)
              (else
                #t)))))))
  
  (define (debconf-on fe)
    (let ((r (parse-awk #\space #t "/var/lib/ogl/etc/debconf")))
      (cond
        ((not (and (list? r) (= 1 (length r)) (= 1 (length (car r)))))
          (inst-log "failed to get debconf's old value.\n")
          #f)
        (else
          (let* ((tmp (unix-tempname))
                 (r1 (sh "gawk '/^Name:/{p=$2}/^Value:/{if(p==\"debconf/frontend\") print \"Value: ~a\" ; else print $0} !/^Value:/{print $0}' <~a >~a"
                         (if (equal? "" fe) (caar r) fe)
                         debconf tmp)))
            (cond
              ((not (= 0 r1))
                (inst-log "failed to converting debconf config.\n")
                #f)
              ((not (and (= 0 (sh "cp -f ~a ~a-old" debconf debconf))
                         (= 0 (sh "mv -f ~a ~a" tmp debconf))
                         (= 0 (sh "chmod 600 ~a" debconf))))
                (inst-log "failed to copy debconf config.\n")
                #f)
              (else
                #t)))))))
  
  (define (list-mir)
    (define (rep l)
      (cond
        ((null? l) l)
        (else
          (let* ((x (car l))
                 (a (cdr x))
                 (p (assoc "Archive-http:" a))
                 (u (assoc "NonUS-http:" a))
                 (c (assoc "Country:" a)))
            (cond
              ((and (not (eq? #f p))
                    (not (eq? #f u))
                    (equal? c '("Country:" "JP" "Japan")))
                (cons (car x) (rep (cdr l))))
              (else
                (rep (cdr l))))))))
    (rep mirror))
  
  (define (desc-media)
    (cond
      ((null? media)
        (_ "Default Not Set"))
      (else
        (case (med-type media)
          ((cdrom) "CD-ROM")
          ((block)
            (format #f "~a - ~a"
              (obj-part 'did-desc (med-did media) (med-part media))
              (med-path media)))
          ((nfs)
            (format #f (_ "NFS Server:~a Path:~a") (med-host media) (med-path media)))
          (else
            (inst-log "desc-media : unknown ~a\n" media)
            (_ "Some media"))))))
  
  (define (desc-misc)
    (map (lambda (x)
           (let* ((s (case (car x)
                       ((debian)
                         (format #f    "Debian       ~a ~a" (cadr x) (caddr x)))
                       ((sec)
                         (format #f (_ "Debian security ~a ~a") (cadr x) (caddr x)))
                       ((nu)
                         (format #f (_ "Debian NonUS ~a ~a") (cadr x) (caddr x)))
                       ((man)
                         (format #f (_ "manually set aptline ~a") (cadr x)))
                       (else
                         (format #f "unknown apt-line ~a" x))))
                  (l (string-length s)))
             (if (< 52 l)
               (substring s 0 52)
               s)))
         media-misc))
  
  (define (add-media . l)
    (cond
      ((null? l)
        #f)
      (else
        (case (car l)
          ((debian sec nu man)
            (set! media-misc (cons l media-misc))
            #t)
          (else
            #f)))))
  
  (define (site-entry s f)
    (let ((r (assoc s mirror)))
      (cond
        ((eq? #f r) '())
        (else
          (let* ((a (cdr r))
                 (p (assoc "Archive-http:" a))
                 (u (assoc "NonUS-http:" a)))
            (list
              (list 'debian (format #f "http://~a~a" s (cadr p)) f)
              (list 'nu (format #f "http://~a~a" s (cadr u)) f)))))))
  
  (define (rm-media n)
    (let ((l (length media-misc)))
      (cond
        ((or (< n 0) (< l n))
          #f)
        (else
          (set! media-misc
                (append (list-sub media-misc 0 n)
                        (if (= l n)
                          '()
                          (list-sub media-misc (1+ n) l))))
          #t))))
  
  (make-object
    'add-media    add-media
    'close        (lambda () (close-it media))
    'commit       commit
    'conf         (lambda () (list (list "/etc/apt/sources.list" make-etc #t)))
    'cp           cp
    'desc-media   desc-media
    'desc-misc    desc-misc
    'debconf-off  debconf-off
    'debconf-on   debconf-on
    'eject        eject
    'init         init
    'info         (lambda () media)
    'info-mirror  (lambda () mirror)
    'link         link
    'list-mir     list-mir
    'list-misc    (lambda () media-misc)
    'load         load-it
    'ls           ls
    'mid-conf     (lambda () (list "/var/lib/ogl/etc/media" "/var/lib/ogl/etc/debconf"))
    'set          set
    'reset        reset
    'open         (lambda () (open-it media))
    'open?        (lambda () open?)
    'pipe         pipe
    'rm-media     rm-media
    'site-entry   site-entry
    'verify       verify
    'verify-deb   verify-deb
    'verify-inst  verify-inst
    ))
