;;
;;     Module Management Object 
;;
;;                          (C)2001,2002 Omoikane Inc.
;;

(define (configure-module)
  ;;
  ;; variables
  ;;
  (define pci-table '())
  (define device-table '())
  (define mid-table '())
  (define the-table '())
  (define usable-table '())
  (define usable-tree '())
  (define compat-table '())
  (define map-table '())
  (define net-list '())
  (define pcmcia-list '())
  (define ide-list '())
  (define scsi-list '())
  (define sound-list '())
  (define alsa-list '())
  (define context 'admin)
  (define object-name "mod")
  (define pcmcia-table '())
  (define pcmcia-chip #f)
  (define usb-table '())
  (define setmod-init? #f)
  
  ;;
  ;; local func
  ;;
  (define (installed)
    (cond
      ((equal? 'regist context)
        '())
      ((unix-access? "/proc/modules" '(read))
        (map (lambda (x) (string->symbol (car x)))
             (parse-awk #\space #t "/proc/modules")))
      (else
        (inst-log "Cound not find /proc/modules.\n/proc does seems to be mounted.")
        '())))
  
  (define (initial-table)
      (define list-inst/set
        (let rep ((l (installed)))
          (cond ((null? l) l)
                ((assoc (car l) mid-table)
                 (cons (car l) (rep (cdr l))))
                (else (rep (cdr l))))))
    
      (define list-inst/not
        (list-set- eq? (installed) list-inst/set))
      
      (define list-un/set
        (list-set- eq? (map car mid-table) list-inst/set))

      (define list-un/not
        (list-set- eq? (append (map car device-table)
                               usb-table
                               pcmcia-table)
                       (append list-inst/set
                               list-inst/not
                               list-un/set)))

      (easy-sort
         (lambda (x y) (symbol<? (car x) (car y)))
         (append (map (lambda (x) (list x 'inst/set)) list-inst/set)
                 (map (lambda (x) (list x 'inst/not)) list-inst/not)
                 (map (lambda (x) (list x 'un/set)) list-un/set)
                 (map (lambda (x) (list x 'un/not)) list-un/not))))
  
  (define (update-table m s)
    (set! the-table
      (let rep ((l the-table))
        (cond
          ((null? l)
            (list (list m s)))
          ((eq? (caar l) m)
            ;(case s    ;mid-table⹹
            ;  ((inst/not un/not)
            ;   (set! mid-table
            ;     (let loop ((l mid-table))
            ;      (cond 
            ;       ((null? l) '())
            ;       ((eq? (caar l) m) (loop (cdr l)))
            ;       (else (cons (car l) (loop (cdr l))))))))
            ; ((inst/set un/set)
            ;  (set! mid-table
            ;    (let loop ((l mid-table))
            ;     (cond
            ;       ((null? l) (list (list m (mod->genre m) (info m))))
            ;       ((eq? (caar l) m) l)
            ;       (else (cons (car l) (loop (cdr l))))))))
            ;  (else #f))
            (cons (list m s) (cdr l)))  ; the-table ¹
          (else
           (cons (car l) (rep (cdr l))))))))
  
  (define (mod->genre m)
    (cond
      ((set-include? m net-list) 'net)
      ((set-include? m ide-list) 'ide)
      ((set-include? m scsi-list) 'scsi)
      ((set-include? m alsa-list) 'alsa)
      ((set-include? m sound-list) 'sound)
      ((set-include? m pcmcia-list) 'pcmcia)
      (else 'other)))
  
  (define (update-mid-table m o)
    (set! mid-table
      (let rep ((l mid-table))
        (cond
          ((null? l)
            (list (list m (mod->genre m) o)))
          ((eq? (caar l) m)
            (cons (list m (mod->genre m) o) (cdr l)))
          (else
            (cons (car l) (rep (cdr l))))))))
  
  (define (genre-list p)
    (let ((tmp (unix-tempname))
          (v (get-proc-version)))
      (sh "find /lib/modules/~a/~a -name '*.o' | sed -e 's/.*\\///' -e 's/.o$//' >~a"
           v p tmp)
      (let ((r (map (lambda (x) (string->symbol (car x)))
                    (parse-awk #\space #t tmp))))
        (sh "rm ~a" tmp)
        r)))
  
  (define (get-pcitable f)
    (define (pcitable-filter x)
      (cond
        ((or (not (list? x))
             (< (length x) 3)
             (not (string? (car x)))
             (not (string? (cadr x)))
             (< (string-length (car x)) 3)
             (< (string-length (cadr x)) 3))
          x)
        (else
          (cons (string-append (substring (car x) 2 (string-length (car x)))
                               (substring (cadr x) 2 (string-length (cadr x))))
                (cddr x)))))
    (let ((r (parse-awk #\tab #t f)))
      (if (eq? #f r)
          '()
          (map pcitable-filter r))))

  (define (probe-pcmcia)
    (define (probe)
      (let ((t (unix-tempname)))
        (cond
          ((not (= 0 (sh "probe -m >~a" t)))
            #f)
          ((= 0 (file-size t))
            #f)
          (else
            (let ((x (parse-awk #\space #t t)))
              (cond
                ((null? x) #f)
                (else (caar x))))))))
    
    (define (insmod-pcmcia mod)
      (sh "mkdir -p /var/run")
      (sh "mv -f /var/run/pcmcia.mod /tmp")
      (cond
        ((not (= 0 (sh "modprobe pcmcia_core")))
          (inst-log "could not install pcmcia_core.\n"))
        ((= 0 (sh "modprobe ~a" mod))
          (= 0 (sh "modprobe ds")))
        ((equal? "i82365" mod)
          (cond
            ((= 0 (sh "modprobe yenta_socket"))
              (set! pcmcia-chip "yenta_socket")
              (sh "modprobe ds"))
            (else
              (inst-log "could not install i82365/yenta_socket.\n")
              #f)))
        (else
          (inst-log "could not install ~a.\n" mod)
          #f)))
    
    (define (cardmgr)
      (sh "cardmgr -oq")
      (let ((l (parse-awk #\space #t "/var/run/pcmcia.mod")))
        (if l l '())))
    
    (let ((chip (probe)))
      (set! pcmcia-chip chip)
      (cond
        ((eq? chip #f)
          #f)
        ((insmod-pcmcia chip)
          (set! pcmcia-table
                (map (lambda (x) (string->symbol (cadr x)))
                     (cardmgr))))
        (else
          #f))))
  
  (define (probe-usb)
    (cond
      ((not (eq? #f (unix-getenv "NOUSB")))
        #t)
      ((case context
         ((inst3 admin)
           (not (= 0 (sh "ls /etc/rc2.d | grep murasaki"))))
         (else
           #f))
        #t)
      (else
        (let ((r (parse-awk #\space #t "| usbmgr -ob")))
          (cond
            ((not (list? r))
              #f)
            (else
              (set! usb-table
                (map (lambda (x) (string->symbol (car x)))
                     r))
              #t))))))
  
  (define (usable-table->tree k x l)
    (cond
      ((null? l)
        (if (null? x)
          x
          (list (cons k (map symbol->string (reverse x))))))
      ((equal? k (cdr (car l)))
        (usable-table->tree k (cons (caar l) x) (cdr l)))
      ((null? x)
        (usable-table->tree (cdr (car l)) (list (caar l)) (cdr l)))
      (else
        (cons
          (cons k (map symbol->string (reverse x)))
          (usable-table->tree (cdr (car l)) (list (caar l)) (cdr l))))))
  
  (define (flash)
    (cond
      ((and (not (equal? 'regist context))
            (not (unix-access? "/proc/modules" '(read))))
        #f)
      (else
        (sh "depmod -a -r 2>&1 >/dev/null")
        (case context
          ((inst1 inst2 recovery)
            (set! pci-table (get-pcitable "/lib/pcitable"))
            (let ((r (parse-awk #\space #t "/lib/moddb")))
              (cond
                ((eq? r #f)
                  #f)
                (else
                  (set! net-list
                       (map (lambda (x) (string->symbol (car x)))
                            (filter (lambda (x) (equal? (cadr x) "net")) r)))
                  (set! pcmcia-list
                       (map (lambda (x) (string->symbol (car x)))
                            (filter (lambda (x) (equal? (cadr x) "pcmcia")) r)))
                  (set! ide-list
                       (map (lambda (x) (string->symbol (car x)))
                            (filter (lambda (x) (equal? (cadr x) "ide"))
                                    r)))
                  (set! scsi-list
                       (map (lambda (x) (string->symbol (car x)))
                            (filter (lambda (x) (or (equal? (cadr x) "block")
                                                    (equal? (cadr x) "scsi")))
                                    r)))
                  (set! usable-table
                        (map (lambda (x) (cons (string->symbol (car x)) (cadr x)))
                             (filter (lambda (x) (= 2 (length x)))
                                     (parse-awk #\space #t "/lib/moddb"))))
                  (set! usable-tree
                    (if (null? usable-table)
                      '()
                      (let ((h (car usable-table)))
                        (usable-table->tree (cdr h) (list (car h)) (cdr usable-table)))))
                  #t))))
          (else  ;inst3 admin and rest
            (let ((tmp (unix-tempname))
                  (v (get-proc-version)))
              (sh "cat /usr/lib/ogl/lib/pcitable-* >~a" tmp)
              (set! pci-table (get-pcitable tmp))
              (sh "rm ~a" tmp))
            (set! net-list (genre-list "kernel/drivers/net"))
            (set! ide-list (genre-list "kernel/drivers/ide"))
            (set! scsi-list (append (genre-list "kernel/drivers/scsi")
                                    (genre-list "kernel/drivers/block")))
            (set! pcmcia-list (genre-list "pcmcia"))
            (set! sound-list (genre-list "kernel/drivers/sound"))
            (set! alsa-list (genre-list "alsa"))
            (set! usable-tree
              (map (lambda (x)
                     (cons x
                       (easy-sort string<?
                         (map symbol->string
                           (genre-list 
                             (if (or (equal? x "misc") (equal? x "pcmcia"))
                                 x
                                 (format #f "kernel/drivers/~a" x)))))))
                   (list "block" "bluetooth" "cdrom" "char" "hotplug" "i2c" "ide" "ieee1394"
                         "input" "isdn" "media" "message" "misc" "mtd" "net" "parport" "pcmcia"
                         "pnp" "scsi" "sensors" "sound" "telephony" "usb" "video")))
            
            (set! usable-table
              (apply append
                (map (lambda (l)
                       (let ((x (car l)))
                         (map (lambda (y) (cons y x))
                            (map string->symbol (cdr l)))))
                     usable-tree)))
            ))
        
        (cond
          ((eq? #f pci-table)
            (sh-log "Cannot open pcitable.\n")
            #f)
          (else
            (set! device-table
              (let rep ((L (parse-awk #\tab #t "/proc/bus/pci/devices")))
                (cond ((null? L) L)
                      ((assoc (cadr (car L)) pci-table) =>
                          (lambda (x) (cons (cons (string->symbol (caddr x)) x)
                                            (rep (cdr L)))))
                      (else (rep (cdr L))))))
            (case context
              ((inst1 recovery live)
                (probe-pcmcia)
                (probe-usb)))
            (set! the-table (initial-table))))
        #t)))
  
  (define (get-mapped m)
    (let ((r (assoc m map-table)))
      (if (eq? #f r)
        m
        (cadr r))))
  
  ;;
  ;; methods
  ;;
  (define (init x)
    (set! context x)
    (let ((r (parse-awk #\space #t "/usr/lib/ogl/lib/compat")))
      (cond
        ((eq? #f r)
          (inst-log "could not find compat table.\n"))
        (else
          (set! compat-table
                (map (lambda (x) (map string->symbol x))
                  (filter (lambda (x) (not (null? x)))
                          r))))))
    (flash))
  
  (define (insmod m o)
    (let ((x (assoc m the-table)))
      (cond
        ;((and (not (eq? x #f))
        ;      (set-include? (car x) '(req inst/set inst/not)))
        ;  #f)
        ((= 0 (sh "modprobe ~a ~a" (map-it m) o))
          (update-table m 'inst/set)
          (update-mid-table m o)
          (if (not (eq? #f pcmcia-chip))
            (sh "cardmgr -oq"))
          (cond
            ((and (not (equal? m 'snd-mixer-oss))
                  (not (equal? m 'snd-pcm-oss))
                  (equal? 'alsa (mod->genre m)))
              (insmod 'snd-mixer-oss "")
              (insmod 'snd-pcm-oss ""))
            ((equal? 'scsi (mod->genre m))
              (sh "modprobe sr_mod")
              (sh "modprobe sg")))
          #t)
        ((assoc m usable-table)
          (if (and (equal? context 'inst1)
                   (not (unix-access? "/cdboot" '(read))))
              (update-table m 'req)
              (update-table m 'un/not))
          #f)
        (else
          (update-table m 'un/set)
          #f))))
  
  (define (setmod m o s)
    (update-table m s)
    (update-mid-table m o)
    #t)
  
  (define (rmmod m)
    (let ((x (assoc m the-table)))
      (cond
        ((eq? x #f)
          #f)
        (else
          (update-table m 'un/not)
          (= 0 (sh "rmmod ~a" m))))))
  
  (define (info m)
    (let ((x (assoc m device-table)))
      (cond
        ((eq? x #f) "")
        ((string=? "*" (list-ref x 4)) "")
        (else (list-ref x 4)))))
  
  (define (get-module-tree)
    (let ((d (unix-ls "../root/mod")))
      (apply append
        (map (lambda (x) (unix-ls (string-append "../root/mod/" x)))
             (remove-if (lambda (x) (string=? "modules.dep" x))
                             d)))))
  ; ƥȤǰۤʤ褦˸ѹ롣
  ; unix-ls  unbound ?
  
  (define (make-etc-1 f)
    (fprintf f
      (apply string-append
        (append
          (list "# /etc/modules: kernel modules to load at boot time.\n"
                "#\n"
                "# This file should contain the names of kernel modules that are\n"
                "# to be loaded at boot time, one per line.  Comments begin with\n"
                "# a `#', and everything on the line after them are ignored.\n"
                "# An entry named `auto' will cause the system to start kerneld immediately.\n"
                "# Kerneld then loads modules on demand. `noauto' disables kerneld completely.\n"
                "\n"
                "#auto\n")
          (map (lambda (x)
                 (cond
                   ((set-include? (car x) pcmcia-list)
                     (inst-log "detected pcmcia module ~s\n" (car x))
                     "")
                   (else
                     (case (cadr x)
                       ((inst/set un/set)
                         (let* ((m (car x))
                                (r0 (assoc m mid-table)))
                           (cond
                             ((eq? #f r0)
                               "")
                             ((equal? "" (caddr r0))
                               (format #f "~a\n" (get-mapped m)))
                             (else
                               (format #f "~a ~a\n" (get-mapped m) (caddr r0))))))
                       (else
                         "")))))
               the-table)
          (list "\n")))))
  
  (define (make-etc-2 f)
    (fprintf f
      (apply string-append
        (map (lambda (x)
               (let ((m (car x)))
                 (case (cadr x)
                   ((inst/set un/set)
                     (cond
                       ((set-include? m scsi-list)
                         (format #f "alias scsi_hostadapter ~a\n" m))
                       ((set-include? m ide-list)
                         (format #f "alias scsi_hostadapter ataraid\nalias scsi_hostadapter ~a\n" m))
                       ((set-include? m alsa-list)
                         (string-append
                             "alias char-major-116 snd\n"
                             "alias char-major-14 soundcore\n"
                             "alias sound-slot-0 snd-card-0\n"
                             "alias sound-service-0-0 snd-mixer-oss\n"
                             "alias sound-service-0-1 snd-seq-oss\n"
                             "alias sound-service-0-3 snd-pcm-oss\n"
                             "alias sound-service-0-8 snd-seq-oss\n"
                             "alias sound-service-0-12 snd-pcm-oss\n"
                             "alias /dev/sound snd-pcm-oss\n"
                             "options snd snd_major=116 snd_cards_limit=1 snd_device_mode=0660 snd_device_gid=29 snd_device_uid=0\n"))
                       (else "")))
                   (else
                     ""))))
             the-table))))
  
  (define (make-etc-3 f)
    (fprintf f
      (string-append
        "# Defaults for pcmcia (sourced by /etc/init.d/pcmcia)\n"
        "PCMCIA="
        (if (eq? #f (assoc 'pcmcia_core the-table)) "no" "yes")
        "\nPCIC="
        (cond
          ((not (eq? #f (assoc 'yenta_socket the-table)))
            "yenta_socket")
          ((not (eq? #f (assoc 'i82365 the-table)))
            "i82365")
          (else
            ""))
        "\nPCIC_OPTS=\nCORE_OPTS=\nCARDMGR_OPTS=\n")))
  
  (define (write-mod)
    (fprintf "/var/lib/ogl/etc/mod"
      (apply string-append
        (map (lambda (x)
               (case (cadr x)
                 ((inst/set un/set)
                   (let ((r0 (assoc (car x) mid-table)))
                     (cond
                       ((eq? #f r0)
                         "")
                       (else
                         (format #f "~a\t~a\t~a\n"
                                 (car r0) (cadr r0) (caddr r0))))))
                 (else
                   "")))
             the-table))))
  
  (define (write-mod-map)
    (if (null? map-table)
      #t
      (ttable-write "/var/lib/ogl/etc/mod-map" map-table)))
      
;      (fprintf "/var/lib/ogl/etc/mod-map"
;        (apply string-append
;          (apply append
;            (map (lambda (x)
;                   (append (map (lambda (y) (format #f "~a " y))
;                                x)
;                           (list "\n")))
;                 map-table))))
  
  (define (commit)
    (and (write-mod)
         (write-mod-map)
         (case context
           ((inst1 recovery)
             #f)
           ((inst2)
             (= 0 (sh "chroot /target update-modules")))
           (else
             (= 0 (sh "update-modules"))))))
  
  (define (load-it)
    (and (let ((r (parse-awk #\tab #t "/var/lib/ogl/etc/mod")))
           (cond
             ((eq? r #f)
               (inst-log "could not load etc/mod\n")
               (set! mid-table '()))
             (else
               (set! mid-table
                     (map (lambda (x)
                            (list (string->symbol (car x))
                                  (string->symbol (cadr x))
                                  (if (null? (cddr x)) "" (caddr x))))
                          r))))
           (flash)
           #t)
          (let ((r1 (ttable-read "/var/lib/ogl/etc/mod-map" '((sym sym)))))
            (cond
              ((list? r1)
                (inst-log "obj-mod : mod-map read. ~s\n" r1)
                (set! map-table r1)))
            #t)))
            
  (define (device-name m)
    (let ((r (assoc m device-table)))
      (if (eq? #f r)
        (list "---" "---")
        (list (list-ref r 5) (list-ref r 6)))))
  
  (define (need-bl-driver?)
    (ormap (lambda (x)
             (case (car x)
               ((ide-floppy ide-tape)
                 #f)
               (else
                 (case (mod->genre (car x))
                   ((scsi ide)
                     #t)
                   (else
                     #f)))))
           the-table))
  
  (define (compatible)
    (inst-log "compat : ~a\n" compat-table)
    (filter (lambda (x)
              (ormap (lambda (y)
                       (and (equal? (car y) (car x))
                            (case context
                              ((inst1 recovery)
                                #t)
                              (else
                                (case (cadr y)
                                  ((un/set inst/not inst/set)
                                    #t)
                                  (else
                                    #f))))))
                     the-table))
            compat-table))
  
  (define (select s d)
    (let ((r (filter (lambda (x) (equal? s (car x)))
                     compat-table)))
      (cond
        ((not (= 1 (length r)))
          #t)
        (else
          (let* ((l (car r))
                 (l' (filter (lambda (x) (list? x))
                       (map (lambda (x)
                              (cond
                                ((not (ormap (lambda (y)
                                               (and (equal? (car y) x)
                                                    (case (cadr y)
                                                      ((inst/not inst/set)
                                                        #t)
                                                      (else
                                                        #f))))
                                             the-table))
                                  #t)
                                ((eq? #f (rmmod x))
                                  (list x (mod->genre x)))
                                (else
                                  #t)))
                            l))))
            (flash)
            (set! map-table
              (cons (list s d)
                (filter (lambda (x)
                          (not (equal? (car x) s)))
                        map-table)))
            (setmod s "" 'un/set)
            (if (null? l')
              #t
              l'))))))
  
  (define (map-it m)
    (let ((r (assoc m map-table)))
      (cond
        ((eq? #f r)
          m)
        (else
          (cadr r)))))
  
  (define (set-first m)
    (let ((r (assoc m the-table)))
      (cond
        ((eq? #f r)
          #f)
        (else
          (set! the-table
            (cons r (filter (lambda (x) (not (equal? x r)))
                            the-table)))))))
  
  (define (desc m)
    (let* ((n  (device-name m))
           (s0 (car n))
           (s1 (cadr n)))
      (sprintf (_ "%-14s %-18s : %-12s ")
        (string-cut (symbol->string (get-mapped m)) 14)
        (string-cut s1 18) (string-cut s0 12))))
  
  (define (setmod-init)
    (if (not setmod-init?)
      (set! the-table 
        (map (lambda (x)
               (case (cadr x)
                 ((un/not)
                   (list (car x) 'un/set))
                 (else
                   x)))
             the-table)))
    (set! setmod-init? #t)
    #t)
  
  (define (which-fd m)
    (let ((g (mod->genre m)))
      (case g
        ((net fs) 1)
        (else 2))))
  
  (define (extra-probe)
    (sh "modprobe ext2")
    (sh "modprobe isofs")
    (sh "modprobe vfat")
    (sh "modprobe ipv6")
    (sh "modprobe nfs")
    (sh "modprobe sr_mod")
    (sh "modprobe sg")
    #t)

  (define (probe-usbmod)
    (or (insmod 'usb-uhci "")
        (insmod 'usb-ohci "")))
  
  (define (need-force-dma?)
    (let ((r (parse-awk #\tab #t "/proc/bus/pci/devices")))
      (cond
        ((eq? #f r)
          #f)
        (else
          (let* ((p (map cadr r))
                 (crusoe-ali? (and (set-include? "12790395" p) ; LongRun North
                                   (set-include? "10b95229" p))))
            crusoe-ali?)))))
  
  (define (sound-target? m)
    (case (mod->genre m)
      ((sound alsa)
        (let ((s (symbol->string m)))
          (cond
            ((and (< 8 (string-length s))
                  (equal? "snd-" (substring s 0 4))
                  (not (equal? "snd-card" (substring s 0 8))))
               #f)
            (else
              #t))))
      (else
        #f)))
  
  (make-object
    'commit            commit
    'compatible        compatible
    'conf              (lambda () (list (list "/etc/modules" make-etc-1 #t)
                                        (list "/etc/modutils/ogl" make-etc-2 #f)
                                        (list "/etc/default/pcmcia" make-etc-3 #t)))
    'desc              desc
    'device-table      (lambda () device-table)
    'extra-probe       extra-probe
    'need-force-dma?   need-force-dma?
    'genre             mod->genre
    'genre-list        genre-list
    'get-module-tree   get-module-tree
    'get-map           (lambda () map-table)
    'get-mapped        get-mapped
    'info              info
    'init              init
    'insmod            insmod
    'inst-table        installed
    'list              (lambda () (list net-list scsi-list))
    'load              load-it
    'map               map-it
    'mid-conf          (lambda () (list "/var/lib/ogl/etc/mod" "/var/lib/ogl/etc/mod-map"))
    'mid-table         (lambda () mid-table)
    'need-bl-driver?   need-bl-driver?
    'pci-table         (lambda () pci-table)
    'pcmcia-table      (lambda () pcmcia-table)
    'probe-usbmod      probe-usbmod
    'rmmod             rmmod
    'select            select
    'set-first         set-first
    'setmod            setmod
    'setmod-init       setmod-init
    'sound-target?     sound-target?
    'table             (lambda () the-table)
    'usable-table      (lambda () usable-table)
    'usable-tree       (lambda () usable-tree)
    'usb-table         (lambda () usb-table)
    'device-name       device-name
    'pcmcia-list       (lambda () pcmcia-list)
    'which-fd          which-fd
    ))
