;;
;;     Network Object 
;;
;;                          (C)2001,2002 Omoikane Inc.

(define (configure-network)
  ;;
  ;;  variables
  ;;
  (define if-list '())
  (define if-hw '())
  (define hw-if '())
  (define table '())
  (define ns-list '())
  (define gw "")            ; default gw
  (define provides-ns '())	; list of DHCP interface which provides nameserver
  (define name '("" ""))		; host/domain name
  (define context 'admin)
  (define ipv6 '())

  ; libs
  (define add4pat
    (let* ((a1 "\\([0-9]\\|[1-9][0-9]\\|1[0-9][0-9]\\|2[0-4][0-9]\\|25[0-5]\\)"))
      (format #f "~a\\.~a\\.~a\\.~a" a1 a1 a1 a1)))
  
  (define add6pat
    (let* ((p1 "[0-9,a-f,A-F]")
           (p2 (format #f "\\(~a\\|~a~a\\|~a~a~a\\|~a~a~a~a\\)" p1 p1 p1 p1 p1 p1 p1 p1 p1 p1))
           (p3 (format #f "\\(:~a\\)" p2))
           (p4 (format #f "~a~a*" p2 p3)))
      (format #f "\\(~a\\|::\\|::~a\\|~a::\\|~a::~a\\)" p4 p4 p4 p4 p4)))
  
  (define addpr6pat
    (format #f "~a/[0-9]\\+" add6pat))

  (define (match-add4? a)
    (let ((x (string-match a add4pat)))
      (and (= 1 (length x))
           (= (string-length a) (string-length (car x))))))
  
  (define (match-add6? a)
    (let ((x (string-match a add6pat)))
      (and (= 1 (length x))
           (= (string-length a) (string-length (car x))))))
  
  (define (match-addpr6? a)
    (let ((x (string-match a addpr6pat)))
      (and (= 1 (length x))
           (= (string-length a) (string-length (car x))))))
  
  (define (get-hw n)
    (cond
      ((< (length if-list) (1+ n))
        "")
      (else
        (car (list-ref if-list n)))))
  
  (define (if-set I ip mask gw bc)
    (and (= 0 (sh "ifconfig eth~a down" I))
         (= 0 (sh "ifconfig eth~a ~a netmask ~a ~a " I ip mask
                  (if (string=? "" bc) bc (format #f "broadcast ~a" bc))))))

  (define (make-etc-resolv p)
    (fprintf p (resolv.conf)))
  
  (define (make-etc-interfaces p)
    (define str-head
      (string-append "# /etc/network/interfaces -- configuration file for ifup(8), ifdown(8)\n\n"
                     "# eth0 example(static)\n"
                     "#iface eth0 inet static\n"
                     "#\taddress 192.168.0.1\n"
                     "#\tnetmask 255.255.255.0\n"
                     "#\tnetwork 192.168.0.0\n"
                     "#\tbroadcast 192.168.0.255\n"
                     "#\tgateway 192.168.0.1\n\n"
                     "# eth0 example(dhcp)\n"
                     "#iface eth0 inet dhcp\n"
                     "#\thostname name\n\n"))
    (define (str-net-dhcp i)
      (string-append "\n# The first network card - this entry was created during the Debian installation\n"
                     (format #f "iface eth~a inet dhcp\n" i)
                     (format #f "\thostname ~a\n" (car name))))
    (define (str-net-static l)
      (string-append "\n# The first network card - this entry was created during the Debian installation\n"
            "# (network, broadcast and gateway are optional)\n"
            (format #f "iface eth~a inet static\n" (car l))
            (format #f "\taddress ~a\n" (caddr l))
            (format #f "\tnetmask ~a\n" (list-ref l 3))
            (format #f "\tnetwork ~a\n" (iutil-ip-and (caddr l) (list-ref l 3)))
            (if (not (equal? "" (list-ref l 5)))
              (format #f "\tbroadcast ~a\n" (list-ref l 5))
              "")
            (if (not (equal? "" (list-ref l 4)))
              (format #f "\tgateway ~a\n" (list-ref l 4))
              "")))
    (fprintf p
      (apply string-append
        (append
          (list str-head)
          (list "\nauto lo")
          (map (lambda (x)
                 (cond
                   ((or (null? (cdr x))
                        (equal? "static" (cadr x))
                        (equal? "dhcp" (cadr x)))
                     (format #f " eth~a" (cdr (assoc (car x) hw-if))))
                   (else
                     "")))
               if-list)
          (list "\n\n# The loopback interface\n"
                "iface lo inet loopback\n")
          (map (lambda (x)
                 (cond
                   ((null? (cdr x))
                     "")
                   ((equal? "static" (cadr x))
                     (str-net-static x))
                   ((equal? "dhcp" (cadr x))
                     (str-net-dhcp (car x)))
                   (else
                     "")))
               (list-active))))))
  
  (define (make-etc-pcmcia p)
    (define (write x)
      (define (dns l i)
        (cond
          ((null? l) '())
          (else
            (cons 
              (format #f "\tDNS_~a=\"~a\"\n" i (car l))
              (dns (cdr l) (1+ i))))))
        
      (let ((t (cadr x)))
        (cond
         ((equal? t "dhcp")
           "*,*,*,*)\n\tPUMP=y\n ;;\n")
         ((equal? t "static")
           (string-append
             "*,*,*,*)\n"
             "\tPUMP=\"n\"\n"
             "\t# Host's IP address, netmask, network address, broadcast address\n"
             (format #f "\tIPADDR=\"~a\"\n" (caddr x))
             (format #f "\tNETMASK=\"~a\"\n" (list-ref x 3))
             (format #f "\tNETWORK=\"~a\"\n" (iutil-ip-and (caddr x) (list-ref x 3)))
             (format #f "\tBROADCAST=\"~a\"\n" (list-ref x 5))
             (format #f "\tGATEWAY=\"~a\"\n" (list-ref x 4))
             (format #f "\tDOMAIN=\"~a\"\n" (cadr name))
             (apply string-append (dns ns-list 1))
             " ;;\n"))
         (else "\n"))))
    (fprintf p
      (string-append
       "# Network adapter configuration\n#\n# Automatically generated during the Debian installation\n#\n"
       "# The address format is \"scheme,socket,instance,hwaddr\".\n"
       "#\n"
       "case \"$ADDRESS\" in\n"
       (apply string-append (map write if-list))
       "esac\n")))
  
  (define (make-etc-hostname p)
    (fprintf p (string-append (car name) "\n")))
  
  (define (make-etc-hosts p)
    (let ((s-list (filter (lambda (x) (and (< 4 (length x))
                                           (equal? (cadr x) "static")))
                          if-list)))
      (fprintf p
        (cond
          ((null? s-list)
            (format #f "127.0.0.1\tlocalhost\t~a\n" (car name)))
          (else
            (apply string-append
              (cons "127.0.0.1\tlocalhost\n"
                (case (length s-list)
                  ((1)
                    (list
                      (let* ((x (car s-list))
                             (n (fqdn-split (gethostbyaddr (caddr x)))))
                        (cond
                          ((and (= 2 (length n))
                                (not (equal? "" (car n)))
                                (not (equal? "" (cadr n))))
                            (format #f "~a\t~a.~a\t~a\n" (caddr x) (car n) (cadr n) (car n)))
                          (else
                            (format #f "~a\t~a\t~a\n" (caddr x)
                              (if (equal? "" (cadr name))
                                (car name)
                                (format #f "~a.~a" (car name) (cadr name)))
                              (car name)))))))
                  (else
                    (map (lambda (x)
                           (let ((n (fqdn-split (gethostbyaddr (caddr x)))))
                             (cond
                               ((and (= 2 (length n))
                                     (not (equal? "" (car n)))
                                     (not (equal? "" (cadr n))))
                                 (format #f "~a\t~a.~a\t~a\n" (caddr x) (car n) (cadr n) (car n)))
                               (else
                                 (inst-log "net-obj : static list entry ~s not written to interfaces.\n" x)
                                 ""))))
                         s-list))))))))))
  
  (define (make-etc-networks p)
    (let ((s-list (filter (lambda (x) (and (< 2 (length x))
                                           (equal? (cadr x) "static")))
                          if-list)))
      (fprintf p
        (cond
          ((null? s-list) "")
          ((= 1 (length s-list))
            (let ((x (car s-list)))
              (format #f "localnet ~a\n" (iutil-ip-and (caddr x) (list-ref x 3)))))
          (else
            (apply string-append
              (let rep ((i 0) (l s-list))
                (cond
                  ((null? l) l)
                  (else
                    (cons
                      (format #f "localnet~a ~a\n"
                                 i
                                 (iutil-ip-and (caddr (car l)) (list-ref (car l) 3)))
                      (rep (1+ i) (cdr l))))))))))))
  
  (define (make-v6 f)
    (inst-log "writing ~a\n" f)
    (and (fprintf f
           (apply string-append
             (append
               (list "#!/bin/sh\n"
                     "# this script was generated by ogl-tools\n\n"
                     "case \"$1\" in\n"
                     "    start)\n")
               (map (lambda (x)
                      (case (string->symbol (car x))
                        ((static)
                          (format #f "        /sbin/ifconfig eth~a add ~a\n"
                                     (cdr (assoc (cadr x) hw-if)) (caddr x)))
                        ((gw)
                          (format #f "        /sbin/route -A inet6 add default gw ~a\n" (cadr x)))
                        ((tunnel)
                          (string-append
                            (format #f "        /sbin/ip tunnel add sit~a mode sit ttl 64 remote ~a local ~a\n"
                              (cadr x) (caddr x) (list-ref x 3))
                            (format #f "        /sbin/ifconfig sit~a up\n" (cadr x))
                            (format #f "        /sbin/route -A inet6 add ~a dev sit1\n" (list-ref x 4))))
                        (else
                          "")))
                    ipv6)
               (list "        ;;\n"
                     "    stop)\n")
               (map (lambda (x)
                      (case (string->symbol (car x))
                        ((static)
                          (format #f "        /sbin/ifconfig eth~a del ~a\n"
                                     (cdr (assoc (cadr x) hw-if)) (caddr x)))
                        ((tunnel)
                          (string-append
                            (format #f "        /sbin/ifconfig sit~a down\n" (cadr x))
                            (format #f "        /sbin/ip tunnel del sit~a\n" (cadr x))))
                        (else
                          "")))
                    ipv6)
               (list ;"        /sbin/route -A inet6 del default gw\n"
                     "        ;;\n"
                     "    restart)\n"
                     "        ;;\n"
                     "    *)\n"
                     "        echo \"Usage : /etc/init.d/ipv6 {start|stop|reload}\"\n"
                     "        ;;\n"
                     "esac\n"))))
         (begin (inst-log "written\n") #t)
         (= 0 (sh "chmod +x ~a" f))
         (if (equal? "/etc/init.d/ipv6" f)
           (= 0 (sh "update-rc.d ipv6 defaults"))
           #t)))
  
  ;;
  ;; methods
  ;;
  (define (init c)
    (set! context c)
    (set! if-hw 
          (map (lambda (x)
                 (cons (string->number (substring x 3 4))
                       (let ((tmp (unix-tempname)))
                         (sh "LANG= ifconfig ~a >~a" x tmp)
                         (let ((r (parse-awk #\space #t tmp)))
                           (sh "rm ~a" tmp)
                           (cond
                             ((or (eq? r #f)
                                  (< (length (car r)) 5))
                                "ff:ff:ff:ff:ff:ff")
                             (else
                               (list-ref (car r) 4)))))))
               (filter (lambda (x)
                          (string=? "eth" (substring x 0 3)))
                       (map car (cddr (parse-awk #\space #t "/proc/net/dev"))))))
    (set! hw-if (map (lambda (x) (cons (cdr x) (car x))) if-hw))
    (set! if-list (map (lambda (x) (list (cdr x) "none")) if-hw))
    (case c
      ((inst1 inst2 recovery)
        (sh "ifconfig lo 127.0.0.1 netmask 255.0.0.0 up")))
    #t)
  
  (define (set-if! I lis)
    (let ((r (assoc (cdr (assoc I if-hw)) if-list)))
      (cond
       ((eq? r #f) #f)
       (else
         (set-cdr! r lis)
         #t))))
  
  (define (apply-gw)
    (let ((r (= 0 (sh "route add default gw ~a" gw))))
      (case context
        ((inst1 inst2 recovery) #t)
        (else r))))
  
  (define (set-gw x)
    (set! gw x)
    #t)
  
  (define (set-apply-gw x)
    (cond
      ((< (length x) 5)
        (inst-log "net-obj/set-apply-gw : invalid argument ~s\n" x)
        #f)
      (else
        (set-gw (list-ref x 4))
        (apply-gw))))
  
  (define (apply-static I ip mask gw bc)
    (cond
      ((if-set I ip mask gw bc)
        (set-if! I (list "static" ip mask gw bc)))
      (else
        #f)))
  
  (define (apply-dhcp I)
    (let ((r (dhcp-get I)))
      (inst-log "dhcp result : ~s\n" r)
      (cond
        ((string? r)
          (inst-log "dhcp-failed. reason:~a" r)
          r)
        ((not (if-set I (car r) (cadr r) (list-ref r 4) (caddr r)))
          (_ "Failed ifconfig."))
        (else
          (set-if! I (list "dhcp" (car r) (cadr r) (list-ref r 4)))
          (let ((n (vector->list (list-ref r 5))))
             (cond
               ((< 0 (length n))
                 (set! provides-ns (list-set+ equal? provides-ns n))
                 (set! ns-list (list-set+ equal? ns-list n))))
             #t)))))
  
  (define (apply-none I)
    (cond
      ((= 0 (sh "ifconfig eth~a down" I))
        (set-if! I (list "none"))
        (set! provides-ns (list-set- eq? provides-ns (list I)))
        #t)
      (else
        #f)))
  
  (define (resolv.conf)
    (string-append
      (if (not (string=? "" (cadr name)))
          (format #f "search ~a\n" (cadr name))
          "")
      (apply string-append
        (map (lambda (x) (format #f "nameserver ~a\n" x))
            ns-list))))
  
  (define (apply-ns)
    (sh "rm -f /etc/resolv.conf")
    (fprintf "/etc/resolv.conf" (resolv.conf))
    #t)
  
  (define (add-ns x)
    (cond
      ((set-include? x ns-list)
        #f)
      (else
        (set! ns-list (cons x ns-list))
        #t)))
  
  (define (elim-ns x)
    (cond
      ((set-include? x ns-list)
        (set! ns-list (list-set- equal? ns-list (list x)))
        #t)
      (else
        #f)))
  
  (define (unreachable-ns)
    (filter
      (lambda (x)
        (let ((r (unix-check-dns x)))
          (if (not (= 0 r))
            (inst-log "DNS ~a does not respond(~a).\n" x r))
          (not (= 0 r))))
      ns-list))
  
  (define (resolv h)
    (let ((d (cadr name)))
      (cond
        ((match-add4? h) h)
        ((and (= (string-length h) (char-search #\. h 0))
              (not (string=? "" d)))
          (gethostbyname (string-append h "." d)))
        (else
          (gethostbyname h)))))
  
  (define (candidate-name)
    (filter
      (lambda (y) (and (= 2 (length y))
                       (not (equal? "" (car y)))))
      (map (lambda (x) (fqdn-split (gethostbyaddr (caddr x))))
           (list-active))))
  
  (define (commit)
    ;(inst-log "provided : ~a ~a\n" ns-list provides-ns)
    (and (fprintf "/var/lib/ogl/etc/nameserver"
           (apply string-append
             (map (lambda (x) (format #f "~a\n" x))
               (list-set- equal? ns-list provides-ns))))
         ;(inst-log "/var/lib/ogl/etc/nameserver created")
         (fprintf "/var/lib/ogl/etc/net"
           (apply string-append
             (cons
               (string-append (car name) " " (cadr name) "\n")
               (map (lambda (x)
                      (let ((x' (if (equal? "dhcp" (cadr x))
                                    (list (car x) (cadr x) "0.0.0.0" "0.0.0.0" (list-ref x 4))
                                    x)))
                        (string-append (list-foldstring " " x') "\n")))
                    if-list))))
         (fprintf "/var/lib/ogl/etc/gw" gw)
         ;(inst-log "/var/lib/ogl/etc/net created")
         (ttable-write "/var/lib/ogl/etc/ipv6" ipv6)))
  
  (define (load-it)
    (let* ((saved '())
           (r1 (let ((l (parse-awk #\space #t "/var/lib/ogl/etc/net")))
                 (cond
                   ((eq? #f l)
                     (inst-log "netobj : not conffile net(1).\n")
                     #f)
                   ((< (length l) 1)
                     (inst-log "netobj : illegal format conffile net(1).\n")
                     #f)
                   ((and (< 1 (length l))
                         (ormap (lambda (x) (< (length x) 2))
                                (cdr l)))
                     (inst-log "Warning / netobj : illegal format conffile(2).\n")
                     #f)
                   (else
                     (set! name (cond ((= 2 (length (car l)))
                                        (list (car (car l)) (cadr (car l))))
                                      (else
                                        (list "" ""))))
                     (set! saved (cdr l))
                     #t))))
           (r2 (let ((r (parse-awk #\space #t "/var/lib/ogl/etc/nameserver")))
                 (cond
                   ((or (eq? #f r) (null? r))
                     #f)
                   (else
                     (set! ns-list (car r))
                     #t))))
           (r3 (let ((r (parse-awk #\space #t "/var/lib/ogl/etc/gw")))
                 (cond
                   ((or (eq? #f r) (null? r) (null? (car r)))
                     #f)
                   (else
                     (set! gw (caar r))))))
           (r4 (ttable-read "/var/lib/ogl/etc/ipv6"
                 '(("auto" str)
                   ("static" str str)
                   ("gw" str)
                   ("tunnel" int str str str)))))
      (cond
        ((not (or r1 r2))
          'non-exist)
        ((not r1)
          'success)
        (else
          (map (lambda (x)
                 (let ((r (assoc (car x) if-list)))
                   (cond
                     ((not (eq? #f r))
                       (set-cdr! r (cdr x))
                       (let ((r (assoc (car x) hw-if)))
                         (if (not (eq? #f r))
                             (let ((i (cdr r)))
                               (cond
                                 ((or (equal? 'inst2 context)
                                      (null? (cdr x)))         ; do nothing
                                   #t)
                                 ((equal? "static" (cadr x))
                                   (apply apply-static (cons i (cddr x))))
                                 ((equal? "dhcp" (cadr x))
                                   (apply-dhcp i)))))
                         #t)))
                   #t))
               saved)
          (if (not (eq? #f r4))
            (set! ipv6
              (filter (lambda (x)
                        (cond
                          ((or (equal? "gw" (car x)) (equal? "tunnel" (car x)))
                            #t)
                          ((eq? #f (assoc (cadr x) hw-if))
                            #f)
                          (else
                            #t)))
                      r4)))
          (cond
            ((< (length saved) (length if-list))	; if $B$,A}$($?(B
              'if-fewer)
            ((< (length if-list) (length saved))	; if $B$,8:$C$?(B
              'if-left)
            (else
              (if (not (equal? "" gw)) (apply-gw))
              'success))))))
  
  (define (set-name n d)
    (cond
      ((and (string? n) (string? d))
        (set! name (list n d)))
      (else
        #f)))
  
  (define (get-if n)
    (let ((x (assoc n if-hw)))
      (cond
        ((eq? #f x) x)
        (else
          (let ((y (assoc (cdr x) if-list)))
            (cond
              ((eq? #f y) y)
              (else (cdr y))))))))
  
  (define (nonet?)
    (null? (filter (lambda (x) (not (equal? "none" (cadr x))))
             if-list)))
  
  (define (sort-if)
    (filter (lambda (y) (not (eq? #f y)))
      (map (lambda (x)
             (let ((r (assoc (cdr x) if-list)))
               (cond
                 ((eq? #f r) r)
                 (else
                   (cons (car x) (cdr r))))))
           if-hw)))
  
  (define (list-active)
    (filter (lambda (x) (or (equal? (cadr x) "dhcp")
                            (equal? (cadr x) "static")))
      (sort-if)))
  
  
  ; IPv6 methods
  (define (get-type6 n)
    (let ((r (filter (lambda (x) (equal? (cadr x) (get-hw n)))
                     ipv6)))
      (cond
        ((null? r) 'none)
        ((equal? "auto" (caar r)) 'auto)
        (else 'static))))
  
  (define (get-static6 n)
    (let ((r (filter (lambda (x) (equal? (cadr x) (get-hw n)))
                     ipv6)))
      (cond
        ((null? r) "")
        ((equal? "static" (caar r))
          (caddr (car r)))
        (else ""))))
  
  (define (update-if6 n l)
    (let ((h (get-hw n)))
      (set! ipv6
        (append (filter (lambda (x) (not (equal? (cadr x) h)))
                        ipv6)
                (list l)))
      #t))
  
  (define (update-none6 n)
    (set! ipv6
      (filter
        (lambda (x) (not (equal? (cadr x) (get-hw n))))
        ipv6))
    #t)
  
  (define (update-auto6 n)
    (update-if6 n (list "auto" (get-hw n))))
  
  (define (update-static6 n a)
    (cond
      ((not (match-addpr6? a))
        (inst-log "update-static6 : incorrect v6 address ~a\n" a)
        #f)
      (else
        (update-if6 n (list "static" (get-hw n) a)))))
  
  (define (set-gw6 a)
    (cond
      ((not (match-add6? a))
        (inst-log "set-gw6 : incorrect v6 address ~a\n" a)
        #f)
      (else
        (set! ipv6
          (append (filter (lambda (x) (not (equal? (car x) "gw")))
                          ipv6)
                  (list (list "gw" a))))
        #t)))
  
  (define (get-gw6)
    (let ((r1 (filter (lambda (x) (equal? (car x) "gw"))
                      ipv6))
          (r2 (filter (lambda (x) (and (equal? (car x) "tunnel")
                                       (equal? (list-ref x 4) "::/0")))
                      ipv6)))
      (cond
        ((not (null? r1))
          (cadr (car r1)))
        ((not (null? r2))
          (format #f "sit~a" (cadr (car r2))))
        (else
          ""))))
  
  (define (update-tunnel n r l a)
    (cond
      ((not (match-addpr6? a))
        (inst-log "update-tunnel : incorrect v6 address prefix ~a\n" a)
        #f)
      (else
        (set! ipv6
          (append (filter (lambda (x) (not (and (equal? (car x) "tunnel")
                                                (equal? (cadr x) n))))
                          ipv6)
                  (list (list "tunnel" n r l a))))
        #t)))
  
  (define (list-tunnel)
    (easy-sort (lambda (x y) (< (car x) (car y)))
      (map cdr
        (filter (lambda (x) (equal? "tunnel" (car x)))
                ipv6))))
  
  (define (radv? n)
    (let ((r (rtsol (format #f "eth~a" n))))
      (case r
        ((0 1) #t)
        (else
          (inst-log "net obj : rtsol returns abornal value ~a.\n" r)))
      (= 0 r)))

  (define (apply-v6)
    (let ((tmp (unix-tempname)))
      (and (make-v6 tmp)
           (begin (sh "~a stop" tmp) #t)
           (= 0 (sh "~a start" tmp))
           (= 0 (sh "rm ~a" tmp)))))
  
  (make-object
  ; for common
    'conf            (lambda () (list
                                  (list "/etc/resolv.conf" make-etc-resolv #t)
                                  (list "/etc/network/interfaces" make-etc-interfaces #t)
                                  (list "/etc/pcmcia/network.opts" make-etc-pcmcia #t)
                                  (list "/etc/hostname" make-etc-hostname #t)
                                  (list "/etc/hosts" make-etc-hosts #t)
                                  (list "/etc/networks" make-etc-networks #t)
                                  (list "/etc/init.d/ipv6" make-v6 #f)))
    'init            init
    'commit          commit
    'load            load-it
    'match-add4?     match-add4?
    'mid-conf        (lambda () (list "/var/lib/ogl/etc/net" "/var/lib/ogl/etc/nameserver"
                                      "/var/lib/ogl/etc/gateway" "/var/lib/ogl/etc/ipv6"))
    
  ; for interface
    'apply-static    apply-static
    'apply-dhcp      apply-dhcp
    'apply-none      apply-none
    'list-if         (lambda () if-list)
    'list-if-hw      (lambda () if-hw)
    'list-sort-if    sort-if
    'list-active     list-active
    'get-if          get-if
    'nonet?          nonet?
    
  ; for name
    'add-ns          add-ns
    'apply-ns        apply-ns
    'elim-ns         elim-ns
    'list-ns         (lambda () ns-list)
    'resolv          resolv
    'candidate-name  candidate-name
    'unreachable-ns  unreachable-ns
    
  ; for gateway
    'apply-gw        apply-gw
    'set-gw          set-gw
    'get-gw          (lambda () gw)
    'set-apply-gw    set-apply-gw
  
  ; for name/domain
    'list-name       (lambda () name)
    'set-name        set-name

  ; for IPv6
    'get-type6       get-type6
    'get-static6     get-static6
    'update-none6    update-none6
    'update-auto6    update-auto6
    'update-static6  update-static6
    'set-gw6         set-gw6
    'get-gw6         get-gw6
    'update-tunnel   update-tunnel
    'list-tunnel     list-tunnel
    'radv?           radv?
    'apply-v6        apply-v6
    ))

