
(define (newt-stage-title s)
  (let ((t (getfile "/var/lib/ogl/etc/title")))
    (snewt-title
      (if (eq? #f t)
	        "OGL Series distribution"
	        t)
	    s)))

(define (newt-fatal-error s)
  (snewt-message (_ "Fatal Error")
    (format #f (_ "~a\n\nInstaller could not go further steps, sorry. To search more system info, You can use background shells by pushing alt+F3/4/5/6.")
               s))
  (newt-fatal-error s))

(define (newt-module-select obj-mod)
  (let ((t (obj-mod 'compatible)))
    (cond
      ((null? t)
        #t)
      (else
        (let ((r (snewt-nmenu (_ "Driver Selection")
                   (string-append
                     (_ "Device(s) below has alternative driver. ")
                     (_ "Some device only works with specific driver. \n\n")
                     (_ "Do you select compatible driver?"))
                   50 4 4 6
                   (list->vector
                     (map (lambda (x) (obj-mod 'desc (car x)))
                          t))
                   (vector (_ "Ok") (_ "Select") (_ "Cancel")))))
          (case (car r)
            ((-1 3)
              #f)
            ((1)
              #t)
            (else
              (let* ((l (list-ref t (cdr r)))
                     (r1 (snewt-nmenu (_ "Driver Selection")
                           (string-append
                             (format #f (_ "Device : ~a\n") (obj-mod 'device-name (car l)))
                             (_ "Select drive from below.\n"))
                           50 4 4 6
                           (list->vector
                             (map (lambda (x) (format #f "  ~a  " x))
                                  l))
                           (vector (_ "Ok") (_ "Cancel")))))
                (case (car r1)
                  ((0 1)
                    (let ((r2 (obj-mod 'select (car l) (list-ref l (cdr r1)))))
                      (cond
                        ((and (list? r2)
                              (null? (filter (lambda (x) (not (equal? 'net (cadr x))))
                                             r2)))
                          (let ((r3 (snewt-choice (_ "Driver Selection")
                                      (apply string-append
                                        (append
                                          (list (_ "Drivers below could not be removed.\n\n"))
                                          (map (lambda (x) (format #f "    ~a\n" (car x)))
                                               r2)
                                          (list (_ "\nDo you stop network now?. If not, configuration will be valid after reboot."))))
                                      (_ "Stop Network") (_ "Leave as current"))))
                            (case r3
                              ((1)
                                (map (lambda (x) (sh "ifconfig eth~a down" x))
                                     '(0 1 2 3 4 5))
                                (map (lambda (x) (sh "rmmod ~a" (car x)))
                                     r2)))))))))
                (newt-module-select obj-mod)))))))))

(define (newt-ui-net inst? obj-net)
  (define title (_ "Network Setting"))
  (define (do-dhcp n)
    (let ((r (snewt-show-while
              (_ "DHCP setting")
              (_ "Waiting for response from DHCP server ...") 50 3
              (lambda () (obj-net 'apply-dhcp n)))))
      (cond
        ((string? r)
          (snewt-message (_ "Error") (format #f (_ "DHCP failed.\n(~a)\n") r))
          #f)
        (else
          #t))))
  
  (define (do-static n)
    (define (guess-gw am)
      (net-ip+mask->gateway (car am) (cadr am)))
    (define (guess-bc am)
      (net-ip+mask->broadcast (car am) (cadr am)))
    (define (cur-ip)
      (let ((x (obj-net 'get-if n)))
        (cond
          ((and (not (eq? #f x))
                (< 2 (length x)))
            (cadr x))
          (else
            ""))))
    (define (cur-mask)
      (let ((x (obj-net 'get-if n)))
        (cond
          ((and (not (eq? #f x))
                (< 2 (length x)))
            (caddr x))
          (else
            "255.255.255.0"))))
    (let* ((am (snewt-entries
                (_ "Ethernet Address")
                (format #f (_ "Enter IP address for Ethernet. (eth~a)") n)
                50 5 5 20
                (list
                 (list (_ "IP address") (cur-ip) 0)
                 (list (_ "netmask") (cur-mask) 0)))))
      (cond
        ((null? am)
          #f)
        (else
          (let* ((cur-gw (let ((x (obj-net 'get-if n)))
                           (cond
                             ((and (not (eq? #f x))
                                   (< 3 (length x)))
                               (list-ref x 3))
                             (else
                               (guess-gw am)))))
                 (cur-bc (let ((x (obj-net 'get-if n)))
                           (cond
                             ((and (not (eq? #f x))
                                   (< 4 (length x)))
                               (list-ref x 4))
                             (else
                               (guess-bc am)))))
                 (gb (and (not (null? am))
                          (snewt-entries
                           (_ "Ethernet Address")
                           (format #f (_ "Enter gateway and broadcast address. (eth~a)") n)
                           50 5 5 20
                           (list
                             (list (_ "gateway address") cur-gw 0)
                             (list (_ "broadcast address") cur-bc 0))))))
             (cond
               ((or (null? am) (null? gb))
                 #f)
               (else
                 (obj-net 'apply-static n (car am) (cadr am) (car gb) (cadr gb)))))))))
  
  (define (net-if-set-v4 n)
    (let ((r (snewt-ternary title
              (_ "How do you want to set up your network device?")
              (_ "Statically") (_ "Use DHCP") (_ "No Network"))))
      (cond
        ((= r 0) #f)
        ((= r 1) (do-static n))
        ((= r 2) (do-dhcp n))
        ((= r 3) (obj-net 'apply-none n))
        (else #t))))

  (define (net-if-set-v6-static n)
    (let ((r (snewt-entries title
                (_ "Enter IPv6 address prefix.")
                50 5 5 20
                (list (list (_ "Address prefix :") (obj-net 'get-static6 n) 0)))))
      (cond
        ((null? r) #f)
        ((not (obj-net 'update-static6 n (car r)))
          (snewt-message (_ "Error") (_ "Incorrect address prefix."))
          (net-if-set-v6-static n))
        (else
          #t))))
  
  (define (net-if-set-v6 n)
    (let ((r (obj-net 'get-if n)))
      (cond
        ((eq? #f r)
          #t)
        ((not (or (equal? "dhcp" (car r))
                  (equal? "static" (car r))))
          #t)
        (else
          (let ((r0 (snewt-choice title
                      (_ "Do you want to set up IPv6 address?")
                      (_ "Yes") (_ "No"))))
            (case r0
              ((0) #f)
              ((2)
                (obj-net 'update-none6 n))
              (else
                (let ((r1 (snewt-show-while title
                            (_ "Looking for IPv6 Router ...") 50 3
                            (lambda () (obj-net 'radv? n)))))
                  (cond
                    ((and (eq? r1 #t)
                          (not (= 1 (snewt-choice title
                                      (string-append
                                        (_ "IPv6 Router was found.\n\n")
                                        (format #f (_ "Interface eth~a is automatically set. ") n)
                                        (_ "Do you add static IPv6 address?"))
                                      (_ "Yes") (_ "No")))))
                      (obj-net 'update-auto6 n))
                    ((not (net-if-set-v6-static n))
                      #f)
                    (else
                      #t))))))))))
  
  (define (net-if-set n)
    (and (net-if-set-v4 n)
         (net-if-set-v6 n)))
  
  (define (net-dns-entry)
    (let ((x (snewt-entries
              (_ "DNS Setting")
              (_ "Enter IP address for a DNS Server.")
              50 5 5 20
              (list
                (list (_ "IP address") "" 0)))))
      (cond
        ((null? x) #f)
        (else
          (obj-net 'add-ns (car x))))))
  
  (define (net-dns-set-multi)
    (let* ((r (snewt-nmenu
                (_ "DNS Setting")
                (string-append
                  (_ "To delete DNS server, select an IP.\nTo add other DNS, select 'add'."))
                68 5 5 8
                (list->vector (obj-net 'list-ns))
                (vector (_ "finish") (_ "add") (_ "cancel")))))
      (let ((n (cdr r))
            (b (car r)))
        (cond
          ((= b 3) #f)
          ((= b -1) #f)
          ((= b 1) #t)
          ((= b 2)
            (if (net-dns-entry)
                (net-dns-set-multi)
                #f))
          (else
            (let* ((e (list-ref (obj-net 'list-ns) n))
                   (x (snewt-choice (_ "Confirm")
                                   (format #f (_"Are you Ok to delete this entry?\n   IP : ~a") e)
                                   (_ "Yes") (_ "No"))))
              (if (= x 1)
                (obj-net 'elim-ns e))
              (net-dns-set-multi)))))))
  
  (define (net-dns-set)
    (define title (_ "DNS Setting"))
    (define (start)
      (case (length (obj-net 'list-ns))
        ((0)
          (if (net-dns-entry)
              (start)
              #t))
        ((1)
          (case (snewt-choice title
                  (_ "Do you edit more DNS?")
                  (_ "No") (_ "Yes"))
            ((0) #f)
            ((1) #t)
            (else (net-dns-set-multi))))
        (else
          (net-dns-set-multi))))
    
    (let ((r (start)))
      (cond
        ((not r) r)
        (else
          (let ((l (snewt-show-while title
                     (_ "Sending a query to the DNS server(s) ...") 45 2
                     (lambda () (obj-net 'unreachable-ns)))))
            (cond
              ((null? l) #t)
              ((= 1 (snewt-choice title
                      (apply string-append
                        (append
                          (list (_ "DNS server below doesn't seem to respond. \n\n"))
                          (map (lambda (x) (format #f "     ~a\n\n" x))
                               l)
                          (list
                            (_ "Using unrechable DNS may cause the system occasional sleep. (In fact it is just waiting for resolving name.) \n\n")
                            (_ "This may be also caused by incorrect default gateway setting. If so, press F12 twice and redo this stage from the begining.\n\n")
                            (_ "Do you edit DNS again?"))))
                      (_ "Yes") (_ "No")))
                (net-dns-set-multi))
              (else
                #t)))))))
  
  (define (net-stage-dns)
    (let ((r (cond
               ((obj-net 'nonet?)
                 #t)
               ((null? (obj-net 'list-ns))
                 (net-dns-set))
               ((null? (filter (lambda (x) (equal? "static" (cadr x)))
                               (obj-net 'list-if)))
                 #t)
               ((null? (filter (lambda (x) (equal? "dhcp" (cadr x)))
                               (obj-net 'list-if)))
                 (net-dns-set))
               (else
                 (let ((x (snewt-choice (_ "Confirm")
                             (_ "You already have DNS with dhcp.\nDo you add more DNS server?")
                             (_ "Yes") (_ "No"))))
                   (cond
                     ((= x 1) (net-dns-set))
                     (else #t)))))))
      (if r (obj-net 'apply-ns))
      r))
  
  (define (net-gateway)
    (and (net-gateway-v4)
         (net-tunnel-v6)
         (net-gateway-v6)))
  
  (define (net-gateway-v4)
    (let ((l (obj-net 'list-active)))
      (case (length l)
        ((0) #t)
        ((1) (obj-net 'set-apply-gw (car l)))
        (else
          (let ((r (snewt-nmenu
                     (_ "Default Gateway Setting")
                     (_ "Select default gateway.")
                     50 5 5 8
                     (list->vector
                       (map (lambda (x) (format #f "  eth~a    ~a  \n" (car x)
                                          (if (equal? "dhcp" (cadr x))
                                              (caddr x) (list-ref x 4))))
                            l))
                     (vector (_ "Ok") (_ "cancel")))))
            (case (car r)
              ((2 -1) #f)
              (else
                (cond
                  ((eq? #f (obj-net 'set-apply-gw (list-ref l (cdr r))))
                    (snewt-smessage (_ "Error")
                      (_ "gateway setting failed. ") 40 3)
                    #f)
                  (else
                    #t)))))))))
  
  (define (net-tunnel-v6)
    (let ((l (map (lambda (x) (obj-net 'get-type6 (car x)))
                  (obj-net 'list-active))))
      (cond
        ((null? (filter (lambda (x) (case x
                                      ((static auto) #t)
                                      (else #f)))
                        l))
          #t)
        (else
          (let ((r (snewt-choice title
                     (_ "Do you set up a manual IPv6-in-IPv4 tunnel?")
                     (_ "Yes") (_ "No"))))
            (case r
              ((0) #f)
              ((2) #t)
              (else
                (let* ((m (map caddr
                            (filter
                              (lambda (x) (equal? (cadr x) "static"))
                              (obj-net 'list-if))))
                       (r1 (snewt-entries title
                             (_ "Enter IPv4 address for remote and local used for tunnel.")
                             50 5 5 20
                             (list (list (_ "Remote IPv4 address") "" 0)
                                   (list (_ "Local  IPv4 address")
                                         (if (= 1 (length m)) (car m) "")
                                         0)))))
                  (cond
                    ((or (null? r1) (equal? "" (car r1)) (equal? "" (cadr r1)))
                      (let ((r2 (snewt-choice title
                                  (_ "Do you skip tunnel setting?")
                                  (_ "Yes") (_ "No"))))
                        (case r2
                          ((0) #f)
                          ((1) #t)
                          (else (net-tunnel-v6)))))
                    (else
                      (let ((r2 (snewt-choice title
                                  (_ "Do you set this tunnel as default IPv6 gateway?")
                                  (_ "Yes") (_ "No")))
                            (check-rep (lambda (a)
                                         (cond
                                           ((obj-net 'update-tunnel 1 (car r1) (cadr r1) a)
                                             #t)
                                           (else
                                             (snewt-message (_ "Error") (_ "Tunnel setting failed."))
                                             (net-tunnel-v6))))))
                        (case r2
                          ((0) #f)
                          ((1)
                            (check-rep "::/0"))
                          (else
                            (let ((r3 (snewt-entries title
                                        (format #f (_ "Enter address prefix for routing with tunnel (~a -> ~a).")
                                          (cadr r1) (car r1))
                                        50 5 5 20
                                        (list (list (_ "Addressprefix :") "" 0)))))
                              (cond
                                ((null? r3) #f)
                                (else
                                  (check-rep (car r3))))))))))))))))))
  
  (define (net-gateway-v6)
    (inst-log "net-gateway-v6\n")
    (let ((l (map (lambda (x) (obj-net 'get-type6 (car x)))
                  (obj-net 'list-active))))
      (cond
        ((null? (filter (lambda (x) (equal? x 'static))
                        l))
          #t)
        (else
          (let ((g (obj-net 'get-gw6)))
            (cond
              ((and (< 3 (string-length g))
                    (equal? "sit" (substring g 0 3)))
                #t)
              (else
                (let ((r (snewt-choice title
                           (_ "Do you set up static default gateway for IPv6?")
                           (_ "Yes") (_ "No"))))
                   (case r
                     ((0) #f)
                     ((2) #t)
                     (else
                       (let ((r1 (snewt-entries title
                                   (format #f (_ "Enter address for static default IPv6 gateway."))
                                   50 5 5 20
                                   (list (list (_ "Address :") (if (equal? "" g) "" g) 0)))))
                         (cond
                           ((null? r1) #f)
                           ((not (obj-net 'set-gw6 (car r1)))
                             (snewt-message (_ "Error")
                               (_ "Could not set default gateway."))
                             (net-gateway-v6))
                           (else
                             #t)))))))))))))
  
  (define (net-if-menu)
    (let* ((r (snewt-nmenu
                (_ "Network Setting")
                (_ "Select network interface to set.\nThis setting is Ok, then select 'finish'.\n\n")
                68 5 5 8
                (list->vector
                  (map (lambda (x)
                         (format #f "  eth~a        ~a  \n"
                                    (car x)
                                    (let ((t (cadr x)))
                                      (cond
                                        ((equal? t "none") (_ "Not configured yet"))
                                        ((equal? t "static") (_ "Static network"))
                                        ((equal? t "dhcp") (_ "DHCP"))
                                        (else "broken data")))))
                       (obj-net 'list-sort-if)))
                (vector (_ "finish") (_ "cancel")))))
      (case (car r)
        ((-1 2) #f)
        ((1) (net-gateway))
        (else
          (net-if-set (cdr r))
          (net-if-menu)))))
  
  (define (net-stage-if)
    (case (length (obj-net 'list-if))
      ((0)
        (inst-log "newt-ui-net : no network.\n")
        #t)
      ((1)
        (and (net-if-set 0)
             (net-gateway)))
      (else
        (net-if-menu))))
  
  (define (net-name-noname-hostname host domain)
    (let ((r (snewt-entries (_ "Hostname")
                (_ "Enter hostname.\nIf you do not enter your hostname, it will be 'noname'.")
                50 5 5 20
                (list
                  (list (_ "hostname") host 0)))))
      (cond
        ((null? r)
          #f)
        (else
          (obj-net 'set-name
                   (if (string=? (car r) "") "noname" (car r))
                   domain)))))

  (define (net-name-noname-both host domain)
    (let ((r (snewt-entries (_ "Hostname and Domainname")
                (_ "Enter hostname and domainname.\nIf you omit these, these will be 'noname.nodomain'.")
                50 5 5 20
                (list (list (_ "hostname") host 0)
                      (list (_ "domain name") domain 0)))))
      (cond
        ((null? r)
          #f)
        (else
          (obj-net 'set-name
                   (if (string=? (car r) "") "noname" (car r))
                   (if (string=? (cadr r) "") "nodomain" (cadr r)))))))
  
  (define (net-name-noname host domain)
    (if (equal? "" domain)
      (net-name-noname-both host domain)
      (net-name-noname-hostname host domain)))
  
  (define (net-enter-name h d)
    (let ((r (snewt-entries (_ "Host and Domain Name")
                (_ "Enter host and domain name.")
                50 5 5 20
                (list (list (_ "hostname") h 0)
                      (list (_ "domain name") d 0)))))
      (cond
        ((null? r)
          #f)
        (else
          (obj-net 'set-name (car r) (cadr r))
          #t))))
  
  (define (net-name-select n)
    (let* ((r (snewt-nmenu
              (_ "Hostname Selection")
              (_ "Choose one of below as name of this host.\nOr you can set hostname manually.")
              50 5 5 3
              (list->vector (map (lambda (x) (string-append (car x) "." (cadr x)))
                                 n))
              (vector (_ "Ok") (_ "Set Name") (_ "Cancel"))))
           (b (car r)))
      (cond
        ((or (= b 0) (= b 1))
          (let ((x (list-ref n (cdr r))))
            (obj-net 'set-name (car x) (cadr x))))
        ((= b 2)
          (net-enter-name "" "")
          #t)
        (else
          #f))))
  
  (define (net-stage-name)
    (let ((n (filter (lambda (y) (and (= 2 (length y))
                                      (not (equal? "" (car y)))))
               (map (lambda (x) (fqdn-split (gethostbyaddr (caddr x))))
                    (obj-net 'list-active)))))
      (let ((r (case (length n)
                 ((0) (net-name-noname "" ""))
                 ((1) (net-name-noname (caar n) (cadr (car n))))
                 (else
                   (net-name-select n)))))
         (cond
           ((not r) #f)
           (else
             (obj-net 'apply-ns))))))
  
  (newt-stage-title (_ "Network Setting"))
  (cond
    ((eq? #f (net-stage-if))
      (inst-log "net-stage-if canceled.\n")
      #f)
    ((eq? #f (net-stage-dns)) #f)
    ((eq? #f (net-stage-name)) #f)
    (else
      (let ((l (map (lambda (x) (obj-net 'get-type6 (car x)))
                    (obj-net 'list-active))))
        (cond
          ((null? (filter (lambda (x) (not (equal? x 'none)))
                          l))
            #t)
          (inst?
            (snewt-message title
              (_ "Other than site local link, IPv6 will function after installation."))
            #t)
          (else
            (obj-net 'apply-v6)
            #t))))))

(define (newt-ui-kbd inst? obj-kbd obj-x)
  (define title (_ "Keyboard Setting"))
  (define first-message
    (cond
      ((equal? 'inst1 (obj-kbd 'get-context))
        (string-append
          (_ "During installation, you can use the keys below.\n\n")
          (_ "    <space> or <return> : to select current items\n")
          (_ "       cursor keys      : move to other items\n")
          (_ "     <tab> or <alt+tab> : move to buttons/menu\n")
          (_ "          <F12>         : to cancel\n\n")))
      (else
        "")))
  
  (define (choose f?)
    (let* ((m (obj-kbd 'list))
           (r (snewt-nmenu (_ "Keyboard Setting")
                (string-append
                  (if f? first-message "")
                  (_ "Select the keyboard layout.\n")
                  (format #f (_ "Current layout is ~a .\n") (obj-kbd 'desc)))
                50 5 5 6
                (list->vector
                  (apply append
                    (map (lambda (x)
                           (let ((k (car x)))
                             (list
                               (sprintf (_ "%-10s : normal") k)
                               (sprintf (_ "%-10s : exchange caps/control") k)
                               (sprintf (_ "%-10s : map caps to control") k))))
                         m)))
                (vector (_ "Ok") (_ "Skip")))))
      (case (car r)
        ((-1) #f)
        ((2) #t)
        (else
          (obj-kbd 'set-caps
            (case (cdr r)
              ((1 4) 'ex)
              ((2 5) 'ctrl)
              (else 'caps)))
          (obj-kbd 'set-type (cdr (list-ref m (i/ (cdr r) 3))))
          (cond
            ((not (obj-kbd 'apply))
              (snewt-message (_ "Error")
                (_ "Could not set keyboard setting. Possibly keymap or some command is not found."))
              #f)
            ((and (not inst?)
                  (not (obj-x 'set-keyboard (obj-kbd 'get-type))))
              (snewt-message (_ "Error")
                (_ "Failed to set X keyboard."))
              #f)
            (else
              #t))))))
  
  (newt-stage-title (_ "Keyboard Setting"))
  (let ((r (cond
             ((and inst? (obj-kbd 'usb?))
               ;(if (not (= 1 (snewt-choice title
               ;                (string-append
               ;                  first-message
               ;                  (_ "Do you use USB driver for keyboard?\n")
               ;                  (_ "If you choose 'no' here, then usb driver removed and system will rely on regacy support of hardware.\n"))
               ;                (_ "Yes") (_ "No"))))
               ;  (obj-kbd 'nousb))
               ; ߤ kernel  usb hub ɥ饤ФäƤΤǰ̣ʤ
               (choose #f))
             (else
               (choose #t)))))
    (if (not inst?)
      (snewt-message title
        (if r (_ "Keyboard configuration was changed successfully.")
              (_ "Keyboard configuration was not changed."))))
    r))

(define (newt-ui-mod-extra l obj-part obj-mod)
  (let ((fd (obj-part 'get-fdd)))
    (cond
      ((equal? "" fd)
        (snewt-message (_ "Fatal Error")
          (_ "Extra modules are required but cannot install because system does not have valid FD drive."))
        #f)
      (else
        (let* ((run (lambda ()
                      (sh "umount ~a" fd)
                      (cond
                        ((not (= 0 (sh "mount ~a /mnt -t cramfs" fd)))
                          (_ "could not mount extra FD"))
                        (else
                          (sh "cp /mnt/*.scm /1.5")
                          (sh "cp -f /mnt/*.mo /usr/share/locale/ja/LC_MESSAGES")
                          (map-newfs "/mnt" "/mod" "/mnt")
                          (cond
                            ((not (= 0 (sh "depmod -a -r")))
                              "depmod failed")
                            (else
                              (obj-mod 'extra-probe)
                              #t))))))
               (fd (list-foldstring ", "
                     (map (lambda (x) (format #f "~a" x))
                       (list-uniq
                         (easy-sort <
                           (map (lambda (x) (obj-mod 'which-fd x))
                             l))))))
               (b (snewt-confirm (_ "Installing Extra Modules")
                    (apply string-append
                      (append (list (_ "To install module(s): \n\n  "))
                              (map (lambda (x) (format #f "~a " (obj-mod 'get-mapped x)))
                                   l)
                              (list
                                (format #f (_ "\n\nExtra module disk ~a has to be loaded.\npress Ok when additional FD is ready.")
                                  fd)))))))
           (cond
             ((not (= b 1)) #f)
             (else
               (let ((r (snewt-show-while
                          (_ "Loading modules")
                          (_ "Loading modules ...") 45 2 run)))
                 (cond
                   ((not (eq? #t r))
                     (snewt-message (_ "Error")
                       (format #f (_ "Failed to load modules.\nReason:\n~a") r))
                     (newt-ui-mod-extra l obj-part obj-mod))
                   (else
                     #t))))))))))

(define (newt-ui-more-driver obj-part obj-mod)
  (define title (_ "Driver Setting"))
  (define (mes-ok)
    (snewt-message title (_ "driver successfully installed."))
    #t)
  (define (mes-ng)
    (snewt-message (_ "Error") (_ "Failed to install module."))
    #f)
  
  (cond
    ((not (= 1 (snewt-choice title
                 (_ "Do you want to manually install more drivers?")
                 (_ "Yes") (_ "No"))))
      #t)
    (else
      (let* ((r1 (snewt-radio-tree title
                   (_ "Select a driver module.")
                   30 1 10 (obj-mod 'usable-tree)))
             (r2 (snewt-entries title
                   (format #f (_ "If use driver option for ~a, enter module option below.") r1)
                   50 5 5 20
                   (list (list (_ "Option : ") "" 0)))))
        (cond
          ((null? r2)
            (newt-ui-more-driver obj-part obj-mod))
          (else
            (let* ((m (string->symbol r1))
                   (ins (lambda () (obj-mod 'insmod m (car r2)))))
              (cond
                ((ins)
                  (mes-ok))
                (else
                  (let ((r3 (assoc m (obj-mod 'table))))
                    (cond
                      ((or (eq? #f r3) (not (equal? 'req (cadr r3))))
                        (mes-ng))
                      ((not (newt-ui-mod-extra (list m) obj-part obj-mod))
                        #f)
                      (else
                        (let ((r4 (ins)))
                          (sh "umount /mnt")
                          (if r4
                            (mes-ok)
                            (mes-ng)))))))))
            (newt-ui-more-driver obj-part obj-mod)))))))

(define (newt-ui-auto-probe obj-part obj-mod)
  (define (table-str l m)
    (apply string-append
      (map (lambda (x)
             (sprintf " %16s : %48s\n"
               (cond
                 ((eq? (car x) m)          (_ "<installing>"))
                 ((eq? (cadr x) 'un/set)   (_ "unsupported"))
                 ((eq? (cadr x) 'un/not)   (_ "to be installed"))
                 ((eq? (cadr x) 'inst/set) (_ "installed"))
                 ((eq? (cadr x) 'inst/not) (_ "auto-installed"))
                 ((eq? (cadr x) 'req)      (_ "driver required"))
                 (else (inst-log "strange table : ~a" x) (_ "unknown")))
               (string-cut (obj-mod 'desc (car x)) 48)))
           (filter (lambda (y)
                     (or (equal? (car y) m)
                         (not (equal? (cadr y) 'un/set))))
                   l))))
  
  (define (main)
    (map (lambda (x)
           (snewt-show-while (_ "AutoProbe")
               (string-append (_ "installing module ...\n\n")
                              (table-str (obj-mod 'table) (car x)))
               70 11
               (lambda () (obj-mod 'insmod (car x) ""))))
         (filter (lambda (x)
                   (case (cadr x)
                     ((un/set req) #t)
                     (else #f)))
                 (obj-mod 'table)))
    (let ((r (snewt-smessage (_ "AutoProbe")
               (string-append
                 (_ "Drivers are loaded as shown:\n\n")
                 (table-str (obj-mod 'table) 'dummy))
               70 14)))
      (case r
        ((-1) #f)
        (else
          (let ((l (filter (lambda (x) (equal? 'req (cadr x)))
                           (obj-mod 'table))))
            (cond
              ((null? l)
                (sh "umount /mnt")
                #t)
              ((newt-ui-mod-extra (map car l) obj-part obj-mod)
                (main))
              (else
                (sh "umount /mnt")
                #f)))))))
  
  (newt-stage-title (_ "Driver Setting"))
  (obj-mod 'setmod-init)
  (main))



(define (newt-ui-media-default inst? obj-med obj-part obj-na)
  (define title (_ "Default Media Setting"))
  
  (define (not-changed)
    (snewt-message (_ "Warning") (if (null? (obj-med 'info))
                                     (_ "Media configuration was not done.")
                                     (_ "Media configuration was not changed.")))
    #f)
  
  (define (select-hd t)
    (let* ((cur (obj-med 'info))
           (set? (and (not (null? cur)) (equal? 'block (car cur))))
           (r (snewt-entries
                (_ "HDD media setting")
                (_ "Enter Device name and directory for HDD installation. (in Linux format, ex. /dev/hda1.)\n")
                60 5 5 50
                (list
                  (list (_ "Device name") (if set? (obj-part 'oldname (cadr cur)) "/dev/") snewt-flag-scroll)
                  (list (_ "Directory") (if set? (list-ref cur 2) "/") snewt-flag-scroll)))))
      (cond
        ((or (null? r) (string=? "" (car r)) (string=? "" (cadr r)))
          (not-changed))
        ((< (string-length (car r)) 6)
          (snewt-message (_ "HDD media setting")
            (_ "Enter correct device name.\n"))
          #f)
        (else
          (let* ((p (car r))
                 (n (string-match p "[0-9]\\+"))
                 (d (string-match p "[^0-9]\\+")))
            (cond
              ((not (and (= 1 (length n))
                         (= 1 (length d))))
                (snewt-message (_ "Error") (_ "Incorrect device name."))
                (select-hd t))
              (else
                (let ((did (obj-part 'get-did-by-oldname (car d))))
                  (cond
                    ((equal? "" did)
                      (snewt-message (_ "Error") (_ "No such a device.\n"))
                      (select-hd t))
                    (else
                      (list 'block did (cadr r) t (string->number (car n)))))))))))))
  
  (define (select-smb)
    (let* ((cur (obj-med 'info))
           (set? (and (not (null? cur)) (equal? 'smb (car cur))))
           (r (snewt-entries
                (_ "SMB setting")
                (_ "Enter SMB parameter.\n")
                60 5 5 50
                (list
                  (list (_ "Hostname") (if set? (cadr cur) "") snewt-flag-scroll)
                  (list (_ "Sharename") (if set? (caddr cur) "") snewt-flag-scroll)
                  (list (_ "Username") (if set? (list-ref cur 3) "") snewt-flag-scroll)
                  (list (_ "Password") (if set? (list-ref cur 4) "") snewt-flag-hidden)
                  ))))
      (cond
        ((or (null? r) (string=? "" (car r)) (string=? "" (cadr r)))
          (not-changed))
        (else
          (let ((a (apply obj-med (cons 'set (cons 'smb r)))))
            (cond
              ((not (eq? #t a))
                (set-failed r a))
              (else
                #f)))))))
  
  (define (select-http)
    (let* ((cur (obj-med 'info))
           (r (snewt-entries
               (_ "HTTP setting")
               (_ "Enter URL for base tree\nYou can use proxy server if needed.\n")
               60 5 5 50
               (list
                (list (_ "URL") (if (and (not (null? cur)) (equal? 'http (car cur)))
                                    (cadr cur)
                                    "http://")
                                 snewt-flag-scroll)
                (list (_ "Proxy Server:Port") (if (and (not (null? cur)) (equal? 'http (car cur)))
                                      (caddr cur)
                                      "")
                                   snewt-flag-scroll)))))
      (cond
        ((or (null? r) (and (string=? "" (car r)) (string=? "" (cadr r))))
          (not-changed))
        (else
          (cons 'set (cons 'http r))))))
  
  (define (select-armanet)
    (cond
      ((not (= 1 (snewt-choice title
                   (_ "Do you set(or reconfigure) the ARMA Net account now?")
                   (_ "Yes") (_ "No"))))
        #t)
      (else
        (newt-ui-na #f obj-na #f)))
    
    (let* ((m (obj-med 'info))
           (h (if (= 7 (length m))
                  (list-ref m 5)
                  ""))
           (p (cond
                ((and (= 7 (length m)) (< 0 (list-ref m 6)))
                  (format #f "~a" (list-ref m 6)))
                (else
                  "")))
           (r (snewt-entries
                (_ "ARMA Net http setting")
                (_ "Do you use HTTP proxy?\n\nIf you use a proxy, please fill below.\n")
                60 5 5 50
                (list
                  (list "Proxy Server :" h 0)
                  (list "Port Number  :" p 0)))))
      (let ((l0 (list 'http "www.omoikane.co.jp" "arma_2.1_updates/deb"
                      (format #f "D~a" (obj-na 'get-id)) (obj-na 'get-hashed))))
        (cond
          ((or (null? r) (equal? "" (car r)))
            (append l0 (list "" -1)))
          (else
            (let ((n (string->number (cadr r))))
              (cond
                ((not (integer? n))
                  (snewt-message (_ "Error")
                    (_ "only number is allowed for port number.\n"))
                  (select-armanet))
                (else
                  (snewt-message (_ "Info")
                    (string-append
                      (_ "Setting proxy is done via environment variable.")
                      (_ "Proxy will be valid from next login.")))
                  (append l0 (list (car r) n))))))))))
  
  (define (select-ftp)
    (let* ((cur (obj-med 'info))
           (set? (and (not (null? cur)) (equal? 'ftp (car cur))))
           (r (snewt-entries
               (_ "FTP setting")
               (_ "Enter Name/IP address of FTP Server and inside Path.\nYou can use proxy server if needed.\n")
               60 5 5 50
               (list
                (list (_ "Server Name") (if set? (cadr cur) "") snewt-flag-scroll)
                (list (_ "Directory Path") (if set? (caddr cur) "/") snewt-flag-scroll)
                (list (_ "FTP Username") (if set? (list-ref cur 3) "anonymous") snewt-flag-scroll)
                (list (_ "FTP Password") (if set? (list-ref cur 4) "") snewt-flag-hidden)
                     ; ʤ 2048 (= password ե饰) ʤ
                (list (_ "Proxy Server:Port") (if (and (not (null? cur)) (equal? 'ftp (car cur)))
                                                  (list-ref cur 5) "")
                      snewt-flag-scroll)))))
      (cond
       ((or (null? r) (string=? "" (car r)) (string=? "" (cadr r))
            (string=? "" (caddr r)))
         (not-changed))
       (else
         (cons 'set (cons 'ftp r))))))

  
  (define (select-cdrom)
    (define (check x)
      (cond
        ((not (equal? #t (obj-med 'set 'cdrom (car x) "/" 'iso9660)))
          #f)
        ((not (eq? #t (obj-med 'open)))
          (obj-med 'close)
          #f)
        (else
          (let ((r (obj-med 'verify)))
            (obj-med 'close)
            r))))
            
    (let* ((l (obj-part 'hw-cdrom))
           (l' (filter check l)))
      (cond
        ((null? l)
          (snewt-message (_ "Media Setting")
            (_ "No CD-ROM found. Check driver first."))
          #f)
        ((null? l')
          (snewt-message (_ "Media Setting")
            (_ "Insert OGL CD-ROM and Try this menu again."))
          #f)
        ((= 1 (length l'))
          (let ((s (apply obj-med (cons 'set (list 'cdrom (caar l') "/" 'iso9660)))))
            (cond
              ((eq? #t s)
                (list 'cdrom (caar l') "/" 'iso9660))
              (else
                (snewt-message (_ "Media Setting")
                  (string-append
                    (_ "Could not set this CD-ROM.\nReason : \n")
                    s "\n"))
                #f))))
        (else
          (snewt-message (_ "Media Setting")
            (string-append
              (_ "Several CD-ROM drives look like to have OGL CD-ROM media.\n")
              (_ "Check that only one drive has correct media and try this menu again.\n")))
          #f))))

  (define (select-nfs)
    (let* ((cur (obj-med 'info))
           (set? (and (not (null? cur)) (equal? 'nfs (car cur))))
           (r (snewt-entries
                (_ "NFS setting")
                (_ "Enter Name/IP address of NFS Server and inside Path.\n")
                60 5 5 50
                (list
                  (list (_ "Server Name") (if set? (cadr cur) "") snewt-flag-scroll)
                  (list (_ "Directory") (if set? (caddr cur) "/") snewt-flag-scroll)))))
      (cond
        ((or (null? r) (string=? "" (car r)) (string=? "" (cadr r)))
          (not-changed))
        (else
          (list 'nfs (car r) (cadr r) "" "" "" -1)))))
  
  (define (test-it m)
    (let ((r0 (snewt-show-while title
                (_ "Verifying media ...")
                50 3
                (lambda ()
                  (let ((r (apply obj-med (cons 'set m))))
                    (cond
                      ((not (eq? #t r))
                        r)
                      (else
                        (let ((a (obj-med 'open))
                              (b (obj-med 'verify)))
                          (obj-med 'close)
                          (cond
                            ((string? a) a)
                            ((not b) (_ "incorrect default media."))
                            (else #t))))))))))
      (cond
        ((eq? #t r0)
          #t)
        (else
          (let ((r1 (snewt-ternary title
                      (format #f (_ "Could not verify media.\n~a\nTry current media again?\n")
                              (if (string? r0) (format #f (_ "Reason : ~a\n") r0) ""))
                      (_ "Test Again") (_ "Set Anyway") (_ "Set Again"))))
            (case r1
              ((1)
                (test-it m))
              ((2)
                #t)
              (else
                #f)))))))
  
  (define (select)
    (let* ((l (if inst?
                  ;'("NFS(latest beta)" "CD-ROM" "NFS" "Hard Disk (ext2)" "Hard Disk (VFAT)")
                  '("CD-ROM" "NFS" "Hard Disk (ext2)" "Hard Disk (VFAT)")
                  '("ARMA Net (http)" "CD-ROM")))
           (r (apply snewt-menuc
                (append (list title (_ "Select default install media below.") 50 5 5 8)
                        l))))
      (case r
        ((-1)
          #f)
        (else
          (let* ((x (list-ref l r))
                 (m (cond
                      ((string=? x "NFS(latest beta)")
                        (inst-log "set nfs : ~a\n" (obj-med 'set 'nfs "192.168.0.1" "/home/tad/na/tree" "" "" "" -1))
                        (select-nfs))
                      ((string=? x "HD(current beta)")
                        (list 'block "VMware_Virtual_IDE_Hard_Drive:ide:0" "/na/tree" 'linux-ext2 3))
                      ((string=? x "CD-ROM") (select-cdrom))
                      ((string=? x "HTTP") (select-http))
                      ((string=? x "ARMA Net (http)") (select-armanet))
                      ((string=? x "FTP") (select-ftp))
                      ((string=? x "NFS") (select-nfs))
                      ((string=? x "SMB") (select-smb))
                      ((string=? x "Hard Disk (ext2)") (select-hd 'linux-ext2))
                      ((string=? x "Hard Disk (VFAT)") (select-hd 'vfat))
                      ((string=? x "Hard Disk (NTFS)") (select-hd 'ntfs))
                      (else #f))))
            (cond
              ((eq? #f m)
                (select))
              ((not (test-it m))
                (select))
              ((case (car m)
                 ((block cdrom nfs)
                   (not (apply obj-part (cons 'set-media m))))
                 (else #f))
                (snewt-message title (_ "Error. Could not set media."))
                (select))
              (else
                #t)))))))
  
  (newt-stage-title (_ "Media Selection"))
  (if (obj-med 'open?) (obj-med 'close))
  (select))

