
(define (newt-ui-atok-key obj-pd obj-sv)
  (define title (_ "ATOK X Keybinding"))
  (let ((dir "/usr/lib/im/locale/ja/atokserver"))
    (cond
      ((not (unix-access? dir '(read)))
        (inst-log "newt-ui-atok-key : could not access to the atox server directory. skipping.")
        #f)
      (else
        (let ((r (snewt-nmenu title
                   (string-append
                     (_ "Select a keybinding to invoke the ATOK X. ")
                     "\n\n"
                     (_ "ATOK X has two way to invoke : 'Ctrl + Space' or 'Shift + space' ")
                     (_ "If you would like to use an editor from emacs family, it is recommended to choose 'Shift + Space'. ")
                     (_ "Other than these bindings, 'Zenkaku' key can be also used. "))
                   50 5 5 5
                   (vector (_ "Shift + Space") (_ "Ctrl + Space"))
                   (vector (_ "Ok")))))
          (cond
            ((< (car r) 0)
              #f)
            (else
              (case (cdr r)
                ((1) #t)
                (else
                  (sh "touch ~a/shift_space" dir)
                  (if (obj-sv 'start? "atokx")
                    (obj-sv 'restart "atokx"))
                  #t)))))))))

(define (newt-ui-kdm obj-sv obj-jq)
  (define title (_ "X Configuration"))
  (obj-jq 'remove-bykey 'kdm)
  (cond
    ((not (and (unix-access? "/etc/X11/XF86Config-4" '(read))
               (unix-access? "/var/lib/ogl/etc/x-win" '(read))))
      #t)
    ((= 1 (snewt-choice title
            (_ "Do you want to start display manager(graphical login) after finishing this tool?")
            (_ "Yes") (_ "No")))
      (obj-jq 'enq 'kdm
        (lambda () (sh "/etc/init.d/kdm start"))
        (lambda () #t)
        (lambda () #t))
      (obj-sv 'set-start "kdm")
      #t)
    (else
      (obj-sv 'remove "kdm")
      #t)))

(define (newt-ui-addtitional inst? obj-add)
  (define title (_ "Additional package installation"))
  (define (show-license f m b)
    (cond
      ((not b)
        b)
      (else
        (newt-ui-pager-info inst?)
        (snewt-finish)
        (unix-system (format #f "lv ~a" f))
        (snewt-start)
        (newt-stage-title title)
        (let ((r (snewt-ternary title
                   m
                   (_ "Yes") (_ "No") (_ "View Again"))))
          (case r
            ((0 2) #f)
            ((1) #t)
            (else (show-license f m b)))))))
  
  (cond
    ((not (obj-add 'init (if inst? 'inst3 'admin) #t))
      (snewt-message title
        (if inst?
          (string-append
            (_ "Currently this host doesn't seems to be ready for addtional package installation. ")
            (_ "Mostly this is because of lacking of network connectivity. ")
            (_ "You can install additional packages later with ogl-admin. "))
          (_ "Could not read the additional package table nor get a package selection. Skipping.")))
      #f)
    (else
      (let ((l (obj-add 'desc)))
        (cond
          ((null? l)
            (if (not inst?)
              (snewt-message title
                (_ "No additional package was found which is to be installed.")))
            #t)
          (else
            (let ((r1 (snewt-checkbox title
                        (_ "Check software which you would like to install.")
                        62 2 5
                        (list->vector
                          (map (lambda (x) (cons x #f))
                               l))
                        (vector (_ "Ok") (_ "Cancel")))))
              (cond
                ((not (= 0 (car r1)))
                  (snewt-message title
                    (_ "Installation for additional packages has been skipped."))
                  #t)
                (else
                  (let ((r2 (map show-license (obj-add 'license) (obj-add 'asking)
                                              (vector->list (cdr r1)))))
                    (cond
                      ((not (ormap (lambda (x) x) r2))
                        (snewt-message title
                          (_ "No additional package has been installed."))
                        #t)
                      (else
                        (map (lambda (f m b)
                               (cond
                                 (b
                                   (snewt-show-while title
                                     m 50 3
                                     (lambda () (sh f))))))
                             (obj-add 'installer)
                             (obj-add 'waiting)
                             r2)
                        (snewt-smessage title
                          (apply string-append
                            (cons
                              (_ "Status of additional packages has been changed as below:\n\n")
                              (map (lambda (x)
                                     (sprintf "  %-12s  --- %s\n" (car x) (cdr x)))
                                   (obj-add 'status-all))))
                          60 5)
                        #t))))))))))))

(define (newt-ui-default-media obj-med obj-na)
  (define title (_ "Default Media Setting"))
  (cond
    ((not (= 1 (snewt-choice title
                 (string-append
                   (_ "Do you want to set the default package source to the ARMA Net web?\n")
                   (_ "After setting it, upgrading will be expected very smooth with the ARMA Net web. Otherwise default package source will remain CD-ROM."))
                 (_ "Yes") (_ "No"))))
      #t)
    (else
      (obj-med 'set 'http "www.omoikane.co.jp" "arma_2.1_updates/deb"
                (format #f "D~a" (obj-na 'get-id)) (obj-na 'get-hashed) "" -1)
      (snewt-message title
        (_ "The default package source has been changed to the ARMA Net.\nIf you are using HTTP proxy, run the administration tool (ogl-admin) after installation is completed."))
      #t)))

(define (newt-ui-orca obj-pd patch obj-prof obj-cr)
  (define title (_ "ORCA Configuration"))
  (newt-stage-title title)
  (cond
    ((not (equal? 'orca (obj-pd 'product-type)))
      (snewt-message (_ "Error")
        (_ "This product does not seem to be ORCA edition. Aborting."))
      #f)
    (else
      (let ((t (obj-pd 'get-orcatype))
            (sl (format #f (_ "Server status setting (~a)") (obj-pd 'desc-server))))
        (case t
          ((unset)
            (newt-ui-orca-type obj-pd patch obj-prof obj-cr))
          (else
            (cond
              ((< (obj-pd 'get-initstep) 100)
                (case t
                  ((primary)
                    (newt-ui-orca-type-primary obj-pd patch obj-cr))
                  ((secondary)
                    (newt-ui-orca-type-secondary #t obj-pd patch))
                  ((single)
                    (newt-ui-orca-type-single obj-pd patch))
                  (else
                    (newt-ui-orca-type-client obj-pd))))
              (else
                (case t 
                  ((primary)
                    (let ((r (snewt-nmenu title
                               (_ "Select ORCA Configuration below.\n")
                               50 5 5 5
                               (vector sl
                                 (_ "Secondary server address") (_ "Master password setting")
                                 (_ "Operator account setting") (_ "Hospital ID setting")
                                 (_ "Secondary server data recovery")
                                 (_ "Data refresh schedule")
                                 (_ "Primary server data recovery"))
                               (vector (_ "Ok")))))
                      (case (car r)
                        ((-1) #f)
                        ((1) #t)
                        (else
                          (case (cdr r)
                            ((0) (newt-ui-orca-server obj-pd))
                            ((1) (newt-ui-orca-primary-host #f obj-pd patch))
                            ((2) (newt-ui-orca-master obj-pd))
                            ((3) (newt-ui-orca-acc obj-pd))
                            ((4) (newt-ui-orca-hospid obj-pd))
                            ((5) (newt-ui-orca-recover-secondary obj-pd))
                            ((6) (newt-ui-orca-schedule obj-pd obj-cr))
                            ((7) (newt-ui-orca-recover-primary obj-pd)))
                          (newt-ui-orca obj-pd patch obj-prof obj-cr)))))
                  ((secondary)
                    (cond
                      ((not (newt-ui-orca-secondary-recover-check obj-pd))
                        #f)
                      (else
                        (let ((r (snewt-nmenu title
                                   (_ "Select ORCA Configuration below.\n")
                                   50 5 5 5
                                   (vector sl (_ "Host configuration") (_ "Master password setting")
                                           (_ "Operator account setting"))
                                   (vector (_ "Ok")))))
                          (case (car r)
                            ((-1) #f)
                            ((1) #t)
                            (else
                              (case (cdr r)
                                ((0) (newt-ui-orca-server obj-pd))
                                ((1) (newt-ui-orca-type-secondary #f obj-pd patch))
                                ((2) (newt-ui-orca-master obj-pd))
                                ((3) (newt-ui-orca-acc obj-pd)))
                              (newt-ui-orca obj-pd patch obj-prof obj-cr)))))))
                  ((single)
                    (let ((r (snewt-nmenu title
                               (_ "Select ORCA Configuration below.\n")
                               50 5 5 5
                               (vector sl
                                       (_ "Master password setting")
                                       (_ "Operator account setting")
                                       (_ "Hospital ID setting"))
                               (vector (_ "Ok")))))
                      (case (car r)
                        ((-1) #f)
                        ((1) #t)
                        (else
                          (case (cdr r)
                            ((0) (newt-ui-orca-server obj-pd))
                            ((1) (newt-ui-orca-master obj-pd))
                            ((2) (newt-ui-orca-acc obj-pd))
                            ((3) (newt-ui-orca-hospid obj-pd)))
                          (newt-ui-orca obj-pd patch obj-prof obj-cr)))))
                  ((client)
                    (newt-ui-orca-type-client obj-pd))
                  (else
                    #f))))))))))

(define (newt-ui-orca-schedule obj-pd obj-cr)
  (define title (_ "Data Refresh Schedule"))
  
  (define (ask-time h m)
    (let ((r (snewt-entries title
               (_ "Enter time when the data refresh begin.")
               50 5 5 20
               (list (list (_ "Hour   (0-23) : ") (if (< 0 h) (format #f "~a" h) "") 0)
                     (list (_ "Minute (0-59) : ") (if (< 0 m) (format #f "~a" m) "") 0)))))
      (cond
        ((null? r) '())
        (else
          (let ((n0 (string->number (car r)))
                (n1 (string->number (cadr r))))
            (cond
              ((not (and (number? n0) (number? n1)))
                (snewt-message title
                  (_ "Enter only digit."))
                (ask-time h m))
              ((not (and (<= 0 n0) (<= n0 23) (<= 0 n1) (<= n1 60)))
                (snewt-message title
                  (_ "Enter values in the correct range."))
                (ask-time h m))
              (else
                (list n0 n1))))))))
  
  (define (ask-week)
    (let ((r (snewt-nmenu title
               (_ "Which day of week the data refresh begin?")
               50 5 5 7
               (vector (_ "Sunday") (_ "Monday") (_ "Tuesday") (_ "Wednesday")
                       (_ "Thursday") (_ "Friday") (_ "Saturday"))
               (vector (_ "Ok") (_ "Cancel")))))
      (case (car r)
        ((-1 2)
          #f)
        (else
          (ntoweek (cdr r))))))
  
  (define (success)
    (obj-cr 'restart-if-run)
    (obj-pd 'set-initstep 90)
    (snewt-message title
      (_ "Automatic data refresh was successfully configured."))
    #t)
  
  (let* ((s (obj-cr 'get 'orca))
         (r (snewt-nmenu title
              (string-append
                (if (null? s)
                  (string-append
                    (_ "You are strongly recommended to set automatic data refresh for secondary server. ")
                    (_ "With automatic data refresh, data synchronization will be more steady data than simple DB duplication. ")
                    "\n\n"
                    (_ "Do you set automatic data refresh? "))
                  (string-append
                    (_ "Data refresh is scheduled as below:\n\n")
                    (format #f (_ "   ~a at ~a:~a\n\n")
                      (let ((w (list-ref s 4)))
                        (case w
                          ((none) (_ "Every day"))
                          (else
                            (list-ref
                              (list (_ "Every Sunday") (_ "Every Monday") (_ "Every Tuesday") (_ "Every Wednesday")
                                    (_ "Every Thursday") (_ "Every Friday") (_ "Every Saturday"))
                              (weekton w)))))
                      (sprintf "%02d" (cadr s))
                      (sprintf "%02d" (car s)))
                    (_ "Do you modify the schedule? "))))
              60 5 5 1
              (vector)
              (vector (_ "Set Every Day") (_ "Set Every Week") (_ "No Schedule") (_ "Leave as current")))))
    (case (car r)
      ((0)
        #f)
      ((4)
        #t)
      ((3)
        (cond
          ((obj-cr 'remove 'orca)
            #t)
          (else
            (snewt-message title "Failed to set.")
            #f)))
      ((1)
        (let ((r (ask-time (if (null? s) -1 (cadr s)) (if (null? s) -1 (car s)))))
          (cond
            ((null? r)
              (newt-ui-orca-schedule obj-pd obj-cr))
            ((not (obj-cr 'update 'orca (cadr r) (car r) -1 -1 'none "root" (obj-pd 'auto-refresh-cmd)))
              (snewt-message title "Failed to set.")
              #f)
            (else
              (success)))))
      ((2)
        (let ((r1 (ask-week)))
          (cond
            ((eq? #f r1)
              (newt-ui-orca-schedule obj-pd obj-cr))
            (else
              (let ((r2 (ask-time (if (null? s) -1 (cadr s)) (if (null? s) -1 (car s)))))
                (cond
                  ((null? r2)
                    (newt-ui-orca-schedule obj-pd obj-cr))
                  ((not (obj-cr 'update 'orca (cadr r2) (car r2) -1 -1 r1 "root" (obj-pd 'auto-refresh-cmd)))
                    (snewt-message title "Failed to set.")
                    #f)
                  (else
                    (success))))))))
      (else
        #f))))

(define (newt-ui-orca-recover-secondary obj-pd)
  (define title (_ "Secondary server recovery"))
  (let ((hs (obj-pd 'get-secondary)))
    (cond
      ((equal? "" hs)
        (snewt-message title
          (_ "Secondary server is not set yet."))
        #f)
      ((not (unix-connect? hs 5432))
        (snewt-message title
          (string-append
             (format #f (_ "Could not connect to the secondary server (~a). ") hs)
             (_ "In order to start recovery, login the secondary server and change the status to 'wait' with ogl-admin. ")))
        #f)
      ((not (= 1 (snewt-choice title
                   (string-append
                     (_ "With this recovery process, data in the secondary server will be copied from the primary server. After the recovery, data in the primary and the secondary will be identical. \n\n")
                     (_ "In order to recover the secondary server, you have to change the primary server status to 'semi wait'. ")
                     (_ "This change will interrupt ORCA service during the recovery. ")
                     (_ "The recovery process will take very long time, accodring to the amount of data. ")
                     (_ "Are you Ok to start the recovery? "))
                   (_ "Yes") (_ "No"))))
        #f)
      (else
        (let ((r1 (snewt-show-while title
                    (_ "Changing server status ... ")
                    50 3
                    (lambda () (obj-pd 'semiwait-server)))))
          (let ((r2 (snewt-show-while title
                      (_ "Recovering data ... ")
                      50 3
                      (lambda () (obj-pd 'recover-secondary)))))
            (cond
              ((not (= 0 r2))
                (snewt-message title
                  (format #f (_ "Recovery failed (~a).") r2))
                #f)
              ((= 1 (snewt-choice title
                      (string-append
                        (_ "Data was successfully recovered. ")
                        (_ "Do you restart ORCA services? "))
                      (_ "Yes") (_ "No")))
                (snewt-show-while title
                  (_ "Changing server status to 'start' ... ")
                  50 3
                  (lambda () (obj-pd 'run-server)))
                #t)
              (else
                (snewt-show-while title
                  (_ "Changing server status to 'wait' ... ")
                  50 3
                  (lambda () (obj-pd 'wait-server)))
                #t))))))))

(define (newt-ui-orca-secondary-recover-check obj-pd)
  (cond
    ((not (obj-pd 'secondary-busy?))
      #t)
    (else
      (= 1 (snewt-choice (_ "Warning")
              (string-append
                (_ "It is possible that the (automatic) data recovery may be proceeding. If so, a system change on the secondary server may cause the serious inconsystemcy with the dual DB. ")
                "\n\n"
                (_ "Do you want to continue?"))
              (_ "Yes") (_ "No"))))))

(define (newt-ui-orca-recover-primary obj-pd)
  (define title (_ "Primary server recovery"))
  (let ((hs (obj-pd 'get-secondary)))
    (cond
      ((equal? "" hs)
        (snewt-message title
          (_ "Secondary server is not set yet."))
        #f)
      ((unix-connect? hs 8000)
        (snewt-message title
          (string-append
            (_ "Secondary server seems to be running. ('work' state). ")
            (_ "In order to start recovery, login the secondary server and change the status to 'wait' with ogl-admin. ") hs))
        #f)
      ((not (unix-connect? hs 5432))
        (snewt-message title
          (string-append
            (format #f (_ "Could not connect to the secondary server (~a). " hs)
            (_ "In order to start recovery, login the secondary server and change the status to 'wait' with ogl-admin. "))))
        #f)
      ((not
        (and 
          (= 1 (snewt-choice title
                 (string-append
                   (_ "With this recovery process, data in the primary server will be copied from the secondary server. After the recovery, data in the primary and the secondary will be identical. \n\n")
                   (_ "This menu should be only used when the primary server (this host) is just after a (physical) crush. ")
                   (_ "Othersize data in the primary will be damaged seriously. ")
                   "\n\n"
                   (_ "In order to recover the primary server, you have to change the primary server status to 'semi wait'. ")
                   (_ "The recovery process will take very long time, accodring to the amount of data. ")
                   (_ "Are you Ok to start the recovery? "))
                 (_ "Yes") (_ "No")))
          (= 1 (snewt-choice title
                 (string-append
                   (_ "Confirmation. \n\n")
                   (_ "I ask you again because this is a very critical process. ")
                   "\n\n\n"
                   (_ "   DATA WILL BE COPIED FROM THE SECONDARY SERVER TO THE PRIMARY SERVER (THIS HOST)")
                   "\n\n"
                   (_ "Are you really Ok to start the recovery? "))
                 (_ "Yes") (_ "No")))))
        #f)
      (else
        (let ((r1 (snewt-show-while title
                    (_ "Changing server status ... ")
                    50 3
                    (lambda () (obj-pd 'semiwait-server)))))
          (let ((r2 (snewt-show-while title
                      (_ "Recovering data ... ")
                      50 3
                      (lambda () (obj-pd 'recover-primary)))))
            (cond
              ((not (= 0 r2))
                (snewt-message title
                  (format #f (_ "Recovery failed (~a).") r2))
                #f)
              ((= 1 (snewt-choice title
                      (string-append
                        (_ "Data was successfully recovered. ")
                        (_ "Do you restart ORCA services? "))
                      (_ "Yes") (_ "No")))
                (snewt-show-while title
                  (_ "Changing server status to 'start' ... ")
                  50 3
                  (lambda () (obj-pd 'run-server)))
                #t)
              (else
                (snewt-show-while title
                  (_ "Changing server status to 'wait' ... ")
                  50 3
                  (lambda () (obj-pd 'wait-server)))
                #t))))))))

(define (newt-ui-orca-local-dbcheck title obj-pd)
  (cond
    ((= 0 (sh "echo \"select usr from orca_acc\" | su orca -c 'psql -h localhost orca' 2>&1"))
      #t)
    ((not (= 1 (snewt-choice title
                 (string-append
                   (_ "Could not access to the local database. ")
                   (_ "In order to modify account information, the server should be in 'wait' or 'start' status. ")
                   "\n\n"
                   (_ "Do you change the server status to 'wait' now? Otherwise setting will be aborted."))
                 (_ "Yes") (_ "No"))))
      #f)
    (else
      (let ((r (snewt-show-while (_ "Changing Server")
                  (_ "Changing service status to 'wait' ...") 50 3
                  (lambda () (obj-pd 'wait-server)))))
        (cond
          ((null? r)
            (snewt-message (_ "Info")
              (_ "Service(s) status are successfully changed."))
            #t)
          (else
            (snewt-message (_ "Error")
              (string-append
                (format #f (_ "Could not change service(s) status below: \n~a")
                  (list-foldstring " " r))
                "\n"
                (_ "Please check the system first.")))
            #f))))))

(define (newt-ui-orca-master obj-pd)
  (define title (_ "Master Password setting"))
  (cond
    ((not (newt-ui-orca-initcheck obj-pd))
      #f)
    ((not (newt-ui-orca-secondary-check obj-pd))
      #f)
    ((not (newt-ui-orca-local-dbcheck title obj-pd))
      #f)
    (else
      (let* ((r (snewt-entries title
                  (_ "Enter new master user password twice.\n")
                  50 5 5 20
                  (list (list (_ "Password") "" snewt-flag-password)
                        (list (_ "Password(confirm)") "" snewt-flag-password))))
             (obj-orca (conf-orca)))
        (cond
          ((null? r)
            #f)
          ((or (equal? "" (car r)) (equal? "" (cadr r)))
            (snewt-message (_ "Info") (_ "Master password has not been modified."))
            #f)
          ((not (equal? (car r) (cadr r)))
            (snewt-message (_ "Info") (_ "Passwords don't match..\n"))
            (newt-ui-orca-master obj-pd))
          (else
            (let ((r1 (obj-orca 'passwd "ormaster" (car r))))
              (cond
                ((string? r1)
                  (snewt-message (_ "Error") r1)
                  #f)
                (else
                  (snewt-message (_ "Success") (_ "Master password has been successfully modified."))
                  #t)))))))))


(define (newt-ui-orca-secondary-check obj-pd)
  (cond
    ((not (equal? 'secondary (obj-pd 'get-orcatype)))
      #t)
    (else
      (= 1 (snewt-choice (_ "Warning")
             (string-append
               (_ "Warning!\n\nThis host is set as secondary ORCA server. ")
               (_ "If primary server is working, don't edit user information on the secondary. Instead do it on the primary. \n\n")
               (_ "Otherwise setting user on the secondary may cause system inconsistency. \n\n")
               (_ "Do you proceed anyway?"))
             (_ "Yes") (_ "No"))))))

(define (newt-ui-orca-initcheck obj-pd)
  (cond
    ((< 40 (obj-pd 'get-initstep))
      #t)
    (else
      (snewt-message (_ "Error")
        (_ "ORCA server is not initialized yet. Please change server status 'Waited' or 'Started' first."))
      #f)))

(define (newt-ui-orca-acc obj-pd)
  (define title (_ "Operator Account setting"))
  (define obj-orca (conf-orca))
  (define (adduser u)
    (let ((r (snewt-entries title
               (_ "Enter ORCA user ID and password (twice).\n")
               50 5 5 20
               (list
                 (list (_ "User ID") "" 0)
                 (list (_ "Password") "" snewt-flag-password)
                 (list (_ "Password(confirm)") "" snewt-flag-password)))))
      (cond
        ((null? r)
          #f)
        ((or (equal? "" (car r)) (equal? "" (caddr r)))
          (snewt-message (_ "Info") (_ "Account not generated.\n"))
          #f)
        ((set-include? (car r) u)
          (snewt-message (_ "Error")
            (format #f (_ "User ~a already exists.") (car r)))
          (adduser u))
        ((not (equal? (cadr r) (caddr r)))
          (snewt-message (_ "Info") (_ "Passwords don't match..\n"))
          (adduser u))
        (else
          (let ((r1 (obj-orca 'adduser (car r) (cadr r))))
            (cond
              ((string? r1)
                (snewt-message (_ "Error") r1)
                #f)
              (else
                (snewt-message (_ "Success")
                  (string-append
                    (_ "Account successfully generated.\n\n")
                    (_ "Notice that this account is not valid yet. In order to validate this operator account, you have to configure further information of this account via ORCA menu. ")
                    (_ "Login ORCA first with 'ormaster' account and configure it.")))
                #t)))))))
  
  (define (edituser n)
    (let ((r (snewt-entries title
               (format #f (_ "Enter password for user ~a (twice).\n") n)
               50 5 5 20
               (list (list (_ "Password") "" snewt-flag-password)
                     (list (_ "Password(confirm)") "" snewt-flag-password)))))
      (cond
        ((null? r)
          #f)
        ((or (equal? "" (car r)) (equal? "" (cadr r)))
          (snewt-message (_ "Info") (_ "Password has not been modified.\n"))
          #f)
        ((not (equal? (car r) (cadr r)))
          (snewt-message (_ "Info") (_ "Passwords don't match..\n"))
          (edituser n))
        (else
          (let ((r1 (obj-orca 'passwd n (car r))))
            (cond
              ((string? r1)
                (snewt-message (_ "Error") r1)
                #f)
              (else
                (snewt-message (_ "Success") (_ "Password successfully modified."))
                #t)))))))
  
  (define (deluser n)
    (let ((r (snewt-choice title
               (format #f (_ "Are you ok to remove user ~a ?") n)
               (_ "Yes") (_ "No"))))
      (case r
        ((0 2) #t)
        (else
          (let ((r1 (obj-orca 'deluser n)))
            (cond
              ((string? r1)
                (snewt-message (_ "Error") r1)
                #f)
              (else
                (snewt-message (_ "Success")
                  (string-append
                    (format #f (_ "User ~a successfully removed.\n\n") n)
                    (_ "Notice that this account is not fully remove yet. In order toremove this account completely, you have to configure further information of this account via ORCA menu. ")
                    (_ "Login ORCA first with 'ormaster' account and configure it.")))
                #t)))))))
  
  (cond
    ((not (newt-ui-orca-initcheck obj-pd))
      #f)
    ((not (newt-ui-orca-secondary-check obj-pd))
      #f)
    ((not (newt-ui-orca-local-dbcheck title obj-pd))
      #f)
    (else
      (let* ((u (obj-orca 'user-list)))
        (cond
          ((not (list? u))
            (snewt-message (_ "Error")
              (string-append
                (_ "Could not access to the local database. ")
                (_ "Please check the system first.")))
            #f)
          (else
            (let ((r1 (snewt-nmenu title
                        (string-append
                          (_ "With this menu, you can edit operator account used for ORCA.\n\n")
                          (_ "To change password, select existing ORCA user name. ")
                          (_ "To create or delete ORCA account, select 'new account/delete'. \n\n"))
                        50 5 5 20
                        (list->vector
                          (map (lambda (x) (format #f "  ~a  " x)) u))
                        (vector (_ "Ok") (_ "New Account") (_ "Delete")))))
             (case (car r1)
               ((-1) #f)
               ((1) #t)
               (else
                 (case (car r1)
                   ((2)
                     (adduser u))
                   ((3)
                     (cond
                       ((< (cdr r1) (length u))
                         (deluser (list-ref u (cdr r1))))
                       (else
                         (snewt-message title
                           (_ "Could not delete user.")))))
                   (else
                     (edituser (list-ref u (cdr r1)))))
                 (newt-ui-orca-acc obj-pd))))))))))

(define (newt-ui-orca-type obj-pd patch obj-prof obj-cr)
  (define title (_ "ORCA Configuration"))
  (cond
    ((not (equal? 'unset (obj-pd 'get-orcatype)))
      (snewt-message (_ "Error")
        (_ "This host is already configured."))
      #f)
    (else
      (let* ((l (list (_ "Primary Server of Dual server") (_ "Secondary Server of Dual server")
                      (_ "Single Server") (_ "Client")))
             (r (snewt-nmenu title
                  (string-append
                    (_ "Select ORCA host type configuration.\n")
                    (_ "Be careful that once you select type of host, it cannot be changed later. "))
                  50 5 5 5
                  (list->vector l)
                  (vector (_ "Ok") (_ "Skip")))))
        (case (car r)
          ((-1 2) #f)
          (else
            (let ((r1 (snewt-choice (_ "Confirm")
                        (format #f (_ "This host will be set as ORCA host type\n\n  : ~a\n\nAre ok with this setting?")
                          (list-ref l (cdr r)))
                        (_ "Yes") (_ "No"))))
              (case r1
                ((0 2)
                  (newt-ui-orca-type obj-pd patch obj-prof obj-cr))
                (else
                  (case (cdr r)
                    ((3) #t)
                    (else
                      (obj-prof 'update-line "psql default host" "PGHOST=localhost\nexport PGHOST")
                      (obj-prof 'update-line "psql default port" "PGPORT=5432\nexport PGPORT")))
                  (case (cdr r)
                    ((0)  (newt-ui-orca-type-primary obj-pd patch obj-cr))
                    ((1)  (newt-ui-orca-type-secondary #t obj-pd patch))
                    ((2)  (newt-ui-orca-type-single obj-pd patch))
                    (else (newt-ui-orca-type-client obj-pd))))))))))))

(define (newt-ui-orca-resolv-check obj-pd h)
  (cond
    ((obj-pd 'ip-or-resolv? h)
      #t)
    (else
      (snewt-message (_ "Info")
        (format #f (_ "Address '~a' is neither IP address nor resolvable name. ") h))
      #f)))

(define (newt-ui-orca-type-client obj-pd)
  (define title (_ "Client Setting"))
  (obj-pd 'set-orcatype 'client)
  (obj-pd 'set-initstep 100)
  (let* ((gl (obj-pd 'get-gl))
         (r (snewt-entries title
              (_ "Enter ORCA Server host or IP.\n")
              70 5 5 40
              (list (list (_ "(Primary) Server : ") (car gl) 0)
                    (list (_ "Secondary Server (if exists) :") (cadr gl) 0)))))
    (cond
      ((or (null? r) (equal? "" (car r)))
        #f)
      (else
        (let* ((h0 (car r))
               (h1 (cadr r))
               (single? (equal? "" (cadr r))))
          (cond
            ((not (andmap (lambda (x) (newt-ui-orca-resolv-check obj-pd x))
                          (if single? (list h0) r)))
              (newt-ui-orca-type-client obj-pd))
            (else
              (let ((c0 (if (obj-pd 'connect? h0)
                          #t
                          (= 1 (snewt-choice (_ "Warning")
                                 (format #f (_ "Could not get response from server ~a. Do you proceed anyway?") h0)
                                 (_ "Yes") (_ "No")))))
                    (c1 (if single? #t
                          (if (obj-pd 'connect? h1)
                            #t
                            (= 1 (snewt-choice (_ "Warning")
                                   (format #f (_ "Could not get response from server ~a. Do you proceed anyway?") h1)
                                   (_ "Yes") (_ "No")))))))
                (inst-log "c0 : ~a, c1 : ~a\n" c0 c1)
                (cond
                  ((not (and c0 c1))
                    (newt-ui-orca-type-client obj-pd))
                  (else
                    (obj-pd 'update-primary h0 8000)
                    (if (not single?)
                      (obj-pd 'update-secondary h1 8000))
                    (snewt-message title
                      (_ "ORCA client has been successfully configured."))
                    #t))))))))))

(define (newt-ui-orca-init obj-pd)
  (define obj-orca (conf-orca))
  (define title (_ "ORCA Server Initialization"))
  (cond
    ((or (case (obj-pd 'get-orcatype)
            ((primary single) #f)
            (else #t))
         (< 60 (obj-pd 'get-initstep)))
      #t)
    ((and (< (obj-pd 'get-initstep) 40)
          (not (snewt-show-while title
                  (_ "Initializing ORCA server.\nThis step takes a few minutes. Please wait a moment.") 50 3
                  (lambda ()
                    (and (obj-pd 'wait-server)
                         (obj-pd 'init-orca)
                         (obj-orca 'make-master)
                         (obj-pd 'commit))))))
      (snewt-message (_ "Error")
        (_ "Failed to initialize ORCA. Make sure that ORCA installation is complete. (or consult ARMA Net support.)"))
      #f)
    (else
      (obj-pd 'set-initstep 40)
      (let rep ((r (or (< 60 (obj-pd 'get-initstep))
                       (newt-ui-orca-hospid obj-pd))))
        (cond
          (r
            (obj-pd 'set-initstep 60)
            (obj-pd 'commit)
            r)
          ((= 1 (snewt-choice title
                  (string-append
                    (_ "Initialization could not be finished until hospital ID is set. ")
                    (_ "ORCA cannot work without setting hospital ID. \n\n")
                    (_ "Do you skip setting of hospital ID anyway?"))
                  (_ "Skip") (_ "Set Again")))
            #f)
          (else
            (rep (newt-ui-orca-hospid obj-pd))))))))

(define (newt-ui-orca-primary-secondary-test obj-pd h)
  (define title (_ "Secondary Access Test"))
  (let ((r (snewt-show-while title
             (_ "Accessing to the secondary server ...") 50 3
             (lambda () (obj-pd 'secondary-test h)))))
    (cond
      ((eq? #t r)
        #t)
      (else
        (snewt-message title
          (format #f (_ "Could not access to the secondary server. \nReason : ~a") r))
        #f))))

(define (newt-ui-orca-primary-host first? obj-pd patch)
  (define title (_ "Secondary Server Address"))
  (define (not-changed)
    (snewt-message title
      (_ "The secondary server address has not been changed. "))
    #t)
  
  (let* ((n (obj-pd 'get-initstep))
         (gl (obj-pd 'get-gl))
         (st (obj-pd 'stat-server))
         (r (snewt-entries title
              (_ "Enter ORCA secondary server host address (hostname or IP).\n")
              70 5 5 40
              (list (list (_ "Secondary Server :") (cadr gl) 0)))))
    (cond
      ((or (null? r) (equal? "" (car r)))
        (not-changed))
      ((not (newt-ui-orca-resolv-check obj-pd (car r)))
        (newt-ui-orca-primary-host first? obj-pd patch))
      (else
        (let* ((h0 (car r))
               (con? (newt-ui-orca-primary-secondary-test obj-pd h0))
               (c0 (if con?
                     #t
                     (= 1 (snewt-choice (_ "Warning")
                            (format #f (_ "Could not connect to the secondary server ~a. Do you want to continue?") h0)
                            (_ "Yes") (_ "No"))))))
          (cond
            ((not c0)
              (newt-ui-orca-primary-host first? obj-pd patch))
            (else
              (obj-pd 'set-initstep 20)
              (obj-pd 'update-primary "127.0.0.1" 8000)
              (obj-pd 'update-secondary h0 8000)
              (patch 'commit)
              (obj-pd 'commit)
              (cond
                ((equal? (car r) (cadr gl))
                  (not-changed))
                ((and (not first?)
                   (not (= 1 (snewt-choice title
                               (case st
                                 ((work wait semiwait)
                                   (string-append
                                     (_ "In order to reflect the system change, ORCA server has to be needed to restart. ")
                                     (_ "Do you restart the ORCA server now?")))
                                 (else
                                   (_ "Do you start the ORCA server now?")))
                               (_ "Yes") (_ "No")))))
                  #t)
                ((not (and (obj-pd 'stop-server)
                        (case st
                          ((wait) (obj-pd 'wait-server))
                          ((semiwait) (obj-pd 'semiwait-server))
                          (else (obj-pd 'run-server)))))
                  (snewt-message title
                    (_ "Failed to restart servers. Please check the system first."))
                  #f)
                (else
                  (snewt-message title
                    (_ "ORCA primary server has been successfully configured."))
                  #t)))))))))

(define (newt-ui-orca-primary-mtacheck title obj-pd)
  (cond
    ((obj-pd 'have-mail?)
      #t)
    ((not (= 1 (snewt-choice title
                 (string-append
                   (_ "Mail server seems not to be running. ")
                   "\n\n"
                   (_ "With the dual ORCA configuration, unix user 'orca' can accept the alert mail from log checker, which sees the DB replicator log and will send alert if it find the error. ")
                   (_ "This function may help the administrator to find a early corruption of the dual system. ")
                   (_ "In order to get this alert, you need to start mail server. ")
                   "\n\n"
                   (_ "Do you start the mail server?"))
                 (_ "Yes") (_ "No"))))
      #f)
    ((not (snewt-show-while title
            (_ "Starting the mail server ...") 50 3
            (lambda () (obj-pd 'start-mail))))
      (snewt-message title
        (_ "Failed to start the mail server. Please investigate the system later."))
      #f)
    (else
      #t)))

(define (newt-ui-orca-type-primary obj-pd patch obj-cr)
  (define title (_ "Primary Server Setting"))
  (define (set-host?)
    (cond
      ((< (obj-pd 'get-initstep) 20)
        #t)
      (else
        (= 1 (snewt-choice title
               (_ "Do you reconfigure secondary server address setting?")
               (_ "Yes") (_ "No"))))))
  
  (obj-pd 'set-orcatype 'primary)
  (obj-pd 'stop-server)
  (cond
    ((and (set-host?)
          (not (newt-ui-orca-primary-host #t obj-pd patch)))
      #f)
    (else
      (let* ((n (obj-pd 'get-initstep))
             (gl (obj-pd 'get-gl)))
        (cond
          ((not (eq? #t (obj-pd 'secondary-test (cadr (obj-pd 'get-gl)))))
            (snewt-message title
              (string-append
                (_ "The ORCA primary server has been partially configured. ")
                "\n\n"
                (_ "To complete the configuration, please configure and start the secondary server first. ")
                (_ "Then run the ogl-admin on the primary server(this host) and select ORCA configuration again. ")))
            #f)
          ((not (and (obj-pd 'wait-server)
                     (if (< n 60) (newt-ui-orca-init obj-pd) #t)
                     (if (< n 80) (newt-ui-orca-master obj-pd) #t)))
            #f)
          (else
            (newt-ui-orca-schedule obj-pd obj-cr)
            (obj-pd 'set-initstep 100)
            (newt-ui-orca-primary-mtacheck title obj-pd)
            (cond
              ((= 1 (snewt-choice title
                      (_ "ORCA primary server was configured. Do you start ORCA server now?")
                      (_ "Yes") (_ "No")))
                (obj-pd 'run-server))
              (else
                #t))))))))

(define (newt-ui-orca-type-single obj-pd patch)
  (obj-pd 'set-orcatype 'single)
  (obj-pd 'update-primary "127.0.0.1" 8000)
  (patch 'commit)
  (obj-pd 'commit)
  (obj-pd 'set-initstep 20)
  (let ((n (obj-pd 'get-initstep)))
    (cond
      ((not (and (obj-pd 'wait-server)
                 (if (< n 60) (newt-ui-orca-init obj-pd) #t)
                 (if (< n 80) (newt-ui-orca-master obj-pd) #t)))
        #f)
      (else
        (obj-pd 'set-initstep 100)
        (cond
          ((= 1 (snewt-choice (_ "Single Server Setting")
                  (_ "ORCA was configured. Do you start ORCA server now?")
                  (_ "Yes") (_ "No")))
            (obj-pd 'run-server))
          (else
            #t))))))

(define (newt-ui-orca-type-secondary first? obj-pd patch)
  (define title (_ "Primary Server Address"))
  (obj-pd 'set-orcatype 'secondary)
  (define (not-changed)
    (snewt-message title
      (_ "The primary server address has not been changed. ")))
    
  (let* ((gl (obj-pd 'get-gl))
         (st (obj-pd 'stat-server))
         (r (snewt-entries title
              (_ "Enter ORCA primary server address (host or IP) .\n")
              70 5 5 40
              (list (list (_ "Primary Server :") (car gl) 0)))))
    (cond
      ((or (null? r) (equal? "" (car r)))
        (not-changed)
        #f)
      ((equal? (car r) (car gl))
        (not-changed)
        #t)
      ((not (newt-ui-orca-resolv-check obj-pd (car r)))
        (newt-ui-orca-type-secondary first? obj-pd patch))
      (else
        (snewt-message (_ "Info")
          (if (not (obj-pd 'connect-backend? (car r)))
            (string-append
              (_ "Could not connect to the primary server. In most case this is not an error, because secondary server is configured first. ")
              "\n\n"
              (_ "Please do not forget to configure the primary server after finishing ogl-admin. "))
            (string-append
              (_ "Primary server seems to be already working. ")
              (_ "If you configure the secondary server (this host) for the first time, please do not forget to run 'Secondary server data recovery' menu of ogl-admin on the primary server. "))))
        (obj-pd 'update-secondary "127.0.0.1" 8000)
        (obj-pd 'update-primary (car r) 8000)
        (obj-pd 'set-initstep 100)
        (patch 'commit)
        (obj-pd 'commit)
        (cond
          ((and (not first?)
             (not (= 1 (snewt-choice title
                         (case st
                           ((wait)
                             (string-append
                               (_ "In order to reflect the system change, ORCA server has to be needed to restart. ")
                               (_ "Do you restart the ORCA server now?")))
                           (else
                             (_ "Do you start the ORCA server now?")))
                         (_ "Yes") (_ "No")))))
            #t)
          ((not (and (obj-pd 'stop-server)
                  (snewt-show-while title
                    (_ "Changing service status to 'wait' ...") 50 3
                    (lambda ()
                      (case st
                        ((wait) (obj-pd 'wait-server))
                        (else (obj-pd 'run-server)))))))
            (snewt-message title
              (_ "Failed to restart servers. Please check the system first."))
            #f)
          (else
            (snewt-message title
              (_ "ORCA secondary server has been successfully configured."))
            #t))))))

(define (newt-ui-secondary-check obj-pd)
  (cond
    ((or (not (equal? 'primary (obj-pd 'get-orcatype)))
         (unix-connect? (obj-pd 'get-secondary) 5432))
      #t)
    ((equal? "" (obj-pd 'get-secondary))
      (snewt-message (_ "Error")
        (_ "Secondary server is not specified yet."))
      #f)
    (else
      (let ((r (snewt-choice (_ "Warning")
                 (string-append
                   (_ "Secondary server does not seem to be in service.\n\n")
                   (_ "With dual host configuration, primary server should not start unless secondary server has trouble. ")
                   (_ "Otherwise primary and secondary DB will be in inconsistent status.")
                   (_ "\n\nDo you proceed anyway?"))
                 (_ "Yes") (_ "No"))))
        (= r 1)))))

(define (newt-ui-orca-server obj-pd)
  (define title (_ "Server Status"))
  (let* ((t (obj-pd 'get-orcatype))
         (r (snewt-nmenu title
              (string-append
                (format #f (_ "Current server status is : ~a\n\n") (obj-pd 'desc-server))
                (_ "   'Start' : start all server\n")
                (_ "   'Stop'  : stop all server\n")
                (_ "   'Wait'  : start DB server/stop application server\n"))
              60 5 5 5
              (vector)
              (vector (_ "Ok") (_ "Start") (_ "Stop") (_ "Wait")))))
    (case (car r)
      ((-1 0 1) #t)
      ((2)
        (cond
          ((not (newt-ui-secondary-check obj-pd))
            #f)
          ((and (equal? 'secondary t)
                (unix-connect? (obj-pd 'get-primary) 5432))
            (snewt-message (_ "Error")
              (string-append
                (_ "Primary server seem to be in service.\n\n")
                (_ "With dual host configuration, secondary ORCA server cannot start when primary server works.")))
            #f)
          ((and (equal? 'secondary t)
                (not (= 1 (snewt-choice title
                            (string-append
                              (_ "Warninig!\n\n")
                              (_ "This host is secondary server and ORCA server should not be run unless the primary server is down.\n\n")
                              (_ "Though primary server does not seem to be in service, you are warned because it is important.\n\n")
                              (_ "Do you proceed?"))
                            (_ "Yes") (_ "No")))))
            #f)
          (else
            (newt-ui-orca-init obj-pd)
            (let ((r1 (snewt-show-while (_ "Staring Server")
                        (_ "Staring server ...") 50 3
                        (lambda () (obj-pd 'run-server)))))
              (cond
                ((null? r1)
                  (snewt-message (_ "Info")
                    (_ "Service(s) are successfully started."))
                  #t)
                (else
                  (snewt-message (_ "Warning")
                    (format #f (_ "Service(s) ~a could not be started.")
                      (list-foldstring " " r1)))
                  #t))))))
      ((3)
        (let ((r1 (snewt-show-while (_ "Stoppping Server")
                    (_ "Stopping server ...") 50 3
                    (lambda () (obj-pd 'stop-server)))))
          (cond
            ((null? r1)
              (snewt-message (_ "Info")
                (_ "Service(s) are successfully stopped."))
              #t)
            (else
              (snewt-message (_ "Error")
                (format #f (_ "Could not stop service(s) below: ~a")
                  (list-foldstring " " r1)))
              #f))))
      (else
        (let ((r1 (snewt-show-while (_ "Changing Server")
                    (_ "Changing server status to 'wait' ...") 50 3
                    (lambda () (obj-pd 'wait-server)))))
          (newt-ui-orca-init obj-pd)
          (cond
            ((null? r1)
              (snewt-message (_ "Info")
                (_ "Service(s) status are successfully changed."))
              #t)
            (else
              (snewt-message (_ "Error")
                (format #f (_ "Could not change service(s) status below: \n~a")
                  (list-foldstring " " r1)))
              #f)))))))

(define (newt-ui-orca-check-wait title m obj-pd)
  (cond
    ((equal? 'wait (obj-pd 'stat-server))
      #t)
    ((not (= 1 (snewt-choice title
                 (string-append
                   m
                   "\n"
                   (_ "Do you want to change the server status now?"))
                 (_ "Yes") (_ "No"))))
      #f)
    (else
      (let ((r (snewt-show-while title
                 (_ "Changing server status to 'wait' ...") 50 3
                 (lambda () (obj-pd 'wait-server)))))
        (cond
          ((null? r)
            #t)
          (else
            (snewt-message (_ "Error")
              (format #f (_ "Could not change service(s) status below: \n~a")
                (list-foldstring " " r)))
            #f))))))

(define (newt-ui-orca-hospid obj-pd)
  (define title (_ "Hostpital ID Setting"))
  (cond
    ((not (newt-ui-orca-check-wait title
            (_ "To change hostpital ID, the ORCA server should be in the wait status. ")
            obj-pd))
      #f)
    ((not (newt-ui-secondary-check obj-pd))
      #f)
    (else
      (let ((r (snewt-entries title
                  (_ "Enter Hostpital ID.\n")
                  70 5 5 40
                  (list (list (_ "Hostpital ID :") (obj-pd 'get-medid) 0)))))
        (cond
          ((or (null? r) (equal? "" (car r)))
            (snewt-message (_ "Info")
              (_ "Hospital ID was not modified."))
            #f)
          (else
            (let ((r1 (snewt-show-while title
                        (_ "Changing ID ...\nThis step takes a few minites. Please wait a moment.") 50 3
                        (lambda () (obj-pd 'update-medid (car r))))))
              (cond
                (r1
                  (snewt-message (_ "Info")
                    (format #f (_ "Hospital ID was successfully modified.")))
                  #t)
                (else
                  (snewt-message (_ "Error")
                    (_ "Could not change hospital ID, sorry. Please execute /home/orca/orca-hospid-set.sh script manually."))
                  #f)))))))))

(define (newt-ui-support-open obj-na obj-acc)
  (define title (_ "Remote Maintenance Mode"))
  (let ((r (snewt-choice title
             (string-append
               (_ "Do you start remote maintenance mode?\n\n")
               (_ "Preesing ok will make the system :\n")
               (_ "  - connect VPN using temporary account\n")
               (_ "  - unlock maintenance account 'sup'\n\n")
               (_ "We(AMRA Net supporter) promise our investigation is limited within the host in general, and does not expand to your local network without obvious need for the problem solving.")
               )
             (_ "Yes") (_ "No"))))
    (case r
      ((0 2) #f)
      (else
        (let ((id (obj-na 'get-stdid))
              (a1 (= 1 (sh "ifconfig | grep 'addr:10.0.' >/dev/null")))
              (a2 (= 1 (sh "ifconfig | grep 'addr:172.16.' >/dev/null")))
              (a3 (= 1 (sh "ifconfig | grep 'addr:192.168.' >/dev/null"))))
          (cond
            ((equal? "" id)
              (snewt-choice title
                (string-append
                  (_ "ARMA Net account is not set yet. ")
                  (_ "Please quit this menu and configure 'ARMA Net account' first. ")))
              #f)
            ((not (or a1 a2 a3))
              (snewt-message (_ "Fatal Error")
                (_ "Could not find valid usable network space. sorry"))
              #f)
            ((not (= 0 (sh "modprobe tun")))
              (snewt-message (_ "Fatal Error")
                (string-append
                  (_ "Could not install network tunnel driver. ")
                  (_ "Run ARMA original kernel or turn on 'tun' as module for your kernel. ")))
              #f)
            (else
              (let* ((tmp (unix-tempname))
                     (prefix (cond (a1 "10.0.") (a2 "172.16.") (else "192.168.")))
                     (r1 (snewt-show-while title
                           (_ "Requesting VPN to ARMA Net ...") 50 3
                           (lambda () (sh "wget -O ~a 'https://202.216.241.68/book.php?ID=~a&NET=~a'"
                                          tmp id prefix)))))
                (sh "chmod og-rw ~a" tmp)
                (let ((r2 (ttable-read tmp '((str str)
                                             (int int str str str)))))
                  (cond
                    ((or (eq? #f r2) (null? r2))
                      (snewt-message (_ "Error")
                        (_ "Could not get a valid response from server. Consult ARMA Net Web support directly, sorry."))
                      #f)
                    (else
                      (let ((vpn (car r2)))
                        (cond
                          ((= 2 (length vpn))
                            (snewt-message (_ "Error")
                              (format #f (_ "Authentication failed. Account invalid?\nServer responded : ~a") (cadr vpn)))
                             #f)
                          (else
                            (let ((sv (format #f "~a~a.~a" prefix (car vpn) (cadr vpn)))
                                  (cl (format #f "~a~a.~a" prefix (car vpn) (1+ (cadr vpn)))))
                              (cond
                                ((not (fprintf tmp
                                        (string-append
                                          "options {\n  timeout 60;\n  ifconfig    /sbin/ifconfig;\n  route       /sbin/route;\n}\n\n"
                                          (format #f "~a {\n  pass ~a;\n  up {\n    ifconfig \"%% inet ~a pointopoint ~a\";\n"
                                            (caddr vpn) (list-ref vpn 3) cl sv)
                                          (format #f "    route \"add -net ~a netmask 255.255.255.255 gw ~a\";\n  };\n"
                                            sv cl)
                                          "  down {\n    ifconfig \"%% down\";\n  };\n}\n")))
                                  (snewt-message (_ "Error")
                                    (_ "Could not write vtun(VPN) configuration. aborting."))
                                  #f)
                                ((not (= 0 (sh "passwd -u sup")))
                                  (snewt-message (_ "Error")
                                    (_ "Could not unlock maintenance account. Check if account 'sup' exitst."))
                                  #f)
                                (else
                                  (let ((s (obj-acc 'passwd "sup" (list-ref vpn 4)))
                                        (unlockexit (lambda () (begin (sh "passwd -l sup") #f))))
                                    (cond
                                      ((string? s)
                                        (snewt-message (_ "Error")
                                          (format #f (_ "Could not change maintainer password.\nReason: ~a\n") s))
                                        (unlockexit))
                                      ((not (= 0 (sh "vtund -f ~a -P 80 ~a 202.216.241.68" tmp (caddr vpn))))
                                        (snewt-message (_ "Error")
                                          (_ "Connecting VPN server failed, sorry. Consult ARMA Net Web support directly.\n"))
                                        (unlockexit))
                                      (else
                                        (let* ((tmp2 (unix-tempname))
                                               (r3 (= 0 (sh " wget -O ~a https://202.216.241.68/vtun-ready.php?ID=~a" tmp2 id)))
                                               (r4 (parse-awk #\tab #t tmp2)))
                                          (inst-log "ready got : ~s, ~s\n" r3 r4)
                                          (cond
                                            ((or (not r3) (eq? #f r4) (not (list? r4)) (null? r4) (null? (car r4)))
                                              (snewt-message (_ "Error")
                                                (_ "Sending 'ready' to server failed. Consult ARMA Net Web support, sorry.\n"))
                                              (unlockexit))
                                            (else
                                              (snewt-message (_ "Success")
                                                (string-append
                                                  (_ "System is successfully connected to ARMA Net support center.\n\n")
                                                  (_ "Consult ARMA Net Web support, or already done, wait for the supporter to research/repair the system.\n\n")
                                                  (_ "Please do not forget to disconnect the VPN line with the tool after support.")))
                                              #t)))))))))))))))))))))))

(define (newt-ui-support-close)
  (define title (_ "Remote maintenance Mode"))
  (let ((r (snewt-choice title
             (string-append
               (_ "Do you turn off remote maintenance mode?\n\n")
               (_ "Preesing ok wiil make the system :\n")
               (_ "  - cut off VPN line\n")
               (_ "  - lock maintenance account (sup).\n"))
             (_ "Yes") (_ "No"))))
    (case r
      ((0 2) #f)
      (else
        (let ((r1 (sh "killall vtund"))
              (r2 (sh "passwd -l sup")))
          (cond
            ((= 0 r2)
              (snewt-message title
                (_ "maintenance mode was successfully turned off."))
              #t)
            (else
              (snewt-message title
                (string-append
                  (_ "Failed to turn off maintenance mode.\n\n")
                  (_ "Check these below manually, sorry.\n")
                  (_ "  - no process named vtund runs (killall vtund)\n")
                  (_ "  - account sup is locked (using 'passwd -l')\n")))
              #f)))))))

(define (newt-ui-support obj-na obj-acc)
  (define title (_ "Remote maintenance Mode"))
  (let ((r0 (sh "grep '^sup:' /etc/passwd >/dev/null")))
    (cond
      ((= 1 r0)
        (snewt-message (_ "Error")
          (_ "Account for support maintenance 'sup' does not exist."))
        #f)
      ((not (= 0 r0))
        (snewt-message (_ "Error")
          (_ "Could not check support maintenance account (sup) in /etc/passwd."))
        #f)
      (else
        (let ((r1 (sh "grep '^sup:!' /etc/shadow >/dev/null")))
          (case r1
            ((0)
              (newt-ui-support-open obj-na obj-acc))
            ((1)
              (newt-ui-support-close))
            (else
              (snewt-message (_ "Error")
                (_ "Could not check support maintenance account (sup) in /etc/passwd."))
              #f)))))))

(define (newt-ui-media obj-med obj-part obj-na)
  (define title (_ "Package Source Setting"))
  (let* ((m (obj-med 'desc-misc))
         (r (snewt-nmenu title
              (_ "Package source (aptline) is configured as below.\n")
              50 5 5 5
              (list->vector (cons (obj-med 'desc-media) m))
              (vector (_ "Ok") (_ "Add Entry") (_ "Cancel")))))
    (case (car r)
      ((-1 3)
        #f)
      ((1)
        #t)
      ((0)
        (case (cdr r)
          ((0)
            (newt-ui-media-default #f obj-med obj-part obj-na)
            (newt-ui-media obj-med obj-part obj-na))
          (else
            (let ((r1 (snewt-choice title
                        (string-append
                          (_ "Do you erase this entry? ")
                          (format #f (_ "Select 'Erase' to erase this entry.\n\n   ~a")
                                     (list-ref m (1- (cdr r)))))
                        (_ "Leave") (_ "Erase"))))
              (if (= r1 2)
                (obj-med 'rm-media (1- (cdr r))))
              (newt-ui-media obj-med obj-part obj-na)))))
      ((2)
        (let ((r1 (snewt-choice title
                    (string-append
                      (_ "Notice:\n\n")
                      (_ "Before proceed further, you must understand that you are about to add purely free package entry.\n\n")
                      (_ "Omoikane neither verify nor support any purely free package to work with ARMA/OGL system.\n\n")
                      (_ "In some case, especially with unstable version, purely free package might damage the system.\n\n")
                      (_ "Do you proceed?"))
                    (_ "Proceed") (_ "Cancel"))))
          (case r1
            ((0 2)
              (newt-ui-media obj-med obj-part obj-na))
            (else
              (let ((r2 (snewt-menuc title
                          (_ "Select type of newly added entry type.\n")
                          50 5 5 5
                          (_ "Select from debian mirror")
                          (_ "Debian security update") (_ "Manually set"))))
                (case r2
                  ((0)
                    (newt-ui-media-debian obj-med))
                  ((1)
                    (let ((new (list 'sec "http://security.debian.org/" "stable")))
                      (if (newt-ui-media-test obj-med (list new))
                          (apply obj-med (cons 'add-media new)))))
                  ((2)
                    (newt-ui-media-aptline obj-med
                      "deb http://dummyhost/dummypath/ unstable main contrib non-free")))
                (newt-ui-media obj-med obj-part obj-na))))))
      (else
        #f))))

(define (newt-ui-media-test obj-med l)
  (define title (_ "Package Source Setting"))
  (let ((r (snewt-show-while title
             (_ "Testing apt-line. This step might require over 20 seconds ... ")
             40 3
             (lambda () (obj-med 'verify-deb l)))))
    (cond
      ((not (eq? #t r))
        (snewt-message title
          (format #f (_ "Could not verify media.\nReason : ~a") r))
        #f)
      (else
        r))))

(define (newt-ui-media-debian obj-med)
  (define title (_ "Package Source Setting"))
  (let* ((l (obj-med 'list-mir)))
    (cond
      ((null? l)
        (snewt-message title
          (string-append
            (_ "Could not read the mirror site list. ")
            (_ "If system doesn't have 'base-config' package, install it and run ogl-admin again. ")
            (_ "The mirror site list is expected to exists at '/usr/share/base-config/Mirrors.masterlist' .")))
        #f)
      (else
        (let ((r (apply snewt-menuc
                   (append
                     (list title (_ "Select a mirror site from below") 50 5 5 5)
                     l))))
         (case r
           ((-1) #f)
           (else 
             (let ((r1 (snewt-menuc title
                         (_ "Select a debian flavor(stableness)")
                         50 5 5 5
                         (_ "Unstable") (_ "Testing") (_ "Stable"))))
               (case r1
                 ((-1) #f)
                 (else
                   (let* ((new (obj-med 'site-entry (list-ref l r)
                                  (case r1 ((0) "unstable")
                                           ((1) "testing")
                                           (else "stable"))))
                          (r (newt-ui-media-test obj-med new)))
                     (cond
                       ((not r)
                         (newt-ui-media-debian obj-med))
                       (else
                         (map (lambda (x) (apply obj-med (cons 'add-media x)))
                              new)
                         #t)))))))))))))

(define (newt-ui-media-aptline obj-med x)
  (define title (_ "Package Source Setting"))
  (let ((r (snewt-entries title
             (_ "Enter aptline.\n")
             70 5 5 60
             (list
               (list "line:" x 0)))))
    (cond
      ((null? r) #f)
      (else
        (let ((new (list 'man (car r))))
          (cond
            ((newt-ui-media-test obj-med (list new))
              (apply obj-med (cons 'add-media new)))
            (else
              (newt-ui-media-aptline obj-med (car r)))))))))

(define (newt-ui-package)
  (define title (_ "Package installation"))
  (snewt-progress-init title "")
  (let ((r (progress-start "/usr/sbin/ogl/pkginst" snewt-progress-update)))
    (snewt-progress-finish)
    (cond
      ((= 0 r)
        #t)
      ((= 1 (snewt-choice title
              (string-append
                (_ "Package installation failed. \n")
                (let ((r (ttable-read "/tmp/inst/pkgfail" '((str int)))))
                  (cond
                    ((and (list? r) (not (null? r)))
                      (let ((p (caar r))
                            (e (cadr (car r))))
                        (format #f (_ "package ~a ~a\n\nDo you want to skip this package?") p
                          (if (= 220 e)
                            (_ "timed out. \n")
                            (format #f (_ "exited with ~a .\n") e)))))
                    (else
                      (_ "Will you skip a failed package and continue installation?")))))
              (_ "Yes") (_ "No")))
        (newt-ui-package))
      (else
        #f))))

(define (newt-check-package obj-med f)
  (inst-log "==== newt-check-package\n")
  (let ((r (parse-awk #\space #t f)))
    (cond
      ((or (eq? #f r)
           (ormap null? r))
        #f)
      (else
        (let ((l (obj-med 'verify-inst (map car r))))
          (cond
            ((null? l)
              #t)
            ((< 20 (length l))
              (snewt-message (_ "Package installation")
                (_ "A lot of packages are not installed successfully.\n"))
              #f)
            (else
              (snewt-message (_ "Package installation")
                (apply string-append
                  (cons
                    (_ "Following Package(s) are not installed:\n\n")
                    (map (lambda (x) (format #f "~a " x))
                         l))))
              #f)))))))

(define newt-ui-boot-title (_ "Boot Loader Setting"))

(define (newt-boot obj-boot obj-part)
  (let ((r (apply snewt-menuc
             (append
               (list newt-ui-boot-title
                 (_ "Select menu below.\n")
                 50 4 4 6
                 (_ "Install GRUB")
                 (_ "Recover Boot Record")
                 (_ "Create Boot FD")
                 (_ "Modify Boot Entry"))
               (if (< (length (obj-part 'hw-get-biosorder)) 2)
                 '()
                 (list (_ "Set BIOS Drive Order")))))))
    (case r
      ((-1) #f)
      ((0)
        (let ((r (newt-ui-install-br obj-boot obj-part #f)))
          (if r
             (snewt-message newt-ui-boot-title (_ "Boot loader was successfully installed.\n")))
          r))
      ((1) (newt-ui-recover-mbr obj-boot obj-part))
      ((2) (newt-ui-bootfd #f obj-boot obj-part))
      ((3) (newt-boot-entry obj-boot obj-part))
      ((4) (newt-ui-bios-order obj-part))
      (else
        (snewt-message t
          (_ "not implemented yet, sorry."))
        #f))))

(define (newt-ui-recover-mbr obj-boot obj-part)
  (define (proceed d all?)
    (let ((r (snewt-choice newt-ui-boot-title
               (string-append
                 (_ "This is last chance to quit. Boot record for device below will be rewritten with old boot record.\n\n    ")
                 (obj-part 'did-desc d)
                 (_ "\n\nIf this device is BIOS boot device, erasing grub might make this system unbootable.\n\n")
                 (_ "Do you proceed anyway?"))
               (_ "Proceed") (_ "Cancel"))))
      (case r
        ((0 2) #f)
        (else
          (cond
            ((obj-boot 'recover d all?)
              (snewt-message newt-ui-boot-title
                (_ "Previous boot record was recovered.\n"))
              #t)
            (else
              (snewt-message newt-ui-boot-title
                (_ "Failed to recover boot record.\n"))
              #f))))))
  (let ((l (obj-boot 'list-recoverable)))
    (cond
      ((null? l)
        (snewt-message newt-ui-boot-title
          (_ "No boot record is recoverable.\n"))
        #f)
      (else
        (let ((r (snewt-nmenu newt-ui-boot-title
                   (_ "\nSelect a device to recover previous boot record.\n")
                   50 5 5 5
                   (list->vector
                     (map (lambda (x)
                             (format #f (_ "top region of ~a") (obj-part 'did-desc x)))
                          l))
                   (vector (_ "Recover") (_ "Cancel")))))
          (case (car r)
            ((-1 2) #f)
            (else
              (let ((d (list-ref l (cdr r))))
                (case (obj-boot 'recover-check d)
                  ((error)
                    (snewt-message newt-ui-boot-title
                      (_ "Could not recover.\n")))
                  ((modified)
                    (let ((r1 (snewt-choice newt-ui-boot-title
                                (string-append
                                  (_ "Boot record was altered since grub had been installed. System might be destroyed completely by forcing recover process.\n\n")
                                  (_ "Do you proceed anyway?"))
                                (_ "Proceed") (_ "Cancel"))))
                      (case r1
                        ((0 2) #f)
                        (else
                          (proceed d #f)))))
                  ((partition)
                    (let ((r1 (snewt-choice newt-ui-boot-title
                                (string-append
                                  (_ "Partition table was altered. Previous boot-loader might not be able to boot correctly. \n\n")
                                  (_ "Do you proceed anyway?"))
                                (_ "Proceed") (_ "Cancel"))))
                      (case r1
                        ((0 2) #f)
                        (else
                          (proceed d #f)))))
                  (else
                    (proceed d #t)))))))))))

(define (newt-boot-entry obj-boot obj-part)
  (let* ((e (obj-boot 'entry))
         (r (snewt-nmenu newt-ui-boot-title ""
             50 4 4 6
             (list->vector
               (map (lambda (x)
                      (case (car x)
                        ((L) (format #f (_ " Linux - ~a ") (cadr x)))
                        ((B) (format #f (_ " MBR   - ~a ") (cadr x)))
                        ((P) (format #f (_ " PBR   - ~a ") (cadr x)))
                        ((F) (format #f (_ " File  - ~a ") (cadr x)))
                        (else (_ "unknown entry"))))
                    e))
             (vector (_ "Ok") (_ "New Entry") (_ "Cancel")))))
    (case (car r)
      ((-1 3) #f)
      ((1)
        #t)
      ((2)
        (newt-boot-entry-new obj-boot obj-part)
        (newt-boot-entry obj-boot obj-part))
      (else
        (newt-boot-entry-entry obj-boot obj-part (cdr r))
        (newt-boot-entry obj-boot obj-part)))))

(define (newt-boot-check-entry obj-part r)
  (let ((d0 (obj-part 'parse-name (cadr r)))
        (d1 (obj-part 'parse-name (caddr r)))
        (t (car r))
        (k (list-ref r 3)))
    (cond
      ((not (= 2 (length d0)))
        (snewt-message (_ "Error")
          (_ "Could not parse grub root partition\n"))
        #f)
      ((not (= 2 (length d1)))
        (snewt-message (_ "Error")
          (_ "Could not parse fs root partition\n"))
        #f)
      ((equal? "" t)
        (snewt-message (_ "Error") (_ "Title is not specified.\n"))
        #f)
      ((equal? "" k)
        (snewt-message (_ "Error") (_ "Kernel path is not specified.\n"))
        #f)
      (else
        (list 'L t (car d0) (cadr d0) k (car d1) (cadr d1)
              (list-ref r 4) (list-ref r 5) (list-ref r 6))))))

(define (newt-boot-entry-entry obj-boot obj-part n)
  (let* ((x (list-ref (obj-boot 'entry) n))
         (r (snewt-nmenu newt-ui-boot-title
              (apply string-append
                (cons (_"\n")
                  (case (car x)
                    ((L)
                      (list
                        (sprintf "%-21s : %s\n" (_ "title") (cadr x)) 
                        (sprintf "%-21s : %s\n" (_ "grub root partition") (obj-part 'oldname (caddr x) (list-ref x 3)))
                        (sprintf "%-21s : %s\n" (_ "fs root partition") (obj-part 'oldname (list-ref x 5) (list-ref x 6)))
                        (sprintf "%-21s : %s\n" (_ "kernel path") (list-ref x 4))
                        (sprintf "%-21s : %s\n" (_ "kernel option") (list-ref x 7))
                        (sprintf "%-21s : %s\n" (_ "initrd path") (list-ref x 8))
                        (sprintf "%-21s : %s\n" (_ "other grub command") (list-ref x 9))))
                    ((B)
                      (list
                        (sprintf "%-21s : %s\n" (_ "title") (cadr x)) 
                        (sprintf "%-21s : %s\n" (_ "MBR") (obj-part 'did-desc (caddr x)))))
                    ((P)
                      (list
                        (sprintf "%-21s : %s\n" (_ "title") (cadr x)) 
                        (sprintf "%-21s : %s\n" (_ "PBR") (obj-part 'did-desc (caddr x) (list-ref x 3)))))
                    (else
                      '()))))
              74 4 4 4 (vector)
              (vector (_ "Ok") (_ "Modify") (_ "Default") (_ "Delete")))))
    (inst-log "entry-entry nmenu got : ~a\n" r)
    (case (car r)
      ((-1 0) #f)
      ((1) #t)
      ((2) (newt-boot-entry-modify obj-boot obj-part n))
      ((3) (obj-boot 'set-default n))
      ((4) (obj-boot 'remove-entry n))
      (else #f))))

(define (newt-boot-entry-modify obj-boot obj-part n)
  (let ((x (list-ref (obj-boot 'entry) n)))
    (case (car x)
      ((L)
        (let ((r (snewt-entries newt-ui-boot-title ""
                   50 4 4 24
                   (list
                     (list (_ "title") (cadr x) 0)
                     (list (_ "grub root partition") (obj-part 'oldname (caddr x) (list-ref x 3)) 0)
                     (list (_ "fs root partition") (obj-part 'oldname (list-ref x 5) (list-ref x 6)) 0)
                     (list (_ "kernel path") (list-ref x 4) 0)
                     (list (_ "kernel option") (list-ref x 7) 0)
                     (list (_ "initrd path") (list-ref x 8) 0)
                     (list (_ "other grub command") (list-ref x 9) 0)))))
          (cond
            ((null? r)
              #f)
            (else
              (let ((x (newt-boot-check-entry obj-part r)))
                (cond
                  ((eq? #f x) #f)
                  (else
                    (obj-boot 'replace n x))))))))
      ((B)
        (let* ((l (obj-part 'hw-info))
               (r (snewt-nmenu newt-ui-boot-title
                    (string-append
                      (_ "Current chain loader : top region of ")
                      (obj-part 'did-desc (caddr x))
                      (_ "\nSelect a position for chain loading.\n"))
                    50 5 5 5
                    (list->vector
                      (map (lambda (x)
                             (format #f (_ "top region of ~a") (obj-part 'hw-desc x)))
                             l))
                    (vector (_ "Leave as current") (_ "Cancel")))))
          (define (set-title d)
            (let ((r (snewt-entries newt-ui-boot-title
                       (_ "Enter title for this entry\n")
                       50 4 4 20
                       (list (list (_ "title") (cadr x) 0)))))
              (cond
                ((null? r) #f)
                ((equal? "" (car r))
                  (snewt-message (_ "Error") (_ "Title is not specified.\n"))
                  #f)
                (else
                  (obj-boot 'replace n (list 'B (car r) d))))))
          (case (car r)
            ((-1 2) #f)
            ((0)
              (set-title (car (list-ref l (cdr r)))))
            (else
              (set-title (caddr x))))))
      ((P)
        (let* ((l (filter (lambda (x) (not (equal? 'empty (car x))))
                          (obj-part 'list-all-part #f)))
               (r (snewt-nmenu newt-ui-boot-title
                    (string-append
                      (_ "Current chain loader : top region of ")
                      (obj-part 'did-desc (caddr x) (list-ref x 3))
                      (_ "\nSelect a drive for chain loading.\n"))
                    50 5 5 5
                    (list->vector
                      (map (lambda (x) (obj-part 'did-desc (cadr x) (list-ref x 3)))
                           l))
                    (vector (_ "Leave as current") (_ "Cancel")))))
          (define (set-title d n)
            (let ((r1 (snewt-entries newt-ui-boot-title
                       (_ "Enter title for this entry\n")
                       50 4 4 20
                       (list (list (_ "title") (cadr x) 0)))))
              (cond
                ((null? r1) #f)
                ((equal? "" (car r1))
                  (snewt-message (_ "Error") (_ "Title is not specified.\n"))
                  #f)
                (else
                  (obj-boot 'replace (cdr r) (list 'P (car r1) d n))))))
          (case (car r)
            ((-1 2) #f)
            ((0)
              (let ((new (list-ref l (cdr r))))
                (set-title (car new) (list-ref new 3))))
            (else
              (set-title (caddr x) (list-ref x 3))))))
      (else
        (inst-log "unimplemeted for this entry: ~a\n" x)
        #f))))

(define (newt-boot-entry-new obj-boot obj-part)
  (let ((r (snewt-nmenu newt-ui-boot-title
             (_ "Select type of new entry")
             50 4 4 6
             (vector "Linux" (_ "Master Boot Record") (_ "Boot Record at Partition"))
             (vector (_ "Ok") (_ "Cancel")))))
    (case (car r)
      ((-1 2) #f)
      (else
        (case (cdr r)
          ((0)
            (let* ((b (obj-part 'path-mnt "/boot"))
                   (f-rt (obj-part 'path-mnt "/"))
                   (g-rt (if (null? b) f-rt b))
                   (r (snewt-entries newt-ui-boot-title ""
                        50 4 4 24
                        (list
                          (list (_ "title") "" 0)
                          (list (_ "grub root partition")
                                (if (null? g-rt) "/dev/" (obj-part 'oldname (car g-rt) (list-ref g-rt 5))) 0)
                          (list (_ "fs root partition")
                                (if (null? f-rt) "/dev/" (obj-part 'oldname (car f-rt) (list-ref f-rt 5))) 0)
                          (list (_ "kernel path") (if (null? b) "/boot/vmlinuz-" "/vmlinuz-") 0)
                          (list (_ "kernel option") "" 0)
                          (list (_ "initrd path") "" 0)
                          (list (_ "other grub command") "" 0)))))
              (cond
                ((null? r)
                  #f)
                (else
                  (let ((x (newt-boot-check-entry obj-part r)))
                    (cond
                      ((eq? #f x)
                        #f)
                      (else
                        (obj-boot 'add-entry x))))))))
          ((1)
            (let* ((l (obj-part 'hw-info))
                   (r (snewt-nmenu newt-ui-boot-title
                        (_ "Select a drive for chain loading.\n")
                        50 5 5 5
                        (list->vector
                          (map (lambda (x)
                                 (format #f (_ "top region of ~a") (obj-part 'hw-desc x)))
                                 l))
                        (vector (_ "Ok") (_ "Cancel")))))
              (define (set-title d)
                (let ((r1 (snewt-entries newt-ui-boot-title
                            (_ "Enter title for this entry\n")
                            50 4 4 20
                            (list (list (_ "title") "" 0)))))
                  (cond
                    ((null? r1) #f)
                    ((equal? "" (car r1))
                      (snewt-message (_ "Error") (_ "Title is not specified.\n"))
                      #f)
                    (else
                      (obj-boot 'add-entry (list 'B (car r1) (car (list-ref l (cdr r)))))))))
              (case (car r)
                ((-1 2) #f)
                (else
                  (set-title (car (list-ref l (cdr r))))))))
          ((2)
            (let* ((l (filter (lambda (x) (not (equal? 'empty (car x))))
                              (obj-part 'list-all-part #f)))
                   (r (snewt-nmenu newt-ui-boot-title
                        (_ "Select a partition for chain loading.\n")
                        50 5 5 5
                        (list->vector
                          (map (lambda (x) (obj-part 'did-desc (cadr x) (list-ref x 3)))
                               l))
                        (vector (_ "Ok") (_ "Cancel")))))
              (define (set-title)
                (let ((r1 (snewt-entries newt-ui-boot-title
                            (_ "Enter title for this entry\n")
                            50 4 4 20
                            (list (list (_ "title") "" 0)))))
                  (cond
                    ((null? r1) #f)
                    ((equal? "" (car r1))
                      (snewt-message (_ "Error") (_ "Title is not specified.\n"))
                      #f)
                    (else
                      (let ((x (list-ref l (cdr r))))
                        (obj-boot 'add-entry (list 'P (car r1) (cadr x) (1- (list-ref x 3)))))))))
              (case (car r)
                ((-1 2) #f)
                (else
                  (set-title)))))
          (else
            #f))))))

(define (newt-ui-ide-single n obj-ide obj-part)
  (define title (_ "IDE Configuration"))
  (let* ((l (obj-ide 'table))
         (x (list-ref l n))
         (d (car x))
         (s (list-ref x 4))
         (ml (obj-ide 'mode-list d))
         (r (snewt-nmenu title
               (string-append
                 (format #f
                   (_ "Select transfer mode for device below. (current ~a~a)\n device : ~a / ~a\n")
                   (obj-ide 'n->mode (caddr x))
                   (if (< 0 s)
                     (format #f (_ " @ ~a Mb/s\n") s)
                     "")
                   (obj-part 'did-desc d)
                   (cadr x))
                 (_ "Warning!\nBe carefull to choose the mode. Selecting inappropriate value will cause the system freeze very easily. ")
                 (_ "If you are not sure of the spec of IDE device, ATA33 or PIO may be safe.\n\n")
                 (_ "If the system freezes or responds much slower after the menu, reboot(reset) the computer and select safer IDE mode.")
                 )
               64 5 5 5
               (list->vector (map cadr ml))
               (vector (_ "Ok") (_ "Test Current Setting") (_ "Cancel")))))
    (case (car r)
      ((-1 3)
        #f)
      ((1)
        #t)
      ((0)
        (let ((m (car (list-ref ml (cdr r)))))
          (cond
            ((obj-ide 'try-mode d m)
              (obj-ide 'set-mode d m))
            (else
              (snewt-message title
                (_ "IDE device or IDE interface does not accept the mode.\n\nMode was not changed.\n")))))
        (newt-ui-ide-single n obj-ide obj-part))
      (else
        (snewt-progress-init title (_ "Measuring speed ..."))
        (let ((y (obj-ide 'test-tool d snewt-progress-update)))
          (inst-log "cmd : ~a\n" (car y))
          (progress-start (car y) (cadr y)))
        (snewt-progress-finish)
        (newt-ui-ide-single n obj-ide obj-part)))))

(define (newt-ui-ide obj-ide obj-part)
  (define title (_ "IDE Configuration"))
  (let* ((l (obj-ide 'table))
         (r (snewt-nmenu title
              (_ "Select device to set transfer mode.")
              60 5 5 5
              (list->vector
                (map (lambda (x) (obj-ide 'desc x))
                     l))
              (vector (_ "Ok") (_ "Cancel")))))
    (case (car r)
      ((-1 2)
        (obj-ide 'abort)
        #f)
      ((1)
        #t)
      (else
        (cond
          ((newt-ui-ide-single (cdr r) obj-ide obj-part)
            (newt-ui-ide obj-ide obj-part))
          (else
            #f))))))

(define (newt-ui-root-pass obj-acc)
  (newt-stage-title (_ "Root Password"))
  (let ((r (snewt-entries
             (_ "Root Password Setting")
             (_ "Enter Password for root account. (twice)\n")
             50 5 5 20
             (list
               (list (_ "Password") "" snewt-flag-password)
               (list (_ "Password(confirm)") "" snewt-flag-password)))))
    (cond
      ((or (null? r) (string=? "" (car r)) (string=? "" (cadr r)))
        (snewt-message (_ "Info")
          (_ "Root password was not changed.\n"))
        #t)
      ((not (equal? (car r) (cadr r)))
        (snewt-message (_ "Info") (_ "Passwords don't match..\n"))
        (newt-ui-root-pass obj-acc))
      (else
        (let ((r (obj-acc 'passwd "root" (car r))))
          (cond
            ((string? r)
              (snewt-message (_ "Error")
                (format #f (_ "Password was not changed.\nReason : ~a\n" r)))
              #f)
            (else
              #t)))))))

(define (newt-ui-adduser obj-acc def1 def2)
  (define title (_ "User Account"))
  (define (ok)
    (let ((r (snewt-choice (_ "Success")
               (_ "Account successfully generated.\nDo you create more user account?")
               (_ "Yes") (_ "No"))))
      (cond
        ((= r 1)
          (newt-ui-adduser obj-acc "" ""))
        (else
          #t))))
  (newt-stage-title title)
  (let ((r (snewt-entries title
             (_ "Enter User name and password (twice).\n")
             50 5 5 20
             (list
               (list (_ "Login name") def1 0)
               (list (_ "Full name") def2 0)
               (list (_ "Password") "" snewt-flag-password)
               (list (_ "Password(confirm)") "" snewt-flag-password)))))
    (cond
      ((null? r)
        #f)
      ((or (equal? "" (car r)) (equal? "" (list-ref r 3)))
        (snewt-message (_ "Info") (_ "Account not generated.\n"))
        #f)
      ((not (equal? (caddr r) (list-ref r 3)))
        (snewt-message (_ "Info") (_ "Passwords don't match..\n"))
        (newt-ui-adduser obj-acc (car r) (cadr r)))
      (else
        (let* ((r1 (snewt-checkbox title
                     (_ "Check groups to which new user will be added. Groups represent access ability to device or command execution")
                     57 3 6
                     (list->vector
                       (map (lambda (x)
                              (cons (sprintf "%-8s : %s" (car x) (cadr x))
                                    (caddr x)))
                            (obj-acc 'suggest)))
                     (vector (_ "Ok") (_ "Cancel"))))
               (b (car r1))
               (cl (vector->list (cdr r1))))
          (cond
            ((not (= b 0)) #f)
            (else
              (let ((r2 (obj-acc 'adduser (car r) (caddr r) (cadr r)
                          (filter (lambda (x) (not (eq? #f x)))
                            (map (lambda (x y) (if y (car x) y))
                              (obj-acc 'suggest)
                              cl)))))
                (cond
                  ((string? r2)
                    (snewt-message (_ "Error")
                      (format #f (_ "Could not create user.\nReason : ~a\n") r))
                    #f)
                  (else
                    (ok)))))))))))

(define (newt-ui-ldap obj-ds)
  (define title (_ "LDAP client setting"))
  (let* ((r0 (obj-ds 'get-ldap))
         (h (if (eq? r0 #f) "" (car r0)))
         (p (if (or (eq? r0 #f) (< (cadr r0) 0))
                ""
                (format #f "~a" (cadr r0))))
         (b (if (eq? r0 #f) "" (caddr r0)))
         (r1 (snewt-entries title
               (_ "Enter LDAP server hostname and base for authentication.\n")
               50 5 5 40
               (list (list (_ "hostname ") h 0)
                     (list (_ "port ") p 0)
                     (list (_ "base ") b 0)))))
    (cond
      ((null? r1) #f)
      (else
        (let ((p' (if (equal? "" (cadr r1))
                      -1
                      (string->number (cadr r1)))))
          (cond
            ((eq? #f p')
              (snewt-message title
                (_ "Digits is allow for port number.\n"))
              (newt-ui-ldap obj-ds))
            ((not (and (obj-ds 'reset)
                       (obj-ds 'update-ldap (car r1) p' (caddr r1))
                       (obj-ds 'set-postconf)))
              (snewt-message title
                (_ "Failed to set ldap configuration\n"))
              #f)
            (else
              #t)))))))

(define (newt-ui-nis obj-ds)
  (define title (_ "NIS client setting"))
  (let* ((r0 (obj-ds 'get-nis))
         (h (if (eq? r0 #f) "" (cadr r0)))
         (b (if (eq? r0 #f) "" (car r0)))
         (r1 (snewt-entries title
               (string-append
                 (_ "Enter NIS domain and server.\n")
                 (_ "If using broadcast, leave 'server' entry empty."))
               50 5 5 20
               (list (list (_ "NIS domain ") h 0)
                     (list (_ "Server ") b 0)))))
    (cond
      ((or (null? r1)
           (and (equal? "" (car r1))
                (equal? "" (cadr r1))))
        #f)
      ((equal? "" (car r1))
        (snewt-message title
          (_ "It is needed to specify NIS domain."))
        (newt-ui-nis obj-ds))
      ((not (and (obj-ds 'reset)
                 (obj-ds 'update-nis (cadr r1) (car r1))
                 (obj-ds 'set-postconf)))
        (snewt-message title (_ "Failed to set NIS configuration."))
        #f)
      (else
        #t))))

(define (newt-ui-user-account obj-ds obj-acc)
  (define title (_ "User Account Setting"))
  (let ((r (apply snewt-menuc
             (append
               (list title
                 (_ "Select menu below\n")
                 50 5 5 5
                 (_ "Local Account Creation")
                 (_ "LDAP client setting")
                 (_ "NIS client setting"))
               (if (null? (obj-ds 'info))
                 '()
                 (list (_ "Disable LDAP/NIS")))))))
    (case r
      ((0)
        (newt-ui-adduser obj-acc "" ""))
      ((1)
        (newt-ui-ldap obj-ds))
      ((2)
        (newt-ui-nis obj-ds))
      ((3)
        (let ((r (snewt-choice title
                   (string-append
                     (_ "Are you OK to disable LDAP/NIS client setting?\n\n")
                     (_ "After pressing Ok, system relies only on local account for user authendication.\n"))
                   (_ "Yes") (_ "No"))))
          (if (= 1 r)
            (and (obj-ds 'reset)
                 (obj-ds 'stop-server))
            #f)))
      (else
        #f))))

; f : (f base? ws? sv? orca? postfix?) $B$J$k(B callback $B4X?t$G$9!#(B
; f $B$O(B #t or string $B$rJV$7$^$9!#(B
(define (newt-ui-selection f)
  (define title (_ "Package Selection"))
  (newt-stage-title title)
  
  (define (ask)
    (let ((r0 (parse-awk #\tab #t "/var/lib/ogl/etc/product")))
      (cond
        ((not (and (not (eq? #f r0))
                   (= 1 (length r0))
                   (= 1 (length (car r0)))
                   (< 2 (string-length (caar r0)))))
          '())
        (else
          (let* ((p (string->symbol (substring (caar r0) 0 2)))
                 (m0 (list (_ "Minimum (core system only)") (_ "Standard Desktop")))
                 (m1 (list (_ "Minimum (core system only)") (_ "Standard Desktop") (_ "Server") (_ "Server + Desktop")))
                 (m2 (list (_ "Minimum (core system only)") (_ "ORCA + Desktop") (_ "ORCA + Server") (_ "ORCA + Desktop + Server")))
                 (r1 (apply snewt-menu
                       (append (list title (_ "Select a course of installation.") 50 5 5 4)
                               (case p ((SV) m1) ((OS OD SS SD) m2) (else m0)))))
                 (orca? (case p ((OS OD SS SD) #t)
                                (else #f))))
            (case r1
              ((-1)
                (ask))
              ((0)
                (let ((r2 (snewt-choice title
                            (string-append
                              (_ "Warining!\n\n")
                              (_ "System will be configured very minimum. ")
                              (_ "After installation, you might have some difficulty with this system.\n\n")
                              (_ "Are you ok with minimum system?\n"))
                            (_ "Yes") (_ "No"))))
                  (cond
                    ((= 1 r2)
                      '(#f #f #f #f #f))
                    (else
                      (ask)))))
              (else
                (let ((r3 (snewt-choice title
                            (_ "Select a MTA (mail tranport agent).")
                            "qmail" "postfix")))
                  (case r3
                    ((0) (ask))
                    (else
                      (let ((postfix? (= r3 2)))
                        (case r1
                          ((1)  (list #t #t #f orca? postfix?))
                          ((2)  (list #t #f #t orca? postfix?))
                          (else (list #t #t #t orca? postfix?))))))))))))))
  (let ((r (ask)))
    (cond
      ((null? r) #f)
      (else
        (let ((x (apply f r)))
          (cond
            ((not (eq? #t x))
              (snewt-message (_ "Fatal Error")
                (format #f (_ "Could not create package selection.\nReason : ~a\n") x))
              #f)
            (else
              #t)))))))


(define (newt-module-sound obj-mod)
  (obj-mod 'setmod-init)
  (let ((l (filter (lambda (x)
                     (case (cadr x)
                       ((un/not inst/not)
                         #f)
                       (else
                         (obj-mod 'sound-target? (car x)))))
                   (obj-mod 'table))))
    (define (insmod)
      (map (lambda (x) (obj-mod 'rmmod (car x)))
           l)
      (obj-mod 'rmmod 'snd-mixer-oss)
      (obj-mod 'rmmod 'snd-pcm-oss)
      (map (lambda (x) (obj-mod 'insmod (car x) ""))
           l)
      #t)
    (inst-log "module-sound : ~a\n" l)
    (cond
      ((< (length l) 2)
        (insmod))
      (else
        (let ((r (snewt-nmenu (_ "Sound Driver Setting")
                   (string-append
                     (_ "System has several sound device. Drivers are going to be loaded according to the order below. \n\n")
                     (_ "Note that a few driver can be only used as primary driver.\n\n")
                     (_ "Proceed with this driver order?"))
                   60 4 4 6
                   (list->vector
                     (map (lambda (x) (obj-mod 'desc (car x)))
                          l))
                   (vector (_ "Proceed") (_ "Set Default") (_ "Remove") (_ "Cancel")))))
          (case (car r)
            ((-1 4) #f)
            ((1)
              (insmod))
            (else
              (let ((m (car (list-ref l (cdr r)))))
                (case (car r)
                  ((3) (obj-mod 'rmmod m))
                  (else (obj-mod 'set-first m)))
                (newt-module-sound obj-mod)))))))))

(define (newt-ui-volume-single obj-vol n)
  (define title (_ "Simple Volume Setting"))
  (let* ((l (obj-vol 'list))
         (v (list-ref l n))
         (t (car v))
         (r (snewt-nmenu title
              (string-append
                (format #f (_ "Current Volume ~a% (for /dev/sound/mixer~a)\n\n")
                           (cadr v) t)
                (_ "Select number to set as volume."))
              50 4 4 11
              (vector "  100  " "   90  " "   80  " "   70  " "   60  " "   50  " "   40  " "   30  " "   20  " "   10  " "    0  ")
              (vector (_ "Ok") (_ "Cancel")))))
    (case (car r)
      ((-1 2) #f)
      ((1) #t)
      (else
        (cond
          ((not (obj-vol 'set n (* 10 (- 10 (cdr r)))))
            (snewt-message title (_ "Failed to set volume."))
            #f)
          (else
            (let ((r1 (snewt-show-while title
                        (if (= 1 (length l))
                          (_ "Ringing ...")
                          (format #f (_ "Ringing ~a sound device (/dev/sound/dsp~a) ...")
                                  (ordinal-string #t n) t))
                        40 5
                        (lambda () (obj-vol 'test t)))))
              (cond
                ((not r1)
                  (snewt-message title (_ "Failed to test sound device."))
                  #f)
                (else
                  (case (snewt-smessage title
                          (string-append
                            (_ "Do you hear a bell at proper volume?\n\n")
                            (_ "If no sound was heard, please check below:\n\n")
                            (_ "    1. speaker is connected appropriately\n")
                            (_ "    2. correct sound driver is installed in correct order\n"))
                          60 8)
                    ((-1) #f)
                    (else
                      (newt-ui-volume-single obj-vol n))))))))))))

(define (newt-ui-volume obj-vol)
  (define title (_ "Simple Volume Setting"))
  (let* ((l (obj-vol 'list))
         (single? (= 1 (length l))))
    (case (length l)
      ((0)
        (snewt-message title
          (string-append
            (_ "No sound device (mixer device) was found. ")
            (_ "If the system has ISA sound devices, please install appropriate driver manually with 'Driver Setting' of ogl-admin. ")
            (_ "(In that case, you may also need to configure with isapntools. )")))
        #t)
      ((1)
        (newt-ui-volume-single obj-vol 0))
      (else
        (let* ((i -1)
               (r (snewt-nmenu title
                    (_ "Volumes are set as below:")
                    40 4 4 6
                    (list->vector
                      (smap (lambda (n x)
                              (sprintf (_ "%6s sound device : %3d")
                                (ordinal-string #t n) (cadr x)))
                            l))
                    (vector (_ "Ok") (_ "Cancel")))))
          (case (car r)
            ((-1 2) #f)
            ((1) #t)
            (else
              (newt-ui-volume-single obj-vol (cdr r))
              (newt-ui-volume obj-vol))))))))

(define (newt-ui-service obj-sv)
  (newt-stage-title (_ "Service Setting"))
  (let* ((r (snewt-checkbox (_ "Service Setting")
              (string-append
                (_ "With this menu, you can set a status of each service.")
                (_ "Checked services are going to run. ")
                (_ "Unchecked services will be stopped.\n"))
              60 4 8
              (list->vector
                (map (lambda (x)
                       (cons
                        (sprintf "%-17s : %s" (car x) (list-ref x 3))
                        (list-ref x 4)))
                     (obj-sv 'list)))
              (vector (_ "Ok") (_ "Cancel"))))
         (b (car r))
         (cl (vector->list (cdr r))))
    (cond
      ((not (= b 0)) #f)
      (else
        (snewt-show-while (_ "Service Setting")
          (_ "Configuring Services ...")  54 3
          (lambda ()
            (map (lambda (c x)
                   (if c (obj-sv 'start x)
                         (obj-sv 'remove x)))
                 cl (map car (obj-sv 'list)))))
        #t))))

(define (newt-ui-printer ask? obj-pr)
  (define title (_ "Printer Setting"))
  (let ((r (if ask?
             (snewt-choice title
               (string-append
                 (_ "Before configuring printer system, you need to stop printer daemon.\n\n")
                 (_ "Do you stop printer daemon?"))
               (_ "Yes") (_ "No"))
             1)))
    (case r
      ((0 2)
        #f)
      (else
        (sh "/etc/init.d/lprng stop") 
        (let ((r1 (snewt-nmenu title
                    (_ "Select menu below.")
                    40 4 4 4
                    (vector (_ " Edit print queue entry ")
                            (_ " Start Printer (lpd) ")
                            (_ " Stop Printer "))
                    (vector (_ "Ok") (_ "Skip")))))
          (case (car r1)
            ((-1 2) #f)
            ((1) #t)
            (else
              (let ((r2 (case (cdr r1)
                          ((0)
                            (newt-ui-printer-entry obj-pr))
                          ((1)
                            (newt-ui-printer-start obj-pr))
                          (else
                            (newt-ui-printer-stop #t obj-pr)))))
                (cond
                  ((eq? #f r2)
                    #f)
                  (else
                    (newt-ui-printer #f obj-pr)))))))))))

(define (newt-ui-printer-start obj-pr)
  (define title (_ "Printer Setting"))
  (let ((r (obj-pr 'lpd-start)))
    (cond
      ((string? r)
        (snewt-message (_ "Error")
          (format #f
            (_ "Starting printer server failed.\nReason : ~a\n")
            r))
        #f)
     (else
       (snewt-message (_ "Success")
         (_ "Printer system started successfully."))
       #t))))

(define (newt-ui-printer-stop ask? obj-pr)
  (define title (_ "Printer Setting"))
  (let ((r (if ask?
             (snewt-choice title
               (string-append
                 (_ "Do you stop printer?\nPressing 'Yes' will commits below:\n\n")
                 (_ "  - remove all print queue\n")
                 (_ "  - stop daemon\n")
                 (_ "  - kill all jobs that uses printer device \n")
                 (_ "  - remove any driver which relates to printer\n"))
               (_ "Yes") (_ "No"))
             1)))
    (case r
      ((1)
        (let ((r1 (snewt-show-while title
                    (_ "Stopping printer ...") 54 3
                    (lambda () (obj-pr 'force-stop)))))
          (cond
            ((string? r1)
              (snewt-message (_ "Error")
                (format #f
                  (_ "Stopping printer failed.\nReason : ~a\n")
                  r1))
              #f)
            (else
              (snewt-message (_ "Success")
                (string-append
                  (_ "Printing system was successfully stopped. \n\n")
                  (_ "If you would like to perfectly erase garbage data, reset the printer hardware once. ")
                  (_ "After that system can start from clean state.")))
              #t))))
      (else
        #t))))

(define (newt-ui-printer-entry obj-pr)
  (define title (_ "Printer Setting"))

  (define (check-commit q)
    (obj-pr 'update q)
    (check-package (car q))
    (ask-testprint (car q)))
  
  (define (check-package q)
    (cond
      ((not (obj-pr 'need-package? q))
        #t)
      (else
        (let* ((p (obj-pr 'get-extfilter q))
               (r (snewt-choice title
                    (string-append
                      (format #f
                        (_ "In order to function print queue ~a, system requires package ~a, which is not installed yet.\n\n")
                        q p)
                      (_ "Do you install this package now?"))
                      (_ "Yes") (_ "No"))))
          (case r
            ((1)
              (snewt-finish)
              (sh "apt-get update")
              (let ((r (= 0 (sh "apt-get install ~a\n" p))))
                (snewt-start)
                (newt-stage-title title)
                (cond
                  ((eq? r #f)
                    (snewt-message (_ "Error")
                      (format #f 
                        (_ "Sorry, could not install package ~a. Install this package manually.")
                        p))
                    #f)
                  (else
                    #t))))
            (else
              #t))))))
  
  (define (ask-testprint q)
    (let ((r (snewt-choice title
               (string-append
                 (format #f (_ "Do you send test data to the print queue '~a'?") q))
               (_ "Yes") (_ "No"))))
      (case r
        ((1)
          (let ((r1 (obj-pr 'test-print q)))
            (cond
              ((string? r1)
                (snewt-message (_ "Error")
                  (format #f
                    (_ "Sending test data failed.\nReason : ~a\n")
                    r1))
                #f)
              (else
                (let ((r2 (snewt-choice title
                            (_ "Test data was sent. If you got garbage date, press 'stop printing'.")
                            (_ "Ok") (_ "Stop printing"))))
                  (case r2
                    ((2)
                      (newt-ui-printer-stop #f obj-pr))
                    (else
                      #t)))))))
        (else
          #t))))
  
  (define (view q)
    (let* ((qname (car q))
           (type (cadr q))
           (driver (list-ref q 4))
           (extf (obj-pr 'get-extfilter qname))
           (gs? (and (not (equal? driver '-)) (equal? "" extf)))
           (r (snewt-nmenu title
                (apply string-append
                  (append
                    (list (format #f (_ "Print queue '~a' \n\n") qname))
                    (case type
                      ((local)
                        (list (format #f (_ "   Interface     : ~a ~a\n")
                                (ordinal-string #t (list-ref q 3))
                                (if (equal? 'usb (caddr q)) "USB" "Parallel"))))
                      ((lp)
                        (list            (_ "  Protocol       : lp\n")
                              (format #f (_ "  Remote host    : ~a\n") (caddr q))
                              (format #f (_ "  Remote queue   : ~a\n") (list-ref q 3))))
                      (else
                        '()))
                    (list
                      (format #f (_ "      Type       : ~a\n")
                        (cond
                          ((equal? driver '-)
                            "PostScript")
                          ((equal? "" extf)
                            (format #f (_ "GhostScript filtered (~a)") driver))
                          (else
                            (format #f (_ "Filtered (using ~a)") extf)))))
                    (cond
                      ((not gs?) '())
                      (else
                        (let ((db (obj-pr 'get-gsdb driver))
                              (kl (obj-pr 'get-optkey qname)))
                          (cond
                            ((or (null? db)
                                 (null? kl))
                              '())
                            (else
                              (cons "  ----------------------------------------------\n"
                                (map (lambda (k v)
                                       (let* ((r (assoc k db))
                                              (n (+ 4 (* v 2))))
                                         (cond
                                           ((or (eq? #f r)
                                                (< (length r) (1+ n)))
                                             "")
                                           (else
                                             (sprintf "%-16s : %s\n"
                                               (string-cut (_ (cadr r)) 16)
                                               (_ (list-ref r n)))))))
                                     kl
                                     (obj-pr 'get-optval qname))))))))))
              50 4 4 6
              (vector)
              (if gs? (vector (_ "Ok") (_ "Modify") (_ "Option") (_ "Remove") (_ "Skip"))
                      (vector (_ "Ok") (_ "Modify") (_ "Remove") (_ "Skip"))))))
      (case (car r)
        ((1) #t)
        ((2)
          (let ((r1 (modify q)))
            (cond
              ((eq? #f r1)
                (inst-log "modify failed\n")
                #f)
              (else
                (check-commit r1))))
          (let ((r2 (obj-pr 'getq qname)))
            (view (if (eq? #f r2) q r2))))
        ((3)
          (cond
            (gs?
              (let* ((ol (obj-pr 'get-optassoc qname)))
                (cond
                  ((null? ol)
                    (snewt-message title
                      (_ "This gs driver doesn't have options."))
                    #f)
                  (else
                    (let ((r1 (ask-gsopt driver ol)))
                      (cond
                        ((eq? r1 #f) #f)
                        (else
                          (check-commit (append (list-sub q 0 5)
                                          (obj-pr 'assoc->list r1)))))))))
              (let ((r2 (obj-pr 'getq qname)))
                (view (if (eq? #f r2) q r2))))
              ;(view q)
            (else
              (obj-pr 'remove (car q)))))
        ((4)
          (if gs?
             (obj-pr 'remove (car q))
             #f))
        (else
          #f))))
  
  (define (ask-interface usb? q n)
    (let ((r (snewt-entries title
               (format #f
                 (_ "Enter queue name and number of ~a interface.")
                 (if usb? "USB" "parallel"))
               50 5 5 20
               (list (list (_ "Queue name ") q 0)
                     (list (_ "Number") (format #f "~a" n) 0)))))
      (cond
        ((null? r) #f)
        (else
          (let ((n' (string->number (cadr r))))
            (cond
              ((equal? "" (car r))
                (snewt-message (_ "Error") (_ "Fill queue name.\n"))
                (ask-interface usb? q n))
              ((not (integer? n'))
                (snewt-message (_ "Error") (_ "Enter correct number.\n"))
                (ask-interface usb? q n))
              (else
                (list (car r) n'))))))))
  
  (define (ask-lphost lq h rq)
    (let ((r (snewt-entries title
               (_ "Enter local queue name and remote host name and queue name.")
               50 5 5 20
               (list (list (_ "Local queue name") lq 0)
                     (list (_ "Remote Host name") h 0)
                     (list (_ "Remote queue name") rq 0)))))
      (cond
        ((null? r) #f)
        ((ormap (lambda (x) (equal? "" x)) r)
          (snewt-message (_ "Error") (_ "Please fill every entry.\n"))
          (ask-lphost (car r) (cadr r) (caddr r)))
        (else
          r))))
  
  (define (choose-driver)
    (let* ((l (obj-pr 'list-driver))
           (r (snewt-nmenu title
                (_ "Select driver from below")
                50 4 4 6
                (list->vector 
                  (map (lambda (x) (format #f "~a" x)) l))
                (vector (_ "Ok") (_ "Skip")))))
      (case (car r)
        ((-1 2) #f)
        (else
          (list-ref l (cdr r))))))
  
  ; o $B$O8=:_$N(B assoc
  (define (ask-gsopt d o)
    (let* ((db (obj-pr 'get-gsdb d)))
      (cond
        ((equal? d '-)
          '())
        ((or (null? db) (null? o))
          '())
          ; (snewt-message title (_ "This gs driver doesn't have options.")
        (else
          (let* ((r (snewt-nmenu title
                      (_ "Edit GhostScript driver option(s) below.")
                      64 4 4 6
                      (list->vector
                        (map (lambda (x)
                               (let* ((y (assoc (car x) o))
                                      (v (cond
                                           ((eq? #f y) (caddr x))
                                           (else (cdr y))))
                                      (n (+ 4 (* v 2))))
                                 (sprintf " %-24s : %s "
                                   (string-cut (cadr x) 24)
                                   (if (< (length x) (1+ n))
                                       "error"
                                       (list-ref x n)))))
                             db))
                      (vector (_ "Ok") (_ "Skip")))))
            (case (car r)
              ((-1 2) #f)
              ((1) o)
              (else
                (let* ((l (list-ref db (cdr r)))
                       (r1 (snewt-nmenu title
                             (format #f (_ "Select value for option ~a.") (cadr l))
                             50 4 4 6
                             (list->vector
                               (obj-pr 'pick-odd (cddr (cddr l))))
                             (vector (_ "Ok") (_ "Skip")))))
                  (case (car r1)
                    ((-1 2) #f)
                    (else
                      (ask-gsopt d
                        (let* ((n (cdr r1))
                               (a (assoc (car l) o)))
                          (cond
                            ((eq? #f a)
                              (cons (cons (car l) n)
                                    o))
                            (else
                              (map (lambda (x)
                                     (if (equal? (car x) (car l))
                                       (cons (car l) n)
                                       x))
                                   o)))))))))))))))
  
  (define (modify q)
    (let ((r0 (snewt-nmenu title
                (_ "Select type of printer.")
                50 4 4 6
                (vector
                  (_ " Local printer, using Parallel port ")
                  (_ " Local printer, using USB ")
                  (_ " Network printer (lp protocol) "))
                (vector (_ "Ok") (_ "Skip")))))
      (case (car r0)
        ((-1 2) #f)
        (else
          (let* ((local? (case (cdr r0)
                           ((0 1) #t)
                           (else #f)))
                 (usb? (= 1 (cdr r0)))
                 (a (if local? 
                      (ask-interface usb? (car q) (list-ref q 3))
                      (case (cadr q)
                        ((lp) (ask-lphost (car q) (caddr q) (list-ref q 3)))
                        (else (ask-lphost "" "" ""))))))
            (cond
              ((eq? #f a) #f)
              (else
                (let* ((l (obj-pr 'get-plist))
                       (r1 (snewt-nmenu title
                             (_ "Select Printer model.")
                             50 4 4 9
                             (list->vector 
                               (cons (_ "Generic PostScript Printer")
                                 (map (lambda (x)
                                        (sprintf "%-12s : %s" (car x) (cadr x)))
                                   l)))
                             (vector (_ "Ok") (_ "Choose from driver") (_ "Cancel")))))
                  (let ((d (case (car r1)
                             ((2) (choose-driver))
                             ((3) #f)
                             (else
                               (case (cdr r1)
                                 ((0) '-)
                                 (else
                                   (list-ref (list-ref l (1- (cdr r1))) 3)))))))
                    (inst-log "d : ~s\n" d)
                    (cond
                      ((not (symbol? d))
                        (inst-log "not symbol. ~a\n" d)
                        #f)
                      (else
                        (let* ((o (ask-gsopt d (obj-pr 'get-optassoc (car q)))))
                          (cond
                            ((eq? #f o)
                              #f)
                            (else
                              (append
                                (list (car a)
                                      (if local? 'local 'lp)
                                      (if local? (if usb? 'usb 'par) (cadr a))
                                      (if local? (cadr a) (caddr a))
                                      d)
                                (obj-pr 'assoc->list o))))))))))))))))
  
  (newt-stage-title title)
  (let* ((q (obj-pr 'qlist))
         (r (snewt-nmenu title
              (if (null? q)
                (_ "Press add to create new queue.")
                (_ "Select queue name or 'add' button."))
              50 4 4 6
              (list->vector
                (map (lambda (x)
                       (format #f " ~a " (obj-pr 'descq x)))
                     q))
              (vector (_ "Ok")  (_ "Add Queue") (_ "Cancel")))))
    (case (car r)
      ((1)
        #t)
      ((-1 3) #f)
      ((2)
        (let ((r1 (modify '("" 'local 'par 0 '-))))
          (cond
            ((eq? #f r1)
              (inst-log "modify failed\n")
              #f)
            (else
              (check-commit r1))))
        (newt-ui-printer-entry obj-pr))
      (else
        (view (obj-pr 'getq (list-ref (obj-pr 'qlist) (cdr r))))
        (newt-ui-printer-entry obj-pr)))))


