;
;     config/patch manager
;
;                                              (C)2002 Omoikane Inc.
;

(define (conf-patch obj)
  ; var
  (define conf
    (map (lambda (x) (list (car x) (cadr x) 'error (caddr x)))
         (obj 'conf)))
  
  (define tmpd
    (let ((d (unix-tempname)))
      (cond
        ((not (and (= 0 (sh "mkdir ~a" d))
                   (= 0 (sh "chmod 700 ~a" d))))
          (inst-log "patcher : could not mkdir ~a" d)
          #f)
        (else
          d))))
  
  (define rm-list '())
  (define remake-list '())
  (define root "")
  ; $B$3$l$O(B init $B%a%=%C%I$H6&$K(B inst2 $B$G$N$_;HMQ$5$l$k(B
  
  ; libs
  (define (flat x) (string-replace x #\/ #\-))
  
  (define (do-diff f0 f1 d)
    (let ((r (sh "diff -u ~a ~a 2>/dev/null 1>~a" f0 f1 d)))
      (case r
        ((0) (sh "rm -f ~a" d))
        ((1) #t)
        (else #f))))
  
  (define (flash)
    (set! conf
          (map (lambda (x)
                 (let* ((t (car x))
                        (t' (flat t))
                        (f (cadr x))
                        (tmp (unix-tempname tmpd))
                        (r (cond
                             ((not (f tmp))
                               (inst-log "could not generate : ~a ~a\n" t tmp)
                               'error)
                             ((unix-access? (format #f "~a/~a" tmpd t') '(read))
                               (cond
                                 ((not (= 0 (sh "patch ~a ~a/~a  2>&1" tmp tmpd (flat t))))
                                   'fail)
                                 ((= 0 (sh "diff ~a ~a 1>/dev/null 2>&1" t tmp))
                                   'patch)
                                 (else
                                   'modify)))
                             (else
                               (cond
                                 ((or (not (unix-access? t '(read)))
                                      (= 0 (sh "diff ~a ~a 1>/dev/null 2>&1" t tmp)))
                                   'clean)
                                 (else
                                   'modify))))))
                   (sh "rm -f ~a" tmp)
                   (list t f r (list-ref x 3))))
               conf)))
  
  ; method
  (define (load-it)
    (map (lambda (x)
           (let* ((t (flat (car x)))
                  (p (format #f "/var/lib/ogl/patch/~a" t)))
             (if (unix-access? p '(read))
               (sh "cp ~a ~a/~a" p tmpd t))))
         conf)
    (and (not (eq? #f tmpd))
         (obj 'load)))
  
  (define (list-all)
    (flash)
    (map (lambda (x) 
           (list (car x) (caddr x)))
         conf))
  
  (define (list-fail)
    (flash)
    (map car
      (filter (lambda (x)
                (and (equal? 'fail (caddr x))
                     (null? (filter (lambda (y) (equal? y x))
                                    rm-list))))
              conf)))
  
  (define (list-modify)
    (flash)
    (inst-log "rm-list : ~a" rm-list)
    (map car
      (filter (lambda (x)
                (and (equal? 'modify (caddr x))
                     (null? (filter (lambda (y) (equal? y (car x)))
                                    rm-list))))
              conf)))
  
  (define (remove t)
    (let ((p (format #f "~a/~a" tmpd (flat t))))
      (cond
        ((and (unix-access? p '(read))
              (not (= 0 (sh "rm -f ~a" p))))
          #f)
        (else
          (set! remake-list
                (filter (lambda (x) (not (equal? t x)))
                        remake-list))
          (set! rm-list (cons t rm-list))
          #t))))
  
  (define (remake t)
    (let ((r (assoc t conf)))
      (cond
        ((eq? #f r)
          #f)
        (else
          (let* ((f (cadr r))
                 (tmp (unix-tempname tmpd))
                 (a (cond
                      ((not (f tmp))
                        (inst-log "remake failed : ~a\n" tmp)
                        #f)
                      ((not (do-diff tmp t (format #f "~a/~a" tmpd (flat t))))
                        (inst-log "diff failed : ~a\n" tmp)
                        #f)
                      (else
                        (set! remake-list (cons t remake-list))
                        (set! rm-list
                              (filter (lambda (x) (not (equal? f x)))
                                      rm-list))
                        #t))))
            (sh "rm -f ~a" tmp)
            a)))))
  
  (define (remade? t)
    (not (null? (filter (lambda (x) (equal? x t))
                        rm-list))))
  
  (define (commit)
    (flash)
    (cond
      ((not (andmap (lambda (x)
                      (let* ((t (string-append root (car x)))
                             (t' (flat (car x)))
                             (f (cadr x)))
                        (cond
                          ((and (list-ref x 3)
                                (unix-access? t '(read))
                                (not (= 0 (sh "mv ~a ~a.bak" t t))))
                            (inst-log "failed to save old config for ~a" t)
                            #f)
                          ((not (f t))
                            (inst-log "failed to make ~a.\n" t)
                            #f)
                          ((not (unix-access? (format #f "~a/~a" tmpd t') '(read)))
                            (sh "rm -f /var/lib/ogl/patch/~a" t')
                            (inst-log "no patch for ~a (~a)\n" t tmpd)
                            #t)
                          ((not (= 0 (sh "patch ~a ~a/~a" t tmpd t')))
                            (inst-log "failed to patch ~a\n" t)
                            #f)
                          ((not (= 0 (sh "cp ~a/~a /var/lib/ogl/patch/~a" tmpd t' t')))
                            (inst-log "failed to cp patch ~a to /var\n" t)
                            #f)
                          (else
                            (inst-log "debug : patch ~a ~a/~a" t tmpd t')
                            (sh "cp ~a /tmp/sample" t)
                            #t))))
                    conf))
        #f)
      ((not (= 0 (sh "rm -rf ~a" tmpd)))
        (inst-log "patcher : could not erase ~a" d)
        #f)
      (else
        (if (and (not (null? rm-list))
                 (not (= 0 (sh (apply string-append
                                 (cons "cd /var/lib/ogl/patch ; rm -f "
                                       (map (lambda (x) (format #f "~a " (flat x)))
                                            rm-list)))))))
          (inst-log "removing old patch failed."))
        (obj 'commit))))
  
  (define (make-cur t tmp)
    (let ((r (assoc t conf)))
      (cond
        ((eq? #f r)
          #f)
        (else
          (let* ((t' (flat t)))
            (cond
              ((not ((cadr r) tmp))
                #f)
              ((not (unix-access? (format #f "~a/~a" tmpd t') '(read)))
                #t)
              ((not (= 0 (sh "patch ~a ~a/~a" tmp tmpd t')))
                (inst-log "failed to patch ~a\n" t)
                #f)
              (else
                #t)))))))
  
  (define (make-diff t tmp)
    (let ((r (assoc t conf))
          (tmp2 (unix-tempname tmpd)))
      (cond
        ((eq? #f r)
          #f)
        ((not (make-cur t tmp2))
          #f)
        ((not (do-diff t tmp2 tmp))
          #f)
        (else
          (sh "rm -f ~a" tmp2)
          #t))))
  
  (define (init c)
    (case c
      ((inst2)
        (set! root "/target")))
    #t)
  
  (define (mid-conf)
    (obj 'mid-conf))
  
  (make-object 
    'init          init
    'make-cur      make-cur
    'make-diff     make-diff
    'mid-conf      mid-conf
    'load          load-it
    'list-all      list-all
    'list-fail     list-fail
    'list-modify   list-modify
    'do-postconf   (lambda () (obj 'postconf))
    'postconf?     (lambda () (obj 'method? 'postconf))
    'remove        remove
    'remake        remake
    'remade?       remade?
    'commit        commit))
