#!/bin/sh
string=?;exec /usr/bin/elk -c -l $0 -p /usr/lib/ogl/scm:/usr/lib/elk/scm:/usr/lib/elk/obj:`pwd` $*

(define a (command-line-args))
(require 'unix)
(load "util.scm")
(load "debug.scm")


(define (flush-format . l)
  (apply format (cons #t l))
  (unix-fflush))

(set! sh-log "logger")

(define (debwrap f l)
  (system-at sh-log (format #f "LANG=C ~a wrap ~a"
                               (if (string? f)
                                   (format #f "QANDA_FILE=~a " f)
                                   "")
                               l)))

(define (apt-update)
  (debwrap #f "/usr/lib/dpkg/methods/apt/update /var/lib/dpkg"))

(define (apt-install)
  (debwrap #f "/usr/lib/dpkg/methods/apt/update /var/lib/dpkg"))

(define (apt-f-install)
  (debwrap #f "apt-get -f install"))

(define ans 
  (let ((f "/tmp/inst/easy-answer"))
    (fprintf f "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n")
    f))
(define (ansrun l)
  (sh (format #f "LANG=C ~a <~a" l ans)))

(define (build-qmail-0)
	(flush-format (_ "1 Prepare to build ucspi-tcp/qmail ...\n"))
  (ansrun "dpkg --configure -a")
  (ansrun "apt-get install ucspi-tcp-src")
  (ansrun "apt-get install qmail-src")
  (ansrun "dpkg --configure -a"))

(define (have-package? x)
  (= 0 (sh "dpkg -l | grep '^ii  ~a '" x)))

(define (build-qmail-1)
  (cond
    ((not (have-package? "ucspi-tcp"))
      (flush-format (_ "0 Building ucspi-tcp ...\n"))
      (sh "rm -rf /tmp/ucspi-tcp")
      (sh "/usr/bin/build-ucspi-tcp <~a" ans)
      (build-qmail-1))
    (else
      (flush-format (_ "1 ucspi-tcp built and installed.\n"))
      (sh "apt-get install jless <~a" ans)
      #t)))

(define (build-qmail-2)
  (cond
    ((not (have-package? "qmail"))
      (flush-format (_ "0 Building qmail ...\n"))
      (sh "rm -rf /tmp/qmail /var/qmail")
      (sh "mkdir /tmp/qmail")
      (sh "cd /tmp/qmail ; dpkg-source -x /usr/src/qmail-src/*.dsc")
      (sh "cd /tmp/qmail/qmail-* ; LANG=C debian/rules binary-arch")
      (sh "cd /tmp/qmail/qmail-* ; LANG=C dpkg -i /tmp/qmail/qmail*i386.deb <~a" ans)
      (build-qmail-2))
    (else
      (flush-format (_ "1 qmail built and installed.\n"))
      #t)))

(define (do-apt to l)
  (let ((cmd (string-append
               (format #f "wrap -t ~a -- apt-get install ~a 2>&1 | logcat | "
                 to (list-foldstring " " l))
               (format #f
                 (string-append
                   "gawk '/BUF.* .(Unpacking|Setting)/"
                   "{sub(/^BUF.*: ./,\"\") ; sub(/ replacement/,\"\") ; sub(/Setting up/, \"Setting\") ; "
                     "if(match($0, /^Setting/)>0) printf(\"0 ~a\", $2) ; "
                     "if(match($0, /^Unpacking/)>0) printf(\"1 ~a\", $2) ; "
                     "fflush(\"\")}' ; exit ${PIPESTATUS[0]}")
                 (_ "Setting %s ...\\n")
                 (_ "Unpacking %s ...\\n")))))
    (inst-log "cmdline : ~a\n" cmd)
    (let ((r (system-at "/dev/stdout" cmd)))
      (inst-log "apt-get -f install returned with ~a\n" r)
      r)))
; PIPESTATUS $B$K$h$k;2>H$O(B bash $B$N3HD%$J$N$GB>$N(B shell $B$K?)$o$;$F$O$$$1$^$;$s(B

(define (install l)
  (let* ((f "/tmp/inst/list-tmp")
         (p (open-output-file f)))
    (map (lambda (x) (display (format #f "~a install\n" x) p)) l)
    (close-port p)
    (sh "dpkg --set-selections <~a" f)
    ; $B",$OITMW$N$O$:(B
    (apt-update)
    (let ((r (do-apt 300 l)))
      ; $B%G%U%)%k%H(B 300 
      (apt-f-install)
      r)))

(define (search-fail l)
  (define (rep l)
    (cond
      ((null? l) l)
      (else
        (let ((r (do-apt 600 (list (car l)))))
          ; $B%G%U%)%k%H(B 600 
          (if (not (= r 0))
            (list (car l) r)
            (rep (cdr l)))))))
  (let ((rm (parse-awk #\space #t "| COLUMNS=300 dpkg -l  | grep -v ^ii")))
    (inst-log "rm : ~a\n" rm)
    (cond
      ((not (and (list? rm)
                 (< 6 (length rm))))
        (inst-log "could not get erase target : ~s\n" rm))
      (else
        (let ((s (list-foldstring " " (map cadr (cddr (cddr (cdr rm)))))))
          (inst-log "removing : ~a\n" s)
          (system-at "/dev/null"
            (format #f "wrap -t 120 -- dpkg --purge ~a 2>&1 | logger -t 'removing'" s))))))
  
  ;(let ((rm (list-foldstring " " l)))
  ;  (system-at "/dev/null"
  ;    (format #f "wrap -t 15 -- apt-get remove --purge ~a 2>&1 | logger -t 'removing'" rm))
  ;  (system-at "/dev/null"
  ;    (format #f "wrap -t 15 -- dpkg --remove ~a 2>&1 | logger -t 'removing'" rm)))
  ; $BK\Ev$O!"$3$N0l2s$N%Q%9$N0JA0$+$iF~$C$F$$$?%Q%C%1!<%8$^$GH4$/I,MW$O$J$$$,(B
  
  (system-at "/dev/null" "wrap -t 120 -- dpkg --configure -a  2>&1 | logger -t 'removing'")
  
  ;(apt-f-install)
  (let ((p (rep l)))
    (if (not (null? p))
      (system-at "/dev/null"
        (format #f "wrap -t 120 -- dpkg --remove ~a 2>&1 | logger -t 'removing'" (car p))))
    p))
; $BK\Ev$O$b$&0l2s%A%'%C%/$7$?$[$&$,$h$$!#$D$^$j8+$D$1$?%Q%C%1!<%8$,(B ^ii $B$K$J$C$F$$$?$i(B
; $B$=$l$O$=$N%Q%C%1!<%8<+?H$,860x$G$O$J$$>l9g$K$J$k!#(B


(define (split-install c l)
  (define (split n l-first l-rest)
    (cond
      ((or (= n 0) (null? l-rest))
        (cons l-first l-rest))
      (else
        (split (1- n) (cons (car l-rest) l-first) (cdr l-rest)))))
  (cond
    ((null? l)
      #t)
    (else
      (let* ((n 12)
             (x (split n '() l)))
        (cond
          ((null? x) #t)
          (else
            (let* ((h (car x))
                   (n' (length h))
                   (r (install h))
                   (r' (cond
                         ((= 0 r)
                           '())
                         (else
                           (inst-log "the first apt-get attempt failed with ~a. searching problematic package ...\n" r)
                           (search-fail h)))))
              (cond
                ((null? r')
                  (flush-format "=~a\n" (+ c n'))
                  (split-install (+ c n') (cdr x)))
                (else
                  (inst-log "pgkinst : installation error on ~a with ~a \n" (car r') (cadr r'))
                  r')))))))))

(define (make-newlist l x)
  (define (rep l)
    (cond
      ((null? l) l)
      ((equal? (car l) x)
        (cdr l))
      (else
        (rep (cdr l)))))
  (fprintf "/tmp/inst/pkglist"
    (apply string-append
      (map (lambda (x) (format #f "~a\n" x))
           (rep l)))))

(define (start-install n l)
  (let ((r (split-install n l)))
    (inst-log "start-install : split-install returns ~a\n" r)
    (cond
      ((or (eq? #t r) (not (list? r)))
        (exit 0))
      (else
        (fprintf "/tmp/inst/pkgfail" (format #f "~a\t~a\n" (car r) (cadr r)))
        (sh "mv /tmp/inst/pkglist /tmp/inst/pkglist.prev")
        (make-newlist l (car r))
        (exit 1)))))

(unix-setenv "PATH" (string-append "/usr/sbin/ogl:/usr/lib/ogl/bin:"
                                   (unix-getenv "PATH")))
(textdomain "ogl-admin" "/usr/share/locale")
(let ((l' (parse-awk #\space #t "/tmp/inst/pkglist")))
  (sh "rm -f /tmp/inst/pkgfail")
  (cond
    ((eq? l' #f)
      (inst-log "Fatal Error. Cannot open /tmp/inst/pkglist.")
      (exit 1))
    (else
      (let ((l (filter (lambda (x) (not (equal? "" x)))
                 (map car l'))))
        (flush-format "~a\n" (length l))
        (flush-format (_ "0 Starting to install packages ...\n"))
        (apt-update)
        (apt-install)
        (cond
          ((and (set-include? "qmail-src" l)
                (not (have-package? "qmail")))
            (let ((first '("libc6-dev" "groff-base" "groff" "procmail"
                           "ucspi-tcp-src" "qmail-src" "dpkg-dev")))
              (install first)
              (build-qmail-0)
              (build-qmail-1)
              (build-qmail-2)
              (apt-update)
              (start-install (length first) (list-set- equal? l first))))
          (else
            (start-install 0 l)))))))
