

(define (snewt-fatal-error . l)
  (inst-log "fatal error occured.\n")
  (let ((m (apply format (cons #f l))))
    (define (rep)
      (snewt-message (_ "Fatal Error") m)
      (rep))
    (inst-log m)
    (rep)))

(define (snewt-menu t m w d u n . l)
  (let ((r (snewt-nmenu t m w d u n (list->vector l) (vector (_ "Ok")))))
    (cond
      ((= -1 (car r))
        -1)
      (else
        (cdr r)))))

(define (snewt-menuc t m w d u n . l)
  (let ((r (snewt-nmenu t m w d u n (list->vector l) (vector (_ "Ok") (_ "Skip")))))
    (case (car r)
      ((-1 2)
        -1)
      (else
        (cdr r)))))

(define (snewt-message2 t m)
  (snewt-smessage t m 40 6))


;
;    Get Skelton Tree (where anything is not selected)
;
(define (snewt-tree-skel l)
  (define (get-one l)
    (cond
      ((not (list? l)) '())
      ((null? l) l)
      (else
        (cons
          (car l)
          (list-filter
            (lambda (x) (not (null? x)))
            (map get-one (cdr l)))))))
  (list-filter
    (lambda (x) (not (null? x)))
    (map get-one l)))
      

(define (snewt-get-raw t s)
  (define v '())
  (define id 1)
  (define (found? a l)
    (cond
      ((null? l) #f)
      ((not (string? (car l)))
        (found? a (cdr l)))
      (else
        (or (string=? a (car l))
            (found? a (cdr l))))))
  (define (get-select s l)
    (cond
      ((null? l) #f)
      ((or (not (list? (car l)))
           (not (string=? s (caar l))))
        (get-select s (cdr l)))
      (else
        (car l))))
  (define (build-sub t s)
    (cond
      ((null? t)
        '())
      ((list? (car t))
        (let ((s' (get-select (caar t) s)))
          (if (eq? #f s')
              #f
              (cons
                (build-top (car t) s')
                (build-sub (cdr t) s)))))
      (else
        (set! id (1+ id))
        (if (found? (car t) s)
            (set! v (cons id v)))
        (cons (cons (car t) id)
              (build-sub (cdr t) s)))))
  (define (build-top t s)
    (cons
      (cons (car t) 0)
      (build-sub (cdr t) (cdr s))))
  (cons (build-sub t s) v))

;
;  Tree Menu
;
; arguments are:
;  t : title string
; m : message
; w : width of message
; h2 : hight of message
; l : tree
; s : selected part of tree
; h : hight of window
; c : string as collection of character which used for selection button
;
; returned value is a list of 
;   list of total selected item
;   list of added item
;   list of eliminated item
; 
(define (snewt-tree t m w h2 l s h c)
  (let ((r (snewt-get-raw l s)))
    (let ((v0 (list->vector (cdr r)))
          (l' (car r)))
      (define (get-string i)
        (define (search-one l)
          (cond
            ((null? l) #f)
            ((= (cdr (car l)) i) (caar l))
            (else
              (search-one (cdr l)))))
        (define (search l)
          (cond
            ((null? l)
              #f)
            ((list? (car l))
              (let ((x (search-one (car l))))
                (if (eq? #f x)
                    (search (cdr l))
                    x)))
            ((= (cdr (car l)) i)
              (caar l))
            (else
              (search (cdr l)))))
        (search l'))
      (let ((v1 (snewt-tree-raw t m w h2 l' v0 h c)))
        (snewt-cls)
        (if v1
            (list
             (map get-string (vector->list v1))
             (map get-string (vector->list (vector-set- = v1 v0)))
             (map get-string (vector->list (vector-set- = v0 v1))))
            #f)))))

(define snewt-flag-returnexit 1)
(define snewt-flag-hidden 2)
(define snewt-flag-scroll 4)
(define snewt-flag-password 2048)

(define (snewt-radio-tree t m w h h2 tree)
  (define (make-tree t)
    (map (lambda (a)
           (if (pair? a) (cons #f (cons (car a) (make-tree (cdr a))))
               a))
         t))
  
  (define (indent string depth)
    (string-append (make-string depth #\space) string))

  (define (flatten tree depth)
    (cond ((null? tree) tree)
          ((pair? (car tree))
           (append
            (flatten-node (car tree) depth)
            (flatten (cdr tree) depth)))
          (else
           (cons (list (indent (car tree) (+ depth 3)) (car tree))
                 (flatten (cdr tree) depth)))))
  
  (define (flatten-node node depth)
    (define (node->string node)
      (format #f "<~a>~a" (if (car node) "-" "+") (cadr node)))
    (if (car node)
        (cons (list (indent (node->string node) depth) node)
              (flatten (cddr node) (+ 3 depth)))
        (list (list (indent (node->string node) depth) node))))

  (define (open-close tree elm)
    (cond ((null? tree) tree)
          ((eqv? (car tree) elm)
           (cons (cons (not (caar tree)) (cdar tree))
                 (cdr tree)))
          (else
           (cons (car tree) (open-close (cdr tree) elm)))))

  (define (max-width tree)
    (+ 3 (apply max (map
                     (lambda (x)
                       (cond ((string? x) (string-length x))
                             (else (max (string-length (car x))
                        								(max-width (cdr x))))))
										 tree))))
	
	(let ((width (max-width tree)))
		(let loop ((tree (make-tree tree))
							 (pos 0))
			(let* ((ftree (flatten tree 0))
						 (elms (map car ftree))
						 (len (string-length (car elms)))
						 (ans (apply snewt-menu-old t m w 1 1 h2 pos
												 (string-append (car elms)
																				(make-string (- width len) #\space))
												 (cdr elms)))
						 (elm (cadr (list-ref ftree ans))))
				(cond ((string? elm) elm)
							(else (loop (open-close tree elm) ans)))))))

