#!/bin/sh
string=?;exec /usr/bin/elk -c -l $0 -p /usr/lib/ogl/scm:/usr/lib/elk/scm:/usr/lib/elk/obj:`pwd` $* 2>/dev/null
(load "util.scm")
(load "obj.scm")
(load "newt1.scm")
(load "newt2.scm")
(load "newt3.scm")
(load "net.scm")
(load "media.scm")
(load "time.scm")
(load "conffile.scm")
(load "part.scm")
(load "debug.scm")
(load "keyboard.scm")
(load "mod.scm")
(load "xwin.scm")
(load "gtk-middle.scm")
(load "gtk-obj.scm")
(load "gtk.scm")
(load "gtk3.scm")
(load "snewt.scm")
(load "account.scm")
(load "service.scm")
(load "boot.scm")
(load "fdisk.scm")
(load "volume.scm")
(load "ideconf.scm")
(load "patch.scm")
(load "na.scm")
(load "profile.scm")
(load "dirsv.scm")
(load "prod.scm")
(load "orca.scm")
(load "add.scm")
(load "cron.scm")


(set! debug-with-readline? #t)
(set! sh-log "logger")

(define (recover-all obj-part obj-mod)
  (let ((tmp (unix-tempname)))
    (sh "cp /etc/inittab ~a" tmp)
    (sh "sed 's/^id:4:initdefault:$/id:2:initdefault:/' <~a >/etc/inittab" tmp)
    (sh "rm ~a" tmp))
  (let* ((obj-boot (configure-boot obj-part obj-mod))
         (patcher (conf-patch obj-boot)))
    (obj-boot 'init 'inst3)
    (patcher 'load)
    (obj-boot 'set-timeout 13)
    (patcher 'commit)
    (obj-prof 'commit)
    (obj-part 'commit)
    (patch-part 'commit)
    ))

(define (runhook n)
  (let ((f (format #f "/tmp/inst/hook/~a" n)))
    (if (unix-access? f '(execute))
      (sh f))))

(define (confpath x) (string-append "/var/lib/ogl/etc/" x))
(define (instpath x) (string-append "/tmp/inst/" x))

(define (check-expand f)
  (cond
    ((unix-access? (confpath f) '(read))
      (= 0 (sh "tar xf ~a -C /" (confpath f))))
    (else
      #t)))

(define (compose-pkglist base? ws? sv? orca? postfix?)
  (inst-log "compose-pkglist : ~a ~a ~a ~a ~a\n" base? ws? sv? orca? postfix?)
  (cond
    ((not base?)
      (sh "rm -f /tmp/inst/pkglist")
      (sh "touch /tmp/inst/pkglist")
      #t)
    ((not (andmap (lambda (x) (unix-access? (format #f "/var/lib/ogl/etc/pkglist-~a" x) '(read)))
            '("base" "ws" "sv" "qmail" "postfix" "orca")))
      (_ "Could open pkglist-* file(s).\n"))
    ((not (and (if base? (= 0 (sh "cp /var/lib/ogl/etc/pkglist-base /tmp/inst/tmp"))
                         #t)
               (if ws?   (= 0 (sh "cat /var/lib/ogl/etc/pkglist-ws >>/tmp/inst/tmp"))
                         #t)
               (if sv?   (= 0 (sh "cat /var/lib/ogl/etc/pkglist-sv >>/tmp/inst/tmp"))
                         #t)
               (if orca? (= 0 (sh "cat /var/lib/ogl/etc/pkglist-orca >>/tmp/inst/tmp"))
                         #t)
               (if postfix?
                  (= 0 (sh "cat /var/lib/ogl/etc/pkglist-postfix >>/tmp/inst/tmp"))
                  (= 0 (sh "cat /var/lib/ogl/etc/pkglist-qmail >>/tmp/inst/tmp")))
               (= 0 (sh "sort /tmp/inst/tmp | awk '{if(NF==1)print $1}' >/tmp/inst/pkglist"))))
      (_ "Could not compose pkglist.\n"))
    (else
      (fprintf "/var/lib/ogl/etc/pkgsel"
        (string-append
          (if base?    "base\n" "")
          (if ws?      "ws\n" "")
          (if sv?      "sv\n" "")
          (if orca?    "orca\n" "")
          (if postfix? "postfix\n" "qmail\n")))
      #t)))

(define (arma-introduce-text)
  (cond
    ((arma-check) "")
    (else
      (string-append
        (_ "\n\nIf you prefer OGL Free, you can buy ARMA.\n")
        (_ "ARMA is commercial version of OGL and more sophisticated.\n")
        (_ "ARMA has designed desktop, ATOK X for Linux, ricoh TrueType, MP3 encoder, fully functional version of cofiguration tool and enough packages.\n")
        ))))

(define (finish-all)
  (sh "rm /var/lib/ogl/etc/under-inst")
  (patch-mod 'commit))

(define (service-check obj-sv obj-mod)
  (obj-sv 'default)
  (obj-sv 'remove "vtun")
  (if (ormap (lambda (x) (equal? 'pcmcia_core (car x)))
             (obj-mod 'table))
      (obj-sv 'set-start "pcmcia"))
  #t)

(define (3rd-newt)
  (define (finish-it)
    (snewt-finish)
    (sh "sync ; clear"))
  
  (define (confirm-error-proceed m)
    (let ((r (snewt-choice (_ "Error")
               (string-append m (_ "\n\nRest of installation might not be processed well.\nDo you proceed anyway?"))
               (_ "Proceed") (_ "Finish here"))))
      (case r
        ((0 2)
          (snewt-message (_ "Error")
            (_ "Installation is incomplete. Exiting abnormally."))
          (finish-it)
          (exit 1)))))
  
  (inst-log "===== 3rd-newt\n")
  (textdomain "ogl-admin" "/usr/share/locale")
  (cond
    ((unix-access? "/proc/cons" '(write))
      (sh "echo 2 >/proc/cons")
      (unix-sleep 1)
      (sh "echo 0 >/proc/cons")))
  (sh "sync ; clear")
  (snewt-start)
  (newt-stage-title (_ "starting third stage"))
  (snewt-show-while (_ "starting third stage")
    (_ "Starting third stage ...") 40 8
    (lambda() (unix-usleep 500000)))
  ; ߡβ̽Ȳ줬
  
  (let ((title-ide (_ "IDE setting (HDD optimization)")))
    (newt-stage-title title-ide)
    (let* ((obj-ide (conf-ide obj-part))
           (patcher (conf-patch obj-ide)))
      (obj-ide 'init 'inst3)
      (and (not (null? (obj-ide 'table)))
           (conf-newt-load/confirm patcher title-ide)
           (newt-ui-ide obj-ide obj-part)
           (conf-newt-finish patcher title-ide))))
  
  (repeat (lambda () (newt-ui-root-pass obj-acc)))
  (let* ((obj-ds (conf-dirsv obj-sv))
         (patcher (conf-patch obj-ds)))
    (and (conf-newt-load/confirm patcher (_ "Account Setting"))
         (newt-ui-user-account obj-ds obj-acc)
         (conf-newt-finish patcher (_ "Account Setting"))))
  
  (cond
    ((unix-access? (confpath "qanda.tar") '(read))
      (sh "rm -f ~a/qanda/*" (instpath ""))
      (sh "cd ~a/qanda ; tar xf ~a" (instpath "") (confpath "qanda.tar"))))
  
  (newt-stage-title (_ "Media Selection"))
  (snewt-show-while (_ "Media Selection")
    (_ "Arranging media ...") 40 3
    (lambda() (runhook 2)))
  
  (obj-med 'load)
  (cond
    ((and (equal? #t (obj-med 'open))
          (equal? #t (obj-med 'verify)))
      #t)
    (else
      (repeat (lambda () (newt-ui-media-default #t obj-med obj-part obj-na)))
      (obj-med 'open)))
  
  (if (not (newt-ui-selection compose-pkglist))
    (confirm-error-proceed (_ "Package selection was incomplete.")))
  
  (obj-med 'debconf-off)
  (sh "cp /tmp/inst/pkglist /tmp/inst/pkglist.orig")
  (cond
    ((< 0 (unix-filesize "/tmp/inst/pkglist"))
      (newt-ui-package))
    ((unix-access? "/tmp/inst/pkglist" '(read)) ; case of minimum
      #t)
    (else
      (inst-log "could not find /tmp/inst/pkglist.")
      (confirm-error-proceed
        (_ "Could not find the package list."))))
  
  (if (not (newt-check-package obj-med "/tmp/inst/pkglist.orig"))
    (confirm-error-proceed
      (_ "Package installation is incomplete.")))
  (obj-med 'debconf-on "Readline")
  
  (check-expand "expandl.tar")
  (check-expand "expand.tar")
  (runhook 3)
  
  (service-check obj-sv obj-mod)
  (obj-sv 'remove "kdm")
  (newt-ui-service obj-sv)
  
  (let* ((title-sound (_ "Sound Setting"))
         (obj-vol (conf-vol))
         (patcher-vol (conf-patch obj-vol)))
    (and (newt-module-sound obj-mod)
         (obj-vol 'init 'inst3)
         (conf-newt-load/confirm patcher-vol title-sound)
         (newt-ui-volume obj-vol)
         (conf-newt-finish patcher-vol title-sound)))
  
  (let* ((obj-sv (configure-service))
         (obj-pd (conf-prod obj-net obj-sv)))
    (obj-pd 'load)
    (newt-ui-atok-key obj-pd obj-sv))
  ; ɲåѥåפ ATOK ν֤촹ƤϤޤ
  ; pcmcia ξ restart äƤޤΤǡservice ꤫
  ; ֤˻ȤäƤޤ
  
  (let ((obj-add (conf-additional)))
    (newt-ui-addtitional #t obj-add))
  
  ; ORCA
  (let* ((obj-sv (configure-service))
         (obj-pd (conf-prod obj-net obj-sv))
         (patch (conf-patch obj-pd))
         (obj-prof (conf-prof))
         (patch-prof (conf-patch obj-prof))
         (obj-cr (conf-cron obj-sv))
         (patch-cron (conf-patch obj-cr)))
    (obj-pd 'load)
    (obj-prof 'load)
    (cond
      ((not (equal? 'orca (obj-pd 'product-type)))
        #t)
      ((not (= 1 (snewt-choice (_ "ORCA Configuration")
                   (_ "Do you configure ORCA now?\n\nYou can configure ORCA after installation too.")
                   (_ "Yes") (_ "No"))))
        #t)
      ((not (newt-ui-orca obj-pd patch obj-prof obj-cr))
        (snewt-message (_ "Info")
          (_ "ORCA configuration has been aborted. Configure ORCA later with ogl-admin."))
        #t)
      (else
        (obj-pd 'commit)
        (patch 'commit)
        (obj-prof 'commit)
        (patch-prof 'commit)
        (obj-cr 'commit)
        (patch-cron 'commit)
        #t)))
  
  (obj-med 'close)
  (newt-ui-default-media obj-med obj-na)
  
  (runhook 4)
  (recover-all obj-part obj-mod)
  
  (obj-med 'close)
  (patch-med 'commit)
  (newt-stage-title (_ "Installation finished"))
  (snewt-message (_ "Installation finished")
    (string-append
      (_ "You have done it.\n\n")
      (_ "The whole installation process has been finished. ")
      (_ "To configure X, login root and type 'ogl-admin'.")
      ;(arma-introduce-text)
      ))
  (finish-it))


(define (3rd-x)
  (define (confirm-error-proceed m)
    (let ((new (sgtk-window-new-toplevel-named (_ "Error")))
          (v (sgtk-vbox-new #f 10))
          (l (sgtk-label-new
               (string-append m
                 (_ "\n\nRest of installation might not be processed well.\nDo you proceed anyway?"))))
          (bb (sgtk-bbox #t
                (sgtk-button-new-with-label (_ "Finish here"))
                (sgtk-button-new-with-label (_ "Proceed")))))
      (sgtk-container-add-many v l (bb 'widget))
      (sgtk-container-add new v)
      (sgtk-widget-show-all new)
      (sgtk-main)
      (sgtk-grab-remove new)
      (sgtk-widget-destroy new)
      (case (bb 'info)
        ((1) #t)
        (else
          (exit 1)))))
  
  (unix-setenv "DISPLAY" ":0.0")
  (sh "mv $HOME/.sawfish/custom $HOME/.sawfish/custom.orig")
  (sh "cp /usr/lib/ogl/lib/sawfish-inst $HOME/.sawfish/custom")
  (sh "sawfish &")
  (sh "xloadimage -fullscreen -onroot /usr/lib/ogl/lib/wallpaper-~a.jpg"
      (if (arma-check-title) "arma" "free"))
  ; λǤ arma-check  #t ˤʤʤΤ title Τߤǥå
  (sh "xset s off")
  (cond
    ((not (sgtk-start))
      (inst-log "Could not initialize Gtk.\n"))
    (else
      (textdomain "ogl-admin" "/usr/share/locale")
      (let ((w (sgtk-window-new-toplevel))
            (p (sgtk-hpaned-new))
            (f (sgtk-frame-new (_ "Installation Step")))
            (v (sgtk-vbox-new #f 10))
            (b-kbd (sgtk-label-new (_ "Keyboard Setting")))
            (b-drv (sgtk-label-new (_ "Driver Setting")))
            (b-net (sgtk-label-new (_ "Network Setting")))
            (b-med (sgtk-label-new (_ "Media Selection")))
            (b-part (sgtk-label-new (_ "Partition Setting")))
            (b-base (sgtk-label-new (_ "Base Installation")))
            (b-time (sgtk-label-new (_ "Time Setting")))
            (b-boot (sgtk-label-new (_ "Boot Loader Installation")))
            (b-kern (sgtk-label-new (_ "Kernel Installation")))
            (b-reboot (sgtk-label-new (_ "Reboot")))
            (b-ide (sgtk-label-new (_ "IDE (HDD optimization)")))
            (b-root (sgtk-label-new (_ "Root Password")))
            (b-adduser (sgtk-label-new (_ "User Account")))
            (b-pkg (sgtk-label-new (_ "Package Installation")))
            (b-sound (sgtk-label-new (_ "Sound Setting")))
            (b-orca (sgtk-label-new (_ "ORCA Configuration")))
            (b-sv (sgtk-label-new (_ "Service Setting")))
            (b-fin (sgtk-label-new (_ "Finish Installation"))))
        (define (pack-set l)
          (sgtk-box-pack-start v l #f #f 0)
          (sgtk-widget-set-sensitive l #f))
        
        ; main window
        (sgtk-oglt-mainpanel w (_ "Installer") f p)
        (sgtk-signal-connect w "delete_event"
          (lambda (w e)
            (sgtk-message-info (_ "Could not abort installer."))))
        
        ; side
        (map pack-set
          (append (list b-kbd b-drv b-net b-med b-part b-base b-time b-boot b-kern 
                        b-reboot b-ide b-root b-adduser b-pkg b-sound b-sv)
                  (let* ((obj-sv (configure-service))
                         (obj-pd (conf-prod obj-net obj-sv)))
                    (if (equal? 'orca (obj-pd 'product-type))
                      (list b-orca b-fin)
                      (list b-fin)))))
        (sgtk-container-add-many f v)
        
        ; run
        (sgtk-widget-show-all w)
        (map (lambda (x) (sgtk-widget-set-sensitive x #f))
             (list b-kbd b-part b-base b-boot b-kern b-pkg b-fin))
        
        (sgtk-widget-set-sensitive b-ide #t)
        (let* ((obj-ide (conf-ide obj-part))
               (patcher (conf-patch obj-ide)))
          (obj-ide 'init 'inst3)
          (and (not (null? (obj-ide 'table)))
               (conf-gtk-load/confirm p patcher)
               (gtk-ui-ide p obj-ide obj-part)
               (conf-gtk-finish p patcher)))
        (sgtk-widget-set-sensitive b-ide #f)
        (sh "mv -f $HOME/.sawfish/custom.orig $HOME/.sawfish/custom")
        
        (sgtk-widget-set-sensitive b-root #t)
        (repeat (lambda () (gtk-ui-root-password p obj-acc)))
        (sgtk-widget-set-sensitive b-root #f)
        
        (sgtk-widget-set-sensitive b-adduser #t)
        (let* ((obj-sv (configure-service))
               (obj-ds (conf-dirsv obj-sv))
               (patcher (conf-patch obj-ds)))
          (and (conf-gtk-load/confirm p patcher)
               (gtk-ui-user-account p obj-ds obj-acc)
               (conf-gtk-finish p patcher)))
        (sgtk-widget-set-sensitive b-adduser #f)
        
        (cond
          ((unix-access? (confpath "qanda.tar") '(read))
            (sh "rm -f ~a/qanda/*" (instpath ""))
            (sh "cd ~a/qanda ; tar xf ~a" (instpath "") (confpath "qanda.tar"))))
        
        ;(let ((l (sgtk-label-new (_ "Arranging media ..."))))
        ;  (sgtk-paned-pack2 w l #t #f)
        ;  (sgtk-widget-show-all l)
        ;  (sgtk-update)
        ;  (sgtk-widget-destroy l))
        
        (sgtk-show-while-new (_ "Accessing")
          (_ "Accessing the install media ...")
          (lambda ()
            (runhook 2)
            (obj-med 'load)))
        
        (repeat (lambda () (gtk-ui-check-media obj-med)))
        
        ; package
        (sgtk-widget-set-sensitive b-pkg #t)
        (obj-med 'open)
        (obj-med 'debconf-off)
        
        (if (not (gtk-ui-selection p compose-pkglist))
          (confirm-error-proceed (_ "Package selection was incomplete.")))
        
        (sh "cp /tmp/inst/pkglist /tmp/inst/pkglist.orig")
        (cond
          ((not (unix-access? "/tmp/inst/pkglist" '(read)))
            (inst-log "could not find /tmp/inst/pkglist.")
            (confirm-error-proceed
              (_ "Could not find package list.")))
          ((not (< 0 (unix-filesize "/tmp/inst/pkglist"))) ; case of minimum
            #t)
          ((not (gtk-ui-package p))
            (confirm-error-proceed
              (_ "Package installation failed."))))
        
        (sgtk-widget-set-sensitive b-pkg #f)
        (if (not (gtk-check-package obj-med "/tmp/inst/pkglist.orig"))
          (confirm-error-proceed
            (_ "Package installation was incomplete.")))
        (obj-med 'debconf-on "Readline")
        
        (check-expand "expandl.tar")
        (check-expand "expand.tar")
        (runhook 3)
        
        ; sound
        (sgtk-widget-set-sensitive b-sound #t)
        (let* ((obj-vol (conf-vol))
               (patcher (conf-patch obj-vol)))
          (and (conf-gtk-load/confirm p patcher)
               (gtk-module-sound p obj-mod)
               (obj-vol 'init 'inst3)
               (conf-gtk-load/confirm p patcher)
               (gtk-ui-volume p obj-vol)
               (conf-gtk-finish p patcher)))
        (sgtk-widget-set-sensitive b-sound #f)
        
        ; service 
        (sgtk-widget-set-sensitive b-sv #t)
        (service-check obj-sv obj-mod)
        (obj-sv 'set-start "kdm")
        (gtk-ui-service p obj-sv)
        (sgtk-widget-set-sensitive b-sv #f)
        
        (let* ((obj-sv (configure-service))
               (obj-pd (conf-prod obj-net obj-sv)))
          (obj-pd 'load)
          (gtk-ui-atok-key p obj-pd obj-sv))

        (let ((obj-add (conf-additional)))
          (gtk-ui-additional p #t obj-add))
        
        ; ORCA
        (sgtk-widget-set-sensitive b-orca #t)
        (let* ((obj-sv (configure-service))
               (obj-pd (conf-prod obj-net obj-sv))
               (patch (conf-patch obj-pd))
               (obj-prof (conf-prof))
               (patch-prof (conf-patch obj-prof))
               (obj-cr (conf-cron obj-sv))
               (patch-cron (conf-patch obj-cr)))
          (obj-prof 'load)
          (obj-pd 'load)
          (cond
            ((not (equal? 'orca (obj-pd 'product-type)))
              #t)
            ((not (= 1 (sgtk-choice p
                         (_ "Do you configure ORCA now?\n\nYou can configure ORCA after installation too.")
                         "Button_No" "Button_Yes")))
              #t)
            ((not (gtk-ui-orca p obj-pd patch obj-prof obj-cr))
              (sgtk-message-info
                (_ "ORCA configuration was aborted. Configure ORCA later with ogl-admin.")))
            (else
              (obj-pd 'commit)
              (patch 'commit)
              (obj-prof 'commit)
              (patch-prof 'commit)
              (obj-cr 'commit)
              (patch-cron 'commit)
              #t)))
        (sgtk-widget-set-sensitive b-orca #f)
        
        (obj-med 'close)
        (gtk-ui-default-media p obj-med obj-na)
        
        (runhook 4)
        (recover-all obj-part obj-mod)
        
        (sgtk-widget-set-sensitive b-fin #t)
        (obj-med 'close)
        (patch-med 'commit)
        ; finished
        (sgtk-choice p 
          (string-append
            (_ "You have done it.\n\n")
            (_ "The whole installation process has been finished. ")
            (_ "Clicking Ok and you can use fully installed system without reboot.\n\n")
            (arma-introduce-text))
          "Button_Ok")
        #t))))

(define obj-net (configure-network))
(define patch-net (conf-patch obj-net))
(obj-net 'init 'inst3)
(patch-net 'load)

(define obj-part (configure-part "/mnt/tmp"))
(define patch-part (conf-patch obj-part))
(obj-part 'init 'inst3)
(patch-part 'load)

(define obj-prof (conf-prof))
(obj-prof 'load)

(define obj-med (conf-media obj-net obj-part obj-prof))
(obj-med 'init 'inst3)
(define patch-med (conf-patch obj-med))
(patch-med 'load)

(define obj-kbd (configure-keyboard))
(obj-kbd 'init 'inst3)
(obj-kbd 'load)

(define obj-mod (configure-module))
(define patch-mod (conf-patch obj-mod))
(obj-mod 'init 'inst3)
(obj-mod 'load)
(patch-mod 'init 'inst3)

(define obj-x (configure-x (obj-kbd 'get-type) obj-mod))
(define patch-x (conf-patch obj-x))
(obj-x 'init 'inst3)
(format #t "Loading configurations. Please wait a moment ...\n")
(cond
  ((obj-x 'pre-configured?)
    (patch-x 'load)
    (patch-x 'commit)))

(define obj-acc (configure-acc))
(define obj-sv (configure-service))
(define obj-na (conf-na))
(obj-na 'load)

(define (start-logsender)
  (let ((date "/var/lib/ogl/etc/install-date"))
    (cond
      ((unix-access? date '(read))
        (obj-na 'write-netrc "/root/.netrc")
        (sh "mkdir /tmp/sup")
        (sh "tail -f /var/log/messages | split -1 - /tmp/sup/mes3: & jobs -l %1 | awk '{print $2}' >/tmp/sender1")
        (sh "tail -f /var/log/daemon.log | split -1 - /tmp/sup/dae3: & jobs -l %1 | awk '{print $2}' >/tmp/sender2")
        (sh "sleep 1 ; rm /tmp/sup/*")
        (sh "cp /var/log/dmesg /tmp/sup/dmesg")
        (sh "elk -c -p /usr/lib/ogl/scm -l /usr/lib/ogl/scm/logsender.scm ~a `cat ~a` >&/tmp/slog &" (obj-na 'get-stdid) date)
        ))))

(define (finish-logsender)
  (sh "kill `cat /tmp/sender1` ; kill `cat /tmp/sender2`")
  (sh "kill -9 `cat /tmp/sender1` ; kill -9 `cat /tmp/sender2`")
  (sh "rm /root/.netrc"))
  ; ɻҥץ progress group Ⱦä롣
  ; ǤŪ˾ä褤

(let ((a (command-line-args)))
  (inst-log "third-main command line : ~a\n" a)
  (obj-mod 'probe-usbmod)
  (cond
    ((null? a) #t)
    ((equal? "kon" (car a))
      (unix-setenv "TERM" "kon"))
    ((equal? "x" (car a))
      (obj-mod 'probe-usbmod)
      (3rd-x)
      (finish-all)
      (exit 0))
    (else
      #t)))

(cond
  ((and (unix-access? "/etc/X11/XF86Config-4" '(read))
        (unix-access? "/var/lib/ogl/etc/x-win" '(read)))
    (inst-log "e\n")
    (let* ((l (parse-awk #\space #t "/var/lib/ogl/etc/lang"))
           (s (cond
                ((eq? #f l) "")
                ((and (list? l) (not (null? l)) (list? (car l)) (not (null? (car l)))
                      (< 2 (string-length (caar l))) (equal? "ja" (substring (caar l) 0 2)))
                  "LANG=ja_JP.eucJP")
                (else
                  ""))))
      (start-logsender)
      (unix-sleep 1)
      (sh "~a /usr/bin/X11/xinit /usr/sbin/ogl/third-main x" s)
      (cond
        ((unix-access? "/var/lib/ogl/etc/under-inst" '(read))
          (format #t "X version of install exited abnormally, sorry.\n")
          (format #t "Please try CUI version.\n"))
        ((unix-access? "/etc/rc2.d/S99kdm" '(execute))
          (sh "rm /etc/rc4.d/S*kdm")))
      (finish-logsender)))
  (else
    (start-logsender)
    (3rd-newt)
    (finish-all)
    (finish-logsender)))

(finish-logsender)
