;;
;;		Timesetting Object
;;                                 (C)2001 Omoikane Inc.


(define (configure-time)
  ;;
  ;;  utils
  ;;
  (define (ogl-tempname)
    (let ((t (unix-tempname)))
      (case context
        ((inst2 recovery)
          (string-append root t))
        (else
          t))))
  
  (define (tz-fold l)
    (define (fold s l l')
      (cond
        ((null? l) l)
        ((string=? s (caar l))
          (fold s (cdr l) (append l' (list (cadr (car l))))))
        (else
          (cons
            (cons s l')
            (if (null? (cdr l))
                '()
                (fold (caar (cdr l)) (cdr l) '()))))))
    (cond
      ((null? l) l)
      (else
        (fold (caar l) l '()))))
  
  ;;
  ;;  vars
  ;;
  (define context 'admin)
  (define root "/")
  (define tz-dir "")
  (define tz-tz "")
  (define hwclock #t)
  (define zone-table '())
  
  ;;
  ;;  methods
  ;;
  (define (init c)
    (set! context c)
    (set! root (case c ((inst2 recovery) "/target")
                       (else "/")))
    #t)
  
  (define (get-root)
    (let ((t0 (unix-tempname))
          (t1 (unix-tempname)))
      ;(display "page\n")
      (fprintf (string-append root t0)
         "ls -F /usr/share/zoneinfo | grep -v / |grep -v tab | grep -v posix | sort >%s\n" t1)
      ;(display "hoge\n")
      (let* ((r (cond
                  ((not (= 0 (sh "~a /bin/sh ~a"
                                 (case context
                                   ((inst2 recovery) "chroot /target")
                                   (else ""))
                                 t0)))
                    #f)
                  (else
                    (let ((r1 (parse-awk #\space #t (string-append root t1))))
                      (cond
                        ((eq? #f r1)
                          (inst-log "could not parse ls command results.\n")
                          #f)
                        (else
                          (let ((l (map car r1)))
                            (if (lang-ja?)
                              (cons "Japan" l)
                              l)))))))))
        (sh "rm ~a~a ~a~a" root t0 root t1)
        (filter (lambda (x) (and (not (equal? "localtime@" x))
                                 (not (equal? "zoneinfo@" x))))
                r))))
  
  (define (get-table)
    (cond
      ((not (null? zone-table))
        zone-table)
      (else
        (let ((t0 (unix-tempname))
              (t1 (unix-tempname)))
          (fprintf (string-append root t0)
             "grep -v ^# </usr/share/zoneinfo/zone.tab | awk '{print $3}' | sort >%s\n" t1)
          (let ((r (cond
                     ((not (= 0 (sh "~a /bin/sh ~a" 
                                    (case context
                                      ((inst2 recovery) "chroot /target")
                                      (else ""))
                                    t0)))
                       #f)
                     (else
                       (collect)
                       (let ((l (parse-awk #\/ #t (string-append root t1))))
                         (cond
                           ((eq? #f l) l)
                           (else (tz-fold l))))))))
            (sh "rm ~a~a ~a~a" root t0 root t1)
            (set! zone-table r)
            r)))))
  
  (define (set-tz-old d t)
    (set! tz-dir d)
    (set! tz-tz t)
    #t)
  
  (define (set-tz t)
    (let* ((l (get-table))
           (r (filter (lambda (x) (set-include? t x))
                      l)))
      (set! tz-tz t)
      (case (length r)
        ((0) (set! tz-dir ""))
        ((1) (set! tz-dir (caar r)))
        (else
          (inst-log "undermined timezone ~a (included in ~a)\n"
                    t r)))
      #t))
  
  (define (commit-tz)
    (let ((etc-tz (format #f "~a/etc/timezone" root))
          (etc-lt (format #f "~a/etc/localtime" root)))
      (cond
        ((string=? "" tz-dir)
          (fprintf etc-tz (format #f "~a\n" tz-tz))
          (= 0 (sh "ln -sf /usr/share/zoneinfo/~a ~a" tz-tz etc-lt)))
        (else
          (fprintf etc-tz (format #f "~a/~a\n" tz-dir tz-tz))
          (= 0 (sh "ln -sf /usr/share/zoneinfo/~a/~a ~a" tz-dir tz-tz etc-lt))))))
  
  (define (set-hwclock f)
    (set! hwclock f))
  
  (define (commit-hwclock)
    (let ((t0 (unix-tempname))
          (t1 (unix-tempname)))
      (fprintf (string-append root t0)
        (string-append
          "sed -e 's:^UTC=%s:UTC=%s:' /etc/default/rcS > %s\n"
          "mv -f %s /etc/default/rcS\n"
          "chmod 744 /etc/default/rcS\n")
        (if hwclock "yes" "no") (if hwclock "no" "yes") t1 t1)
      (let ((r (= 0 (sh "~a /bin/sh ~a" (case context ((inst2 recovery) "chroot /target")
                                                      (else ""))
                                        t0))))
        (sh "rm ~a~a" root t0)
        r)))
  
  (make-object
    'init    init
    'get-root       get-root
    'get-table      get-table
    'set-tz         set-tz
    'commit-tz      commit-tz
    'set-hwclock    set-hwclock
    'commit-hwclock commit-hwclock
    ))
