;
;    util
;                                                 (C) 2001,2002 Omoikane Inc.
;


(define (ntoweek n)
  (cond
    ((not (and (<= 0 n) (<= n 6)))
      'none)
    (else
      (list-ref (list 'sun 'mon 'tue 'wed 'thu 'fri 'sat) n))))

(define (weekton w)
  (case w
    ((sun) 0)
    ((mon) 1)
    ((tue) 2)
    ((wed) 3)
    ((thu) 4)
    ((fri) 5)
    ((sat) 6)
    (else -1)))

(define (product-type)
  (let ((p (parse-awk #\space #t "/var/lib/ogl/etc/product")))
    (cond
      ((or (eq? p #f) (null? p) (< (string-length (caar p)) 3))
        'other)
      (else
        (case (string->symbol (substring (caar p) 0 2))
          ((OS OD SS SD)
            'orca)
          ((SV)
            'sv)
          ((WS)
            'ws)
          (else
            'other))))))

(define (get-proc-version)
  (let ((r (parse-awk #\space #t "/proc/version")))
    (cond
      ((not (and (list? r)
                 (not (null? r))
                 (< 3 (length (car r)))))
        "")
      (else
        (caddr (car r))))))

(define (get-proc-version-list)
  (string-match
    (get-proc-version)
    "[^.]\\+"))

(define (ttable-write p l)
  (define (enc s)
    (cond
      ((string? s)
        (awk-table-encode s))
      ((symbol? s)
        (awk-table-encode (symbol->string s)))
      ((boolean? s)
        (if s "true" "false"))
      (else
        (format #f "~a" s))))
  (define (rep l)
    (cond
      ((null? l) (list "\n"))
      (else
        (cons "\t" (cons (enc (car l)) (rep (cdr l)))))))
  (define (mkline l)
    (case (length l)
      ((0) (list "\n"))
      ((1) (list (enc (car l)) "\n"))
      (else
        (cons (enc (car l)) (rep (cdr l))))))
  (cond
    ((not (and (list? l)
               (andmap list? l)
               (andmap (lambda (x)
                         (andmap (lambda (y) (not (list? y)))
                                 x))
                       l)))
      #f)
    (else
      (fprintf p
        (apply string-append
          (apply append
            (map mkline l)))))))

(define (ttable-read p f)
  
  (define (convert-line l f)
    (cond
      ((and (null? l) (null? f))
        '())
      ((or (null? l) (null? f))
        #f)
      (else
        (let ((h (car f))
              (x (awk-table-decode (car l)))
              (r (convert-line (cdr l) (cdr f))))
          (if (eq? #f r)
            #f
            (cond
              ((string? h)
                (cond
                  ((not (equal? x h))
                    #f)
                  (else
                    (cons h r))))
              (else
                (case h
                  ((sym)
                    (cons (string->symbol x) r))
                  ((num)
                    (let ((y (string->number x)))
                      (if (eq? #f y)
                        #f
                        (cons y r))))
                  ((int)
                    (let ((y (string->number x)))
                      (if (not (integer? y))
                        #f
                        (cons y r))))
                  ((bool)
                    (cond
                      ((equal? "true" x)
                        (cons #t r))
                      ((equal? "false" x)
                        (cons #f r))
                      (else
                        #f)))
                  ((str string)
                    (cons x r))
                  (else
                    #f)))))))))
  
  (define (convert l f)
    (cond
      ((null? f) #f)
      (else
        (let ((r (convert-line l (car f))))
          (cond
            ((eq? r #f)
              (convert l (cdr f)))
            (else
              r))))))
  
  (let ((r0 (parse-awk #\tab #f p)))
    (cond
      ((eq? #f r0)
        #f)
      ((eq? #f f)
        r0)
      ((not (and (list? f)
                 (andmap list? f)
                 (andmap (lambda (x)
                           (andmap (lambda (y) (not (list? y)))
                                   x))
                         f)))
        #f)
      (else
        (let ((r1 (map (lambda (x) (convert x f)) r0)))
          (cond
            ((ormap (lambda (x) (equal? x #f)) r1)
              #f)
            (else
              r1)))))))

(define (fs-by-magic f)
  (let ((l (filter symbol?
             (list
               (if (= 0 (magictest-xfs f)) 'linux-xfs #f)
               (if (or (= 0 (magictest-reiser35 f))
                       (= 0 (magictest-reiser36 f)))
                   'linux-reiser
                   #f)
               (if (= 0 (magictest-jfs f)) 'linux-jfs #f)
               (if (= 0 (magictest-ext2 f)) 'linux-ext2 #f)))))
    (cond
      ((not (= 1 (length l)))
        'linux-unknown)
      (else
        (car l)))))

; part ֥Ȥ fs ܥΥꥹȤ֤


(define sh-log "/dev/tty")
; 󥹥ȡ /dev/tty2

; z? ξ zero  0 Ȥ
(define (ordinal-string z? n)
  (let ((n' (if z? (1+ n) n)))
    (case n'
      ((1) (_ "first"))
      ((2) (_ "second"))
      ((3) (_ "third"))
      (else (format #f (_ "~ath") n')))))

(define (arma-check-title)
  (let ((r (parse-awk #\space #t "/var/lib/ogl/etc/title")))
    (and (not (eq? #f r))
         (list? (car r))
         (not (null? (car r)))
         (<= 4 (string-length (caar r)))
         (equal? "ARMA" (substring (caar r) 0 4)))))

(define (arma-check)
  (define (wallpapers-check)
    (unix-access? "/usr/share/arma-papers" '(execute)))
  (define (atok-check)
    (unix-access? "/usr/share/doc/atokx-arma" '(execute)))
  (define (font-check)
    (unix-access? "/usr/X11R6/lib/X11/fonts/TrueType/oamincho.ttc" '(read)))
  (let ((c 0))
    (map (lambda (x) (if (x) (set! c (1+ c))))
         (list arma-check-title wallpapers-check atok-check font-check))
    (< 1 c)))

(define (repeat f)
  (let ((r (f)))
    (cond
      ((eq? r #t) r)
      (else
        (inst-log "repeat got ~s\n" r)
        (repeat f)))))

; chroot Ƥ dpkg ¹
(define (chroot-target-dpkg o s a)
  (fprintf "/target/tmp/tmp.sh"
    "#!/bin/sh\nPATH=/bin:/usr/bin:/sbin:/usr/sbin dpkg %s %s <<EOF\n%s\nEOF\n"
    o s a)
  (= 0 (sh "chroot /target /bin/sh /tmp/tmp.sh")))

;
; FS Υ󥯤ˤޥå
;
; rr  : from
; cr  : to
; new : newbase
(define (map-newfs new rr cr)
  ;(collect)
  (let rep ((d ""))
    (let ((l (unix-read-directory (string-append cr "/" d))))
      ;(log (format #f "mapping ~a\n" d))
      (map (lambda (x)
             (let* ((path (string-append d "/" x))
                    (from (string-append cr path))
                    (to (string-append rr path))
                    (ln (lambda () (sh "ln -sf ~a ~a" (string-append new path) to)))
                    (t (unix-filetype to)))
               ;(format #t "checking ~a -> ~a\n" to t)
               (case t 
                 ((error)
                   (ln))
                 ((dir)
                   (rep path))
                 ((regular socket other)
                   #t)
                 (else
                   (ln)))))
           (filter (lambda (y)
                     (and (not (string=? ".." y))
                          (not (string=? "." y))))
                   l)))))
; init1 ƬϺϤ


;
; Extended assoc
;
(define (assoc-all obj list)
  (let ((r (let loop ((o obj) (l list))
             (cond
              ((null? l) '())
              ((equal? o (caar l)) (cons (car l) (loop o (cdr l))))
              (else (loop o (cdr l)))))))
    (if (null? r) #f (reverse r))))

;
;               1ƥåפļ¹
;
(define (ui-step-it l)
  (let rep ((n 0))
    (cond
      ((< n 0) #f)
      ((= n (length l)) #t)
      (((list-ref l n))
        (rep (1+ n)))
      (else
        #f))))
;        (rep (1- n))))))

;
;   
;
(define (sh . l)
  (cond
    ((or (null? l))
      (stg-fatal "sh : argument too short."))
    (else
      (system-at sh-log (apply format (cons #f l))))))
;       (system-at "/dev/tty2" (apply format (cons #f l)))))) ; to avoid bug
; åϤʤ⡣

(define (sh-nc . l)
  (cond
    ((or (null? l))
      (stg-fatal "sh : argument too short."))
    (else
      (system-atnc sh-log #f (apply format (cons #f l))))))


;
;   easy and dumb fprintf 
;
(define (fprintf . l)
  (cond
    ((null? l)
      #f)
    (else
      (let ((f (car l)))
        (cond
          ((eq? #f (unix-write-open? (car l)))
            #f)
          (else
            (sh "rm ~a" f)
            (let ((p (open-output-file f)))
              (display (apply sprintf (cdr l)) p)
              (close-output-port p)
              #t)))))))

(define (getfile f)
  (cond
    ((not (unix-access? f '(read)))
      #f)
    (else
      (let ((p (open-input-file f)))
        (let ((r (list->string
                   (let rep ((c (read-char p)))
                     (cond
                       ((eof-object? c) '())
                       (else (cons c (rep (read-char p)))))))))
          (close-port p)
          r)))))


;
;   ascii isprint
;
(define (isprint-filter c)
  (let ((n (char->integer c)))
    (and (< 31 n) (< n 127))))

;
;   ʸ s f(c) ˤʤΤ˥ե륿n ڤͤ
;
(define (filter-cut-string s f n)
  (let ((s' (list->string (filter f (string->list s)))))
    (substring s' 0 (min n (string-length s')))))

(define (string-replace s c0 c1)
  (list->string
    (map (lambda (x) (if (eq? x c0) c1 x))
         (string->list s))))

(define (list-foldstring s l)
  (apply string-append
    (let r ((l l))
      (cond
        ((null? l) l)
        ((null? (cdr l))
          (list (car l)))
        (else
          (cons (car l) (cons s (r (cdr l)))))))))

(define (list-last l)
  (list-ref l (1- (length l))))

(define inst-log-context 'inst)
(define (inst-log fmt . rest)
  (let* ((s (apply format (cons #f (cons fmt rest))))
         (r (syslog  "elk/installer" "LOG_DAEMON" "LOG_INFO" s)))
    (cond
      ((equal? inst-log-context 'inst)
        (with-output-to-file "/dev/tty2"
          (lambda () (display s))))
      (else
        r))))


(define (fatal . l)
  (apply inst-log l)
  (unix-sleep 1200)
  (exit))

(define (ipaddr? s)
  (define (check n l)
    (cond
      ((and (null? l) (= n 3)) #t)
      ((null? l) #f)
      ((char-numeric? (car l)) (check n (cdr l)))
      ((char=? #\. (car l)) (check (1+ n) (cdr l)))
      (else #f)))
  (check 0 (string->list s)))
; δؿϤȤǺ롣obj-net  match-add4? ΤߤǽʬΤϤ


;
;   split fqdn into hostname and domain name
;
(define (fqdn-split s)
  (let ((n (char-search #\. s 0))
        (l (string-length s)))
    (list
      (substring s 0 n)
      (substring s (min (1+ n) l) l))))


(define (string-cut s n)
  (if (< n (string-length s))
    (substring s 0 n)
    s))

;
;   euc б
;
(define (string->wchar-list s)
  (define (rep l)
    (cond
      ((null? l) l)
      ((< 127 (char->integer (car l)))
        (cond
          ((null? (cdr l))
            (list->string l))
          (else
            (cons (list->string (list (car l) (cadr l)))
                  (rep (cddr l))))))
      (else
        (cons (list->string (list (car l)))
              (rep (cdr l))))))
  (rep (string->list s)))

(define (wchar-list-length s)
  (string-length (apply string-append s)))


;
;		ʸִ
;
(define (string-sub s pat to)
  "")

(define (string-gsub s pat to)
  (cond
    ((or (not (string? s))
         (not (string? pat))
         (not (string? to)))
      #f)
    (else
      (let ((g (make-regexp pat)))
        (define (rep n)
          (let ((m (regexp-exec g s n)))
            (cond
              ((eq? m #f)
                (list (substring s n (string-length s))))
              (else
                (let ((st (+ n (regexp-match-start m 0)))
                      (ed (+ n (regexp-match-end m 0))))
                  (cons (substring s n st)
                    (cons to (rep ed))))))))
        (apply string-append (rep 0))))))

;
;		ѥޥåȴ
;
(define (string-match s pat)
  (cond
    ((or (not (string? s))
         (not (string? pat)))
      #f)
    (else
      (let ((g (make-regexp pat)))
        (define (rep n)
          (let ((m (regexp-exec g s n)))
            (cond
              ((eq? m #f)
                '())
              (else
                (let ((st (+ n (regexp-match-start m 0)))
                      (ed (+ n (regexp-match-end m 0))))
                  (cons (substring s st ed)
                        (rep ed)))))))
        (rep 0)))))

;
;   ʸʸѴ
;
(define (string-upcase s)
  (list->string
    (map char-upcase
      (string->list s))))

;
;   embed line-break (for dump/useless word-based gtk line wrap)
;
(define (break-line s n)
  (define (rep l l' m)
    (cond
      ((null? l)
        (cons "\n" l'))
      ((equal? (car l) "\n")
        (rep (cdr l) 
             (append l' (list (car l)))
             0))
      ((< n m)
        (rep l (append l' (list "\n")) 0))
      (else
        (rep (cdr l)
             (append l' (list (car l)))
             (+ m (string-length (car l)))))))
  (apply string-append
    (rep (string->wchar-list s)
         '() 0)))

;
;   Search character c of s from n
;
(define (char-search c s n)
  (let ((l (string-length s)))
    (define (check i)
      (cond
        ((>= i l) l)
        ((char=? c (string-ref s i)) i)
        (else (check (1+ i)))))
    (check n)))

;
;   do 'and' over result of map f l
;
(define (andmap f l)
  (cond
    ((eq? '() l)
      #t)
    ((not (f (car l)))
      #f)
    (else
      (andmap f (cdr l)))))


;
;               list Ϥä f η̤ or 
;
(define (ormap f l)
  (cond
    ((eq? '() l)
      #f)
    ((f (car l))
      #t)
    (else
      (ormap f (cdr l)))))

;
;    suffix դ map 
;
(define (smap f . ls)
  (let rep ((i 0)
            (ls' ls))
    (cond
      ((ormap null? ls')
        '())
      (else
        (cons (apply f (cons i (map car ls')))
              (rep (1+ i) (map cdr ls')))))))

(define (set-include? x l)
  (ormap (lambda (i) (equal? i x)) l))

;
;v 椫ؿfˤxͿ줿or
;
(define (vector-found? f v x)
  (define (check i)
    (cond
      ((= i -1) #f)
      (else
        (or (f (vector-ref v i) x)
            (check (1- i))))))
  (check (1- (vector-length v))))

;
;v0 ǤΤ ؿfˤ v1Ǥ#f֤ʤ
;

(define (vector-set- f v0 v1)
  (define (check i)
    (cond
      ((= i -1) '())
      ((vector-found? f v1 (vector-ref v0 i))
        (check (1- i)))
      (else
        (cons (vector-ref v0 i) (check (1- i))))))
  (list->vector (reverse (check (1- (vector-length v0))))))

; ineffective list-
(define (list-set- f l0 l1)
  (vector->list (vector-set- f (list->vector l0) (list->vector l1))))

(define (list-set+ f l0 l1)
  (append (list-set- f l0 l1) l1))

(define (list-set-and l0 l1)
  (filter (lambda (x) (set-include? x l0))
          l1))

(define (easy-sort f l)
  (define (insert lis elm)
    (cond ((null? lis) (list elm))
          ((f (car lis) elm)
           (cons (car lis) (insert (cdr lis) elm)))
          (else
           (cons elm lis))))
  (let loop ((rest (reverse l))
             (result '()))
    (cond ((null? rest) result)
          (else
           (loop (cdr rest)
                 (insert result (car rest)))))))

(define (symbol<? x y)
  (string<? (symbol->string x) (symbol->string y)))

(define (filter f l)
  (cond
    ((null? l)
      l)
    ((f (car l))
      (cons (car l) (filter f (cdr l))))
    (else
      (filter f (cdr l)))))

(define (i/ x y) (real->int (/ x y)))

;
;   sub list (like substring)
;
(define (list-sub l n m)
  (define (rep l n)
    (cond
      ((= n 0) '())
      ((null? l) '())
      (else (cons (car l) (rep (cdr l) (1- n))))))
  (cond
    ((< n 0) '())
    ((= n 0) (rep l m))
    ((null? l) '())
    (else (list-sub (cdr l) (1- n) m))))

;
;   primitive uniq function
;
(define (list-primitive-uniq f l)
  (define (rep x l)
    (cond
      ((null? l) (list x))
      ((f x (car l)) (rep x (cdr l)))
       (else (cons x (rep (car l) (cdr l))))))
  (cond
    ((null? l) l)
    (else (rep (car l) (cdr l)))))

(define (list-uniq l)
  (list-primitive-uniq equal? l))

;; older version
;(define (list-uniq l)
;  (define (rep f x l)
;    (cond
;      ((null? l) (list x))
;      ((f x (car l)) (rep f x (cdr l)))
;      (else (cons x (rep f (car l) (cdr l))))))
;  (cond
;    ((null? l) l)
;    (else (rep equal? (car l) (cdr l)))))

;
;   get country code from locale name
;
(define (lang->country s)
  (let ((n (char-search #\_ s 0)))
    (substring s 0 n)))

;
;   wheter locale is japan
;
(define (lang-ja?)
  (let ((s (unix-getenv "LANG")))
    (cond
      ((eq? #f s) #f)
      ((and (string? s) (not (string=? "" s)))
        (string=? "ja" (lang->country (unix-getenv "LANG"))))
      (else
        #f))))

;
;   revised version of unix-exec
;
(define (unix-exec2 bin args)
  (if (unix-access? bin '(execute))
    (unix-exec bin args)
    #f))

;
;   safe/revised version of unix-read-directory
;
(define (unix-read-directory2 d)
  (case (unix-filetype d)
    ((dir)
      (reverse (unix-read-directory d)))
    (else
      '())))


;
;   get drive device from partition device name
;
(define (pdev->ddev d)
  (cond
    ((< (string-length d) 8) "")
    (else
      (let ((s (substring d 5 7)))
        (cond
          ((or (string=? "sd" s)
              (string=? "hd" s))
            (substring d 0 8))
          ((string=? s "rd")
            (substring d 0 12))
          (else
            ""))))))

;
;   get partial drive name from device name
;
(define (ddev->partial d)
  (let ((l (string-length d)))
    (cond
      ((< l 5) "")
      (else
        (substring d 5 l)))))

;
;   get drive name from device name
;
(define (ddev->drive d)
  (cond
    ((< (string-length d) 8) "")
    (else
      (let ((s (substring d 5 7)))
        (cond
          ((or (string=? "sd" s)
              (string=? "hd" s))
            (substring d 5 8))
          ((string=? s "rd")
            (substring d 5 12))
          (else
            ""))))))

(define (ddev->nth d)
  (let ((l (string-length d)))
    (cond
      ((< l 8) -1)
      (else
        (let ((s (substring d 5 7)))
          (cond
            ((or (string=? "sd" s)
                (string=? "hd" s))
              (string->number (substring d 8 l)))
            ((string=? s "rd")
              (string->number (substring d 13 l)))
            (else
              -1)))))))

;
;   re-define max (easy hack to avoid max primitive bug)
;
(define (max . x)
  (define (mm a c)
    (if (null? c) a
        (mm (if (< a (car c)) (car c) a) (cdr c))))
  (cond ((null? x) #f)
        (else (mm (car x) (cdr x)))))

;;
;;   Make device file name
;;
(define (ddev-make d n)
  (format #f "ddev-make ~a ~a" d n))
;;  (cond
;;    ((string=? "rd" (substring d 0 2))
;;      (format #f "/dev/~ap~a" d n))
;;    (else
;;      (format #f "/dev/~a~a" d n))))
;
;(define (partition-name dev n)
;  (cond
;    ((not (and (string? dev) (number? n)))
;      "")
;    (else
;      (let ((len (string-length dev))
;            (n' (if (< n 0) 0 n)))
;        (cond
;          ((and (< 5 len)
;                (equal? "disc" (substring dev (- len 4) len)))
;            (format #f "/dev/~apart~a" (substring dev 0 (- len 4)) n'))
;          (else
;            (format #f "/dev/~a~a" dev n')))))))
;; /dev//disc?/disc Ĵ٤ơ/part? Ѵ롣


;
;               PCI 
;
(define (pci-expandbusid s) ; s:bus id string from 1st column of /proc/pci/bus/devices
  (cond
    ((or (not (string? s))
         (not (= 4 (string-length s))))
      (list -1 -1 -1))
    (else
      (let ((b (string->number (substring s 0 2) 16))
            (n (string->number (substring s 2 4) 16)))
        (cond
          ((or (eq? #f b) (eq? #f n))
            (list -1 -1 -1))
          (else
            (list b (quotient n 8) (remainder n 8))))))))

;
;   these abstructions are for when we implemnt on other architecture
;
(define (id-linux) 131)

(define (id-swap) 130)

(define (id-extended) 5)

(define (id-extended? x)
  (case x
    ((5 15 133) #t)
    (else #f)))

(define (id-primary? x) (< x 5))

(define (id-dos? x)
  (case x
    ((1 4 6 11 12 14) #t)
    (else #f)))

(define (id-nt? x)
  (case x
    ((7 23 134 135) #t)
    (else #f)))

(define (id-win? x)
  (or (id-dos? x) (id-nt? x)))

(define (fs->id fs)
  (case fs
    ((linux-ext2 linux-xfs linux-reiser linux-jfs)
      (id-linux))
    ((swap)
      (id-swap))
    ((ext)
      (id-extended))
    (else
      (id-linux))))

(define (fs->mountopt fs)
  (case fs
    ((linux-xfs) "xfs")
    ((linux-reiser) "reiserfs")
    ((linux-ext2) "ext2")
    ((linux-ext3) "ext3")
    ((linux-jfs) "jfs")
    ((vfat) "vfat")
    ((ntfs) "ntfs")
    (else
      (format #f "~a" fs))))

(define id-name-db
  (list (cons 0 "Empty")
        (cons 1 "FAT12")
        (cons 2 "XENIX root")
        (cons 3 "XENIX usr")
        (cons 4 "FAT16 <32M")
        (cons 5 (_ "Extended"))
        (cons 6 "FAT16")
        (cons 7 "HPFS/NTFS)")
        (cons 8 "AIX")
        (cons 9 "AIX bootable")
        (cons 10 "OS/2 Boot Manager")
        (cons 11 "Win95 FAT32")
        (cons 12 "Win95 FAT32 (LBA)")
        (cons 14 "Win95 FAT16 (LBA)")
        (cons 15 "Win95 Ext'd (LBA)")
        (cons 16 "OPUs)n")
        (cons 17 "Hidden FAT12")
        (cons 18 "Compaq diagnostics)n")
        (cons 20 "Hidden FAT16 <32M")
        (cons 22 "Hidden FAT16")
        (cons 23 "Hidden HPFS/NTFS)")
        (cons 24 "AST Windows swapfile")
        (cons 27 "Hidden Win95 FAT32")
        (cons 28 "Hidden Win95 FAT32 (LBA)")
        (cons 30 "Hidden Win95 FAT16 (LBA)")
        (cons 36 "NEC DOS")
        (cons 60 "PartitionMagic recovery")
        (cons 64 "Venix 80286")
        (cons 65 "PPC PReP Boot")
        (cons 66 "SFS")
        (cons 77 "QNX4.x")
        (cons 78 "QNX4.x 2nd part")
        (cons 79 "QNX4.x 3rd part")
        (cons 80 "OnTrack DM")
        (cons 81 "OnTrack DM6 Aux1")
        (cons 82 "CP/M")
        (cons 83 "OnTrack DM6 Aux3")
        (cons 84 "OnTrackDM6")
        (cons 85 "EZ-Drive")
        (cons 86 "Golden Bow")
        (cons 92 "Priam Edisk")
        (cons 97 "SpeedStor")
        (cons 99 "GNU HURD or SysV")
        (cons 100 "Novell Netware 286")
        (cons 101 "Novell Netware 386")
        (cons 112 "DiskSecure Multi-Boot")
        (cons 117 "PC/IX")
        (cons 128 "Old Minix")
        (cons 129 "Minix / old Linux")
        (cons 130 "Swap")
        (cons 131 "Linux")
        (cons 132 "OS/2 hidden C: drive")
        (cons 133 "Linux extended")
        (cons 134 "NTFS volume set")
        (cons 135 "NTFS volume set")
        (cons 147 "Amoeba")
        (cons 148 "Amoeba BBT")
        (cons 160 "IBM Thinkpad hibernation")
        (cons 165 "BSD/386")
        (cons 166 "OpenBSD")
        (cons 167 "NeXTSTEP")
        (cons 183 "BSDI fs)n")
        (cons 184 "BSDI swap")
        (cons 193 "DRDOS/sec (FAT-12)")
        (cons 196 "DRDOS/sec (FAT-16 < 32M)")
        (cons 198 "DRDOS/sec (FAT-16)")
        (cons 199 "Syrinx")
        (cons 219 "CP/M / CTOS / ...")
        (cons 225 "DOS access")
        (cons 227 "DOS R/O")
        (cons 228 "SpeedStor")
        (cons 235 "BeOS fs")
        (cons 241 "SpeedStor")
        (cons 244 "SpeedStor")
        (cons 242 "DOS secondary")
        (cons 253 "Linux raid autodetect")
        (cons 254 "LANstep")
        (cons 255 "BBT")))

