;
;     X Window Object 
;
;                          (C)2001,2002 Omoikane Inc.
;

(define (configure-x kbd-type obj-mod)
  ;
  ;  utils
  ;
  (define (parse-card-table f)
    (define (first-word s)
      (let ((n (char-search #\space s 0)))
        (substring s 0 n)))
    
    (define (left-word s)
      (let ((n (char-search #\space s 0))
            (l (string-length s)))
        (substring s (min l (1+ n)) l)))
    
    (define (get-block ll)
      (let rep ((d '()) (l ll))
        (if (null? l)
            d
            (let* ((s (caar l))
                   (w (first-word s))
                   (left (left-word s)))
              (cond
                ((or (string=? w "")
                     (string=? w "END")
                     (string=? w "NAME"))
                  d)
                ((string=? w "LINE")
                  (let ((r (assoc "LINE" d)))
                    (cond
                      ((eq? #f r)
                        (rep (cons (cons "LINE" left) d)
                             (cdr l)))
                      (else
                        (set-cdr! r (string-append (cdr r) "\n" left))
                        (rep d (cdr l))))))
                (else
                  (rep (cons (cons w left) d)
                       (cdr l))))))))
    
    (define (start-block l)
      (cond
        ((null? l) '())
        (else
          (let* ((s (caar l))
                 (w (first-word s)))
            (cond
              ((string=? w "END") '())
              ((string=? w "NAME")
                (cons
                  (cons (left-word s) (get-block (cdr l)))
                  (start-block (cdr l))))
              (else
                (start-block (cdr l))))))))
    
    (define (resolv-see x) x)
    
    (let ((r (parse-awk #\newline #t f)))
      (cond
        ((eq? #f r)
          #t)
        (else
          (resolv-see (start-block r))))))
  
  (define (get-memory)
  	(define (search l)
  	  (cond
  	    ((or (null? l) (null? (cdr l)) (null? (cddr l)))
  	      -1)
  	    ((and (string=? "VideoRAM:" (car l))
  	          (string=? "kByte" (caddr l)))
  	      (string->number (cadr l)))
  	    (else
  	      (search (cdr l)))))
  	
    (let ((x "/usr/X11R6/bin/X")
          (c "/tmp/XF86Config-4.new")
          (xlog "/var/log/XFree86.0.log"))
      (cond
        ((not (unix-access? x '(execute)))
          -1)
        ((unix-access? c '(read))
          -1)
        (else
          (sh "HOME=/tmp ~a -configure -probeonly" x)
          (sh "~a -probeonly -xf86config ~a >& /dev/null" x c)
          (sh "rm ~a" c)
          (let ((r (parse-awk #\space #t xlog)))
            (cond
              ((eq? #f r) -1)
              (else
                (search (apply append r)))))))))
    
  (define (refer-db x)
    ;(format #t "refer-db ~a\n" x)
    (cond
      ((assoc x device-db) =>
        (lambda (z)
          (cond
            ((assoc "DRIVER" (cdr z)) =>
              (lambda (w) (cons (cdr w) (cdr z))))
            ((assoc "SEE" (cdr z)) =>
              (lambda (w) (refer-db (cdr w))))
            (else
              (inst-log "device ~a found in db, but does not contain valid data : ~a\n"
                        x z)
              (cons "vesa" (cdr z))))))
         ;    (cons "unknown-driver" (cdr z))
      (else
        (inst-log "device ~a not found in db.\n" x)
        (list "vesa"))))
        ;(list "unknown-driver")
  
  (define (update-list n l x)
    (cond
      ((< (length l) n)
        (update-list n (append l (list '())) x))
      (else
        (append
           (list-sub l 0 n)
           (list x)
           (list-sub l (1+ n) (length l))))))
  
  ;
  ;  vars
  ;
  (define context 'admin)
  (define device-db '())
  (define pci-db '())
  (define info-mouse '(usb))
  (define info-display '())    ; (<name:str> <int> <int> <int> <int> <modeline:str>) $B$N%j%9%H(B
  (define info-resolution '())
  (define info-color '())
  (define info-device '())
  (define info-num-screen 1)
  (define info-layout 'h)
  (define info-mem 0)
  (define probe-error "")
  
  ;
  ;  methods
  ;
  (define (init c)
    (set! context c)
    (set! device-db (let ((r (parse-card-table "/usr/lib/ogl/lib/videodevice")))
                      (if (eq? #f r) '() r)))
    (set! pci-db (let ((r (parse-awk #\tab #t "/usr/lib/ogl/lib/videotable")))
                   (if (eq? #f r) '() r)))
    #t)
  
  (define (pre-configured?)
    (unix-access? (string-append (case context
                                   ((recovery) "/target")
                                   (else ""))
                                 "/var/lib/ogl/etc/x-win")
                  '(read)))
  
  (define (probe-pci)
    (let* ((vd (filter (lambda (x)
                         (let ((cl (apply pci-getclass (pci-expandbusid (car x)))))
                           (or (equal? "03" (substring cl 0 2))
                               (equal? "0001" (substring cl 0 4)))))
                       (parse-awk #\tab #t "/proc/bus/pci/devices")))
           (vname (map (lambda (x)
                         ;(format #t "x:~a\n" x)
                         (cond
                           ((assoc (cadr x) pci-db) =>
                             (lambda (y)
                               (append (cons (car x) (cdr y))
                                       (refer-db (cadr y)))))
                           (else
                             (list (car x) "unknown-name" "unknown-desc" "vesa" '()))))
                             ;(list (car x) "unknown-name" "unknown-desc" "unknown-driver" '())
                       vd))
           (vname' (list-primitive-uniq
                     (lambda (x y)
                       (and (< 4 (length x))
                            (< 4 (length y))
                            (equal? (list-ref x 3) (list-ref y 3))
                            (equal? "i810" (list-ref x 3))))
                     vname)))
      (letrec ((i8xx-fail-text
                 (lambda (m)
                   (string-append (format #f (_ "Failed to install agpgart/~a kernel modules.") m)
                                  (_ "You need to load both modules before functioning video driver.\n\n")
                                  (_ "If you are running development kernel, this is not a bug. ")
                                  (_ "Please check if your kernel is stable version and restart admin tool or installer again. ")
                                  )))
               (r (filter
                    (lambda (x) (eq? #f x))
                    (map (lambda (x)
                           (cond
                             ((set-include? (cadr x) '("Intel 815" "Intel 810"))
                               (cond
                                 ((and (obj-mod 'insmod "agpgart" "")
                                       (obj-mod 'insmod "i810" ""))
                                   #t)
                                 (else
                                   (set! probe-error
                                     (string-append probe-error (i8xx-fail-text "i810")))
                                   #f)))
                             ((equal? (cadr x) "Intel 830")
                               (cond
                                 ((and (obj-mod 'insmod "agpgart" "")
                                       (obj-mod 'insmod "i830" ""))
                                   #t)
                                 (else
                                   (set! probe-error
                                     (string-append probe-error (i8xx-fail-text "i830")))
                                   #f)))
                             (else
                               #t)))
                         vname'))))
        (cons (null? r) vname'))))
  
  (define (probe-lite)
    (set! info-mouse
      (list-uniq (map (lambda (x)
                        (cond
                          ((equal? x "genericps/2") 'ps2old)
                          ((equal? x "generic3ps/2") 'ps2)
                          (else 'usb)))
                      (easy-sort string<? (probe-mouse)))))
    #t)
  
  (define (probe-lite-display)
    (let* ((p (probe-display))
           (x (cond
                ; $B:G=i$KIT@5$JL>A0$rJV$9%G%#%9%W%l%$(B/$B%S%G%*$r$O$8$/(B
                ((and (string? (car p))
                      (not (equal? "" (filter-cut-string (car p) isprint-filter 20))))
                  (list (filter-cut-string (car p) isprint-filter 20)
                        (1- (cadr p)) (1+ (caddr p))
                        (1- (list-ref p 3)) (1+ (list-ref p 4)) ""))
                ((list? p)
                  (list "unknown"
                        (1- (cadr p)) (1+ (caddr p))
                        (1- (list-ref p 3)) (1+ (list-ref p 4)) ""))
                (else
                  (list "unknown" 0 0 0 0 "")))))
      (set! info-display
        (update-list 0 info-display x))))
  
  (define (probe-all mem?)
    (set! probe-error "")
    (if mem?
      (set! info-mem (get-memory)))
    (probe-lite)
    (probe-lite-display)
    (let ((pci (probe-pci)))
      (set! info-device
        (map (lambda (x) (list (car x) 0 100 (cadr x) (caddr x) (list-ref x 3)))
             (cdr pci)))
      (if (car pci) #t probe-error)))
  
  (define (update-nth l n f)
    (if (= n 0)
        (cons (f (car l)) (cdr l))
        (cons (car l) (update-nth (cdr l) (1- n) x))))
  
  (define (set-hsync n b t)
    (let ((x (if (< n (length info-display))
               (list-ref info-display n)
               (list "unknown" 0 0 0 0 ""))))
      (set! info-display
            (update-list n info-display
              (if (null? x)
                (list "unknown" b t 0 0 "")
                (list (car x) b t (list-ref x 3) (list-ref x 4) ""))))
      #t))
  
  (define (set-vsync n b t)
    (let ((x (if (< n (length info-display))
               (list-ref info-display n)
               (list "unknown" 0 0 0 0 ""))))
      (set! info-display
            (update-list n info-display
              (if (null? x)
                (list "unknown" 0 0 b t "")
                (list (car x) (cadr x) (caddr x) b t ""))))
      #t))
  
  (define (set-modeline n m)
    (let ((x (if (< n (length info-display))
               (list-ref info-display n)
               (list "unknown" 0 0 0 0 ""))))
      (set! info-display
            (update-list n info-display
              (list (car x) (cadr x) (caddr x) (list-ref x 3) (list-ref x 4) m)))
      #t))
  
  (define (set-resolution n x y)
    (set! info-resolution
      (update-list n info-resolution (list x y))))
  
  (define (avail-depth x y)
    (let ((b (quotient (* 1024 info-mem) (* x y))))
      (cond
        ((<= 4 b) '(16 24 32 24 8))
        ((= 3 b) '(16 24 8))
        ((= 2 b) '(16 8))
        ((= 1 b) '(8))
        (else '()))))
  
  (define (set-color n) (set! info-color n))
  
  (define (color-name n)
    (case n
      ((32) (_ "True Color           16M colors (32bpp)"))
      ((24) (_ "True Color(Packed)   16M colors (24bpp)"))
      ((16) (_ "High(Packed)       65536 colors (16bpp)"))
      ((8)  (_ "Pseudo Color         256 colors ( 8bpp)"))
      (else "")))
  
  (define (write-med)
    (fprintf (string-append "/var/lib/ogl/etc/x-win")
      (apply string-append
        (append
          (list
            (if (null? info-mouse)
                "mouse\n"
                (format #f "mouse\t~a\n" (list-foldstring "\t" (map symbol->string info-mouse)))))
          (list (format #f "layout\t~a\n" info-layout))
          (map (lambda (x) (format #f "device\t~a\t~a\t~a\t~a\t~a\t~a\n"
                             (car x) (cadr x) (caddr x) (list-ref x 3) (list-ref x 4) (list-ref x 5)))
               info-device)
          (smap (lambda (n x)
                  (format #f "display\t~a\t~a\t~a\t~a\t~a\t\~a\t~a\n"
                          n (car x) (cadr x) (caddr x)
                          (list-ref x 3) (list-ref x 4) (list-ref x 5)))
                info-display)
          (smap (lambda (n x)
                  (format #f "resolution\t~a\t~a\t~a\t~a\n"
                    n (car x) (cadr x) info-color))
                info-resolution)))))
  
  (define (commit-med)
    (inst-log "commit-med : ")
    (cond
      ((equal? 'inst2 context)
        (write-med)
        (inst-log "written."))
      (else
        (inst-log "rejected. (context ~s)" context)))
    #t)
  
  (define (make-paper-link)
    (case context
      ((inst1 inst2 recovery)
        (inst-log "make-paper-link : context doesn't match. ignoring...\n")
        #t)
      (else
        (cond
          ((null? info-resolution)
            (inst-log "make-paper-link : info-resolution is null.\n")
            #f)
          ((and (equal? info-layout 'g)
                (not (= 4 (length info-resolution))))
            (inst-log "make-paper-link : layout is grid but info-resolution is not 4.\n")
            #f)
          (else
            (let* ((sz (case info-layout
                         ((h)
                           (list (apply + (map (lambda (x) (car x)) info-resolution))
                                 (apply max (map (lambda (x) (cadr x)) info-resolution))))
                         ((v)
                           (list (apply max (map (lambda (x) (car x)) info-resolution))
                                 (apply + (map (lambda (x) (cadr x)) info-resolution))))
                         ((g)
                           (let ((r0 (car info-resolution))
                                 (r1 (cadr info-resolution))
                                 (r2 (caddr info-resolution))
                                 (r3 (list-ref info-resolution 3)))
                             (list (max (+ (car r0) (car r1)) (+ (car r2) (car r3)))
                                   (max (+ (cadr r0) (cadr r3)) (+ (cadr r2) (cadr r3))))))))
                   (x (car sz))
                   (y (cadr sz))
                   (tmp1 (unix-tempname))
                   (tmp2 (unix-tempname)))
              (inst-log "make-paper-link : creating sz ~a x ~a ...\n" (car sz) (cadr sz))
              (cond
                ((not
                   (and
                     (= 0 (sh "pngtopnm /usr/lib/ogl/lib/paper.png | pnmtile ~a ~a >~a"
                              x y tmp1))
                     (= 0 (sh "composite /usr/lib/ogl/lib/dark.png ~a ~a"
                              tmp1 tmp2))
                     (= 0 (sh "composite -geometry +~a+60 /usr/lib/ogl/lib/logo.png ~a /usr/lib/ogl/lib/default-back.png"
                              (- x 204) tmp2))
                     (= 0 (sh "rm ~a ~a" tmp1 tmp2))))
                  (inst-log "make-paper-link : failed to create default back.\n"))
                (else
                  #t))))))))
  
  (define (commit)
    (write-med)
    (make-paper-link)
    #t)
  
  (define (load-it)
    (let ((p (parse-awk #\tab #t (string-append (case context
                                                  ((recovery) "/target")
                                                  (else ""))
                                                "/var/lib/ogl/etc/x-win"))))
      (cond
        ((eq? #f p)
          #t)
        (else
          (andmap (lambda (x) (not (equal? #f x)))
            (map (lambda (l)
                   (let ((x (car l))
                         (len (length l)))
                     (cond
                       ((equal? "mouse" x)
                         (set! info-mouse (map string->symbol (cdr l)))
                         #t)
                       ((equal? "layout" x)
                         (cond
                           ((not (= 2 len))
                             #f)
                           (else
                             (let ((s (string->symbol (cadr l))))
                               (case s
                                 ((h v g)
                                   (set! info-layout s)
                                   #t)
                                 (else
                                   (inst-log "illega layout : ~a\n" l)
                                   #f))))))
                       ((equal? "device" x)
                         (cond
                           ((not (= 7 len))
                             #f)
                           (else
                             (let ((n0 (string->number (caddr l)))
                                   (n1 (string->number (list-ref l 3))))
                               (cond
                                 ((or (eq? #f n0) (eq? #f n1))
                                   (inst-log "illegal device line (not number): ~a\n" l)
                                   #f)
                                 (else
                                   (set! info-device
                                     (append info-device
                                       (list (list (cadr l) n0 n1 (list-ref l 4) (list-ref l 5) (list-ref l 6)))))
                                   #t))))))
                       ((equal? "display" x)
                         (cond
                           ((not (or (= 7 len) (= 8 len)))
                             (inst-log "illegal length of display line : ~a\n" l)
                             #f)
                           (else
                             (let ((n0 (string->number (cadr l)))
                                   (n1 (string->number (list-ref l 3)))
                                   (n2 (string->number (list-ref l 4)))
                                   (n3 (string->number (list-ref l 5)))
                                   (n4 (string->number (list-ref l 6))))
                               (cond
                                 ((ormap (lambda (x) (eq? #f x))
                                         (list n0 n1 n2 n3 n4))
                                   (inst-log "illegal display line : ~a\n" l)
                                   #f)
                                 (else
                                   (set! info-display
                                     (update-list n0 info-display
                                       (list (caddr l) n1 n2 n3 n4 (if (= 7 len) "" (list-ref l 7)))))
                                   #t))))))
                       ((equal? "resolution" x)
                         (let ((n0 (string->number (cadr l)))
                               (n1 (string->number (caddr l)))
                               (n2 (string->number (list-ref l 3))))
                           (cond
                             ((or (eq? n0 #f) (eq? n1 #f) (eq? n2 #f))
                               (inst-log "illegal format in resolution line : ~a\n" l)
                               #f)
                             (else
                               (set! info-resolution
                                 (update-list n0 info-resolution (list n1 n2)))
                               (set! info-num-screen (length info-resolution))
                               #t)))
                         (set! info-color (string->number (list-ref l 4)))
                         #t)
                       (else
                         (inst-log "unknown line in xwin : ~a\n" l)
                         #f))))
                 p))))))
  
  (define (make-etc f ext?)
    (define i 0)
    
    (define (make-layout)
      (apply string-append
        (append
          (list "Section \"ServerLayout\"\n"
                "    Identifier     \"XFree86 Configured\"\n")
          (smap (lambda (n x)
                  (format #f "    Screen ~a \"Screen~a\" ~a\n"
                    n n
                    (case n
                      ((0) "")
                      ((1)
                        (format #f "~a \"Screen0\""
                           (case info-layout
                             ((h g) "RightOf")
                             ((v) "Below")
                             (else ""))))
                      ((2)
                        (case info-layout
                          ((h) "RightOf \"Screen1\"")
                          ((v) "Below \"Screen1\"")
                          ((g) "Below \"Screen0\"")
                          (else "")))
                      ((3)
                        (case info-layout
                          ((h) "RightOf \"Screen2\"")
                          ((v) "Below \"Screen2\"")
                          ((g) "RightOf \"Screen2\"")))
                      (else
                        ""))))
                info-resolution)
          (begin
            (set! i -1)
            (if (null? info-mouse)
              (list "    InputDevice    \"Mouse0\" \"CorePointer\"\n")
              (map (lambda (x)
                     (set! i (1+ i))
                     (format #f "    InputDevice    \"Mouse~a\" ~a\n"
                                i (if (= i 0)
                                      "\"CorePointer\""
                                      "\"SendCoreEvents\"")))
                   (if (equal? info-mouse '(ps2old))
                      (list 'ps2old 'usb)
                      info-mouse))))
          (list "    InputDevice    \"Keyboard0\" \"CoreKeyboard\"\n"
                "EndSection\n\n"))))
    
    (define (make-files)
      (string-append
        "Section \"Files\"\n"
        "    RgbPath  \"/usr/X11R6/lib/X11/rgb\"\n"
        "    FontPath \"/usr/X11R6/lib/X11/fonts/misc:unscaled\"\n"
        "    FontPath \"/usr/X11R6/lib/X11/fonts/75dpi:unscaled\"\n"
        "    FontPath \"/usr/X11R6/lib/X11/fonts/100dpi:unscaled\"\n"
        "    FontPath \"/usr/X11R6/lib/X11/fonts/misc\"\n"
        "    FontPath \"/usr/X11R6/lib/X11/fonts/Speedo\"\n"
        "    FontPath \"/usr/X11R6/lib/X11/fonts/Type1\"\n"
        "    FontPath \"/usr/X11R6/lib/X11/fonts/CID\"\n"
        "    FontPath \"/usr/X11R6/lib/X11/fonts/75dpi\"\n"
        "    FontPath \"/usr/X11R6/lib/X11/fonts/100dpi\"\n"
        "    FontPath \"/usr/X11R6/lib/X11/fonts/TrueType\"\n"
        "EndSection\n\n"))
    
    (define (make-serverf)
      (case (length info-display)
        ((0 1) "")
        (else
          (string-append
            "Section \"ServerFlags\"\n"
            "    Option     \"Xinerama\" \"on\"\n" ""
            "EndSection\n\n"))))
    
    (define (make-module)
      (string-append
        "Section \"Module\"\n"
        "    Load \"GLcore\"\n"
        "    Load \"dbe\"\n"
        "    Load \"dri\"\n"
        "    Load \"extmod\"\n"
        "    Load \"glx\"\n"
        "    Load \"pex5\"\n"
        "    Load \"record\"\n"
        ;"    Load \"xie\"\n"
        "    Load \"xtt\"\n"
        "EndSection\n\n"))
    
    (define (make-module-test)
      (string-append
        "Section \"Module\"\n"
        "        Load  \"extmod\"\n"
        "EndSection\n\n"))
    
    (define (make-inputdevice)
      (apply string-append
        (append
          (let ((layout (car kbd-type))
                (caps (cadr kbd-type)))
            (list            "Section \"InputDevice\"\n"
                             "    Identifier  \"Keyboard0\"\n"
                             "    Driver      \"keyboard\"\n"
                             "    Option      \"XkbRules\" \"xfree86\"\n"
                  (format #f "    Option      \"XkbModel\" \"~a\"\n" layout)
                  (format #f "    Option      \"XkbLayout\" \"~a\"\n"
                    (cond
                       ((equal? layout "jp106") "jp")
                       ((equal? layout "pc101") "us")
                       (else "unknown")))
                  (case caps
                    ((caps) "")
                    (else
                      (format #f "    Option      \"XkbOptions\" \"~a\"\n"
                                 (case caps
                                   ((ex) "ctrl:swapcaps")
                                   ((ctrl) "ctrl:nocaps")
                                   (else "")))))
                  "EndSection\n"
                  "\n"))
          (let ((s (string-append "Section \"InputDevice\"\n"
                                  "    Identifier  \"Mouse~a\"\n"
                                  "    Driver      \"mouse\"\n"
                                  "    Option      \"Protocol\" \"~a\"\n"
                                  "    Option      \"Device\" \"~a\"\n"
                                  "    Option      \"Buttons\" \"5\"\n"
                                  "    Option      \"ZAxisMapping\" \"4 5\"\n"
                                  "    Option      \"Emulate3Buttons\"\n"
                                  "EndSection\n\n")))
            (if (null? info-mouse)
              (list (format #f s 0 "ImPS/2" "/dev/input/mice"))
              (begin (set! i -1)
                (map (lambda (x)
                       (set! i (1+ i))
                       (inst-log "x:~a\n" x)
                       (format #f s i
                                  (case x
                                    ((ps2old) "PS/2")
                                    (else "ImPS/2"))
                                  (case x
                                    ((usb) "/dev/input/mice")
                                    (else "/dev/psaux"))))
                     (if (equal? info-mouse '(ps2old))
                        (list 'ps2old 'usb)
                        info-mouse))))))))
    
    (define (make-monitor)
      (apply string-append
        (smap (lambda (n x)
                (cond
                  ((null? x)
                    (format #f "# monitor ~a not set\n\n" n))
                  (else
                    (let ((m (list-ref x 5)))
                      (string-append
                                     "Section \"Monitor\"\n"
                        (format #f   "    Identifier   \"Monitor~a\"\n" n)
                                     "    VendorName   \"Monitor Vendor\"\n"
                        (format #f   "    ModelName    \"~a\"\n"
                          (if (equal? "unknown" (car x))
                              "Unknown Display" (car x)))
                        (if (and (= 0 (cadr x)) (= 0 (caddr x))) ""
                          (format #f "    HorizSync    ~a-~a\n" (cadr x) (caddr x)))
                        (if (and (= 0 (list-ref x 3)) (= 0 (list-ref x 4))) ""
                          (format #f "    VertRefresh  ~a-~a\n" (list-ref x 3) (list-ref x 4)))
                        (if (equal? "" m) ""
                          (format #f "    ModeLine     ~a\n" m))
                                     "EndSection\n\n")))))
              info-display)))
    
    (define (make-device)
      (define (adhoc-options x)
        (cond
          ((and (equal? (list-ref x 5) "neomagic")
                (ormap (lambda (y) (not (equal? "" (list-ref y 5))))
                       info-display))
            (string-append
              "    Option     \"SWCursor\"\n"
              "    Option     \"override_validate_mode\"\n"))
          (else
            "")))
    	(apply string-append
    	  (smap (lambda (n x)
                (cond
                  ((not (= 6 (length x)))
                    (inst-log "make-device : illegal device format : ~a\n" x)
                    (format #f "# illegal data : ~a\n\n" x))
                  (else
                    (let* ((bid (pci-expandbusid (car x))))
                      (format #f (string-append "Section \"Device\"\n"
                                                "    Identifier  \"Card~a\"\n"
                                                "    Driver      \"~a\"\n"
                                                "    BusID       \"PCI:~a:~a:~a\"\n"
                                                "    Screen      ~a\n"
                                                "    VendorName  \"SomeVendor\"\n"
                                                "    BoardName   \"~a\"\n"
                                                (adhoc-options x)
                                                "EndSection\n\n")
                                n
                                (list-ref x 5)
                                (car bid) (cadr bid) (caddr bid)
                                (cadr x)
                                (list-ref x 3))))))
              info-device)))
    
    (define (make-screen)
      (apply string-append
        (apply append
          (smap (lambda (n x)
                  (define (search-dev m l)
                    (cond
                      ((null? l) 100)
                      ((equal? n (caddr (car l)))
                        m)
                      (else
                        (search-dev (1+ m) (cdr l)))))
                  (let ((s (format #f "                Modes     \"~ax~a\"\n" (car x) (cadr x))))
                    (list "Section \"Screen\"\n"
                          (format #f "   Identifier \"Screen~a\"\n" n)
                          (format #f "   Device     \"Card~a\"\n" (search-dev 0 info-device))
                          (format #f "   Monitor    \"Monitor~a\"\n" n)
                          (format #f "   DefaultDepth ~a\n" info-color)
                          "    SubSection \"Display\"\n"
                          "        Depth     1\n"
                          "    EndSubSection\n"
                          "    SubSection \"Display\"\n"
                          "        Depth     4\n"
                          "    EndSubSection\n"
                          "    SubSection \"Display\"\n"
                          "        Depth     8\n"
                          (if (= 8 info-color) s "")
                          "    EndSubSection\n"
                          "    SubSection \"Display\"\n"
                          "        Depth     15\n"
                          "    EndSubSection\n"
                          "    SubSection \"Display\"\n"
                          "        Depth     16\n"
                          (if (= 16 info-color) s "")
                          "    EndSubSection\n"
                          "    SubSection \"Display\"\n"
                          "        Depth     24\n"
                          (if (= 24 info-color) s "")
                          "    EndSubSection\n"
                          "    SubSection \"Display\"\n"
                          "        Depth     32\n"
                          (if (= 32 info-color) s "")
                          "    EndSubSection\n"
                          "EndSection\n\n")))
                info-resolution))))
    
    (define (make-dri)
      (string-append "Section \"DRI\"\n"
                     "    Mode 0666\n"
                     "EndSection\n"))
    (cond
      ((or (null? device-db)
           (null? pci-db)
           (null? info-display)
           (null? info-device))
        #f)
      (else
        (fprintf f
          (string-append
            (let ((s (make-layout)))
              (inst-log "XF86Config : layout written : ~a\n" s)
              s)
            (make-files)
            (begin (inst-log "XF86Config : files written\n") "")
            (make-serverf)
            (let ((s (cond
                       (ext?
                         (string-append (make-module) (make-dri)))
                       (else
                         (make-module-test)))))
              (inst-log "module written : ~a\n" s)
              s)
            (make-inputdevice)
            (begin (inst-log "XF86Config : inputdevice written\n") "")
            (make-monitor)
            (begin (inst-log "XF86Config : monitor written\n") "")
            (let ((s (make-device)))
              (inst-log "XF86Config : device written : ~a\n" s)
              s)
            (let ((s (make-screen)))
              (inst-log "screen : ~a\n" s)
              s))))))
  
  (define (runx xopt . l)
    (let ((pid (unix-fork)))
      (cond
        ((= 0 pid)
          (let ((bin (case context
                       ((inst1 inst2 recovery) "/bin/XFree86")
                       (else "/usr/X11R6/bin/XFree86"))))
            (unix-exec2 bin (cons "XFree86" xopt))
            (exit 1)))
        (else
          (define (wait n)
            (cond
              ((< n 0)
                #f)
              ((unix-access? "/tmp/.X11-unix/X0" '(read))
                #t)
              (else
                (unix-usleep 500000)
                (inst-log "xobj : waiting ...\n")
                (wait (1- n)))))
          (cond
            ((wait 6)
              (let ((r (if (string? (car l))
                         (apply sh l)
                         (apply (car l) (cdr l)))))
                (unix-kill2 pid 'sigterm)
                r))
            (else
              -1))))))
  
  (define (test-old prog)
    (let ((tmp (if (equal? context 'inst2)
                   "/tmp/XF86Config"
                   (unix-tempname))))
      (make-etc tmp #f)
      (let ((p (unix-getenv "PATH")))
        (unix-setenv "PATH" (if (eq? #f p)
                                "/usr/sbin/ogl:/bin:/usr/bin:/sbin:/usr/sbin"
                                (string-append "/usr/sbin/ogl:" p))))
      (runx (list "-xf86config" tmp "-allowMouseOpenFail") (format #f "DISPLAY=:0.0 ~a" prog))))
  
  (define (test2 prog)
    (let ((tmp (if (equal? context 'inst2)
                   "/tmp/XF86Config"
                   (unix-tempname)))
          (tmp2 (unix-tempname)))
      (cond
        ((eq? #f (make-etc tmp #f))
          (_ "Could not write XF86Config."))
        ((eq? #f (unix-access? "/usr/X11R6/bin/xinit" '(exec)))
          (_ "Could not execute xinit."))
        ((not (string? prog))
          (_ "Argument must be a string."))
        (else
          (inst-log "xwin : LANG: ~a\n" (unix-getenv "LANG"))
          (fprintf tmp2 (format #f "#!/bin/sh\nLANG=~a\nexport LANG\n~a\n"
                          (let ((l (unix-getenv "LANG")))
                            (cond
                              ((or (eq? #f l)
                                   (< (string-length l) 2))
                                "C")
                              ((equal? "ja" (substring l 0 2))
                                "ja_JP.eucJP")
                              (else
                                "C")))
                           prog))
          (sh "chmod +x ~a" tmp2)
          (sh "cp ~a /tmp/samp" tmp2)
          ;(snewt-finish)
          (let* ((p (unix-getenv "PATH"))
                 (p' (if (eq? #f p)
                       "/usr/sbin/ogl:/bin:/usr/bin:/sbin:/usr/sbin:/usr/X11R6/bin"
                       (string-append "/usr/sbin/ogl:/usr/X11R6/bin:" p)))
                 (r (sh "PATH=~a XINITRC=~a xinit -- -xf86config ~a -allowMouseOpenFail >>/var/log/XFree86.0.log 2>&1"
                        p' tmp2 tmp)))
            ;(snewt-start)
            (sh "rm ~a" tmp2)
            r)))))
  
  (define (test l c)
    (let ((tmp (if (equal? context 'inst2)
                   "/tmp/XF86Config"
                   (unix-tempname))))
      (make-etc tmp #f)
      (let ((p (unix-getenv "PATH")))
        (unix-setenv "PATH" (if (eq? #f p)
                                "/usr/sbin/ogl:/bin:/usr/bin:/sbin:/usr/sbin"
                                (string-append "/usr/sbin/ogl:" p))))
      (runx (list "-xf86config" tmp "-allowMouseOpenFail") (lambda () (gtk-ui-xtest l c)))))
  
  (define (xinit . l)
    (apply runx (cons '("-logverbose" "9") l)))
  
  (define (set-mouse l)
    (set! info-mouse l))
  
  (define (get-driver bid)
    (let ((r (assoc bid info-device)))
      (cond
        ((not (list? r))
          (inst-log "xwin, get-driver : no such device ~a\n" bid)
          "vesa")
        (else
          (list-ref r 5)))))
  
  (define (set-driver bid drv)
    (set! info-device
      (map (lambda (x)
             (if (equal? bid (car x))
               (list bid (cadr x) (caddr x) (list-ref x 3) (list-ref x 4) drv)
               x))
           info-device))
    #t)
  
  (define (list-driver)
    (let ((l (unix-read-directory2 "/usr/X11R6/lib/modules/drivers")))
      (easy-sort string<?
        (map (lambda (x)
               (substring x 0 (- (string-length x) 6)))
          (filter (lambda (x)
                    (let ((len (string-length x)))
                      (and (< 6 len)
                           (equal? "_drv.o" (substring x (- len 6) len)))))
                  l)))))
  
  (define (get-resolution n)
    (if (< n (length info-resolution))
      (list-ref info-resolution n)
      '()))

  (define (get-hsync n)
    (let ((x (list-ref info-display n)))
      (list (cadr x) (caddr x))))
  
  (define (get-vsync n)
    (let ((x (list-ref info-display n)))
      (list (list-ref x 3) (list-ref x 4))))
  
  (define (valid-mode)
          ;     xdot ydot vsync  hsync
    (list (list  640  480   60    31.5  "")
          (list  640  480   75    37.5  "")
          (list  800  600   60    37.9  "")
          (list  800  600   75    46.9  "")
          (list  800  600   85    53.7  "")
          (list 1024  768   60    48.4  "")
          (list 1024  768   75    60.0  "")
          (list 1024  768   85    68.7  "")
          (list 1152  864   75    67.5  "")
          (list 1280  960   60    60.0  "")
          (list 1280  960   85    85.9  "")
          (list 1280 1024   60    64.0  "")
          (list 1280 1024   75    80.0  "")
          (list 1280 1024   85    91.1  "")
          (list 1400 1050   60    48.4  "\"1400x1050\" 122.0   1400 1488 1640 1880  1050 1052 1064 1082")
          (list 1400 1050   75    81.5  "\"1400x1050\" 155.8   1400 1464 1784 1912  1050 1052 1064 1090")
          (list 1400 1050   85    93.9  "\"1400x1050\" 184.0   1400 1464 1656 1960  1050 1051 1054 1100")
          (list 1600 1200   60    75.0  "")
          (list 1600 1200   65    81.3  "")
          (list 1600 1200   70    87.5  "")
          (list 1600 1200   75    93.8  "")
          (list 1600 1200   85   106.3  "")
          (list 1792 1344   60    83.6  "")
          (list 1792 1344   75   106.3  "")
          (list 1856 1392   60    86.3  "")
          (list 1856 1392   75   112.5  "")
          (list 1920 1200   73    91.0  "\"1920x1200\" 230.0   1920 1936 2096 2528  1200 1201 1204 1250")
          (list 1920 1440   60    90.0  "")
          (list 1920 1440   75   112.5  "")
          (list 1024  480   92    48.4  "\"1024x480\"  65      1024 1032 1176 1344  480  491  493  525")
          (list 1024  512   64    41.0  "\"1024x512\"  69.65   1280 1312 1472 1696  600  604  610  640")
          (list 1024  600   78    48.9  "\"1024x600\"  65.0    1024 1048 1184 1328  600  601  605  628")
          (list 1280  600   60    37.7  "\"1280x600\"  63.61   1280 1280 1384 1688  600  600  604  628")
          (list 1280  768   60    48.0  "\"1280x768\"  81      1280 1328 1440 1688  768  771  777  802")
          (list 1600 1024   60    64.0  "\"1600x1024\" 106.910 1600 1620 1640 1670  1024 1027 1030 1067")
          ))
  ; modeline $B$+$i<~GH?t$r7W;;$9$kJ}K!$O(B http://www.sra.co.jp/public/doc/bsd/X11/Xconfig $B$r;2>H$N$3$H(B
  
  
  (define (set-device-scr sid bid sopt)
    (let ((d (assoc bid info-device)))
      (cond
        ((null? d) #f)
        (else
          (set! info-device
            (cons (list bid sopt sid (list-ref d 3) (list-ref d 4) (list-ref d 5))
              (filter (lambda (x) (not (and (equal? bid (car x))
                                            (= sopt (cadr x)))))
                (map (lambda (x)
                       (if (equal? (caddr x) sid)
                         (list (car x) (cadr x) -1 (list-ref x 3) (list-ref x 4) (list-ref x 5))
                         x))
                     info-device))))
          #t))))
  
  (define (set-num-screen n)
    (define (list-null i)
      (cond
        ((< i 1) '())
        (else
          (cons '() (list-null (1- i))))))
    (set! info-num-screen n)
    (let ((m (- n (length info-resolution))))
      (set! info-resolution
        (append info-resolution (list-null m))))
    (let ((m (- n (length info-display))))
      (set! info-display
        (append info-display (list-null m)))))
  
  (define (set-layout x)
    (case x
      ((h v g)
        (set! info-layout x)
        #t)
      (else
        #f)))
  
  (define (desc-display n)
    (cond
      ((<= info-num-screen n)
        (_ "not exists"))
      (else
        (let ((d (filter (lambda (x) (equal? (caddr x) n))
                         info-device))
              (r (list-ref info-resolution n)))
          (cond
            ((or (not (= 1 (length d)))
                 (null? r))
              (_ "not yet set"))
            (else
              (format #f "~ax~a : ~a" (car r) (cadr r) (list-ref (car d) 3))))))))
  
  (define (set-keyboard k)
    (set! kbd-type k)
    #t)
  
  (make-object
    'avail-depth        avail-depth
    'commit             commit
    'commit-med         commit-med
    'color-name         color-name
    'conf               (lambda () (list (list "/etc/X11/XF86Config-4"
                                               (lambda (p) (make-etc p #t))
                                               #t)))
    'desc-display       desc-display
    'get-context        (lambda () context)
    'get-driver         get-driver
    'get-hsync          get-hsync
    'get-vsync          get-vsync
    'probe              probe-all
    'probe-lite         probe-lite
    'load               load-it
    'make-etc           (lambda (p) (make-etc p #f))
    'init               init
    'info-mem           (lambda () info-mem)
    'info-resolution    (lambda () info-resolution)
    'info               (lambda () device-db)
    'info-pci           (lambda () pci-db)
    'info-display       (lambda () info-display)
    'info-device        (lambda () info-device)
    'info-layout        (lambda () info-layout)
    'info-mouse         (lambda () info-mouse)
    'info-num-screen    (lambda () info-num-screen)
    'list-driver        list-driver
    'mid-conf           (lambda () (list "/var/lib/ogl/etc/x-win"))
    'set-color          set-color
    'set-driver         set-driver
    'set-device-scr     set-device-scr
    'set-hsync          set-hsync
    'set-keyboard       set-keyboard
    'set-vsync          set-vsync
    'set-resolution     set-resolution
    'set-layout         set-layout
    'set-modeline       set-modeline
    'set-mouse          set-mouse
    'set-num-screen     set-num-screen
    
    'test               test
    'test-old           test-old
    'test2              test2
    'xinit              xinit
    'pre-configured?    pre-configured?
    'valid-mode         valid-mode
  ))
